#!/bin/sh

# QLogic ISP2xxx/ISP4xxx device driver build script
# Copyright (C) 2003-2010 QLogic Corporation
# (www.qlogic.com)
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 2, or (at your option) any
# later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#

UNAME=`uname -a`
K_VERSION=`uname -r`
K_LIBS=/lib/modules/${K_VERSION}
K_BUILD_DIR=${K_LIBS}/build
K_SOURCE_DIR=${K_LIBS}/source
K_THREADS=5

SLES=/etc/SuSE-release
RHEL=/etc/redhat-release
UDEV_RULE_DIR=/etc/udev/rules.d
UDEV_RULE_FILE=99-qla2xxx.rules
UDEV_SCRIPT_DIR=/lib/udev
UDEV_SCRIPT=qla2xxx_udev.sh
UDEV_EARLY_RULE=/etc/udev/rules.d/05-udev-early.rules
UDEV_TMP_RULE=/tmp/tmp.rules

BOOTDIR="/boot"

Q_NODE=QLA2XXX
MODULE=qla2xxx
Q_CLEAN_FILES="Module.markers Module.symvers modules.order"
K_INSTALL_DIR=${K_LIBS}/extra/qlgc-qla2xxx/

if test -f "${SLES}" ; then
	K_INSTALL_DIR=${K_LIBS}/updates/
fi

if [ "`uname -m`" = "ia64" ]; then
        if [ -f "$RHEL" ]; then
                BOOTDIR="/boot/efi/efi/redhat"
        fi
fi

###
# drv_build -- Generic 'make' command for driver
#	$1 -- directive
#
set_variables() {
	if [ -f ./qla_os.c ]; then
		Q_NODE=QLA2XXX
		MODULE=qla2xxx
	elif [ -f ./ql4_os.c ]; then
		Q_NODE=QLA4XXX
		MODULE=qla4xxx
		UDEV_RULE_FILE=99-qla4xxx.rules
		UDEV_SCRIPT=qla4xxx_udev.sh
	elif [ -f ./ql4im_os.c ]; then
		Q_NODE=QISIOCTL
		MODULE=qisioctl
	fi

        K_INSTALL_DIR=${K_LIBS}/extra/qlgc-${MODULE}/
        if test -f "${SLES}" ; then
	        K_INSTALL_DIR=${K_LIBS}/updates/
        fi
}

drv_build() {
	test -z "$1" && return 1

	# Go with build...
	if test -f "${SLES}" ; then
		# SuSE -------------------------------------------------------
		make -j${K_THREADS} -C ${K_SOURCE_DIR} O=${K_BUILD_DIR} M=$PWD $1
	else
		# Redhat -----------------------------------------------------
		make -j${K_THREADS} -C ${K_BUILD_DIR} M=$PWD $1
	fi
}

udev_install()
{
	RET=0
	diff ./extras/${UDEV_RULE_FILE} ${UDEV_RULE_DIR}/${UDEV_RULE_FILE} &> /dev/null
	RET=$?
	diff ./extras/${UDEV_SCRIPT} ${UDEV_SCRIPT_DIR}/${UDEV_SCRIPT} &> /dev/null
	(( RET += $? ))

	if [ $RET -ne 0 ]; then
		echo "${Q_NODE} -- Installing udev rule to capture FW dump..."
		cp ./extras/${UDEV_RULE_FILE} ${UDEV_RULE_DIR}/${UDEV_RULE_FILE}
		cp ./extras/${UDEV_SCRIPT} ${UDEV_SCRIPT_DIR}/${UDEV_SCRIPT}

		# comment out the modules ignore_device rule
		if [ -e ${UDEV_EARLY_RULE} ]; then
			cp ${UDEV_EARLY_RULE} ${UDEV_EARLY_RULE}.bak
			cat ${UDEV_EARLY_RULE} | sed "s/\(^SUBSYSTEM==\"module\".*OPTIONS=\"ignore_device\".*\)/#\1/" > ${UDEV_TMP_RULE}
			if [ -s ${UDEV_TMP_RULE} ]; then
				mv -f ${UDEV_TMP_RULE} ${UDEV_EARLY_RULE}
			fi
		fi
		udevcontrol reload_rules
	else
		echo "${Q_NODE} -- udev rules already installed"
	fi
}

udev_remove()
{
		rm -f ${UDEV_RULE_DIR}/${UDEV_RULE_FILE} &> /dev/null
		rm -f ${UDEV_SCRIPT_DIR}/${UDEV_SCRIPT} &> /dev/null
		if [ -e ${UDEV_EARLY_RULE} ]; then
			cat ${UDEV_EARLY_RULE} |sed "s/\#\(SUBSYSTEM==\"module\".*OPTIONS=\"ignore_device\".*\)/\1/" > ${UDEV_TMP_RULE}
			if [ -s ${UDEV_TMP_RULE} ]; then
				echo "${Q_NODE} -- Removing FW capture udev rule..."
				mv -f ${UDEV_TMP_RULE} ${UDEV_EARLY_RULE}
				udevcontrol reload_rules
			fi
		fi
}

###
# drv_install -- Generic steps for installation
#
drv_install() {
	if test $EUID -ne 0 ; then
		echo "${Q_NODE} -- Must be root to install..."
		return 1
	fi

	#backup all modules except the one in default path
	for module in `find /lib/modules/$K_VERSION -name $MODULE.ko`
	do
		echo $module | grep "scsi" >& /dev/null
		if [ $? -ne 0 ]; then
			mv $module $module.org
		fi
	done

	echo "${Q_NODE} -- Installing the $MODULE modules to ${K_INSTALL_DIR}..."
	install -d -o root -g root ${K_INSTALL_DIR}
	install -o root -g root -m 0644 *.ko ${K_INSTALL_DIR}

	# depmod
	/sbin/depmod -a

	#install the udev rules to capture FW dump
	udev_install
}

###
# drv_tar -- Create tarball from versioned directory
#
drv_tar() {
        Q_DOC_FILES="LICENSE.*"
	Q_DOC_DIR=$PWD/../../../Documentation/scsi
	Q_DRVR_FILES="Kconfig Makefile ql?_*.h ql?_*.c ql4im_*.h ql4im_*.c qli*.h"

	if [ -f ./qla_os.c ]; then
                Q_VER_FILE=qla_version.h
	        Q_VER_STR="QLA2XXX_.*VERSION"
	elif [ -f ./ql4_os.c ]; then
		Q_VER_FILE=ql4_version.h
		Q_VER_STR="QLA4XXX_.*VERSION"
	elif [ -f ./ql4im_os.c ]; then
                Q_VER_FILE=ql4im_version.h
	        Q_VER_STR="QL4IM_VERSION"
	        Q_DRVR_FILES="Kconfig Makefile ql?_*.h ql4im_*.h ql4im_*.c qli*.h"
	fi

	if [ -f "$Q_VER_FILE" ] ; then
	        # Extract version number from version.h file
	       	local VER_STR=`cat $Q_VER_FILE | grep $Q_VER_STR |
		awk 'BEGIN {FS="\""} {print $2}'`

		local VRSND_DIR="$MODULE-v$VER_STR"
		local VRSND_TAR="$MODULE-src-v$VER_STR.tar.gz"

		if [ -n "$VRSND_DIR" ] ; then
			if [ -d "$PWD/$VRSND_DIR" ] ; then
				echo Removing old directory $PWD/$VRSND_DIR
				rm -rf $PWD/$VRSND_DIR
			fi

			# Create versioned directory
			mkdir -p $PWD/$VRSND_DIR/extras

			# Copy files to versioned directory
			cp extras/* $PWD/$VRSND_DIR/extras
			for i in $Q_DRVR_FILES
			do
				cp $i $VRSND_DIR
			done

			if [ -d "$Q_DOC_DIR" ] ; then
			   cp $Q_DOC_DIR/$Q_DOC_FILES $VRSND_DIR
			elif [ -f "./$Q_DOC_FILES" ]; then
			   cp $Q_DOC_FILES $VRSND_DIR
			fi

			# Create versioned tarball
			tar czvf $VRSND_TAR $VRSND_DIR/*
			echo Created tarball $VRSND_TAR
		else
			echo Tarball not created - Version string not detected.
		fi
	else
		echo Tarball not created - Version file not located.
	fi
}

###
#
#
case "$1" in
    -h | help)
	echo "QLogic Corporation -- driver build script"
	echo ""
	echo "    Execute from within driver source directory:"
	echo "        # cd <driver source>"
	echo "        # ./extras/build.sh <directive>"
	echo ""
	echo "    Display this help message"
	echo ""
	echo "        # ./extras/build.sh help"
	echo "        # ./extras/build.sh -h"
	echo ""
	echo "    Build the driver sources"
	echo ""
	echo "        # ./extras/build.sh"
	echo ""
	echo "    Clean driver source directory of all build files (i.e. "
	echo "    *.ko, *.o, etc)."
	echo ""
	echo "        # ./extras/build.sh clean"
	echo ""
	echo "    Rebuild the driver sources from scratch (i.e. clean "
	echo "    before build)."
	echo ""
	echo "        # ./extras/build.sh new"
	echo ""
	echo "    Create a versioned tarball from version number found in "
	echo "    version.h."
	echo ""
	echo "        # ./extras/build.sh tar"
	echo ""
	echo "     Build and install the driver module files."
	echo "     This command performs the following:"
	echo ""
	echo "       # ./extras/build.sh install"
	echo ""
	echo "        1. Builds the driver .ko files."
	echo "        2. Copies the .ko files to the appropriate "
	echo "           /lib/modules/... directory."
        echo ""
        echo "     Remove/uninstall the driver module files."
        echo "     This command performs the following:"
        echo ""
	echo "       # ./extras/build.sh remove"
        echo ""
        echo "     Build, install, and update the initrd image."
        echo "     This command performs the following:"
        echo ""
        echo "       # ./extras/build.sh initrd"
        echo ""
        echo "        1. All steps in the 'install' directive."
        echo "        2. Rebuilds the initrd image with the /sbin/mk_initrd"
        echo "           command."
        echo ""
       ;;
    -i | install)
	set_variables
	echo "${Q_NODE} -- Building the $MODULE driver..."
	drv_build modules
	drv_install
	;;
    -r | remove)
	set_variables
	echo "${Q_NODE} -- Removing the $MODULE driver..."
	if  [ -f ${K_INSTALL_DIR}/$MODULE.ko ]; then
		rm ${K_INSTALL_DIR}/$MODULE.ko
		/sbin/depmod -a
		echo "${Q_NODE} -- Rebuilding INITRD image..."
		if test -f "${SLES}" ; then
			if [ -f /sbin/mk_initrd ]; then
				/sbin/mk_initrd
			elif [ -f /sbin/mkinitrd ]; then
				/sbin/mkinitrd
			else
				echo "${Q_NODE} -- Unable to find mkinitrd command..."
				echo "${Q_NODE} -- Skipping rebuilding of INITRD image..."
			fi

		else
			if [ -f ${BOOTDIR}/initrd-${K_VERSION}.img ]; then
				/sbin/mkinitrd -f ${BOOTDIR}/initrd-${K_VERSION}.img ${K_VERSION}
			else
				/sbin/mkinitrd ${BOOTDIR}/initrd-${K_VERSION}.img ${K_VERSION}
			fi
		fi
		udev_remove
	fi
	;;
    initrd)
	set_variables
	echo "${Q_NODE} -- Building the $MODULE driver..."
	drv_build modules
	drv_install

	echo "${Q_NODE} -- Rebuilding INITRD image..."
	if test -f "${SLES}" ; then
		if [ ! -f ${BOOTDIR}/initrd-${K_VERSION}.bak ]; then
			cp ${BOOTDIR}/initrd-${K_VERSION} ${BOOTDIR}/initrd-${K_VERSION}.bak
		fi
		if [ -f /sbin/mk_initrd ]; then
			/sbin/mk_initrd
		elif [ -f /sbin/mkinitrd ]; then
			/sbin/mkinitrd
		else
			echo "${Q_NODE} -- Unable to find mkinitrd command..."
			echo "${Q_NODE} -- Skipping rebuilding of INITRD image..."
		fi

	else
		if [ -f ${BOOTDIR}/initrd-${K_VERSION}.img ]; then
			if [ ! -f ${BOOTDIR}/initrd-${K_VERSION}.bak.img ]; then
				cp ${BOOTDIR}/initrd-${K_VERSION}.img ${BOOTDIR}/initrd-${K_VERSION}.bak.img
			fi
			/sbin/mkinitrd -f ${BOOTDIR}/initrd-${K_VERSION}.img ${K_VERSION}
		else
			/sbin/mkinitrd ${BOOTDIR}/initrd-${K_VERSION}.img ${K_VERSION}
		fi
	fi
	;;
    clean)
	echo "${Q_NODE} -- Cleaning driver build directory..."
	drv_build clean
	#Remove these extra files after build clean completes
	for i in $Q_CLEAN_FILES
	do
	    if test -f $i ; then
		rm $i
	    fi
	done
	;;
    new)
	set_variables
	echo "${Q_NODE} -- Clean rebuild of the $MODULE driver..."
	drv_build clean
	drv_build modules
	;;
    tar)
	set_variables
	echo "${Q_NODE} -- Build tarball from versioned source directory..."
	drv_tar
	;;
    *)
	set_variables
	echo "${Q_NODE} -- Building the $MODULE driver..."
	drv_build modules
	;;
esac
