/*
 * QLogic iSCSI HBA Driver
 * Copyright (c)  2003-2012 QLogic Corporation
 *
 * See LICENSE.qla4xxx for copyright and licensing details.
 */

#ifndef __QL4_DEF_H
#define __QL4_DEF_H

#include <linux/version.h>
#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/types.h>
#include <linux/module.h>
#include <linux/list.h>
#include <linux/pci.h>
#include <linux/dma-mapping.h>
#include <linux/sched.h>
#include <linux/slab.h>
#include <linux/dmapool.h>
#include <linux/mempool.h>
#include <linux/spinlock.h>
#include <linux/completion.h>
#include <linux/delay.h>
#include <linux/interrupt.h>
#include <asm/semaphore.h>
#include <linux/string.h>
#include <linux/in6.h>

#include <scsi/scsi.h>
#include <scsi/scsi_host.h>
#include <scsi/scsi_device.h>
#include <scsi/scsi_cmnd.h>

#include "ql4_nx.h"
#include "ql4_83xx.h"
#include "ql4_kcompat.h"

#if defined(CONFIG_PCIEAER)
#include <linux/aer.h>
#else
/* AER releated */
static inline int pci_enable_pcie_error_reporting(struct pci_dev *dev)
{
	return -EINVAL;
}
static inline int pci_disable_pcie_error_reporting(struct pci_dev *dev)
{
	return -EINVAL;
}
static inline int pci_cleanup_aer_uncorrect_error_status(struct pci_dev *dev)
{
	return -EINVAL;
}
#endif

/* XXX(dg): move to pci_ids.h */
#ifndef PCI_DEVICE_ID_QLOGIC_ISP4010
#define PCI_DEVICE_ID_QLOGIC_ISP4010	0x4010
#endif

#ifndef PCI_DEVICE_ID_QLOGIC_ISP4022
#define PCI_DEVICE_ID_QLOGIC_ISP4022	0x4022
#endif

#ifndef PCI_DEVICE_ID_QLOGIC_ISP4032
#define PCI_DEVICE_ID_QLOGIC_ISP4032	0x4032
#endif

#ifndef PCI_DEVICE_ID_QLOGIC_ISP8022
#define PCI_DEVICE_ID_QLOGIC_ISP8022	0x8022
#endif

#ifndef PCI_DEVICE_ID_QLOGIC_ISP8324
#define PCI_DEVICE_ID_QLOGIC_ISP8324	0x8032
#endif

#define IS_QLA4010(ha)	((ha)->pdev->device == PCI_DEVICE_ID_QLOGIC_ISP4010)
#define IS_QLA4022(ha)	((ha)->pdev->device == PCI_DEVICE_ID_QLOGIC_ISP4022)
#define IS_QLA4032(ha)	((ha)->pdev->device == PCI_DEVICE_ID_QLOGIC_ISP4032)
#define IS_QLA8022(ha)	((ha)->pdev->device == PCI_DEVICE_ID_QLOGIC_ISP8022)
#define IS_QLA8032(ha)	((ha)->pdev->device == PCI_DEVICE_ID_QLOGIC_ISP8324)
#define IS_QLA40XX(ha)	(IS_QLA4010(ha) || IS_QLA4022(ha) || IS_QLA4032(ha))
#define IS_QLA80XX(ha)	(IS_QLA8022(ha) || IS_QLA8032(ha))


/* Note: Currently AER/EEH is now supported only for 8022 and 8032 cards
 * This function needs to be updated when AER/EEH is enabled
 * for other cards.
 */
#define IS_AER_SUPPORTED(ha)	\
		(((ha)->pdev->device == PCI_DEVICE_ID_QLOGIC_ISP8022) || \
		 ((ha)->pdev->device == PCI_DEVICE_ID_QLOGIC_ISP8324))

#if defined(CONFIG_COMPAT) && !defined(CONFIG_IA64)
#define QLA_CONFIG_COMPAT
#endif

/*
 * This file set some defines that are required to compile the
 * command source for 4000 module
 *----------------------------------------------------------------------------*/
#define QLA4010
#define QLA4XXX_BOARD_PORTS		1
#define QLA4XXX_PROC_NAME		"qla4010"

#define MEMORY_MAPPED_IO		1 /* 1=Memory Mapped (preferred),
					   * 0=I/O Mapped */

#define LINESIZE		256
#define MIN(x,y)		((x)<(y)?(x):(y))
#define MAX(x,y)		((x)>(y)?(x):(y))

/*
 * Return status codes for internal routines
 ********************************************/
#define QLA_SUCCESS			0
#define QLA_ERROR			1
#define QLA_STATUS_DATA_UNDERRUN	2

/*
 * Data bit definitions
 */
#define BIT_0	0x1
#define BIT_1	0x2
#define BIT_2	0x4
#define BIT_3	0x8
#define BIT_4	0x10
#define BIT_5	0x20
#define BIT_6	0x40
#define BIT_7	0x80
#define BIT_8	0x100
#define BIT_9	0x200
#define BIT_10	0x400
#define BIT_11	0x800
#define BIT_12	0x1000
#define BIT_13	0x2000
#define BIT_14	0x4000
#define BIT_15	0x8000
#define BIT_16	0x10000
#define BIT_17	0x20000
#define BIT_18	0x40000
#define BIT_19	0x80000
#define BIT_20	0x100000
#define BIT_21	0x200000
#define BIT_22	0x400000
#define BIT_23	0x800000
#define BIT_24	0x1000000
#define BIT_25	0x2000000
#define BIT_26	0x4000000
#define BIT_27	0x8000000
#define BIT_28	0x10000000
#define BIT_29	0x20000000
#define BIT_30	0x40000000
#define BIT_31	0x80000000

/*
 * Macros to help code, maintain, etc.
 */
#define ql4_printk(level, ha, format, arg...) \
	dev_printk(level , &((ha)->pdev->dev) , format , ## arg)

#define ql4_info(ha, format, arg...)    \
	ql4_printk(KERN_INFO, ha, format, ## arg)
#define ql4_warn(ha, format, arg...)    \
	ql4_printk(KERN_WARNING, ha, format, ## arg)
#define ql4_err(ha, format, arg...)     \
	ql4_printk(KERN_ERR, ha, format, ## arg)
#define ql4_dbg(ha, format, arg...)     \
	ql4_printk(KERN_DEBUG, ha, format, ## arg)

/*
 * Host adapter default definitions
 ***********************************/
#define MAX_HBAS			16
#define MAX_BUSES       		1
#define MAX_TARGETS     		MAX_PRST_DEV_DB_ENTRIES + MAX_DEV_DB_ENTRIES
#define MAX_LUNS        		256
#define MAX_AEN_ENTRIES 		256 /* should be > EXT_DEF_MAX_AEN_QUEUE */
#define MAX_DDB_ENTRIES 		MAX_PRST_DEV_DB_ENTRIES + MAX_DEV_DB_ENTRIES
#define MAX_PDU_ENTRIES                 32
#define INVALID_ENTRY			0xFFFF
#define MAX_CMDS_TO_RISC		1024
#define MAX_SRBS			MAX_CMDS_TO_RISC
#define MBOX_AEN_REG_COUNT		8
#define MAX_INIT_RETRIES		5
#define IOCB_HIWAT_CUSHION		16
#define LEGACY_IFCB_SIZE		0x200
#define MIN_IOCBS			64
#define MAX_IOCBS			1024

/*
 * Buffer sizes
 ***************/
#define REQUEST_QUEUE_DEPTH       	MAX_CMDS_TO_RISC
#define RESPONSE_QUEUE_DEPTH      	64
#define RESPONSE_QUEUE_DEPTH_ISP8XXX	1024
#define QUEUE_SIZE			64
#define DMA_BUFFER_SIZE 		512

/*
 * Misc
 *******/
#define MAC_ADDR_LEN			6 /* in bytes */
#define IP_ADDR_LEN			4 /* in bytes */
#define DRIVER_NAME			"qla4xxx"

#define MAX_LINKED_CMDS_PER_LUN		3
#define MAX_REQS_SERVICED_PER_INTR	1


/* Number of seconds to subtract for internal command timer */
#define QLA_CMD_TIMER_DELTA             5


#define	ISCSI_IPADDR_SIZE		4	/* IP address size */
#define	ISCSI_IPv6ADDR_SIZE		16	/* IPv6 address size */
#define	ISCSI_ALIAS_SIZE		32	/* ISCSI Alais name size */
#define	ISCSI_NAME_SIZE			255	/* ISCSI Name size  - usually a string */
#define IP_ADDR_STR_SIZE		40

#define SYS_DELAY(x)		do {udelay(x);barrier();} while(0);
#define QLA4XXX_DELAY(sec)  	do {mdelay(sec * 1000);} while(0);
#define NVRAM_DELAY() 		do {udelay(500);} while(0); /* 500 microsecond delay */

/* delay 30 seconds */
#define RESET_DELAY()		do {int delay; for(delay=30; delay!=0; delay--) \
				{current->state = TASK_UNINTERRUPTIBLE; \
				schedule_timeout(1 * HZ);}} while(0);

#define LSB(x)	((uint8_t)(x))
#define MSB(x)	((uint8_t)((uint16_t)(x) >> 8))
#define LSW(x)	((uint16_t)(x))
#define MSW(x)	((uint16_t)((uint32_t)(x) >> 16))
#define LSDW(x)	((uint32_t)((uint64_t)(x)))
#define MSDW(x)	((uint32_t)((((uint64_t)(x)) >> 16) >> 16))

#define IPv4AddrIsZero( _X1_ )   ((_X1_)[0] == 0 && \
                                  (_X1_)[1] == 0 && \
                                  (_X1_)[2] == 0 && \
                                  (_X1_)[3] == 0)

#define IPv4AddrIsEqual(_X1_, _X2_) ((_X1_)[0] == (_X2_)[0] && \
                                     (_X1_)[1] == (_X2_)[1] && \
                                     (_X1_)[2] == (_X2_)[2] && \
                                     (_X1_)[3] == (_X2_)[3])

#define IPv4Addr2Str(addr, str) { sprintf(str, "%d.%d.%d.%d", \
				  addr[0], addr[1], addr[2], addr[3]);}

#define IPv4CompatIPv6Addr(_X1_) ((_X1_)[0] == 0 && \
				  (_X1_)[1] == 0 && \
				  (_X1_)[2] == 0 && \
				  (_X1_)[3] == 0 && \
				  (_X1_)[4] == 0 && \
				  (_X1_)[5] == 0 && \
				  (_X1_)[6] == 0 && \
				  (_X1_)[7] == 0 && \
				  (_X1_)[8] == 0 && \
				  (_X1_)[9] == 0 && \
				  (_X1_)[10] == 0 && \
				  (_X1_)[11] == 0 && \
				  ((_X1_)[12] != 0 || \
				   (_X1_)[13] != 0 || \
				   (_X1_)[14] != 0 || \
				   (_X1_)[15] != 0 ))

#define IPv4MappedIPv6Addr(_X1_) ((_X1_)[0] == 0 && \
				  (_X1_)[1] == 0 && \
				  (_X1_)[2] == 0 && \
				  (_X1_)[3] == 0 && \
				  (_X1_)[4] == 0 && \
				  (_X1_)[5] == 0 && \
				  (_X1_)[6] == 0 && \
				  (_X1_)[7] == 0 && \
				  (_X1_)[8] == 0 && \
				  (_X1_)[9] == 0 && \
				  (_X1_)[10] == 0xFF && \
				  (_X1_)[11] == 0xFF)

#define IPv6Addr2Str(addr, str) { \
				if ((IPv4MappedIPv6Addr(addr)) \
				|| (IPv4CompatIPv6Addr(addr))) \
				sprintf(str, "%d.%d.%d.%d", \
				addr[12], addr[13], addr[14], addr[15]);\
				else \
				sprintf(str, \
				"%02x%02x:%02x%02x:%02x%02x:%02x%02x:" \
				"%02x%02x:%02x%02x:%02x%02x:%02x%02x", \
				addr[0], addr[1], addr[2], addr[3], \
				addr[4], addr[5], addr[6], addr[7],	\
				addr[8], addr[9], addr[10], addr[11], \
				addr[12], addr[13], addr[14], addr[15]);}

#define IPv6AddrIsZero( _X1_ )   ((_X1_)[0] == 0 && (_X1_)[1] == 0 && \
				  (_X1_)[2] == 0 && (_X1_)[3] == 0 && \
				  (_X1_)[4] == 0 && (_X1_)[5] == 0 && \
				  (_X1_)[6] == 0 && (_X1_)[7] == 0 && \
				  (_X1_)[8] == 0 && (_X1_)[9] == 0 && \
				  (_X1_)[10] == 0 && (_X1_)[11] == 0 && \
				  (_X1_)[12] == 0 && (_X1_)[13] == 0 && \
                                  (_X1_)[14] == 0 && (_X1_)[15] == 0)

#define IPv6AddrIsEqual(_X1_, _X2_) ((_X1_)[0] == (_X2_)[0] && \
                                     (_X1_)[1] == (_X2_)[1] && \
                                     (_X1_)[2] == (_X2_)[2] && \
                                     (_X1_)[3] == (_X2_)[3] && \
                                     (_X1_)[4] == (_X2_)[4] && \
                                     (_X1_)[5] == (_X2_)[5] && \
                                     (_X1_)[6] == (_X2_)[6] && \
                                     (_X1_)[7] == (_X2_)[7] && \
                                     (_X1_)[8] == (_X2_)[8] && \
                                     (_X1_)[9] == (_X2_)[9] && \
                                     (_X1_)[10] == (_X2_)[10] && \
                                     (_X1_)[11] == (_X2_)[11] && \
                                     (_X1_)[12] == (_X2_)[12] && \
                                     (_X1_)[13] == (_X2_)[13] && \
                                     (_X1_)[15] == (_X2_)[15] && \
                                     (_X1_)[15] == (_X2_)[15])

#define IPAddr2Str(ha, addr, str) { \
				if (is_ipv6_enabled(ha)) \
				     IPv6Addr2Str(addr, str) \
				else IPv4Addr2Str(addr, str);}

#define IPAddrIsZero(ha, _X1_) ( \
				(is_ipv6_enabled(ha)) \
				? (IPv6AddrIsZero(_X1_)) \
				: (IPv4AddrIsZero(_X1_)))

#define IPAddrIsEqual(ha, _X1_, _X2_) ( \
				(is_ipv6_enabled(ha)) \
				? (IPv6AddrIsEqual(_X1_, _X2_)) \
				: (IPv4AddrIsEqual(_X1_, _X2_)))

/*
 * I/O port access macros
 *************************/
#if MEMORY_MAPPED_IO
#   define RD_REG_BYTE(addr)	     readb(addr)
#   define RD_REG_WORD(addr)         readw(addr)
#   define RD_REG_DWORD(addr)        readl(addr)
#   define RD_REG_WORD_RELAXED(addr) 	readw_relaxed(addr)
#   define RD_REG_DWORD_RELAXED(addr)	readl_relaxed(addr)
#   define WRT_REG_BYTE(addr, data)  writeb(data, addr)
#   define WRT_REG_WORD(addr, data)  writew(data, addr)
#   define WRT_REG_DWORD(addr, data) writel(data, addr)
#else
#   define RD_REG_BYTE(addr)	     (inb((u_long)addr))
#   define RD_REG_WORD(addr)         (inw((u_long)addr))
#   define RD_REG_DWORD(addr)        (inl((u_long)addr))
#   define WRT_REG_BYTE(addr, data)  (outb(data,(u_long)addr))
#   define WRT_REG_WORD(addr, data)  (outw((data),(u_long)addr))
#   define WRT_REG_DWORD(addr, data) (outl((data),(u_long)addr))
#endif

#define PCI_POSTING(a) (RD_REG_DWORD(a))

#include "ql4_os.h"
#include "ql4_fw.h"
#include "ql4_nvram.h"
#include "ql4_settings.h"

#include "ql4_isns.h"

/*---------------------------------------------------------------------------*/

/*
 * Retry & Timeout Values
 *************************/
#define MBOX_TOV			60
#define SOFT_RESET_TOV			30
#define RESET_INTR_TOV			3
#define SEMAPHORE_TOV			10
#define ADAPTER_INIT_TOV		30
#define ADAPTER_RESET_TOV		180
#define ADAPTER_ONLINE_TOV		30
#define INTERNAL_PASSTHRU_TOV		60
#define EXTEND_CMD_TOV			60
#define WAIT_CMD_TOV			30
#define EH_WAIT_CMD_TOV			10
#define FIRMWARE_UP_TOV			60
#define RESET_FIRMWARE_TOV        	30
#define LOGOUT_TOV			10
#define IOCB_TOV_MARGIN			10
#define RELOGIN_TOV			18
#define ISNS_DEREG_TOV			5
#define MIN_CMD_TOV			25
#define ISP_SEM_WAIT_TOV		10
#define PDU_WAIT_TOV			10

#define MAX_RESET_HA_RETRIES		2
#define DEVICE_ONLINE_TOV		10
#define FW_ALIVE_WAIT_TOV		3
#define QSNT_ACK_TOV			30

/*---------------------------------------------------------------------------*/
/*
 * SCSI Request Block structure  (srb)  that is placed
 * on cmd->SCp location of every I/O     [We have 22 bytes available]
 */
typedef struct _srb_t {
	struct list_head   list_entry;	/* (8)   */
	struct scsi_qla_host *ha;	/* (4) HA the SP is queued on */

	uint16_t     flags;		/* (2) Status flags. */
	#define SRB_DMA_VALID		BIT_3	/* DMA Buffer mapped. */

	#define SRB_GOT_SENSE		BIT_4	/* sense data recieved. */
	#define SRB_IOCTL_CMD		BIT_5	/* generated from an IOCTL. */
	#define SRB_BUSY		BIT_7	/* in busy retry state. */

	#define SRB_RETRY		BIT_9	/* needs retrying. */
	#define SRB_TAPE		BIT_10	/* FCP2 (Tape) command. */
	#define SRB_NO_TIMER		BIT_11


	uint8_t     state;		/* (1) Status flags. */
	#define SRB_NO_QUEUE_STATE	 0	/* Request is in between states */
	#define SRB_FREE_STATE		 1
	#define SRB_PENDING_STATE	 2
	#define SRB_ACTIVE_STATE	 3
	#define SRB_ACTIVE_TIMEOUT_STATE 4
	#define SRB_RETRY_STATE	 	 5
	#define SRB_DONE_STATE	 	 6
	#define SRB_SUSPENDED_STATE	 7	/* Request in suspended state */

	#define SRB_STATE_TBL()	  	  \
	{	    			  \
            "NO_QUEUE"	        	, \
            "FREE"		        , \
            "PENDING"	        	, \
	    "ACTIVE"	        	, \
	    "ACTIVE_TIMEOUT"        	, \
	    "RETRY"	        	, \
	    "DONE"	        	, \
	    NULL			  \
	}

	uint8_t     entry_count;	/* (1) number of request queue
					 *     entries used */
	struct scsi_cmnd  *cmd;		/* (4) SCSI command block */
	dma_addr_t  saved_dma_handle;	/* (4) for unmap of single transfers */
	atomic_t    ref_count;		/* (4) reference count for this srb */
	uint32_t    fw_ddb_index;	/* (4) */

	/* Target/LUN queue pointers. */
	struct os_tgt *tgt_queue;	/* (4) ptr to visible ha's target */
	struct os_lun *lun_queue;	/* (4) ptr to visible ha's lun */
	struct fc_lun *fclun;		/* (4) FC LUN context pointer. */

	/* Raw completion info for use by failover ? */
	uint8_t rsvd1;
	uint8_t err_id;			/* (1) error id */
	#define SRB_ERR_PORT       1    /* Request failed because "port down" */
	#define SRB_ERR_LOOP       2    /* Request failed because "loop down" */
	#define SRB_ERR_DEVICE     3    /* Request failed because "device error" */
	#define SRB_ERR_OTHER      4

	uint16_t    rsvd2;		/* (2) */
	uint32_t    lun;		/* (4) */
	struct timer_list timer;	/* (32) used to timeout command */
	uint16_t    os_tov;		/* (2) */
	uint16_t    iocb_tov;           /* (2) */
	uint16_t    rsvd3;		/* (2) Number of used iocbs */
	uint16_t    cc_stat;            /* (2) */
	u_long      r_start;	      	/* (4) Time we recieve a cmd from OS*/
	u_long      u_start;	      	/* (4) Time when we handed the cmd to F/W */

	/* Used for extended sense / status continuation */
	uint8_t *req_sense_ptr;
	uint16_t req_sense_len;
	uint16_t reserved2;
} srb_t;

/*
 * SCSI Target Queue structure
 */
typedef struct os_tgt {
	struct os_lun		*olun[MAX_LUNS];	 /* LUN context pointer. */
	struct scsi_qla_host	*ha;
	uint32_t		down_timer;
	struct fc_port		*fcport;		/* Current fcport for this target */
	unsigned long		flags;
	uint8_t			port_down_retry_count;
	uint8_t			id;

	/* Persistent binding information */
	uint16_t		ddb_index;
	uint8_t			iscsi_name[ISCSI_NAME_SIZE];
	uint8_t			*name;
} os_tgt_t;

/*
 * SCSI Target Queue flags
 */
#define TQF_ONLINE		0		/* Device online to OS. */
#define TQF_SUSPENDED		1
#define TQF_RETRY_CMDS		2

/*
 * LUN structure
 */
typedef struct os_lun {
	struct fc_lun *fclun;		/* (4) FC LUN context pointer. */
	struct list_head list_entry;	/* (8) For suspended lun list */
	struct scsi_device *sdev;	/* (4) */

	spinlock_t lun_lock  ____cacheline_aligned; /* (4) For suspended lun list */
	unsigned long flags;		/* 4 */
	#define LF_LUN_DELAYED		0
	#define LF_LUN_SUSPEND		1
	#define LF_LUN_BLOCKED		2
	#define LUN_MPIO_RESET_CNTS	3	/* Lun */

	uint8_t lun_state;		/* (1) */
	#define LS_LUN_READY		0   /* LUN is ready to accept commands */
	#define LS_LUN_SUSPENDED	1   /* LUN is suspended */
	#define LS_LUN_RETRY		2   /* LUN is retrying commands */
	#define LS_LUN_TIMEOUT		3   /*  */
	#define LUN_STATE_TBL()		  \
	{				  \
		"READY"			, \
		"SUSPENDED"		, \
		"RETRY"			, \
		"TIMEOUT"		, \
		NULL			  \
	}

	uint8_t out_count;		/* (1) Number of outstanding commands */
	uint8_t lun;			/* (1) Lun number */

	uint8_t retry_count;		/* (1) Number of times lun is suspended */
	uint8_t max_retry_count;	/* (1) Max number of times lun can be */
					/*     suspended before returning commands */
	uint8_t reserved[3];		/* (3) */
	uint32_t tot_io_count;		/* (4) Total num outstanding I/Os */
	atomic_t suspend_timer;		/* (4) Timer for suspending lun */
} os_lun_t;

/* Never set this to Zero */
#define SUSPEND_SECONDS	6
#define SUSPEND_RETRIES	1

/* LUN BitMask structure definition, array of 32bit words,
 * 1 bit per lun.  When bit == 1, the lun is masked.
 * Most significant bit of mask[0] is lun 0, bit 24 is lun 7.
 */
typedef struct lun_bit_mask {
	/* Must allocate at least enough bits to accomodate all LUNs */
#if ((MAX_LUNS & 0x7) == 0)
	uint8_t   mask[MAX_LUNS >> 3];
#else
	uint8_t mask[(MAX_LUNS + 8) >> 3];
#endif
} lun_bit_mask_t;

/*---------------------------------------------------------------------------*/

/*
 * Device Database (DDB) structure
 */

typedef struct ddb_entry {
	struct list_head   list_entry;	/* 00 x00 */
	uint16_t bus;			/* 08 x08 SCSI bus number */
	uint16_t target;		/* 10 x0a SCSI target ID */
	struct fc_port *fcport;		/* 12 x0c */
	uint16_t fw_ddb_index;		/* 16 x10 DDB index from firmware's DEV_DB structure */
	uint16_t out_count;		/* 18 x12 Number of active commands */
	uint8_t  num_valid_luns;	/* 20 x14 Number of valid luns */
	uint16_t options;		/* 21 x15 */
	uint8_t  reserved1;		/* 23 x17 */

	/* refer to MBOX_CMD_GET_DATABASE_ENTRY for fw_ddb_fw_ddb_device_state definitions   */
	uint32_t fw_ddb_device_state;	/* 24 x18 Device State */
	#define DDB_STATE_TBL(){	\
                "UNASSIGNED",           \
		"NO_CONNECTION_ACTIVE", \
		"DISCOVERY",            \
		"NO_SESSION_ACTIVE",    \
		"SESSION_ACTIVE",       \
		"LOGGING_OUT",          \
		"SESSION_FAILED",       \
		NULL                    \
	}
	uint32_t CmdSn;			/* 28 x1c */
	uint16_t target_session_id;	/* 32 x20 */
	uint16_t connection_id;		/* 34 x22 */
	uint16_t exe_throttle;		/* 36 x24 Max mumber of cmds outstanding simultaneously */
	uint16_t task_mgmt_timeout;	/* 38 x26 Min time for task mgmt cmds to complete */
	uint16_t default_relogin_timeout; /*40 x28 Max time to wait for relogin to complete */
	uint16_t tcp_source_port_num;	/* 42 x2a */
	uint32_t default_time2wait;	/* 44 x2c Default Min time between relogins (+aens) */
	atomic_t port_down_timer;	/* 48 x30 Device down time */
	atomic_t retry_relogin_timer;	/* 52 x34 Min Time between relogins (4000 only)*/
	atomic_t relogin_timer;		/* 56 x38 Max Time to wait for relogin to complete */
	atomic_t relogin_retry_count;	/* 60 x3c Num of times relogin has been retried */
	atomic_t state;			/* 64 x40 Device State*/
	#define DEV_STATE_DEAD		0 /* We can no longer talk to this device */
	#define DEV_STATE_ONLINE	1 /* Device ready to accept commands */
	#define DEV_STATE_MISSING	2 /* Device logged off, trying to re-login */
	#define DEV_STATE_DELETED	3 /* Deleted by GUI */
	#define DEV_STATE_TBL(){	  \
		"DEAD"			, \
		"ONLINE"		, \
		"MISSING"		, \
		"DELETED"		, \
		NULL			  \
	}
	unsigned long flags;		/* 68 x44 */
	#define DF_RELOGIN			0  /* Relogin to device */
	#define DF_NO_RELOGIN			1  /* Do not relogin if IOCTL logged it out */
	#define DF_DISCOVERY_DDB        	2
	#define DF_TARGET_DDB           	3
	#define DF_LUN_SCAN           		4
	#define DF_DYNAMIC_LUN_SCAN_NEEDED	5

	unsigned long dev_scan_wait_to_start_relogin;    /* 72 x48 */
	unsigned long dev_scan_wait_to_complete_relogin; /* 76 x4c */

	uint8_t  reserved2[4];				 /* 80 x50 */
	uint8_t  ip_addr[ISCSI_IPADDR_SIZE];
	uint8_t  iscsi_name[ISCSI_NAME_SIZE];	  	 /*  (255) */
	uint8_t  remote_ip_addr[ISCSI_IPv6ADDR_SIZE];	 /*  (16) IPv4 or IPv6 */
	uint8_t  ipv6_local_ip_addr[ISCSI_IPv6ADDR_SIZE];/*  (16) */
} ddb_entry_t;				 /* 368 x170 */

/*
 * Fibre channel port type.
 */
typedef enum {
	FCT_UNKNOWN,
	FCT_RSCN,
	FCT_SWITCH,
	FCT_BROADCAST,
	FCT_INITIATOR,
	FCT_TARGET
} fc_port_type_t;

/*
 * Fibre channel port structure.
 */
typedef struct fc_port {
	struct list_head list;
	struct list_head fcluns;

	struct scsi_qla_host *ha;
	struct scsi_qla_host *vis_ha;	/* only used when suspending lun */
	ddb_entry_t     *ddbptr;

	uint8_t  *iscsi_name;
	fc_port_type_t port_type;

	atomic_t state;
	uint32_t flags;

	os_tgt_t *tgt_queue;
	uint16_t os_target_id;
	uint8_t device_type;
	uint8_t unused;

	uint8_t mp_byte;		/* multi-path byte (not used) */
	uint8_t cur_path;		/* current path id */

	lun_bit_mask_t lun_mask;
	int loop_id;

	unsigned long last_queue_full;
	unsigned long last_ramp_up;
} fc_port_t;


/*
 * Fibre channel port/lun states.
 */
#define FCS_UNCONFIGURED	1
#define FCS_DEVICE_DEAD		2
#define FCS_DEVICE_LOST		3
#define FCS_ONLINE		4
#define FCS_NOT_SUPPORTED	5
#define FCS_FAILOVER		6
#define FCS_FAILOVER_FAILED	7
#define FCS_DEVICE_DELETED	8

/*
 * FC port flags.
 */
#define FCF_FABRIC_DEVICE	BIT_0
#define	FCF_INITIATOR_DEVICE	BIT_1
#define FCF_FO_MASKED		BIT_2
#define FCF_FAILOVER_NEEDED	BIT_3
#define FCF_RESET_NEEDED	BIT_4
#define FCF_PERSISTENT_BOUND	BIT_5
#define FCF_TAPE_PRESENT	BIT_6
#define FCF_XP_DEVICE            BIT_7
#define FCF_CONFIG_DEVICE        BIT_8
#define FCF_MSA_DEVICE            BIT_9
#define FCF_MSA_PORT_ACTIVE     BIT_10
#define FCF_LOGIN_NEEDED	BIT_12
#define FCF_EVA_DEVICE          BIT_13

#define FCF_RLC_SUPPORT		BIT_14
#define FCF_CONFIG		BIT_15	/* Needed? */
#define FCF_RESCAN_NEEDED	BIT_16
#define FCF_FAILBACK_DISABLE	BIT_17
#define FCF_FAILOVER_DISABLE	BIT_18

#define	FCF_VSA			BIT_19
#define	FCF_HD_DEVICE		BIT_20
#define FCF_NONFO_DEVICE	BIT_21	/* Non Failover device */


/*
 * Fibre channel LUN structure.
 */
typedef struct fc_lun {
	struct list_head list;

	fc_port_t *fcport;
	atomic_t state;
	uint16_t lun;
	uint8_t device_type;
	uint8_t flags;
	#define	FLF_VISIBLE_LUN		BIT_0
	#define	FLF_ACTIVE_LUN		BIT_1

	uint8_t lun_state;		/* 00 x00 */
	uint8_t resvd[3];
	#define LS_LUN_RESET_MARKER_NEEDED 4   /* LUN Reset marker needed */

} fc_lun_t, lun_entry_t;


/*---------------------------------------------------------------------------*/

/*
 * Asynchronous Event Queue structure
 */
typedef struct {
	uint32_t mbox_sts[MBOX_AEN_REG_COUNT];
}aen_t;

struct ql4_aen_log {
	int count;
	aen_t entry[MAX_AEN_ENTRIES];
};


/*
 * NOTE: This structure definition really belongs in the ql4isns.h file,
 *       but it's easier to compile when the structure is defined here.
 */
typedef struct _ATTRIBUTE_LIST {
	uint32_t isns_tag;
	#define ISNS_ATTR_TYPE_EMPTY      1   /* Used for delimiter attr. & */
					      /* operating attr. for query. */
	#define ISNS_ATTR_TYPE_STRING     2   /* UTF-8 encoded string */
	#define ISNS_ATTR_TYPE_ULONG      3
	#define ISNS_ATTR_TYPE_ADDRESS    4   /* 128-bit IPv6 */
	uint32_t type;
	unsigned long data;
	uint8_t ipv6_addr[16];
} ATTRIBUTE_LIST;

/* Used to time out short PDUs */

typedef struct {
	struct timer_list timer;
	PDU_ENTRY *pdu_entry;
	uint32_t  in_residual;
	uint32_t  pdu_type;
	uint16_t  connection_id;
	uint8_t   rsvd[2];
} timed_pdu_t;

struct ql82xx_hw_data {
	/* Offsets for flash/nvram access (set to ~0 if not used). */
	uint32_t        flash_conf_off;
	uint32_t        flash_data_off;

	uint32_t        fdt_wrt_disable;
	uint32_t        fdt_erase_cmd;
	uint32_t        fdt_block_size;
	uint32_t        fdt_unprotect_sec_cmd;
	uint32_t        fdt_protect_sec_cmd;

	uint32_t        flt_region_flt;
	uint32_t        flt_region_fdt;
	uint32_t        flt_region_boot;
	uint32_t	flt_region_bootload;
	uint32_t        flt_region_fw;
	uint32_t        reserved;
};

struct qla4_8xxx_legacy_intr_set {
	uint32_t        int_vec_bit;
	uint32_t        tgt_status_reg;
	uint32_t        tgt_mask_reg;
	uint32_t        pci_int_reg;
};

/* MSI-X Support */
#define QLA_MSIX_DEFAULT        0x00
#define QLA_MSIX_RSP_Q          0x01

#define QLA_MSIX_ENTRIES        2
#define QLA_MIDX_DEFAULT        0
#define QLA_MIDX_RSP_Q          1

struct ql4_msix_entry {
	int have_irq;
	uint16_t msix_vector;
	uint16_t msix_entry;
};

/*
 * ISP Operations
 */

struct isp_operations {
	int  (*iospace_config) (struct scsi_qla_host *ha);
	void (*pci_config) (struct scsi_qla_host *);
	void (*disable_intrs) (struct scsi_qla_host *);
	void (*__disable_intrs) (struct scsi_qla_host *);
	void (*enable_intrs) (struct scsi_qla_host *);
	void (*__enable_intrs) (struct scsi_qla_host *);
	int  (*start_firmware) (struct scsi_qla_host *);
	int (*restart_firmware) (struct scsi_qla_host *);
	irqreturn_t (*intr_handler) (int , void *, struct pt_regs *);
	void (*interrupt_service_routine) (struct scsi_qla_host *, uint32_t);
	int (*need_reset) (struct scsi_qla_host *);
	int  (*reset_chip) (struct scsi_qla_host *);
	int  (*reset_firmware) (struct scsi_qla_host *);
	void  (*queue_iocb) (struct scsi_qla_host *);
	void  (*complete_iocb) (struct scsi_qla_host *);
	uint16_t (*rd_shdw_req_q_out) (struct scsi_qla_host *);
	uint16_t (*rd_shdw_rsp_q_in) (struct scsi_qla_host *);
	int (*get_sys_info) (struct scsi_qla_host *);
	uint32_t (*rd_reg_direct) (struct scsi_qla_host *, ulong);
	void (*wr_reg_direct) (struct scsi_qla_host *, ulong, uint32_t);
	int (*rd_reg_indirect) (struct scsi_qla_host *, uint32_t, uint32_t *);
	int (*wr_reg_indirect) (struct scsi_qla_host *, uint32_t, uint32_t);
	int (*idc_lock) (struct scsi_qla_host *);
	void (*idc_unlock) (struct scsi_qla_host *);
	void (*rom_lock_recovery) (struct scsi_qla_host *);
	void (*queue_mailbox_command) (struct scsi_qla_host *, uint32_t *, int);
	void (*process_mailbox_interrupt) (struct scsi_qla_host *, int);
};

/*
 * Linux Host Adapter structure
 */
typedef struct scsi_qla_host {
	struct list_head list;

	/* Linux adapter configuration data */
	struct Scsi_Host *host;		    /* pointer to host data */
	struct scsi_qla_host *next;

	uint32_t        tot_ddbs;

	unsigned long   flags;
	#define AF_ONLINE			0 /* 0x00000001 */
	#define AF_INIT_DONE			1 /* 0x00000002 */
	#define AF_MBOX_COMMAND			2 /* 0x00000004 */
	#define AF_MBOX_COMMAND_DONE		3 /* 0x00000008 */
	#define AF_INIT_ACTIVE			4 /* 0x00000010 */
	#define AF_DPC_SCHEDULED		5 /* 0x00000020 */
	#define AF_INTERRUPTS_ON		6 /* 0x00000040 */
	#define AF_GET_CRASH_RECORD		7 /* 0x00000080 */
	#define AF_LINK_UP			8 /* 0x00000100 */
	#define AF_IRQ_ATTACHED			10 /* 0x00000400 */
	#define AF_64BIT_PCI_ADDR		11 /* 0x00000800 */
	#define AF_WAIT4DISABLE_ACB_COMPLETE	12 /* 0x00001000 */
	#define AF_MSI_ENABLED			13 /* 0x00002000 */
	#define AF_INTx_ENABLED			15 /* 0x00008000 */
	#define AF_MSIX_ENABLED			16 /* 0x00010000 */
	#define AF_HA_REMOVAL			18 /* 0x00040000 */
	#define AF_MBOX_COMMAND_NOPOLL		19 /* 0x00080000 */
	#define AF_FW_RECOVERY			20 /* 0x00100000 */
	#define AF_EEH_BUSY			21 /* 0x00200000 */
	#define AF_PCI_CHANNEL_IO_PERM_FAILURE	22 /* 0x00400000 */
	#define AF_PT_ACTIVE			23 /* 0x00800000 */
	#define AF_QUIESCE_OWNER		24 /* 0x01000000 */
	#define AF_82XX_FW_DUMPED		26 /* 0x04000000 */
	#define AF_8XXX_RST_OWNER		27 /* 0x08000000 */
	#define AF_83XX_NO_FW_DUMP		28 /* 0x10000000 */
	#define AF_82XX_DUMP_READING		29 /* 0x40000000 */

	unsigned long   dpc_flags;
	#define DPC_RESET_HA			1 /* 0x00000002 */
	#define DPC_RETRY_RESET_HA		2 /* 0x00000004 */
	#define DPC_RELOGIN_DEVICE		3 /* 0x00000008 */
	#define DPC_RESET_HA_FW_CONTEXT		4 /* 0x00000010 */
	#define DPC_RESET_HA_INTR		5 /* 0x00000020 */
	#define DPC_IOCTL_ERROR_RECOVERY	6 /* 0x00000040 */
	#define DPC_ISNS_RESTART		7 /* 0x00000080 */
	#define DPC_ISNS_START			8 /* 0x00000100 */
	#define DPC_AEN				9 /* 0x00000200 */

	#define DPC_WAIT_TO_RELOGIN_DEVICE      13 /* 0x00002000 */
	#define DPC_LUN_SCAN                    14 /* 0x00004000 */
	#define DPC_GET_DHCP_IP_ADDR            15 /* 0x00008000 */
	#define DPC_RESET_HA_INTR_COMPLETION    18 /* 0x00040000 */
	#define DPC_HA_UNRECOVERABLE            19 /* 0x00080000 ISP-82xx only*/
	#define DPC_HA_NEED_QUIESCENT		20 /* 0x00100000 ISP-82xx only*/
	#define DPC_ASYNC_PDU                   21 /* 0x00200000 */
	#define DPC_DYNAMIC_LUN_SCAN            22 /* 0x00400000 */
	#define DPC_ISNS_REREGISTER		23 /* 0x00800000 */
	#define DPC_ISNS_STOP			24 /* 0x01000000 */
	#define DPC_RESET_ACTIVE		25 /* 0x02000000 */
	#define DPC_ISNS_DEREGISTER		26 /* 0x04000000 */
	#define DPC_QUIESCE_ACTIVE		27 /* 0x08000000 */
	#define DPC_RESET_QUIESCENT		28 /* 0x10000000 */
	#define DPC_POST_IDC_ACK		29 /* 0x20000000 */

	uint16_t        iocb_cnt;
	uint16_t        rsvd;
	uint16_t        req_q_count;	/* Number of available request queue entries. */
	uint16_t        resvd;

	u_long          i_start;	/* jiffies at start of IOCTL */
	u_long          i_end;		/* jiffies at end of IOCTL */
	u_long          f_start;	/* jiffies at sending cmd to f/w */
	u_long          f_end;		/* jiffies at receiving cmd from f/w */

	/* pci information */
	struct pci_dev  *pdev;
	struct qla_board_info *brd_info;
	unsigned long   pci_resource_flags;

	uint8_t         marker_needed;
	uint8_t         rsvd1;

	/* adapter instance w.r.t. all scsi hosts in OS */
	uint16_t        host_no;

	/* adapter instance w.r.t. this driver */
	uint16_t        instance;

	uint16_t        resvd2;

	void            __iomem *virt_mmapbase;

	uint32_t        func_num;

	/* ISP registers, Base Memory-mapped I/O address */
	isp_reg_t       __iomem *reg;

	/* temp only */
	unsigned long io_addr;
	unsigned long mem_addr;
	unsigned long io_len;
	unsigned long mem_len;
	unsigned int irq;		 /* IRQ for adapter */

	uint8_t		driver_verstr[80];
	uint8_t		driver_version[4];

	/* NVRAM registers */
	eeprom_data_t     *nvram;

	/* Counters for general statistics */
	uint64_t        adapter_error_count;
	uint64_t        device_error_count;
	uint64_t        total_io_count;
	uint64_t        total_mbytes_xferred;
	uint64_t        isr_count;	    /* Interrupt count */
	uint64_t        link_failure_count;
	uint64_t        invalid_crc_count;

	uint32_t        spurious_int_count;
	uint32_t        aborted_io_count;
	uint32_t        io_timeout_count;
	uint32_t        mailbox_timeout_count;
	uint32_t        seconds_since_last_intr;
	uint32_t        seconds_since_last_heartbeat;
	uint32_t	temperature;
	uint32_t        mac_index;

	/* Info Needed for Management App */
	/* --- From GetFwVersion --- */
	uint32_t        firmware_version[2];
	uint32_t        patch_number;
	uint32_t        build_number;
	/* --- From Init_FW --- */
	uint16_t        ifcb_size;
	uint16_t       	firmware_options;
	uint16_t       	tcp_options;
	uint8_t         ip_address[IP_ADDR_LEN];
	uint8_t         subnet_mask[IP_ADDR_LEN];
	uint8_t         gateway[IP_ADDR_LEN];
	uint8_t		rsvd4[4];
	uint16_t       	isns_server_port_number;
	uint16_t        ip_options;
	uint8_t         alias[32];
	uint8_t         name_string[256];
	uint8_t         heartbeat_interval;
	uint8_t         acb_version;
	/* --- From FlashSysInfo --- */
	uint8_t         my_mac[MAC_ADDR_LEN];
	uint8_t         serial_number[16];
	/* --- From GetFwState --- */
	uint32_t        firmware_state;
	uint32_t        board_id;
	uint32_t        addl_fw_state;
	uint32_t	txcvr_state; /* qla4_8xxx transceiver state */

	/* Linux kernel thread */
	pid_t                   dpc_pid;
	int                     dpc_should_die;
	struct completion       dpc_inited;
	struct completion       dpc_exited;
	struct semaphore        *dpc_wait;
	uint8_t dpc_active;		     /* DPC routine is active */
	uint8_t resvd3[3];

	/* Linux timer thread */
	struct timer_list timer;
	uint32_t        timer_active;

	/* Recovery Timers */
	uint32_t        port_down_retry_count;
	uint32_t        discovery_wait;
	atomic_t        check_relogin_timeouts;
	uint32_t        retry_reset_ha_cnt;
	uint32_t        isp_reset_timer;    /* reset test timer */
	uint32_t        nic_reset_timer;    /* simulated nic reset test timer */
	uint32_t        eh_display_timer;   /* simulated nic reset test timer */

	int             eh_start;	    /* To wake up the mid layer error
					     * handler thread */

	/* This spinlock must be held with irqs disabled in order to access
	 * the pending, retry and free srb queues.
	 *
	 * The list_lock spinlock is of lower priority than the io_request
	 * lock.
	 *-------------------------------------------------------------------*/
	spinlock_t      list_lock  ____cacheline_aligned;

	/* internal srb queues */
	struct          list_head retry_srb_q;	 /* retry queue request list */
	uint32_t        retry_srb_q_count;

	struct          list_head free_srb_q;
	uint16_t        free_srb_q_count;
	uint16_t        num_srbs_allocated;

	/* Done queue
	 * In order to avoid deadlocks with the list_lock,
	 * place all srbs to be returned to OS on this list.
	 * After the list_lock is released, return all of
	 * these commands to the OS */

	struct list_head done_srb_q;
	uint32_t         done_srb_q_count;

	/* This spinlock is used to protect "io transactions", you must
	 * aquire it before doing any IO to the card, eg with RD_REG*() and
	 * WRT_REG*() for the duration of your entire command transaction.
	 * It is also used to protect the active_srb_array.
	 *
	 * The hardware_lock spinlock is of lower priority than the
	 * io request lock.
	 *-------------------------------------------------------------------*/
	spinlock_t         hardware_lock  ____cacheline_aligned;

	/* Active array */
	srb_t           *active_srb_array[MAX_SRBS];
	uint16_t        active_srb_count;
	uint16_t        current_active_index;

	int             mem_err;

	/* DMA Memory Block */
	void            *queues;
	dma_addr_t      queues_dma;
	unsigned long   queues_len;
#define MEM_ALIGN_VALUE	\
	((MAX(REQUEST_QUEUE_DEPTH, RESPONSE_QUEUE_DEPTH)) * \
	 sizeof(QUEUE_ENTRY))

	/* request and response queue variables */
	dma_addr_t      request_dma;
	QUEUE_ENTRY     *request_ring;
	QUEUE_ENTRY     *request_ptr;

	dma_addr_t      response_dma;
	QUEUE_ENTRY     *response_ring;
	QUEUE_ENTRY     *response_ptr;

	dma_addr_t      shadow_regs_dma;
	shadow_regs_t   *shadow_regs;

	uint16_t        request_in;		/* Current indexes. */
	uint16_t        request_out;
	uint16_t        response_in;
	uint16_t        response_out;
	uint16_t	response_qdepth;

	/* aen queue variables */
	uint32_t	aen_reg_mask;
	uint16_t        aen_report;	/* Number of aens reported to SDMAPI */
	uint16_t        aen_count;	/* Total number of aens received */
	uint16_t        aen_in;		/* Current indexes */
	uint16_t        aen_out;
	aen_t           aen_q[MAX_AEN_ENTRIES];

	/* pdu variables */
	uint16_t        resvd4[3];

	uint16_t        pdu_active;
	PDU_ENTRY       *free_pdu_top;
	PDU_ENTRY       *free_pdu_bottom;
	PDU_ENTRY       pdu_queue[MAX_PDU_ENTRIES];

	timed_pdu_t	timed_pdu; /* Used to time out short PDUs */

	/* This semaphore protects several threads to do mailbox commands
	 * concurrently.
	 *-------------------------------------------------------------------*/
	struct semaphore  mbox_sem;
	wait_queue_head_t mailbox_wait_queue;

	/* temporary mailbox status registers */
	volatile uint8_t  mbox_status_count;
	uint8_t resvd5[3];
	volatile uint32_t mbox_status[MBOX_REG_COUNT];

	/* IOCTL DMA buffer for iSNS targets buffer */
	void	*ioctl_dma_bufv;

	/* local device database list (contains internal ddb entries)*/
	struct list_head ddb_list;
	/* Fibre Channel Device List. */
	struct list_head        fcports;

	/* OS target queue pointers. */
	os_tgt_t        *otgt[MAX_TARGETS+1];

	/* Map ddb_list entry by FW ddb index */
	ddb_entry_t     *fw_ddb_index_map[MAX_DDB_ENTRIES];

	struct ql4_aen_log aen_log;

	/*
	 * There are several Scsi_Host members that are RHEL3 specific
	 * yet depend on the SCSI_HAS_HOST_LOCK define for visibility.
	 * Unfortuantely, it seems several RH kernels have the define
	 * set, but do not have a host_lock member.
	 *
	 * Use the SH_HAS_HOST_LOCK define determined during driver
	 * compilation rather than SCSI_HAS_HOST_LOCK.
	 */

	/* Scsi midlayer lock */
	#if defined(SH_HAS_HOST_LOCK)
	spinlock_t      host_lock ____cacheline_aligned;
	#endif

	/* IPv6 Needed for Management App */
	uint8_t         isns_server_ip_addr[16];
	uint8_t         isns_ha_ip_addr[16];
	uint8_t		ipv4_addr_state;

	uint32_t	ipv6_options;
	uint32_t	ipv6_addl_options;
	uint32_t	ipv6_tcp_options;
	uint8_t		ipv6_link_local_addr[16];
	uint8_t		ipv6_addr0[16];
	uint8_t		ipv6_addr1[16];
	uint8_t		ipv6_default_router_addr[16];
	uint8_t		ipv6_link_local_state;
	uint8_t		ipv6_addr0_state;
	uint8_t		ipv6_addr1_state;
	uint8_t		ipv6_default_router_state;

        struct list_head async_iocb_list;
        void            *gen_req_rsp_iocb;
        dma_addr_t      gen_req_rsp_iocb_dma;

	/* DMA Pool for Passthru IOCBs */
	struct dma_pool *pt_iocb_dmapool;

	/* Placeholder for original srb of status continuation */
	srb_t           *status_srb;

	/* QLA82XX specific fields */
	struct device_reg_82xx  __iomem *qla4_82xx_reg; /* Base I/O address */
	unsigned long nx_pcibase;       /* Base I/O address */
	uint8_t * nx_db_rd_ptr;         /* Doorbell read pointer */
	unsigned long nx_db_wr_ptr;     /* Door bell write pointer */
	unsigned long first_page_group_start;
	unsigned long first_page_group_end;

	uint32_t crb_win;
	uint32_t curr_window;
	uint32_t ddr_mn_window;
	unsigned long mn_win_crb;
	unsigned long ms_win_crb;
	int qdr_sn_window;
	rwlock_t hw_lock;
	int link_width;

	void *rsvd5;
	struct qla4_8xxx_legacy_intr_set nx_legacy_intr;
	u32 nx_crb_mask;

	uint8_t revision_id;
	uint8_t rsvd2[3];

	int fw_heartbeat_counter;

	struct isp_operations *isp_ops;
	struct ql82xx_hw_data hw;

	struct ql4_msix_entry msix_entries[QLA_MSIX_ENTRIES];

	uint32_t nx_dev_init_timeout;
	uint32_t nx_reset_timeout;

	void *fw_dump;
	int fw_dump_size;
	int fw_dump_capture_mask;
	void *fw_dump_tmplt_hdr;
	uint32_t fw_dump_tmplt_size;

	struct completion mbx_intr_comp;

	struct isns isns;

	uint8_t (*ql4_isns_start_svc)(struct scsi_qla_host *ha);
	uint8_t (*ql4_isns_stop_svc)(struct scsi_qla_host *ha);
	void (*ql4_isns_populate_server_ip) (struct scsi_qla_host *ha,
			ADDRESS_CTRL_BLK *init_fw_cb);
	void (*ql4_isns_send_dev_get_next) (struct scsi_qla_host *ha,
			__u8 *last_iscsi_name, __u8 *buf,
			__u32 *buf_len);
	void (*ql4_isns_send_dev_attr_qry) (struct scsi_qla_host *ha,
			__u8 *last_iscsi_name, __u8 *buf,
			__u32 *buf_len);
	uint8_t (*ql4_is_isns_active) (struct scsi_qla_host *ha);
	uint8_t (*ql4_mbx) (struct scsi_qla_host *ha, uint8_t inCount,
			    uint8_t outCount, uint32_t *mbx_cmd,
			    uint32_t *mbx_sts);
	struct workqueue_struct *pt_thread;
	struct work_struct pt_work;
	struct semaphore pt_sem;

	uint16_t maxcmds;	/* maximum outstanding commands */
	uint32_t *reg_tbl;

	/* QLA83XX specific fields */
	struct qla4_83xx_reset_template reset_tmplt;
	struct device_reg_83xx  __iomem *qla4_83xx_reg; /* Base I/O address
							   (ISP83xx) */
	uint32_t pf_bit;

	struct completion idc_comp;
	int notify_idc_comp;
	struct completion link_up_comp;
	int notify_link_up_comp;

	struct qla4_83xx_idc_information idc_info;
} scsi_qla_host_t;

/*
 * structure to buffer Async PDUs
 */
typedef struct  {
        struct list_head list_entry;
        ASYNC_PDU_ENTRY iocb;
} async_pdu_iocb_t;

#define ADAPTER_UP(ha) ((test_bit(AF_ONLINE, &ha->flags) != 0) && \
			(test_bit(AF_LINK_UP, &ha->flags) != 0))

typedef struct {
	uint8_t ha_mac[MAX_HBAS][MAC_ADDR_LEN];
} mac_cfgs_t;

/*
 * Other macros
 */
#define TGT_Q(ha, t) (ha->otgt[t])
#define LUN_Q(ha, t, l)	(TGT_Q(ha, t)->olun[l])
#define GET_LU_Q(ha, t, l) ((TGT_Q(ha,t) != NULL)? TGT_Q(ha, t)->olun[l] : NULL)

#define to_qla_host(x)		((scsi_qla_host_t *) (x)->hostdata)

#define ql4_printk(level, ha, format, arg...) \
	dev_printk(level , &((ha)->pdev->dev) , format , ## arg)


/*---------------------------------------------------------------------------*/

/* Defines for qla4xxx_initialize_adapter() and qla4xxx_recover_adapter() */
#define PRESERVE_DDB_LIST	0
#define REBUILD_DDB_LIST	1

/* Defines for process_aen() */
#define PROCESS_ALL_AENS	 0
#define FLUSH_DDB_CHANGED_AENS	 1
#define RELOGIN_DDB_CHANGED_AENS 2

/* Defines for qla4xxx_take_hw_semaphore */
#define NO_WAIT		0
#define WAIT_FOREVER	1
#define TIMED_WAIT	2

/* Defines from EXT_REG_AEN_ISCSI IOCTL, needed in isr_decode_mailbox routine */
#define EXT_DEF_ENABLE_AENS		0x00000000
#define EXT_DEF_ENABLE_NO_AENS		0xFFFFFFFF

/* EXT_ISCSI_IP_ADDR IOCTL*/
#define EXT_DEF_TYPE_ISCSI_IPV6		1

/* Defines for udev events */
#define QL4_UEVENT_CODE_FW_DUMP		0 /* Uevent code for dumping firmware */

static inline int is_ipv4_enabled(struct scsi_qla_host *ha)
{
        return ((ha->ip_options & IPOPT_IPv4_PROTOCOL_ENABLE) != 0);
}

static inline int is_ipv6_enabled(struct scsi_qla_host *ha)
{
        return ((ha->ipv6_options & IPV6_OPT_IPV6_PROTOCOL_ENABLE) != 0);
}

static inline int is_isnsv4_enabled(struct scsi_qla_host *ha)
{
        return ((ha->tcp_options & TOPT_ISNSv4_ENABLE) != 0);
}

static inline int is_isnsv6_enabled(struct scsi_qla_host *ha)
{
        return ((ha->ipv6_tcp_options & IPV6_TCPOPT_ISNSv6_ENABLE) != 0);
}

static inline int qla4_8xxx_rd_direct(struct scsi_qla_host *ha,
				      const uint32_t crb_reg)
{
	return ha->isp_ops->rd_reg_direct(ha, ha->reg_tbl[crb_reg]);
}

static inline void qla4_8xxx_wr_direct(struct scsi_qla_host *ha,
				       const uint32_t crb_reg,
				       const uint32_t value)
{
	ha->isp_ops->wr_reg_direct(ha, ha->reg_tbl[crb_reg], value);
}

extern int qla4_83xx_get_port_config(struct scsi_qla_host *, uint32_t *);
extern int qla4_83xx_set_port_config(struct scsi_qla_host *, uint32_t *);
extern int qla4_83xx_post_idc_ack(struct scsi_qla_host *);

#ifndef __QIM_DEF_H__
#include "ql4_version.h"
#include "ql4_glbl.h"
#include "ql4_dbg.h"
#include "ql4_inline.h"
#include "ql4_listops.h"

#endif

#endif /*_QLA4XXX_H */

/*
 * Overrides for Emacs so that we get a uniform tabbing style.
 * Emacs will notice this stuff at the end of the file and automatically
 * adjust the settings for this buffer only.  This must remain at the end
 * of the file.
 * ---------------------------------------------------------------------------
 * Local variables:
 * c-indent-level: 4
 * c-brace-imaginary-offset: 0
 * c-brace-offset: -4
 * c-argdecl-indent: 4
 * c-label-offset: -4
 * c-continued-statement-offset: 4
 * c-continued-brace-offset: 0
 * indent-tabs-mode: nil
 * tab-width: 8
 * End:
 */
