/*
 * QLogic iSCSI HBA Driver
 * Copyright (c)  2003-2012 QLogic Corporation
 *
 * See LICENSE.qla4xxx for copyright and licensing details.
 */

/******************************************************************************
 * Global include file.
 ****************************************************************************/
#ifndef __QLA4x_GBL_H
#define	__QLA4x_GBL_H

#include <linux/interrupt.h>

/*
 * Defined in ql4_os.c
 */

extern void qla4xxx_start_io(scsi_qla_host_t *ha);
extern srb_t *del_from_active_array(scsi_qla_host_t *ha, uint32_t index);
extern uint8_t qla4xxx_complete_request(scsi_qla_host_t *ha, srb_t *srb);
extern uint8_t qla4xxx_reset_lun(scsi_qla_host_t *ha, ddb_entry_t *ddb_entry,
		int lun);
extern int qla4xxx_hw_reset(scsi_qla_host_t *);
extern int qla4xxx_soft_reset(scsi_qla_host_t *);
extern const char *host_sts_msg[];
extern void qla4xxx_delete_timer_from_cmd(srb_t *srb);
extern scsi_qla_host_t *qla4xxx_get_adapter_handle(uint16_t instance);
extern void qla4xxx_free_ddb_list(scsi_qla_host_t *ha);

extern void qla4xxx_tgt_free(scsi_qla_host_t *ha, uint16_t t);
extern os_lun_t * qla4xxx_lun_alloc(scsi_qla_host_t *, uint16_t, uint16_t);
extern void qla4xxx_extend_timeout(struct scsi_cmnd *cmd, int timeout);
extern int qla4xxx_done(scsi_qla_host_t *old_ha);
extern void qla4xxx_device_suspend( scsi_qla_host_t *, srb_t * );
extern void qla4xxx_add_timer_to_cmd(scsi_qla_host_t *ha, srb_t *, int );
extern void qla4xxx_request_cleanup(scsi_qla_host_t *ha, srb_t *srb);
extern void qla4xxx_dynamic_lun_scan(scsi_qla_host_t *ha, struct device *dev, uint16_t target_id);
extern uint8_t qla4xxx_cmd_wait(scsi_qla_host_t *ha);

extern int ql4xdontresethba;
extern int ql4xqfullrampup;
extern int ql4xmaxqdepth;
extern int ql4xmaxcmds;

extern int ql4xextended_error_logging;
extern int ql4xenablemsix;
extern int ql4xmdcapmask;
extern int ql4xenablemd;

struct class_device_attribute;
extern struct class_device_attribute *qla4xxx_host_attrs[];

extern char *qdev_state[];

/*
 * Defined in  ql4_iocb.c
 */
extern uint8_t qla4xxx_send_marker_iocb(scsi_qla_host_t *ha,
		ddb_entry_t *ddb_entry, int lun, uint16_t modifier);

extern uint8_t qla4xxx_get_req_pkt(scsi_qla_host_t *, QUEUE_ENTRY **);

extern PDU_ENTRY *qla4xxx_get_pdu(scsi_qla_host_t *, uint32_t);
extern void qla4xxx_free_pdu(scsi_qla_host_t *, PDU_ENTRY *);
extern uint8_t qla4xxx_send_passthru0_iocb(scsi_qla_host_t *, uint16_t,
    uint16_t, dma_addr_t, uint32_t, uint32_t, uint16_t, uint32_t);

/*
 * Defined in  ql4_isr.c
 */

extern uint8_t qla4xxx_poll_and_ack_scsi_reset(scsi_qla_host_t *ha);
extern irqreturn_t qla4xxx_intr_handler(int irq, void *dev_id, struct pt_regs *regs);
extern void qla4xxx_interrupt_service_routine(scsi_qla_host_t *ha, uint32_t  intr_status);
extern void __qla4xxx_suspend_lun(scsi_qla_host_t *ha, srb_t *srb, os_lun_t *lun_entry, uint16_t time,
		    uint16_t retries, int delay);


/*
 * Defined in  ql4_init.c
 */
extern uint8_t qla4xxx_initialize_adapter(scsi_qla_host_t *ha, uint8_t renew_ddb_list);
extern uint8_t qla4xxx_init_firmware(scsi_qla_host_t *ha);
extern uint8_t qla4xxx_initialize_ddb_list(scsi_qla_host_t *ha);
extern ddb_entry_t *qla4xxx_alloc_ddb(scsi_qla_host_t *ha, uint32_t fw_ddb_index);
extern uint8_t qla4xxx_update_ddb_entry(scsi_qla_host_t *ha, ddb_entry_t
    *ddb_entry, uint32_t fw_ddb_index);
extern uint8_t qla4xxx_get_fwddb_entry(scsi_qla_host_t *ha, uint16_t fw_ddb_index, DEV_DB_ENTRY *fw_ddb_entry, dma_addr_t fw_ddb_entry_dma, uint32_t *num_valid_ddb_entries, uint32_t *next_ddb_index, uint32_t *fw_ddb_device_state, uint32_t *time2wait, uint16_t *tcp_source_port_num, uint16_t *connection_id);
extern void qla4xxx_relogin_device(scsi_qla_host_t *ha, ddb_entry_t *ddb_entry);
extern int qla4xxx_get_and_set_ddb(struct scsi_qla_host *ha);
extern int qla4xxx_send_tgts(struct scsi_qla_host *ha, char *ip, uint16_t port);
extern uint8_t qla4xxx_send_command_to_isp(scsi_qla_host_t *, srb_t *);
extern int qla4xxx_get_prop_12chars(scsi_qla_host_t *ha, uint8_t *propname, uint8_t *propval, uint8_t *db);
extern void qla4xxx_free_ddb(scsi_qla_host_t *ha, ddb_entry_t *ddb_entry);
extern uint8_t qla4xxx_resize_ioctl_dma_buf(scsi_qla_host_t *ha, uint32_t size);
extern uint8_t qla4xxx_set_ddb_entry(scsi_qla_host_t *ha, uint16_t fw_ddb_index, DEV_DB_ENTRY *fw_ddb_entry, dma_addr_t fw_ddb_entry_dma);
extern uint8_t qla4xxx_process_ddb_changed(scsi_qla_host_t *ha, uint32_t fw_ddb_index,
			uint32_t fw_ddb_device_state, uint32_t err_code);
extern uint8_t qla4xxx_init_rings(scsi_qla_host_t *ha);
extern uint8_t qla4xxx_reinitialize_ddb_list(scsi_qla_host_t *ha);
extern fc_lun_t * qla4xxx_add_fclun(fc_port_t *fcport, uint16_t lun);
extern os_lun_t *
qla4xxx_fclun_bind(scsi_qla_host_t *ha, fc_port_t *fcport, fc_lun_t *fclun);
extern void qla4xxx_flush_all_srbs(scsi_qla_host_t *ha, ddb_entry_t *ddb_entry, os_lun_t *lun_entry);
extern int qla4xxx_lock_drvr_wait(struct scsi_qla_host *a);
extern uint8_t qla4xxx_reindex_ddb(scsi_qla_host_t *ha, uint32_t old_fw_ddb_index, uint32_t new_fw_ddb_index);
void qla4xxx_add_ddb_to_list(scsi_qla_host_t *ha, uint32_t fw_ddb_index, uint32_t *next_fw_ddb_index);
extern ddb_entry_t * qla4xxx_find_ddb_internally(scsi_qla_host_t *ha, uint32_t fw_ddb_index);
void qla4xxx_set_device_state(scsi_qla_host_t *ha, ddb_entry_t *ddb_entry, uint32_t state);
extern int qla4xxx_start_firmware(struct scsi_qla_host *ha);
/*
 * Defined in  ql4_mbx.c
 */
extern void qla4xxx_process_aen(scsi_qla_host_t *ha, uint8_t flush_ddb_chg_aens);
extern uint8_t qla4xxx_mailbox_command(scsi_qla_host_t *ha, uint8_t inCount, uint8_t outCount, uint32_t *mbx_cmd, uint32_t *mbx_sts);
extern void qla4xxx_mailbox_premature_completion(struct scsi_qla_host *ha);
extern int qla82xx_collect_md_data(struct scsi_qla_host *ha);
extern uint8_t qla4xxx_issue_iocb(scsi_qla_host_t *ha, uint32_t cmpl_sts_offset, dma_addr_t phys_addr);

extern uint8_t qla4xxx_get_flash(scsi_qla_host_t *, dma_addr_t, uint32_t,
    uint32_t);

extern uint8_t qla4xxx_get_dhcp_ip_address(scsi_qla_host_t *);

extern uint8_t qla4xxx_get_firmware_state(scsi_qla_host_t *);

extern void qla4xxx_get_crash_record(scsi_qla_host_t *);

extern uint8_t qla4xxx_conn_close_sess_logout(scsi_qla_host_t *, uint16_t,
    uint16_t, uint16_t);

extern uint8_t qla4xxx_clear_database_entry(scsi_qla_host_t *, uint16_t);

extern uint8_t qla4xxx_get_fw_version(scsi_qla_host_t *ha);

extern uint8_t qla4xxx_restore_factory_defaults(scsi_qla_host_t *ha, uint32_t, uint32_t, uint32_t);

extern uint8_t qla4xxx_get_firmware_status(scsi_qla_host_t *ha);

extern void qla4xxx_get_conn_event_log(scsi_qla_host_t *ha);

extern uint8_t qla4xxx_get_ifcb(scsi_qla_host_t *, uint32_t *, uint32_t *, dma_addr_t);

extern uint8_t qla4xxx_set_ifcb(scsi_qla_host_t *, uint32_t *, uint32_t *, dma_addr_t);
extern int qla4xxx_req_template_size(struct scsi_qla_host * ha);
extern int qla4xxx_get_minidump_template(struct scsi_qla_host * ha,
					dma_addr_t phys_addr);
void qla4_8xxx_alloc_sysfs_attr(struct scsi_qla_host *);
void qla4_8xxx_free_sysfs_attr(struct scsi_qla_host *);

/*
 * Defined in  ql4_inioct.c
 */
extern void qla4xxx_iocb_pass_done(scsi_qla_host_t *ha, PASSTHRU_STATUS_ENTRY *sts_entry);

/*
 * Defined in  ql4_xioct.c
 */
extern void qla4xxx_scsi_pass_done(struct scsi_cmnd *cmd);
extern void qla4xxx_ioctl_sem_init (scsi_qla_host_t *ha);

/*
 * Defined in  ql4_isns.c
 */
uint8_t ql4_isns_start_service(struct scsi_qla_host *ha);
uint8_t ql4_isns_stop_service(struct scsi_qla_host *ha);
void ql4_isns_restart_service(struct scsi_qla_host *ha);
void ql4_isns_restart_timer(struct scsi_qla_host *ha, __u32 time);
void ql4_isns_send_scn_dereg(struct scsi_qla_host *ha);
void ql4_isns_send_dev_get_next(struct scsi_qla_host *ha,
			__u8 *last_iscsi_name,
			__u8 *buf, __u32 *buf_len);
void ql4_isns_send_dev_attr_qry(struct scsi_qla_host *ha,
			__u8 *last_iscsi_name,
			__u8 *buf, __u32 *buf_len);
void ql4_isns_populate_server_ip(struct scsi_qla_host *ha,
			ADDRESS_CTRL_BLK *init_fw_cb);
void ql4_isns_process_isns_aen(struct scsi_qla_host *ha, __u32 *mbox_sts);
void ql4_isns_queue_passthru_sts_iocb(struct scsi_qla_host *ha,
			struct isns_prb *prb);
void ql4_isns_process_passthru_sts_iocb(struct scsi_qla_host *ha,
			struct isns_prb *prb);
void ql4_isns_process_ip_state_chg(struct scsi_qla_host *ha, __u32 *mbox_sts);
void __dump_prb(struct scsi_qla_host *ha, struct isns_prb *prb);
void ql4_isns_dequeue_passthru_sts_iocb(struct work_struct *work);
__u8 ql4_is_isns_active(struct scsi_qla_host *ha);
__u8 ql4_isns_deregister_isns_server(struct scsi_qla_host *ha);
__u8 ql4_isns_register_isns_server(struct scsi_qla_host *ha);

	/*
 * Defined in  ql4_nvram.c
	 */

extern u16 RD_NVRAM_WORD(scsi_qla_host_t *, int);
extern uint8_t qla4xxx_is_NVRAM_configuration_valid(scsi_qla_host_t *ha);
extern int ql4xxx_sem_lock(scsi_qla_host_t *ha, u32 sem_mask, u32 sem_bits) ;
extern void ql4xxx_sem_unlock(scsi_qla_host_t *ha, u32 sem_mask) ;
extern int ql4xxx_sem_spinlock(scsi_qla_host_t *ha, u32 sem_mask, u32 sem_bits) ;

/*
 * Defined in  ql4_nx.c
 */
extern void qla4_8xxx_clear_drv_active(struct scsi_qla_host *ha);
extern int qla4_8xxx_start_firmware(struct scsi_qla_host *ha,
			uint32_t image_start);
void qla4_8xxx_set_drv_active(struct scsi_qla_host *ha);
void qla4_8xxx_clear_qsnt_ready(struct scsi_qla_host *);
void qla4xxx_qsnt_state_cleanup(struct scsi_qla_host *ha);
void qla4xxx_get_conn_event_log(struct scsi_qla_host *);

/*
 * Defined in  ql4_dbg.c
 */
extern void qla4xxx_dump_buffer(uint8_t *b, uint32_t size);

extern void qla4xxx_queue_iocb(struct scsi_qla_host *ha);
extern void qla4xxx_complete_iocb(struct scsi_qla_host *ha);
int qla4xxx_get_sys_info(struct scsi_qla_host *ha);
extern int qla4xxx_iospace_config(struct scsi_qla_host *ha);
extern void qla4xxx_pci_config(struct scsi_qla_host *ha);
extern uint32_t * qla4xxx_isp_mbx_out(struct scsi_qla_host *ha);
extern uint16_t qla4xxx_rd_shdw_req_q_out(struct scsi_qla_host *ha);
extern uint16_t qla4xxx_rd_shdw_rsp_q_in(struct scsi_qla_host *ha);
extern int qla4xxx_mbx_reg_test(struct scsi_qla_host *);

int qla4xxx_request_irqs(struct scsi_qla_host *ha);
void qla4xxx_free_irqs(struct scsi_qla_host *ha);
int qla4xxx_process_response_queue(struct scsi_qla_host *ha);
void qla4xxx_isr_decode_mailbox(struct scsi_qla_host * ha, uint32_t mbox_status);
void qla4xxx_wake_dpc(struct scsi_qla_host *ha);

/* ISP 8022 related functions */
extern void qla4_8xxx_pci_config(struct scsi_qla_host *);
extern void qla4_8xxx_reset_chip(struct scsi_qla_host *);
extern int qla4_8xxx_iospace_config(struct scsi_qla_host *ha);
extern void qla4_8xxx_reset_adapter(struct scsi_qla_host *);
extern int qla4_8xxx_nvram_config(struct scsi_qla_host *);
extern int qla4_8xxx_load_risc(struct scsi_qla_host *);
extern char *qla4_8xxx_get_fw_version_str(struct scsi_qla_host *, char *);
extern int qla4_8xxx_abort_command(srb_t *);
extern int qla4_8xxx_start_scsi(srb_t *);
extern irqreturn_t qla4_82xx_intr_handler(int irq, void *dev_id, struct pt_regs *regs);
extern void qla4_82xx_queue_iocb(struct scsi_qla_host *ha);
extern void qla4_82xx_complete_iocb(struct scsi_qla_host *ha);

extern void qla4_82xx_wr_32(struct scsi_qla_host *, ulong, u32);
extern uint32_t qla4_82xx_rd_32(struct scsi_qla_host *, ulong);
extern int rom_fast_read(struct scsi_qla_host *, int , int*);
extern int qla4_82xx_pci_mem_write_2M(struct scsi_qla_host *ha, u64, void*, int);
extern int qla4_8xxx_pci_write_immediate_2M(struct scsi_qla_host *, u64, u32);
extern u32 qla4_8xxx_pci_read_immediate_2M(struct scsi_qla_host *, u64);
extern void qla80xx_update_crb_int_ptr(struct scsi_qla_host *);

extern int qla4_8xxx_isp_reset(struct scsi_qla_host *ha);
extern void qla4_8xxx_hard_reset(struct scsi_qla_host *ha);
extern uint16_t qla4_82xx_rd_shdw_req_q_out(struct scsi_qla_host *ha);
extern uint16_t qla4_82xx_rd_shdw_rsp_q_in(struct scsi_qla_host *ha);
void qla4_82xx_interrupt_service_routine(struct scsi_qla_host * ha, uint32_t intr_status);
int qla4_8xxx_get_sys_info(struct scsi_qla_host *ha);
void qla4_8xxx_init_local_data(struct scsi_qla_host *ha);
void qla4_8xxx_watchdog(struct scsi_qla_host *ha);
int qla4_8xxx_stop_firmware(struct scsi_qla_host *ha);

int qla4_82xx_get_flash_info(struct scsi_qla_host *ha);
void qla4_82xx_enable_intrs(struct scsi_qla_host *ha);
void qla4_82xx_disable_intrs(struct scsi_qla_host *ha);
void qla4_8xxx_disable_msix(struct scsi_qla_host *ha);
int qla4_8xxx_enable_msix(struct scsi_qla_host *ha);
irqreturn_t qla4_82xx_msi_handler(int irq, void *dev_id, struct pt_regs *regs);
irqreturn_t qla4_8xxx_default_intr_handler(int irq, void *dev_id,
					 struct pt_regs *regs);
irqreturn_t qla4_8xxx_msix_rsp_q(int irq, void *dev_id, struct pt_regs *regs);
void qla4xxx_abort_active_cmds(struct scsi_qla_host *ha, int res);
void qla4xxx_dead_adapter_cleanup(struct scsi_qla_host *ha);
int qla4_82xx_idc_lock(struct scsi_qla_host *ha);
void qla4_82xx_idc_unlock(struct scsi_qla_host *ha);
int qla4_8xxx_device_state_handler(struct scsi_qla_host *ha);
void qla4_8xxx_need_qsnt_handler(struct scsi_qla_host *ha);
int qla4_82xx_try_start_fw(struct scsi_qla_host *ha);
int qla4_8xxx_need_reset(scsi_qla_host_t *ha);
int qla4_82xx_md_rd_32(struct scsi_qla_host *ha, uint32_t off, uint32_t *data);
int qla4_82xx_md_wr_32(struct scsi_qla_host *ha, uint32_t off, uint32_t data);
void qla4_82xx_rom_lock_recovery(struct scsi_qla_host *ha);
void qla4_82xx_queue_mbox_cmd(struct scsi_qla_host *ha, uint32_t *mbx_cmd,
			       int incount);
void qla4_82xx_process_mbox_intr(struct scsi_qla_host *ha, int outcount);
void qla4xxx_queue_mbox_cmd(struct scsi_qla_host *ha, uint32_t *mbx_cmd,
			     int incount);
void qla4xxx_process_mbox_intr(struct scsi_qla_host *ha, int outcount);
int qla4_8xxx_collect_md_data(struct scsi_qla_host *ha);
int qla4_8xxx_mbx_intr_disable(struct scsi_qla_host *ha);
int qla4_8xxx_mbx_intr_enable(struct scsi_qla_host *ha);
int qla4_8xxx_set_param(struct scsi_qla_host *ha, int param);

void qla4_83xx_queue_iocb(struct scsi_qla_host *ha);
void qla4_83xx_complete_iocb(struct scsi_qla_host *ha);
uint32_t qla4_83xx_rd_reg(struct scsi_qla_host *ha, ulong addr);
void qla4_83xx_wr_reg(struct scsi_qla_host *ha, ulong addr, uint32_t val);
int qla4_83xx_idc_dontreset(struct scsi_qla_host *ha);
int qla4_83xx_can_perform_reset(struct scsi_qla_host *ha);
void qla4_83xx_disable_intrs(struct scsi_qla_host *ha);
void qla4_83xx_enable_intrs(struct scsi_qla_host *ha);
int qla4_83xx_start_firmware(struct scsi_qla_host *ha);
irqreturn_t qla4_83xx_intr_handler(int irq, void *dev_id, struct pt_regs *regs);
void qla4_83xx_interrupt_service_routine(struct scsi_qla_host *ha,
					uint32_t intr_status);
int qla4_83xx_rd_reg_indirect(struct scsi_qla_host *ha, uint32_t addr,
			     uint32_t *data);
int qla4_83xx_wr_reg_indirect(struct scsi_qla_host *ha, uint32_t addr,
			     uint32_t data);
int qla4_83xx_drv_lock(struct scsi_qla_host *ha);
void qla4_83xx_drv_unlock(struct scsi_qla_host *ha);
void qla4_83xx_rom_lock_recovery(struct scsi_qla_host *ha);
void qla4_83xx_queue_mbox_cmd(struct scsi_qla_host *ha, uint32_t *mbx_cmd,
			     int incount);
void qla4_83xx_process_mbox_intr(struct scsi_qla_host *ha, int outcount);
uint16_t qla4_83xx_rd_shdw_req_q_out(struct scsi_qla_host *ha);
uint16_t qla4_83xx_rd_shdw_rsp_q_in(struct scsi_qla_host *ha);
void qla4_83xx_get_flash_info(struct scsi_qla_host *ha);
void qla4_83xx_set_idc_dontreset(struct scsi_qla_host *ha);
int qla4_8xxx_device_bootstrap(struct scsi_qla_host *ha);
void qla4_83xx_need_reset_handler(struct scsi_qla_host *ha);
void qla4_8xxx_set_rst_ready(struct scsi_qla_host *ha);
void qla4_8xxx_clear_rst_ready(struct scsi_qla_host *ha);
int qla4_83xx_lockless_flash_read_u32(struct scsi_qla_host *ha,
				     uint32_t flash_addr, uint8_t *p_data,
				     int u32_word_count);
void qla4_83xx_clear_idc_dontreset(struct scsi_qla_host *ha);
int qla4_83xx_isp_reset(struct scsi_qla_host *ha);
int qla4_8xxx_update_idc_reg(struct scsi_qla_host *ha);
#endif /* _QLA4x_GBL_H */
