/*
 * QLogic iSCSI IOCTL Module
 * Copyright (c)  2003-2012 QLogic Corporation
 *
 * See LICENSE.qla4xxx for copyright and licensing details.
 */

/******************************************************************************
 *             Please see release.txt for revision history.                   *
 *                                                                            *
 ******************************************************************************
 * Function Table of Contents:
 *	ql4im_printchar
 *	ql4im_dump_bytes
 *	ql4im_dump_words
 *	ql4im_dump_dwords
 *	ql4im_print_iocb_passthru
 *	ql4im_dump_registers
 ****************************************************************************/

#include "ql4im_def.h"

/****************************************************************************/
/*                      Debug Print Routines                          	    */
/****************************************************************************/

inline void ql4im_printchar(char ch)
{
	if (ch>=32)
		printk("%c", ch);
	else
		printk(".");
}

#if 0
void
ql4im_dump_buffer(uint8_t * b, uint32_t size)
{
	uint32_t cnt;
	uint8_t c;

	printk(" 0   1   2   3   4   5   6   7   8   9  "
	    "Ah  Bh  Ch  Dh  Eh  Fh\n");
	printk("----------------------------------------"
	    "----------------------\n");

	for (cnt = 0; cnt < size;) {
		c = *b++;
		printk("%02x",(uint32_t) c);
		cnt++;
		if (!(cnt % 16))
			printk("\n");
		else
			printk("  ");
	}
	if (cnt % 16)
		printk("\n");
}
#endif

/**************************************************************************
 * ql4im_dump_bytes
 *	This routine displays bytes in hex format
 *
 * Input:
 *	dbg_mask - this call's debug print mask
 *	buffer   - data buffer to display
 *	size     - number of bytes to display
 *
 * Output:
 *	None
 *
 * Returns:
 *	None
 **************************************************************************/
void
ql4im_dump_bytes(void *buffer, uint32_t size)
{
	uint32_t i;
	uint8_t  *data = (uint8_t *)buffer;

	//printk("        0  1  2  3  4  5  6  7 -  8  9  A  B  C  D  E  F\n");
	//printk("---------------------------------------------------------\n");

	for (i = 0; i < size; i++, data++) {
		if (i % 0x10 == 0) {
			printk("%04X:  %02X", i, *data);
		}
		else if (i % 0x10 == 0x08) {
			printk(" - %02X", *data);
		}
		else if (i % 0x10 == 0xF) {
			printk(" %02X:  ", *data);
			ql4im_printchar(*(data-15));
			ql4im_printchar(*(data-14));
			ql4im_printchar(*(data-13));
			ql4im_printchar(*(data-12));
			ql4im_printchar(*(data-11));
			ql4im_printchar(*(data-10));
			ql4im_printchar(*(data-9));
			ql4im_printchar(*(data-8));
			ql4im_printchar(*(data-7));
			ql4im_printchar(*(data-6));
			ql4im_printchar(*(data-5));
			ql4im_printchar(*(data-4));
			ql4im_printchar(*(data-3));
			ql4im_printchar(*(data-2));
			ql4im_printchar(*(data-1));
			ql4im_printchar(*data);
			printk("\n");
		}
		else {
			printk(" %02X", *data);
		}
	}

	if ((i != 0) && (i % 0x10)) {
		printk("\n");
	}
	printk("\n");
}

/* hardware_lock taken */
/**************************************************************************
 * ql4im_dump_words
 *	This routine displays words in hex format
 *
 * Input:
 *	dbg_mask - this call's debug print mask
 *	buffer   - data buffer to display
 *	size     - number of bytes to display
 *
 * Output:
 *	None
 *
 * Returns:
 *	None
 **************************************************************************/
void
ql4im_dump_words(void *buffer, uint32_t size)
{
	uint16_t *data = (uint16_t *)buffer;
	uint32_t i;

	for (i = 0; i < size; i+=2, data++) {
		if (i % 0x10 == 0) {
			printk("%04X:  %04X", i, *data);
		}
		else if (i % 0x10 == 0x08) {
			printk(" - %04X", *data);
		}
		else if (i % 0x10 == 0x0E) {
			uint8_t *bdata = (uint8_t *) data;
			printk(" %04X:  ", *data);
			ql4im_printchar(*(bdata-13));
			ql4im_printchar(*(bdata-14));
			ql4im_printchar(*(bdata-11));
			ql4im_printchar(*(bdata-12));
			ql4im_printchar(*(bdata-9));
			ql4im_printchar(*(bdata-10));
			ql4im_printchar(*(bdata-7));
			ql4im_printchar(*(bdata-8));
			ql4im_printchar(*(bdata-5));
			ql4im_printchar(*(bdata-6));
			ql4im_printchar(*(bdata-3));
			ql4im_printchar(*(bdata-4));
			ql4im_printchar(*(bdata-1));
			ql4im_printchar(*(bdata-2));
			ql4im_printchar(*(bdata+1));
			ql4im_printchar(*(bdata));
			printk("\n");
		}
		else {
			printk(" %04X", *data);
		}
	}
	if ((i != 0) && (i % 0x10 != 0)) {
		printk("\n");
	}
}

/* hardware_lock taken */
/**************************************************************************
 * ql4im_dump_dwords
 *	This routine displays double words in hex format
 *
 * Input:
 *	dbg_mask - this call's debug print mask
 *	buffer   - data buffer to display
 *	size     - number of bytes to display
 *
 * Output:
 *	None
 *
 * Returns:
 *	None
 **************************************************************************/
void
ql4im_dump_dwords(void *buffer, uint32_t size)
{
	uint32_t *data = (uint32_t *)buffer;
	uint32_t i;

	for (i = 0; i < size; i+=4, data++) {
		if (i % 0x10 == 0) {
			printk("%04X:  %08X", i, *data);
		}
		else if (i % 0x10 == 0x08) {
			printk(" - %08X", *data);
		}
		else if (i % 0x10 == 0x0C) {
			printk(" %08X\n", *data);
		}
		else {
			printk(" %08X", *data);
		}
	}
	if ((i != 0) && (i % 0x10 != 0)) {
		printk("\n");
	}
}

/**************************************************************************
 * ql4im_dump_registers
 *	This routine displays ISP registers
 *
 * Input:
 *	dbg_mask - this call's debug print mask
 *	ha       - adapter structure pointer
 *
 * Output:
 *	None
 *
 * Returns:
 *	None
 **************************************************************************/
void
ql4im_dump_registers(scsi_qla_host_t *ha)
{
	uint8_t  i;

	for (i=0; i<MBOX_REG_COUNT; i++) {
		printk("0x%02X mailbox[%d]      = 0x%08X\n",
		       (uint8_t) offsetof(isp_reg_t, mailbox[i]), i,
		       RD_REG_DWORD(&ha->reg->mailbox[i]));
	}
	printk("0x%02X flash_address   = 0x%08X\n",
	       (uint8_t) offsetof(isp_reg_t, flash_address),
	       RD_REG_DWORD(&ha->reg->flash_address));

	printk("0x%02X flash_data      = 0x%08X\n",
	       (uint8_t) offsetof(isp_reg_t, flash_data),
	       RD_REG_DWORD(&ha->reg->flash_data));

	printk("0x%02X ctrl_status     = 0x%08X\n",
	       (uint8_t) offsetof(isp_reg_t, ctrl_status),
	       RD_REG_DWORD(&ha->reg->ctrl_status));

	if (IS_QLA4010(ha)) {

		printk("0x%02X nvram           = 0x%08X\n",
		       (uint8_t) offsetof(isp_reg_t, u1.isp4010.nvram),
		       RD_REG_DWORD(&ha->reg->u1.isp4010.nvram));
	}
	else if (IS_QLA4022(ha)) {

		printk("0x%02X intr_mask       = 0x%08X\n",
		    (uint8_t) offsetof(isp_reg_t, u1.isp4022.intr_mask),
		    RD_REG_DWORD(&ha->reg->u1.isp4022.intr_mask));
		
		printk("0x%02X nvram           = 0x%08X\n",
		    (uint8_t) offsetof(isp_reg_t, u1.isp4022.nvram),
		    RD_REG_DWORD(&ha->reg->u1.isp4022.nvram));
		
		printk("0x%02X semaphore       = 0x%08X\n",
		    (uint8_t) offsetof(isp_reg_t, u1.isp4022.semaphore),
		    RD_REG_DWORD(&ha->reg->u1.isp4022.semaphore));
	}

	printk("0x%02X req_q_in        = 0x%08X\n",
	       (uint8_t) offsetof(isp_reg_t, req_q_in),
	       RD_REG_DWORD(&ha->reg->req_q_in));

	printk("0x%02X rsp_q_out       = 0x%08X\n",
	       (uint8_t) offsetof(isp_reg_t, rsp_q_out),
	       RD_REG_DWORD(&ha->reg->rsp_q_out));

	if (IS_QLA4010(ha)) {

		printk("0x%02X ext_hw_conf     = 0x%08X\n",
		       (uint8_t) offsetof(isp_reg_t, u2.isp4010.ext_hw_conf),
		       RD_REG_DWORD(&ha->reg->u2.isp4010.ext_hw_conf));

		printk("0x%02X port_ctrl       = 0x%08X\n",
		       (uint8_t) offsetof(isp_reg_t, u2.isp4010.port_ctrl),
		       RD_REG_DWORD(&ha->reg->u2.isp4010.port_ctrl));

		printk("0x%02X port_status     = 0x%08X\n",
		       (uint8_t) offsetof(isp_reg_t, u2.isp4010.port_status),
		       RD_REG_DWORD(&ha->reg->u2.isp4010.port_status));

		printk("0x%02X req_q_out       = 0x%08X\n",
		       (uint8_t) offsetof(isp_reg_t, u2.isp4010.req_q_out),
		       RD_REG_DWORD(&ha->reg->u2.isp4010.req_q_out));

		printk("0x%02X gp_out          = 0x%08X\n",
		       (uint8_t) offsetof(isp_reg_t, u2.isp4010.gp_out),
		       RD_REG_DWORD(&ha->reg->u2.isp4010.gp_out));

		printk("0x%02X gp_in           = 0x%08X\n",
		       (uint8_t) offsetof(isp_reg_t, u2.isp4010.gp_in),
		       RD_REG_DWORD(&ha->reg->u2.isp4010.gp_in));

		printk("0x%02X port_err_status = 0x%08X\n",
		       (uint8_t) offsetof(isp_reg_t, u2.isp4010.port_err_status),
		       RD_REG_DWORD(&ha->reg->u2.isp4010.port_err_status));
	}
	else if (IS_QLA4022(ha)) {

		printk("Page 0 Registers:\n");
	
		printk("0x%02X ext_hw_conf     = 0x%08X\n",
		    (uint8_t) offsetof(isp_reg_t, u2.isp4022.p0.ext_hw_conf),
		    RD_REG_DWORD(&ha->reg->u2.isp4022.p0.ext_hw_conf));
		
		printk("0x%02X port_ctrl       = 0x%08X\n",
		    (uint8_t) offsetof(isp_reg_t, u2.isp4022.p0.port_ctrl),
		    RD_REG_DWORD(&ha->reg->u2.isp4022.p0.port_ctrl));
		
		printk("0x%02X port_status     = 0x%08X\n",
		    (uint8_t) offsetof(isp_reg_t, u2.isp4022.p0.port_status),
		    RD_REG_DWORD(&ha->reg->u2.isp4022.p0.port_status));
		
		printk("0x%02X gp_out          = 0x%08X\n",
		    (uint8_t) offsetof(isp_reg_t, u2.isp4022.p0.gp_out),
		    RD_REG_DWORD(&ha->reg->u2.isp4022.p0.gp_out));
		
		printk("0x%02X gp_in           = 0x%08X\n",
		    (uint8_t) offsetof(isp_reg_t, u2.isp4022.p0.gp_in),
		    RD_REG_DWORD(&ha->reg->u2.isp4022.p0.gp_in));
		
		printk("0x%02X port_err_status = 0x%08X\n",
		    (uint8_t) offsetof(isp_reg_t, u2.isp4022.p0.port_err_status),
		    RD_REG_DWORD(&ha->reg->u2.isp4022.p0.port_err_status));
		
		printk("Page 1 Registers:\n");
		
		WRT_REG_DWORD(&ha->reg->ctrl_status, HOST_MEM_CFG_PAGE &
			   SET_RMASK(CSR_SCSI_PAGE_SELECT));
		
		printk("0x%02X req_q_out       = 0x%08X\n",
		    (uint8_t) offsetof(isp_reg_t, u2.isp4022.p1.req_q_out),
		    RD_REG_DWORD(&ha->reg->u2.isp4022.p1.req_q_out));
		
		WRT_REG_DWORD(&ha->reg->ctrl_status, PORT_CTRL_STAT_PAGE &
			   SET_RMASK(CSR_SCSI_PAGE_SELECT));
		
	}
}

void
ql4im_print_iocb_passthru(scsi_qla_host_t *ha, INT_IOCB_PASSTHRU *iocb)
{
	printk("SendDMAOffset=0x%x, RspDMAOffset=0x%x\n",
	       iocb->SendDMAOffset, iocb->RspDMAOffset);
	printk("IOCBCmdBuffer:\n");
	ql4im_dump_bytes(iocb->IOCBCmdBuffer, sizeof(iocb->IOCBCmdBuffer));
	printk("IOCBStatusBuffer:\n");
	ql4im_dump_bytes(iocb->IOCBStatusBuffer, sizeof(iocb->IOCBStatusBuffer));
	printk("SendData:  (SendData %p, Len=%d)\n", iocb->SendData, iocb->SendDataLen);
	ql4im_dump_bytes(iocb->SendData, iocb->SendDataLen);
	printk("RspData:  (RspData %p, Len=%d)\n", iocb->RspData, iocb->RspDataLen);
	ql4im_dump_bytes(iocb->RspData, iocb->RspDataLen);
}

/*
 * Overrides for Emacs so that we almost follow Linus's tabbing style.
 * Emacs will notice this stuff at the end of the file and automatically
 * adjust the settings for this buffer only.  This must remain at the end
 * of the file.
 * ---------------------------------------------------------------------------
 * Local variables:
 * c-indent-level: 2
 * c-brace-imaginary-offset: 0
 * c-brace-offset: -2
 * c-argdecl-indent: 2
 * c-label-offset: -2
 * c-continued-statement-offset: 2
 * c-continued-brace-offset: 0
 * indent-tabs-mode: nil
 * tab-width: 8
 * End:
 */


