/*
 * QLogic iSCSI IOCTL Module
 * Copyright (c)  2003-2012 QLogic Corporation
 *
 * See LICENSE.qla4xxx for copyright and licensing details.
 */

#ifndef __QIM_DEF_H__
#define __QIM_DEF_H__

#include "ql4im_settings.h"
#include "ql4im_version.h"
#include "ql4_def.h"
#include "qlisioct.h"
#include "qlinioct.h"
#include "ql4im_dbg.h"

#include <linux/spinlock.h>
#include <asm/semaphore.h>
#include <linux/timer.h>


extern struct list_head **drvr_ha_list_ptr;
extern rwlock_t **drvr_ha_list_lock_ptr;

extern rwlock_t ql4im_ha_list_lock;
extern struct list_head ql4im_ha_list;

#ifndef MAX_STR_SIZE
#define MAX_STR_SIZE		80
#endif

#ifndef MAX_FIBRE_DEVICES
#define MAX_FIBRE_DEVICES	512
#endif

#ifndef MAX_FIBRE_LUNS
#define MAX_FIBRE_LUNS  	256
#endif

#ifndef MAX_RSCN_COUNT
#define	MAX_RSCN_COUNT		32
#endif

#ifndef MAX_HOST_COUNT
#define	MAX_HOST_COUNT		16
#endif

#ifndef FALSE
#define FALSE			0
#endif

#ifndef TRUE
#define TRUE			1
#endif

#define	QIM_SUCCESS		0
#define	QIM_FAILED		1


typedef struct hba_ioctl{

	/* This semaphore protects several threads to do ioctl commands
	 * concurrently.
	 *-------------------------------------------------------------------*/
	struct semaphore  ioctl_sem;

	/* Passthru cmd/completion */
	struct semaphore	ioctl_cmpl_sem;
	struct timer_list	ioctl_cmpl_timer;
	uint32_t		ioctl_tov;
	struct scsi_cmnd	*ioctl_err_cmd;
	uint8_t			ioctl_scsi_pass_in_progress;
	uint8_t			ioctl_iocb_pass_in_progress;

	/* AEN queue */
	uint8_t		aen_q_head;	/* index to the current head of q */
	uint8_t		aen_q_tail;	/* index to the current tail of q */
	void		*aen_tracking_queue;/* points to async events buffer */

	/* Misc. */
	uint32_t	flags;
#define	IOCTL_OPEN			BIT_0
#define	IOCTL_AEN_TRACKING_ENABLE	BIT_1
    	uint8_t		*scrap_mem;	/* per ha scrap buf for ioctl usage */
	uint32_t	scrap_mem_size; /* total size */
	uint32_t	scrap_mem_used; /* portion used */

        uint8_t         *core_dump_mem; /* per ha core dump buf */
} hba_ioctl_context;

typedef struct ql4im_scsi_qla_host {
	struct list_head	list;

	uint16_t		host_no;
	uint16_t		instance;
	struct scsi_qla_host	*drvr_ha;

	hba_ioctl_context	*ioctl;
	void      		*ioctl_dma_bufv;
	dma_addr_t 		ioctl_dma_bufp;
	uint32_t   		ioctl_dma_buf_len;

	uint8_t			drv_verstr[MAX_STR_SIZE];
	uint8_t			drv_major;
	uint8_t			drv_minor;
	uint8_t			drv_patch;
	uint8_t			drv_beta;

	/* Info Needed for Management App */
	/* --- From GetFwVersion --- */
	uint32_t        firmware_version[2];
	uint32_t        patch_number;
	uint32_t        build_number;
	/* --- From Init_FW --- */
	uint16_t	ifcb_size;
	uint8_t		acb_version;
	uint8_t		resvd1;
#if ISNS_SUPPORTED
	uint16_t        tcp_options;
	uint16_t        isns_server_port_number;
	uint8_t         isns_server_ip_addr[IP_ADDR_LEN];
	uint32_t	ipv6_options;

	atomic_t   isns_state;
	#define ISNS_STATE_TCP_DISCONNECTED	0
	#define ISNS_STATE_TCP_CONNECTED	1
	#define ISNS_STATE_STARTING_SRV         2
	#define ISNS_STATE_RESTART_SRV_WAIT     3
#endif
	uint8_t         ip_address[IP_ADDR_LEN];
} ql4im_scsi_qla_host_t;


#include "ql4im_glbl.h"

#endif /* ifndef __QIM_DEF_H__ */


