/*
 * QLogic iSCSI IOCTL Module
 * Copyright (c)  2003-2012 QLogic Corporation
 *
 * See LICENSE.qla4xxx for copyright and licensing details.
 */

#ifndef _QLINIOCT_H_
#define _QLINIOCT_H_

#include "qlisioln.h"

/*
   Ioctl
*/

/*
  General
*/

/*
 * Command Codes definitions
 */
#define INT_CC_GET_DATA			EXT_CC_RESERVED0A_OS
#define INT_CC_SET_DATA			EXT_CC_RESERVED0B_OS
#define INT_CC_DIAG_PING		EXT_CC_RESERVED0C_OS
#define INT_CC_ISCSI_LOOPBACK		EXT_CC_RESERVED0D_OS
#define INT_CC_HBA_RESET		EXT_CC_RESERVED0E_OS
#define INT_CC_COPY_FW_FLASH		EXT_CC_RESERVED0F_OS
#define INT_CC_LOGOUT_ISCSI		EXT_CC_RESERVED0G_OS
#define INT_CC_FW_PASSTHRU		EXT_CC_RESERVED0H_OS
#define INT_CC_IOCB_PASSTHRU		EXT_CC_RESERVED0I_OS
#define INT_CC_RESTORE_FACTORY_DEFAULTS EXT_CC_RESERVED0J_OS
#define INT_CC_DIAG_MODE		EXT_CC_RESERVED0K_OS
#define INT_CC_DIAG_TEST		EXT_CC_RESERVED0L_OS

/*
 * Sub codes for diag mode
 * Use in combination with INT_CC_DIAG_MODE in ioctl call
 */
#define INT_SC_SET_DIAG_MODE	0
#define INT_SC_RESET_DIAG_MODE	1

/*
 * Sub codes for Get Data.
 * Use in combination with INT_GET_DATA as the ioctl code
 */
#define INT_SC_GET_FLASH			1
#define INT_SC_GET_CORE_DUMP			2

/*
 * Sub codes for Set Data.
 * Use in combination with INT_SET_DATA as the ioctl code
 */
#define INT_SC_SET_FLASH			1

#define INT_DEF_DNS_ENABLE                  0x0100

/*
 * ***********************************************************************
 * INIT_FW_ISCSI_ALL
 * ***********************************************************************
 */
typedef struct _INT_INIT_FW_ISCSI_ALL {
	uint8_t   Version;					/* 1   */
	uint8_t   Reserved0;					/* 1   */
	uint16_t  FWOptions;					/* 2   */
	uint16_t  exeThrottle;					/* 2   */
	uint8_t   retryCount;					/* 1   */
	uint8_t   retryDelay;					/* 1   */
	uint16_t  EthernetMTU;					/* 2   */
	uint16_t  addFWOptions;					/* 2   */
	uint8_t   HeartBeat;					/* 1   */
	uint8_t   Reserved1;					/* 1   */
	uint16_t  Reserved2;					/* 2   */
	uint16_t  ReqQOutPtr;					/* 2   */
	uint16_t  RespQInPtr;					/* 2   */
	uint16_t  ReqQLen;					/* 2   */
	uint16_t  RespQLen;					/* 2   */
	uint32_t  ReqQAddr[2];					/* 8   */
	uint32_t  RespQAddr[2];					/* 8   */
	uint32_t  IntRegBufAddr[2];				/* 8   */
	uint16_t  iSCSIOptions;					/* 2   */
	uint16_t  TCPOptions;					/* 2   */
	uint16_t  IPOptions;					/* 2   */
	uint16_t  MaxRxDataSegmentLen;				/* 2   */
	uint16_t  recvMarkerInt;				/* 2   */
	uint16_t  sendMarkerInt;				/* 2   */
	uint16_t  Reserved3;					/* 2   */
	uint16_t  firstBurstSize;				/* 2   */
	uint16_t  DefaultTime2Wait;				/* 2   */
	uint16_t  DefaultTime2Retain;				/* 2   */
	uint16_t  maxOutstandingR2T;				/* 2   */
	uint16_t  keepAliveTimeout;				/* 2   */
	uint16_t  portNumber;					/* 2   */
	uint16_t  maxBurstSize;					/* 2   */
	uint32_t  Reserved4;					/* 4   */
	uint8_t   IPAddr[16];					/* 16  */
	uint8_t   SubnetMask[16];				/* 16  */
	uint8_t   IPGateway[16];				/* 16  */
	uint8_t   DNSsvrIP[4];					/* 4   */
	uint8_t   DNSsecSvrIP[4];				/* 4   */
	uint8_t   Reserved5[8];					/* 8   */
	uint8_t   Alias[EXT_DEF_ISCSI_ALIAS_LEN];		/* 32  */
	uint32_t  targetAddr0;					/* 4   */
	uint32_t  targetAddr1;					/* 4   */
	uint32_t  CHAPTableAddr0;				/* 4   */
	uint32_t  CHAPTableAddr1;				/* 4   */
	uint8_t   EthernetMACAddr[6];				/* 6   */
	uint16_t  TargetPortalGrp;				/* 2   */
	uint8_t   SendScale;					/* 1   */
	uint8_t   RecvScale;					/* 1   */
	uint8_t   TypeOfService;				/* 1   */
	uint8_t   Time2Live;					/* 1   */
	uint16_t  VLANPriority;					/* 2   */
	uint16_t  Reserved6;					/* 2   */
	uint8_t   SecondaryIPAddr[16];				/* 16  */
	uint8_t   iSNSServerAdr[4];				/* 4   */
	uint16_t  iSNSServerPort;				/* 2   */
	uint8_t   Reserved7[10];				/* 10  */
	uint8_t   SLPDAAddr[16];				/* 16  */
	uint8_t   iSCSIName[EXT_DEF_ISCSI_NAME_LEN];		/* 256 */
} INT_INIT_FW_ISCSI_ALL, *PINT_INIT_FW_ISCSI_ALL;		/* 512 */

/*
 * ***********************************************************************
 * INT_DEVICE_ENTRY_ISCSI_ALL
 * ***********************************************************************
 */
typedef struct _INT_DEVICE_ENTRY_ISCSI_ALL {
	uint8_t   Options;					/* 1   */
	uint8_t   Control;					/* 1   */
	uint16_t  exeThrottle;					/* 2   */
	uint16_t  exeCount;					/* 2   */
	uint8_t   retryCount;					/* 1   */
	uint8_t   retryDelay;					/* 1   */
	uint16_t  iSCSIOptions;					/* 2   */
	uint16_t  TCPOptions;					/* 2   */
	uint16_t  IPOptions;					/* 2   */
	uint16_t  MaxRxDataSegmentLen;				/* 2   */
	uint16_t  RecvMarkerInterval;				/* 2   */
	uint16_t  SendMarkerInterval;				/* 2   */
	uint16_t  MaxTxDataSegmentLen;				/* 2   */
	uint16_t  firstBurstSize;				/* 2   */
	uint16_t  DefaultTime2Wait;				/* 2   */
	uint16_t  DefaultTime2Retain;				/* 2   */
	uint16_t  maxOutstandingR2T;				/* 2   */
	uint16_t  keepAliveTimeout;				/* 2   */
	uint8_t   InitiatorSessID[EXT_DEF_ISCSI_ISID_SIZE];	/* 6   */
	uint16_t  TargetSessID;					/* 2   */
	uint16_t  portNumber;					/* 2   */
	uint16_t  maxBurstSize;					/* 2   */
	uint16_t  taskMngmntTimeout;				/* 2   */
	uint16_t  Reserved0;					/* 2   */
	uint8_t   IPAddress[16];				/* 16  */
	uint8_t   Alias[EXT_DEF_ISCSI_ALIAS_LEN];		/* 32  */
	uint8_t   targetAddr[EXT_DEF_ISCSI_TADDR_SIZE];		/* 32  */
	/* need to find new definition XXX */
	uint8_t   res[64];
	uint8_t   iSCSIName[EXT_DEF_ISCSI_NAME_LEN];		/* 256 */
	uint16_t  ddbLink;					/* 2   */
	uint16_t  chapTableIndex;				/* 2   */
	uint16_t  targetPortalGrp;				/* 2   */
	uint16_t  Reserved1;					/* 2   */
	uint32_t  statSN;					/* 4   */
	uint32_t  expStatSN;					/* 4   */
} INT_DEVICE_ENTRY_ISCSI_ALL, *PINT_DEVICE_ENTRY_ISCSI_ALL;	/* 464 */

/*
 * ****************************************************************************
 * INT_DEVDDB_ENTRY
 * ****************************************************************************
 */

typedef struct _FLASH_DEVDB_ENTRY {
	INT_DEVICE_ENTRY_ISCSI_ALL        entryData;		/* 0-1C7   */
	uint8_t                           RES0[0x2C];		/* 1C8-1FB */
	uint16_t                          ddbValidCookie;	/* 1FC-1FD */
	uint16_t                          ddbValidSize;		/* 1FE-1FF */
} FLASH_DEVDB_ENTRY, *PFLASH_DEVDB_ENTRY;

/*
 * ****************************************************************************
 * INT_FLASH_INITFW
 * ****************************************************************************
 */

typedef struct _FLASH_INITFW {
	INT_INIT_FW_ISCSI_ALL   initFWData;
	uint32_t                  validCookie;
} FLASH_INITFW, *PFLASH_INITFW;


/*
 * ***********************************************************************
 * INT_ACCESS_FLASH
 * ***********************************************************************
 */

#define INT_DEF_AREA_TYPE_FW_IMAGE1		0x01
#define INT_DEF_AREA_TYPE_FW_IMAGE2		0x02
#define INT_DEF_AREA_TYPE_DRIVER		0x03
#define INT_DEF_AREA_TYPE_DDB			0x04
#define INT_DEF_AREA_TYPE_INIT_FW		0x05
#define INT_DEF_AREA_TYPE_SYS_INFO		0x06

#define INT_DEF_FLASH_BLK_SIZE			0x4000
#define INT_DEF_FLASH_PHYS_BLK_SIZE		0x20000

#define INT_ISCSI_FW_IMAGE2_FLASH_OFFSET	0x01000000
#define INT_ISCSI_SYSINFO_FLASH_OFFSET		0x02000000
#define INT_ISCSI_DRIVER_FLASH_OFFSET		0x03000000
#define INT_ISCSI_INITFW_FLASH_OFFSET		0x04000000
#define INT_ISCSI_DDB_FLASH_OFFSET		0x05000000
#define INT_ISCSI_CHAP_FLASH_OFFSET		0x06000000
#define INT_ISCSI_FW_IMAGE1_FLASH_OFFSET	0x07000000
#define INT_ISCSI_BIOS_FLASH_OFFSET		0x0d000000
#define INT_ISCSI_OFFSET_MASK			0x00FFFFFF
#define INT_ISCSI_PAGE_MASK			0x0F000000

#define INT_ISCSI_ACCESS_FLASH			0x00000000
#define INT_ISCSI_ACCESS_RAM			0x10000000
#define INT_ISCSI_ACCESS_MASK			0xF0000000

/* WRITE_FLASH option definitions */
#define INT_WRITE_FLASH_OPT_HOLD		0 /* Write data to FLASH but
						     do not Commit */
#define INT_WRITE_FLASH_OPT_CLEAR_REMAINING	1 /* Write data to FLASH but
						     do not Commit any data
						     not written before
						     commit will be cleared
						     (set to 0xFF)	*/
#define INT_WRITE_FLASH_OPT_COMMIT_DATA		2 /* Commit (Burn) data to
						     FLASH */


typedef struct _INT_ACCESS_FLASH {
	uint32_t  AreaType;					/* 4      */
	uint32_t  DataLen;					/* 4      */
	uint32_t  DataOffset;					/* 4      */
	uint8_t   FlashData[INT_DEF_FLASH_BLK_SIZE];		/* 0x4000 */
	uint32_t  Options;					/* 4      */
} INT_ACCESS_FLASH, *PINT_ACCESS_FLASH;				/* 0x4010 */

/*
 * ****************************************************************************
 * INT_FLASH_DRIVER_PARAM
 * ****************************************************************************
 */

typedef struct _INT_FLASH_DRIVER_PARAM {
	uint16_t  DiscoveryTimeOut;				/* 2   */
	uint16_t  PortDownTimeout;				/* 2   */
	uint32_t  Reserved[32];					/* 128 */
} INT_FLASH_DRIVER_PARAM, *PINT_FLASH_DRIVER_PARAM;		/* 132 */


#define VALID_FLASH_INITFW		0x11BEAD5A

#define FLASH_ISCSI_MAX_DDBS		64
#define FLASH_DDB_VALID_COOKIE		0x9034 /* this value indicates this
						  entry in flash is valid */
#define FLASH_DDB_INVALID_COOKIE	0x0    /* this value is used to set
						  the entry to invalid    */

/*
 * ****************************************************************************
 * INT_HBA_SYS_INFO
 * ****************************************************************************
 */

typedef struct _INT_HBA_SYS_INFO {
	uint32_t  cookie;					/* 4   */
	uint32_t  physAddrCount;				/* 4   */
	uint8_t   macAddr0[6];					/* 6   */
	uint8_t   reserved0[2];					/* 2   */
	uint8_t   macAddr1[6];					/* 6   */
	uint8_t   reserved1[2];					/* 2   */
	uint8_t   macAddr2[6];					/* 6   */
	uint8_t   reserved2[2];					/* 2   */
	uint8_t   macAddr3[6];					/* 6   */
	uint8_t   reserved3[2];					/* 2   */
	uint8_t   vendorId[128];				/* 128 */
	uint8_t   productId[128];				/* 128 */
	uint32_t  serialNumber;					/* 4   */
	uint32_t  pciDeviceVendor;				/* 4   */
	uint32_t  pciDeviceId;					/* 4   */
	uint32_t  pciSubsysVendor;				/* 4   */
	uint32_t  pciSubsysId;					/* 4   */
	uint32_t  crumbs;					/* 4   */
	uint32_t  enterpriseNumber;				/* 4   */
	uint32_t  crumbs2;					/* 4   */
} INT_HBA_SYS_INFO, *PINT_HBA_SYS_INFO;				/* 328 */

/*
 * ****************************************************************************
 * INT_FW_DW_HDR
 * ****************************************************************************
 */

/* File header for FW */
typedef struct _INT_FW_DL_HDR {
	uint32_t  Size;		/* download size, excluding DL_HDR & EXT_HDR*/
	uint32_t  Checksum;	/* Checksum of download file, excluding DL_HDR
				   & EXT_HDR */
	uint32_t  HdrChecksum;	/* Checksum of header area should be zero */
	uint32_t  Flags;	/* See Flags bits defined above */
	uint32_t  Cookie;	/* Target specific identifier */
	uint32_t  Target;	/* Target specific identifier */
	uint32_t  Reserved0;	/* Reserved */
	uint32_t  Reserved1;	/* Reserved */
	uint8_t   Copyright[64];/* Copyright */
	uint8_t   Version[32];	/* Version String */
} INT_FW_DL_HDR, *PINT_FW_DL_HDR;

/* File header for BIOS */
typedef struct _INT_BIOS_HDR {
	uint8_t   BIOSidCode55;
	uint8_t   BIOSidCodeAA;
	uint8_t   reserved[52];
	uint8_t   BIOSminorVer;
	uint8_t   BIOSmajorVer;
} INT_BIOS_HDR, *PINT_BIOS_HDR;

typedef struct _INT_SDMBIOS_NVRAM {
	uint16_t  Flags;
	uint8_t   PriID;
	uint64_t  PriLUN;
	uint8_t   SecID;
	uint64_t  SecLUN;
} INT_SDMBIOS_NVRAM, *PINT_SDMBIOS_NVRAM;

/*
 * ****************************************************************************
 * INT_HBA_RESET
 * ****************************************************************************
 */

typedef struct _INT_HBA_RESET {
	uint32_t ResetFlag;					/* 4 */
#define HBA_RESET_FUNCTION_RESET	0
#define HBA_RESET_CHIP_RESET		1
	uint32_t Reserved[1];					/* 4 */
} INT_HBA_RESET, *PINT_HBA_RESET;				/* 8 */

/*
 * ****************************************************************************
 * INT_COPY_FW_FLASH
 * ****************************************************************************
 */

typedef struct _INT_COPY_FW_FLASH {
	uint32_t  Options;					/* 4  */
} INT_COPY_FW_FLASH, *PINT_COPY_FW_FLASH;			/* 4  */

#define INT_COPY_FLASH_PRIMARY_TO_SECONDARY	0
#define INT_COPY_FLASH_SECONDARY_TO_PRIMARY	1

/*
 * ****************************************************************************
 * INT_LOGOUT_ISCSI
 * ****************************************************************************
 */

/* Logout Options */

#define INT_DEF_CLOSE_SESSION			0x0001
#define INT_DEF_RELOGIN_CONNECTION		0x0002
#define INT_DEF_DELETE_DDB		      	0x0004
#define INT_DEF_REINDEX_DDB		      	0x0008  /* Solaris & Linux */

typedef struct _INT_LOGOUT_ISCSI {
	uint16_t    TargetID;					/* 2   */
	uint16_t    ConnectionID;				/* 2   */
	uint16_t    Options;					/* 2   */
	uint32_t    NewTargetID;				/* 4   */
} INT_LOGOUT_ISCSI, *PINT_LOGOUT_ISCSI;				/* 10  */

/*
 * ****************************************************************************
 * INT_PING
 * ****************************************************************************
 */

typedef struct _INT_PING {
	EXT_ISCSI_IP_ADDR       IPAddr;				/* 20  */
	uint16_t                  PacketSize;			/* 2   */
	uint16_t                  Reserved;			/* 2   */
} INT_PING, *PINT_PING;						/* 24  */

/*
 * ****************************************************************************
 * INT_DIAG_TEST_RESULT
 * ****************************************************************************
 */

/* Opcodes for ql4_iscsi_diag */
#define TEST_LOCAL_RAM_SIZE		2
#define TEST_LOCAL_RAM_RDWR		3
#define TEST_RISC_RAM			4
#define TEST_NVRAM			5
#define TEST_FLASH_ROM			6
#define TEST_INTERNAL_LOOPBACK_DIAG	7
#define TEST_EXTERNAL_LOOPBACK_DIAG	8
#define TEST_DMA_TRANSFER		9

#define MB_REG_COUNT			8

typedef struct _INT_DIAG_TEST_RESULT {
	uint32_t	mbCmd[MB_REG_COUNT];
	uint32_t	mbSts[MB_REG_COUNT];
	uint32_t	Reserved[16];
} INT_DIAG_TEST_RESULT, *PINT_DIAG_TEST_RESULT;

/*
 * ****************************************************************************
 * INT_IOCB_PASSTHRU
 * ****************************************************************************
 */

#define INT_DEF_IOCB_BUF_SIZE			64
#define INT_DEF_IOCB_DATA_SIZE			1500

typedef struct _INT_IOCB_PASSTHRU {
	uint32_t    SendDMAOffset;				/* 4    */
	uint32_t    RspDMAOffset;				/* 4    */
	uint8_t     IOCBCmdBuffer[INT_DEF_IOCB_BUF_SIZE];	/* 64   */
	uint8_t     IOCBStatusBuffer[INT_DEF_IOCB_BUF_SIZE];	/* 64   */
	uint32_t    SendDataLen;				/* 4    */
	uint8_t     SendData[INT_DEF_IOCB_DATA_SIZE];		/* 1500 */
	uint32_t    RspDataLen;					/* 4    */
	uint8_t     RspData[INT_DEF_IOCB_DATA_SIZE];		/* 1500 */
	uint32_t    Reserved;					/* 4    */
} INT_IOCB_PASSTHRU, *PINT_IOCB_PASSTHRU;			/* 3148 */


/*
 * ****************************************************************************
 * INT_CC_FW_PASSTHRU
 * ****************************************************************************
 */

/* FW PASSTHRU Defines */
#define INT_DEF_FW_PASSHTRU_BLK_SIZE		0x4000

#define INT_DEF_DATA_TYPE_CHAP_TABLE		0x0001
#define INT_DEF_DATA_TYPE_DDB			0x0002
#define INT_DEF_DATA_TYPE_INITFW		0x0003
#define INT_DEF_DATA_TYPE_FW_IMAGE		0x0004

#define INT_DEF_DATA_LOCATION_HBA_FLASH		0x0001
#define INT_DEF_DATA_LOCATION_HBA_RAM		0x0002

#define INT_DEF_DATA_READ			0x0001
#define INT_DEF_DATA_WRITE			0x0002

#define INT_DEF_DATA_INIT			0x0001
#define INT_DEF_DATA_COMMIT			0x0002

/*
 * ****************************************************************************
 * INT_RESTORE_FACTORY_DEFAULTS
 * ****************************************************************************
 */

#define INT_DEF_FACT_DFLT_MASK_IFCB		0x00000001
#define INT_DEF_FACT_DFLT_MASK_DDB		0x00000002
#define INT_DEF_FACT_DFLT_MASK_CHAP		0x00000004
#define INT_DEF_FACT_DFLT_MASK_IPSEC		0x00000008
#define INT_DEF_FACT_DFLT_MASK_NVRAM		0x00000010
#define INT_DEF_FACT_DFLT_MASK_INVERSE_INITFW   0x02000000

#define INT_DEF_FACT_DFLT_IFCB_ALL		0xffffffff
#define INT_DEF_FACT_DFLT_IFCB_FW_OPT		0x00000001
#define INT_DEF_FACT_DFLT_IFCB_MTU		0x00000002
#define INT_DEF_FACT_DFLT_IFCB_ADD_FW_OPT	0x00000004
#define INT_DEF_FACT_DFLT_IFCB_HEARTBEAT	0x00000008
#define INT_DEF_FACT_DFLT_IFCB_ISCSI_OPT	0x00000010
#define INT_DEF_FACT_DFLT_IFCB_TCP_OPT		0x00000020
#define INT_DEF_FACT_DFLT_IFCB_IP_OPT		0x00000040
#define INT_DEF_FACT_DFLT_IFCB_MAX_RECV_SEG_LEN	0x00000080
#define INT_DEF_FACT_DFLT_IFCB_FIRST_BURST_LEN	0x00000100
#define INT_DEF_FACT_DFLT_IFCB_MAX_OUTSTANDING_R2T	0x00000200
#define INT_DEF_FACT_DFLT_IFCB_ISCSI_PORT_NUM	0x00000400
#define INT_DEF_FACT_DFLT_IFCB_MAX_BURST_LEN	0x00000800
#define INT_DEF_FACT_DFLT_IFCB_IP_ADDR		0x00001000
#define INT_DEF_FACT_DFLT_IFCB_SUBNET_MASK	0x00002000
#define INT_DEF_FACT_DFLT_IFCB_GATEWAY_ADDR	0x00004000
#define INT_DEF_FACT_DFLT_IFCB_PRI_DNS_ADDR	0x00008000
#define INT_DEF_FACT_DFLT_IFCB_SEC_DNS_ADDR	0x00010000
#define INT_DEF_FACT_DFLT_IFCB_MIN_EPHMRL_PORT_NUM	0x00020000
#define INT_DEF_FACT_DFLT_IFCB_MAX_EPHMRL_PORT_NUM	0x00040000
#define INT_DEF_FACT_DFLT_IFCB_ABORT_TIMER	0x00080000
#define INT_DEF_FACT_DFLT_IFCB_ISCSI_ALIAS	0x00100000
#define INT_DEF_FACT_DFLT_IFCB_TCP_WIN_SCALE	0x00200000
#define INT_DEF_FACT_DFLT_IFCB_SEC_IP_ADDR	0x00400000
#define INT_DEF_FACT_DFLT_IFCB_ISNS_IP_ADDR	0x00800000
#define INT_DEF_FACT_DFLT_IFCB_ISNS_PORT_NUM	0x01000000
#define INT_DEF_FACT_DFLT_IFCB_ISCSI_NAME	0x02000000

typedef struct _INT_RESTORE_FACTORY_DEFAULTS
{
	uint32_t	BlockMask;				/* 4   */
	uint32_t	IFCBMask1;				/* 4   */
	uint32_t	IFCBMask2;				/* 4   */
	uint32_t	Reserved[5];				/* 20  */
} INT_RESTORE_FACTORY_DEFAULTS, *PINT_RESTORE_FACTORY_DEFAULTS; /* 32  */


/*
 * ****************************************************************************
 * INT_ACCESS_CORE_DUMP
 * ****************************************************************************
 */

typedef struct _INT_ACCESS_CORE_DUMP {
	uint32_t  DataLen;					/* 4  - 0x00 */
	uint32_t  Offset;						/* 4  - 0x04 */
	uint8_t   LastBlockFlag;					/* 1  - 0x08 */
	uint8_t   Reserved[7];					/* 7  - 0x09 */
	uint8_t   Data[];						/* 0  - 0x10 */
} INT_ACCESS_CORE_DUMP, *PINT_ACCESS_CORE_DUMP;			/* 0x10  */

#define CORE_DUMP_HEADER_SIZE			0x256
#endif /* _QLINIOCT_H_ */
