/*
 * QLogic iSCSI HBA Driver
 * Copyright (c)  2003-2012 QLogic Corporation
 *
 * See LICENSE.qla4xxx for copyright and licensing details.
 */

/******************************************************************************
 *             Please see release.txt for revision history.                   *
 *                                                                            *
 ******************************************************************************
 * Function Table of Contents:
 *      pci_set_dma_mask
 *      qla4xxx_config_dma_addressing
 *      qla4xxx_detect
 *      qla4xxx_display_config
 *      qla4xxx_alloc_srb_pool
 *      qla4xxx_free_srb_pool
 *      qla4xxx_mem_alloc
 *      qla4xxx_mem_free
 *      qla4xxx_register_resources
 *      qla4xxx_set_info
 *      copy_mem_info
 *      copy_info
 *      qla4xxx_get_adapter_handle
 *      qla4xxx_release
 *      del_from_active_array
 *      qla4xxx_normalize_dma_addr
 *      qla4xxx_alloc_cont_entry
 *      qla4xxx_send_command_to_isp
 *      qla4xxx_complete_request
 *      qla4xxx_queuecommand
 *      qla4xxx_extend_timeout
 *      qla4xxx_start_io
 *      qla4xxx_os_cmd_timeout
 *      qla4xxx_add_timer_to_cmd
 *      qla4xxx_delete_timer_from_cmd
 *      qla4xxx_timer
 *      qla4xxx_do_dpc
 *      qla4xxx_panic
 *      qla4xxx_eh_wait_on_command
 *      qla4xxx_wait_for_hba_online
 *      qla4xxx_eh_abort
 *      qla4xxx_hw_reset
 *      qla4xxx_soft_reset
 *      qla4xxx_cmd_wait
 *      qla4xxx_recover_adapter
 *      qla4xxx_eh_wait_for_active_target_commands
 *      qla4xxx_eh_device_reset
 *      qla4xxx_eh_bus_reset
 *      qla4xxx_reset_target
 *      qla4xxx_abort_active_cmds
 *      qla4xxx_eh_host_reset
 *      apidev_open
 *      apidev_close
 *      apidev_ioctl
 *      apidev_init
 *      apidev_cleanup
 ****************************************************************************/

#include "ql4_def.h"
#include "ql4_83xx.h"

#include <linux/version.h>
#include <linux/moduleparam.h>
#include <linux/vmalloc.h>
#include <linux/smp_lock.h>
#include <linux/delay.h>

#include <scsi/scsi_tcq.h>
#include <scsi/scsicam.h>

/*
 * List of host adapters
 *---------------------------------------------------------------------------*/
/*
 * True list of host adapters.  Available for use after qla4xxx_detect has completed
 */
LIST_HEAD(qla4xxx_hostlist);
rwlock_t qla4xxx_hostlist_lock = RW_LOCK_UNLOCKED;

struct list_head *qla4xxx_hostlist_ptr = &qla4xxx_hostlist;
rwlock_t *qla4xxx_hostlist_lock_ptr = &qla4xxx_hostlist_lock;
char qla4xxx_version_str[64];

void qla80xx_update_crb_int_ptr(struct scsi_qla_host *);

static int qla4xxx_hba_count = 0;

/*
 * Command line options
 *---------------------------------------------------------------------------*/
/*
 * Just in case someone uses commas to separate items on the insmod
 * command line, we define a dummy buffer here to avoid having insmod
 * write wild stuff into our code segment
 */
int ql4xdiscoverywait=10;
module_param(ql4xdiscoverywait, int, S_IRUGO|S_IWUSR);
MODULE_PARM_DESC(ql4xdiscoverywait,
		 "Max target discovery wait time");

int ql4xkeepalive=0xDEAD;
module_param(ql4xkeepalive, int, S_IRUGO|S_IWUSR);
MODULE_PARM_DESC(ql4xkeepalive,
		 "Keep Alive Timeout");

int ql4xdontresethba=0;
module_param(ql4xdontresethba, int, S_IRUGO|S_IWUSR);
MODULE_PARM_DESC(ql4xdontresethba,
		 "Dont reset the HBA when the driver gets 0x8002 AEN\n"
		 "or for driver recovery\n"
		 "\t\t 0 - Reset HBA (Default)\n"
		 "\t\t 1 - Avoid resetting HBA\n"
		 "\t\t 2 - Only reset in eh_host_reset");

static int ql4xcmdretrycount = 20;
module_param(ql4xcmdretrycount, int, S_IRUGO|S_IWUSR);
MODULE_PARM_DESC(ql4xcmdretrycount,
		 "Maximum number of mid-layer retries allowed for a command.  "
		 "Default value is 20");

int ql4xqfullrampup = 120;
module_param(ql4xqfullrampup, int, S_IRUGO|S_IWUSR);
MODULE_PARM_DESC(ql4xqfullrampup,
                "Number of seconds to wait to begin to ramp-up the queue\n"
                "depth for a device after a queue-full condition has been\n"
                "detected.  Default is 120 seconds.");

int ql4xmaxqdepth = 0;
module_param(ql4xmaxqdepth, int, S_IRUGO|S_IWUSR);
MODULE_PARM_DESC(ql4xmaxqdepth,
		 "Maximum queue depth to report for target devices.");

int ql4xmaxcmds = 0;
module_param(ql4xmaxcmds, int, S_IRUGO|S_IWUSR);
MODULE_PARM_DESC(ql4xmaxcmds,
		"Maximum outstanding commands per host adapter.\n"
		"\t\t Default value is 0, which uses driver default of 1024.\n"
		"\t\t Range for ISP4xxx - 64 to 1024\n"
		"\t\t Range for ISP8xxx - 64 to 1024.");

int ql4xextended_error_logging = 0;	/* 0 = off, 1 = log errors, 2 = debug logging */
module_param(ql4xextended_error_logging, int, S_IRUGO | S_IWUSR);
MODULE_PARM_DESC(ql4xextended_error_logging,
		 "Option to enable extended error logging,\n"
		 "\t\t 0 - no logging (Default)\n"
		 "\t\t 1 - log errors\n"
		 "\t\t 2 - debug logging");

int ql4xenablemsix = 1;
module_param(ql4xenablemsix, int, S_IRUGO|S_IWUSR);
MODULE_PARM_DESC(ql4xenablemsix,
		 "Set to enable MSI or MSI-X interrupt mechanism.\n"
		 "\t\t 0 = enable INTx interrupt mechanism.\n"
		 "\t\t 1 = enable MSI-X interrupt mechanism (Default).\n"
		 "\t\t 2 = enable MSI interrupt mechanism.");

int ql4xmdcapmask = 0x1F;
module_param(ql4xmdcapmask, int, S_IRUGO);
MODULE_PARM_DESC(ql4xmdcapmask,
		 "Set the Minidump driver capture mask level.\n"
		 "\t\t Default is 0x1F.\n"
		 "\t\t Set 0 for firmware default capture mask.\n"
		 "\t\t Can be set to 0x3, 0x7, 0xF, 0x1F, 0x3F, 0x7F.");

int ql4xenablemd = 1;
module_param(ql4xenablemd, int, S_IRUGO|S_IWUSR);
MODULE_PARM_DESC(ql4xenablemd,
		 "Set to enable minidump.\n"
		 "\t\t 0 - disable minidump\n"
		 "\t\t 1 - enable minidump (Default)");

char *ql4xdevconf = NULL;

MODULE_AUTHOR("QLogic Corporation");
MODULE_DESCRIPTION("QLogic ISP4XXX/ISP82XX iSCSI Host Bus Adapter driver");
MODULE_LICENSE("GPL");
MODULE_VERSION(QLA4XXX_DRIVER_VERSION);

/*
 * Proc info processing
 *---------------------------------------------------------------------------*/
struct info_str {
	char    *buffer;
	int     length;
	off_t   offset;
	int     pos;
};

/*
 * String messages for various state values (used for print statements)
 *---------------------------------------------------------------------------*/
//static const char *ddb_state_msg[] = DDB_STATE_TBL();
static const char *srb_state_msg[] = SRB_STATE_TBL();



uint8_t qla4xxx_mem_alloc(scsi_qla_host_t *ha);
void qla4xxx_mem_free(scsi_qla_host_t *ha);
void qla4xxx_timer(unsigned long p);
int qla4xxx_do_dpc(void *data);
void qla4xxx_display_config(void);
int qla4xxx_ioctl_probe_init(struct scsi_qla_host *ha);
void qla4xxx_add_timer_to_cmd(scsi_qla_host_t *ha, srb_t *srb, int timeout);
void qla4xxx_abort_active_cmds(scsi_qla_host_t *ha, int res);
uint8_t qla4xxx_reset_target(scsi_qla_host_t *ha, ddb_entry_t *ddb_entry,
	 int eh_active);
uint8_t qla4xxx_recover_adapter(scsi_qla_host_t *ha, uint8_t renew_ddb_list);
void qla4xxx_config_dma_addressing(scsi_qla_host_t *ha);

CONTINUE_ENTRY *qla4xxx_alloc_cont_entry(scsi_qla_host_t *ha);

void qla4xxx_free_other_mem(scsi_qla_host_t *ha);
int qla4xxx_iospace_config(scsi_qla_host_t *ha);
extern fc_lun_t * qla4xxx_add_fclun(fc_port_t *fcport, uint16_t lun);

extern int ql4im_init(void);
extern int ql4im_exit(void);

/*
 * PCI driver interface definitions
 *---------------------------------------------------------------------------*/
static struct pci_device_id qla4xxx_pci_tbl[] __devinitdata =
{
	{
		.vendor		= PCI_VENDOR_ID_QLOGIC,
		.device		= PCI_DEVICE_ID_QLOGIC_ISP4010,
		.subvendor	= PCI_ANY_ID,
		.subdevice	= PCI_ANY_ID,
	},
	{
		.vendor		= PCI_VENDOR_ID_QLOGIC,
		.device		= PCI_DEVICE_ID_QLOGIC_ISP4022,
		.subvendor	= PCI_ANY_ID,
		.subdevice	= PCI_ANY_ID,
	},
	{
		.vendor		= PCI_VENDOR_ID_QLOGIC,
		.device		= PCI_DEVICE_ID_QLOGIC_ISP4032,
		.subvendor	= PCI_ANY_ID,
		.subdevice	= PCI_ANY_ID,
	},
	{
		.vendor         = PCI_VENDOR_ID_QLOGIC,
		.device         = PCI_DEVICE_ID_QLOGIC_ISP8022,
		.subvendor      = PCI_ANY_ID,
		.subdevice      = PCI_ANY_ID,
        },
	{
		.vendor		= PCI_VENDOR_ID_QLOGIC,
		.device		= PCI_DEVICE_ID_QLOGIC_ISP8324,
		.subvendor	= PCI_ANY_ID,
		.subdevice	= PCI_ANY_ID,
	},
	{0, 0},
};
MODULE_DEVICE_TABLE(pci, qla4xxx_pci_tbl);

static int __devinit qla4xxx_probe_adapter(struct pci_dev *, const struct pci_device_id *);
void __devexit qla4xxx_remove_adapter(struct pci_dev *);
void qla4xxx_free_adapter(scsi_qla_host_t *ha);


int qla4xxx_queuecommand(struct scsi_cmnd *cmd,
			void (*done_fn)(struct scsi_cmnd *));
int qla4xxx_eh_abort(struct scsi_cmnd *cmd);
int qla4xxx_eh_bus_reset(struct scsi_cmnd *cmd);
int qla4xxx_eh_device_reset(struct scsi_cmnd *cmd);
int qla4xxx_eh_host_reset(struct scsi_cmnd *cmd);
int qla4xxx_slave_configure(struct scsi_device * device);
static void
qla4xxx_pci_resume(struct pci_dev *pdev);
static pci_ers_result_t
qla4xxx_pci_slot_reset(struct pci_dev *pdev);
static pci_ers_result_t
qla4xxx_pci_mmio_enabled(struct pci_dev *pdev);
static pci_ers_result_t
qla4xxx_pci_error_detected(struct pci_dev *pdev, pci_channel_state_t state);
static inline struct pci_dev *pci_get_domain_bus_and_slot(int domain,
		unsigned int bus, unsigned int devfn);

static struct pci_error_handlers qla4xxx_err_handler = {
	.error_detected = qla4xxx_pci_error_detected,
	.mmio_enabled   = qla4xxx_pci_mmio_enabled,
	.slot_reset     = qla4xxx_pci_slot_reset,
	.resume         = qla4xxx_pci_resume,
};

static struct pci_driver qla4xxx_pci_driver = {
	.name           = DRIVER_NAME,
	.id_table       = qla4xxx_pci_tbl,
	.probe          = qla4xxx_probe_adapter,
	.remove         = qla4xxx_remove_adapter,
	.err_handler    = &qla4xxx_err_handler,
};

static struct scsi_host_template qla4xxx_driver_template = {
	.module			= THIS_MODULE,
	.name			= "qla4xxx",
	.proc_name		= "qla4xxx",
	.queuecommand		= qla4xxx_queuecommand,

	.eh_abort_handler	= qla4xxx_eh_abort,
	.eh_device_reset_handler = qla4xxx_eh_device_reset,
	.eh_bus_reset_handler	= qla4xxx_eh_bus_reset,
	.eh_host_reset_handler	= qla4xxx_eh_host_reset,

	.slave_configure	= qla4xxx_slave_configure,

	.this_id		= -1,
	.cmd_per_lun		= 3,
	.use_clustering		= ENABLE_CLUSTERING,
	.sg_tablesize		= SG_ALL,

	/* 2^32-1 size limit */
	.max_sectors 		= 0xffff,
	.shost_attrs		= qla4xxx_host_attrs,
};

struct isp_operations qla4xxx_isp_ops = {
        .iospace_config         = qla4xxx_iospace_config,
        .pci_config             = qla4xxx_pci_config,
        .disable_intrs          = qla4xxx_disable_intrs,
        .enable_intrs           = qla4xxx_enable_intrs,
        .start_firmware         = qla4xxx_start_firmware,
        .intr_handler           = qla4xxx_intr_handler,
        .interrupt_service_routine= qla4xxx_interrupt_service_routine,
        .reset_chip             = qla4xxx_soft_reset,
        .reset_firmware         = qla4xxx_hw_reset,
        .queue_iocb             = qla4xxx_queue_iocb,
        .complete_iocb          = qla4xxx_complete_iocb,
        .rd_shdw_req_q_out      = qla4xxx_rd_shdw_req_q_out,
        .rd_shdw_rsp_q_in       = qla4xxx_rd_shdw_rsp_q_in,
        .get_sys_info           = qla4xxx_get_sys_info,
	.queue_mailbox_command	= qla4xxx_queue_mbox_cmd,
	.process_mailbox_interrupt = qla4xxx_process_mbox_intr,
};
struct isp_operations qla4_82xx_isp_ops = {
	.iospace_config		= qla4_8xxx_iospace_config,
	.pci_config		= qla4_8xxx_pci_config,
	.disable_intrs		= qla4_82xx_disable_intrs,
	.enable_intrs		= qla4_82xx_enable_intrs,
	.start_firmware		= qla4_8xxx_load_risc,
	.restart_firmware	= qla4_82xx_try_start_fw,
	.intr_handler		= qla4_82xx_intr_handler,
	.interrupt_service_routine = qla4_82xx_interrupt_service_routine,
	.need_reset		= qla4_8xxx_need_reset,
	.reset_chip		= qla4_8xxx_isp_reset,
	.reset_firmware		= qla4_8xxx_stop_firmware,
	.queue_iocb		= qla4_82xx_queue_iocb,
	.complete_iocb		= qla4_82xx_complete_iocb,
	.rd_shdw_req_q_out	= qla4_82xx_rd_shdw_req_q_out,
	.rd_shdw_rsp_q_in	= qla4_82xx_rd_shdw_rsp_q_in,
	.get_sys_info		= qla4_8xxx_get_sys_info,
	.rd_reg_direct		= qla4_82xx_rd_32,
	.wr_reg_direct		= qla4_82xx_wr_32,
	.rd_reg_indirect	= qla4_82xx_md_rd_32,
	.wr_reg_indirect	= qla4_82xx_md_wr_32,
	.idc_lock		= qla4_82xx_idc_lock,
	.idc_unlock		= qla4_82xx_idc_unlock,
	.rom_lock_recovery	= qla4_82xx_rom_lock_recovery,
	.queue_mailbox_command	= qla4_82xx_queue_mbox_cmd,
	.process_mailbox_interrupt = qla4_82xx_process_mbox_intr,
};
struct isp_operations qla4_83xx_isp_ops = {
	.iospace_config		= qla4_8xxx_iospace_config,
	.pci_config		= qla4_8xxx_pci_config,
	.disable_intrs		= qla4_83xx_disable_intrs,
	.enable_intrs		= qla4_83xx_enable_intrs,
	.start_firmware		= qla4_8xxx_load_risc,
	.restart_firmware	= qla4_83xx_start_firmware,
	.intr_handler		= qla4_83xx_intr_handler,
	.interrupt_service_routine = qla4_83xx_interrupt_service_routine,
	.need_reset		= qla4_8xxx_need_reset,
	.reset_chip		= qla4_83xx_isp_reset,
	.reset_firmware		= qla4_8xxx_stop_firmware,
	.queue_iocb		= qla4_83xx_queue_iocb,
	.complete_iocb		= qla4_83xx_complete_iocb,
	.rd_shdw_req_q_out	= qla4_83xx_rd_shdw_req_q_out,
	.rd_shdw_rsp_q_in	= qla4_83xx_rd_shdw_rsp_q_in,
	.get_sys_info		= qla4_8xxx_get_sys_info,
	.rd_reg_direct		= qla4_83xx_rd_reg,
	.wr_reg_direct		= qla4_83xx_wr_reg,
	.rd_reg_indirect	= qla4_83xx_rd_reg_indirect,
	.wr_reg_indirect	= qla4_83xx_wr_reg_indirect,
	.idc_lock	= qla4_83xx_drv_lock,
	.idc_unlock	= qla4_83xx_drv_unlock,
	.rom_lock_recovery	= qla4_83xx_rom_lock_recovery,
	.queue_mailbox_command	= qla4_83xx_queue_mbox_cmd,
	.process_mailbox_interrupt = qla4_83xx_process_mbox_intr,
};

uint16_t qla4xxx_rd_shdw_req_q_out(struct scsi_qla_host *ha)
{
	return (uint16_t) le32_to_cpu(ha->shadow_regs->req_q_out);
}

uint16_t qla4_82xx_rd_shdw_req_q_out(struct scsi_qla_host *ha)
{
	return (uint16_t) le32_to_cpu(readl(&ha->qla4_82xx_reg->req_q_out));
}

uint16_t qla4_83xx_rd_shdw_req_q_out(struct scsi_qla_host *ha)
{
	return (uint16_t) le32_to_cpu(readl(&ha->qla4_83xx_reg->req_q_out));
}

uint16_t qla4xxx_rd_shdw_rsp_q_in(struct scsi_qla_host *ha)
{
	return (uint16_t) le32_to_cpu(ha->shadow_regs->rsp_q_in);
}

uint16_t qla4_82xx_rd_shdw_rsp_q_in(struct scsi_qla_host *ha)
{
	return (uint16_t) le32_to_cpu(readl(&ha->qla4_82xx_reg->rsp_q_in));
}

uint16_t qla4_83xx_rd_shdw_rsp_q_in(struct scsi_qla_host *ha)
{
	return (uint16_t) le32_to_cpu(readl(&ha->qla4_83xx_reg->rsp_q_in));
}

/**************************************************************************
 * qla4xxx_set_info
 *      This routine set parameters for the driver from the /proc filesystem.
 *
 * Input:
 *      Unused
 *
 * Returns:
 *      -ENOSYS - no-op
 *
 * Context:
 *      Kernel context.
 **************************************************************************/
static int
qla4xxx_set_info(char *buffer, int length, struct Scsi_Host *host)
{
	return(-ENOSYS);  /* Currently this is a no-op */
}


/**************************************************************************
 * qla4xxx_module_init
 *    Module initialization.
 **************************************************************************/
static int __init
qla4xxx_module_init(void)
{
	int status;

	printk(KERN_INFO
	    "QLogic iSCSI HBA Driver (%p) v%s\n",
	       qla4xxx_set_info /* code start */,
	       QLA4XXX_DRIVER_VERSION);

#if ISP_RESET_TEST
	printk(KERN_INFO "qla4xxx: Adapter Reset Test Enabled!  "
	       "Adapter Resets will be issued every 3 minutes!\n");
#endif

	/* Derive version string. */
	strcpy(qla4xxx_version_str, QLA4XXX_DRIVER_VERSION);

	status = pci_module_init(&qla4xxx_pci_driver);

	ql4im_init();

	return status;
}

/**************************************************************************
 * qla4xxx_module_exit
 *    Module cleanup.
 **************************************************************************/
static void __exit
qla4xxx_module_exit(void)
{
	ql4im_exit();
	pci_unregister_driver(&qla4xxx_pci_driver);
}
module_init(qla4xxx_module_init);
module_exit(qla4xxx_module_exit);


/**************************************************************************
 * qla4xxx_probe_adapter
 *    This routine will probe for Qlogic 4010 iSCSI host adapters.
 *    It returns the number of host adapters of a particular
 *    type that were found.  It also initializes all data necessary for
 *    the driver.  It is passed-in the host number, so that it
 *    knows where its first entry is in the scsi_hosts[] array.
 *
 * Input:
 *
 * Returns:
 *
 * Context:
 *	Kernel context.
 **************************************************************************/
static int __devinit
qla4xxx_probe_adapter(struct pci_dev *pdev, const struct pci_device_id *ent)
{
	struct Scsi_Host *host;
	scsi_qla_host_t *ha;
	int ret = -ENODEV;
	uint8_t init_retry_count;
	char buf[34];

	if (pci_enable_device(pdev))
		return -1;

	host = scsi_host_alloc(&qla4xxx_driver_template,
	    sizeof(scsi_qla_host_t));
	if (host == NULL) {
		printk(KERN_WARNING
		    "qla4xxx: Couldn't allocate host from scsi layer!\n");
		goto probe_disable_device;
	}


	switch (ql4xextended_error_logging) {
	case 2:
		qla4xxx_set_debug_level(QLP1|QLP2|QLP7|QLP20|QLP19|QLP4|QLP25);
		break;
	case 1:
		qla4xxx_set_debug_level(QLP1|QLP2);
		break;
	}

	/* Clear our data area */
	ha = (scsi_qla_host_t *)host->hostdata;
	memset(ha, 0, sizeof(scsi_qla_host_t));

	/* Save the information from PCI BIOS.  */
	ha->pdev = pdev;
	ha->host = host;
	ha->host_no = host->host_no;
	ha->instance = qla4xxx_hba_count;
	ha->func_num = PCI_FUNC(pdev->devfn);

	if (IS_QLA80XX(ha)) {
		if (ql4xmaxcmds) {
			if ((ql4xmaxcmds >= MIN_IOCBS) &&
			    (ql4xmaxcmds <= MAX_IOCBS))
				ha->maxcmds = ql4xmaxcmds;
		} else
			ha->maxcmds = REQUEST_QUEUE_DEPTH;
		ha->response_qdepth = RESPONSE_QUEUE_DEPTH_ISP8XXX;

	} else {
		if (ql4xmaxcmds) {
			if ((ql4xmaxcmds >= MIN_IOCBS) &&
			    (ql4xmaxcmds <= MAX_IOCBS))
				ha->maxcmds = ql4xmaxcmds;
		} else
			ha->maxcmds = REQUEST_QUEUE_DEPTH;
		ha->response_qdepth = RESPONSE_QUEUE_DEPTH;
	}

	pci_enable_pcie_error_reporting(pdev);

	/* Setup Runtime configurable options */
	if (IS_QLA8032(ha)) {
		ha->isp_ops = &qla4_83xx_isp_ops;
		ha->reg_tbl = (uint32_t *) qla4_83xx_reg_tbl;
		rwlock_init(&ha->hw_lock);
		ha->pf_bit = ha->func_num << 16;
	} else if (IS_QLA8022(ha)) {
		ha->isp_ops = &qla4_82xx_isp_ops;
		ha->reg_tbl = (uint32_t *) qla4_82xx_reg_tbl;
		/* ISP 8022 initializations */
		qla4_8xxx_init_local_data(ha);
	} else
		ha->isp_ops = &qla4xxx_isp_ops;

	/* Configure PCI I/O space. */
	ret = ha->isp_ops->iospace_config(ha);
	if (ret)
		goto probe_failed_ioconfig;

        ql4_printk(KERN_INFO, ha,
	    "Found an %s%04x, irq %d, iobase 0x%p\n",
		IS_QLA80XX(ha) ? "" : "ISP", pdev->device,
		host->irq, ha->reg);

	/* save PCI configuration space for EEH */
	pci_save_state(ha->pdev);

	/* Configure OS DMA addressing method. */
	qla4xxx_config_dma_addressing(ha);

	/* Initialize lists and spinlocks. */
	INIT_LIST_HEAD(&ha->ddb_list);
	INIT_LIST_HEAD(&ha->free_srb_q);
	INIT_LIST_HEAD(&ha->fcports);
	INIT_LIST_HEAD(&ha->async_iocb_list);
	INIT_LIST_HEAD(&ha->done_srb_q);
	INIT_LIST_HEAD(&ha->retry_srb_q);
	INIT_LIST_HEAD(&ha->isns.async_prb_list);

	init_MUTEX(&ha->mbox_sem);
	init_MUTEX(&ha->pt_sem);
	init_waitqueue_head(&ha->mailbox_wait_queue);

	spin_lock_init(&ha->hardware_lock);
	spin_lock_init(&ha->list_lock);

	ha->dpc_pid = -1;
	init_completion(&ha->dpc_inited);
	init_completion(&ha->dpc_exited);
	init_completion(&ha->mbx_intr_comp);
	init_completion(&ha->idc_comp);
	init_completion(&ha->link_up_comp);

	/*
	 * Allocate memory for dma buffers
	 */
	if (qla4xxx_mem_alloc(ha) == QLA_ERROR) {
		ql4_printk(KERN_WARNING, ha,
		    "[ERROR] Failed to allocate memory for adapter\n");

		ret = -ENOMEM;
		goto probe_failed;
	}

	if (IS_QLA8022(ha))
		(void) qla4_82xx_get_flash_info(ha);
	else if (IS_QLA8032(ha)) {
		qla4_83xx_get_flash_info(ha);

		/* NOTE: If ql4dontresethba==1, set IDC_CTRL DONTRESET_BIT0.
		 * If DONRESET_BIT0 is set, drivers should not set dev_state
		 * to NEED_RESET. But if NEED_RESET is set, drivers should
		 * should honor the reset. */
		if (ql4xdontresethba == 1)
			qla4_83xx_set_idc_dontreset(ha);
	}

	DEBUG2(printk("scsi: %s: Starting kernel thread for "
			"qla4xxx_ptc\n", __func__));
	sprintf(buf, "qla4xxx_%u_pt", ha->host_no);
	ha->pt_thread = create_singlethread_workqueue(buf);
	if (!ha->pt_thread) {
		dev_warn(&ha->pdev->dev, "Unable to start pt thread!\n");
		ret = -ENODEV;
		goto probe_failed;
	}

	INIT_WORK(&ha->pt_work, (void *)ql4_isns_dequeue_passthru_sts_iocb, ha);

	/*
	 * Initialize the Host adapter request/response queues and firmware
	 * NOTE: AF_ONLINE flag set and interrupts enabled upon successful
	 *	 completion of qla4xxx_initialize_adapter
	 */
	(void) qla4xxx_initialize_adapter(ha, REBUILD_DDB_LIST);

	/* Dont retry adapter initialization if IRQ allocation failed */
	if (!test_bit(AF_IRQ_ATTACHED, &ha->flags))
		goto skip_retry_init;

	for (init_retry_count = 1;
	     !test_bit(AF_ONLINE, &ha->flags) &&
	     init_retry_count <= MAX_INIT_RETRIES;
	     init_retry_count++) {
		DEBUG2(printk("scsi: %s: retrying adapter "
		    "initialization (%d)\n", __func__, init_retry_count));

		if (ha->isp_ops->reset_chip(ha) == QLA_ERROR)
			continue;

		(void) qla4xxx_initialize_adapter(ha, REBUILD_DDB_LIST);
	}

skip_retry_init:
	if (!test_bit(AF_ONLINE, &ha->flags)) {
		ql4_printk(KERN_WARNING, ha, "Failed to initialize adapter\n");

		DEBUG2(printk("scsi%d: Failed to initialize adapter\n",
				ha->host_no));

		if ((IS_QLA8022(ha) && ql4xdontresethba) ||
		    (IS_QLA8032(ha) && qla4_83xx_idc_dontreset(ha))) {
			ha->isp_ops->idc_lock(ha);
			ql4_printk(KERN_INFO, ha, "Setting HW State to FAILED\n");
			qla4_8xxx_wr_direct(ha, QLA8XXX_CRB_DEV_STATE,
					QLA8XXX_DEV_FAILED);
			ha->isp_ops->idc_unlock(ha);
			ql4_printk(KERN_INFO, ha, "%s: HW State: FAILED\n",
				   __func__);
		}

		ret = -ENODEV;
		goto probe_failed;
	}

	host->cmd_per_lun = 3;
	host->io_port = ha->io_addr;
	host->max_channel =  0;
	host->max_lun = MAX_LUNS-1;
	host->max_id = MAX_TARGETS;
	host->unique_id = ha->instance;
	host->max_cmd_len = IOCB_MAX_CDB_LEN;
	host->can_queue = (ha->maxcmds + 128);
	QL4PRINT(QLP7, printk("scsi%d: %s max cmds=%d\n", ha->host_no,
			__func__, ha->maxcmds));
	QL4PRINT(QLP7, printk("scsi%d: %s CAN Q =%d\n", ha->host_no,
			__func__, host->can_queue));

	/* Startup the kernel thread for this host adapter. */
	QL4PRINT(QLP7, printk("scsi%d: %s: Starting kernel thread for "
	    "qla4xxx_dpc\n", ha->host_no, __func__));
	ha->dpc_should_die = 0;
	ha->dpc_pid = kernel_thread(qla4xxx_do_dpc, ha, 0);
	init_retry_count = 0;
	while ((ha->dpc_pid < 0) &&
	       (init_retry_count++ < MAX_INIT_RETRIES)) {
		ha->dpc_pid = kernel_thread(qla4xxx_do_dpc, ha, 0);
		if (ha->dpc_pid < 0) {
			printk(KERN_WARNING
				" host_no =%d Unable to start DPC thread!"
				"init_retry_count =%d\n", ha->host_no,
				init_retry_count);
			set_current_state(TASK_UNINTERRUPTIBLE);
			schedule_timeout(1 * HZ);
		} else {
		 	break;
		}
	}
	if (ha->dpc_pid < 0) {
		ql4_printk(KERN_WARNING, ha, "Unable to start DPC thread!\n");

		goto probe_failed;
	}
	wait_for_completion(&ha->dpc_inited);

#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,12))
	#if ENABLE_MSI
	if (!IS_QLA4010(ha)) {
		if (pci_enable_msi(pdev)) {
			ql4_printk(KERN_WARNING, ha,
			    "Failed to Enable MSI!!!.\n");
			goto probe_failed;
		}
	}
	ql4_printk(KERN_INFO, ha, "MSI Enabled...\n");
	set_bit(AF_MSI_ENABLED, &ha->flags);
	#endif
#endif

	/* For ISP-8XXX, request_irqs is called in qla4_8xxx_load_risc
	 * (which is called indirectly by qla4xxx_initialize_adapter),
	 * so that irqs will be registered after crbinit but before
	 * mbx_intr_enable. */
	if (IS_QLA40XX(ha)) {
		/* Install the interrupt handler with the new ha */
		ret = qla4xxx_request_irqs(ha);
		if (ret)
			goto probe_failed;
	}

	ha->isp_ops->enable_intrs(ha);

	/* Start timer thread. */
	QL4PRINT(QLP7, printk("scsi: %s: Starting timer thread for adapter "
	    "%d\n", __func__, ha->instance));
	init_timer(&ha->timer);
	ha->timer.expires = jiffies + HZ;
	ha->timer.data = (unsigned long)ha;
	ha->timer.function = (void (*)(unsigned long))qla4xxx_timer;
	add_timer(&ha->timer);
	ha->timer_active = 1;

	strcpy(ha->driver_verstr, QLA4XXX_DRIVER_VERSION);
	ha->driver_version[0] = QL4_DRIVER_MAJOR_VER;
	ha->driver_version[1] = QL4_DRIVER_MINOR_VER;
	ha->driver_version[2] = QL4_DRIVER_PATCH_VER;
	ha->driver_version[3] = QL4_DRIVER_BETA_VER;

	/* Insert new entry into the list of adapters. */
	write_lock(&qla4xxx_hostlist_lock);
	list_add_tail(&ha->list, &qla4xxx_hostlist);
	write_unlock(&qla4xxx_hostlist_lock);

	DEBUG(printk("qla4xxx: lock=%p listhead=%p, done adding ha list=%p.\n",
	    &qla4xxx_hostlist_lock, &qla4xxx_hostlist, &ha->list);)

	qla4xxx_display_config();

	/* Set the driver version */
	if (IS_QLA80XX(ha))
		qla4_8xxx_set_param(ha, SET_DRVR_VERSION);

	set_bit(AF_INIT_DONE, &ha->flags);
	qla4xxx_hba_count++;

	pci_set_drvdata(pdev, ha);

	ret = scsi_add_host(host, &pdev->dev);
	if (ret)
		goto probe_failed;

	qla4_8xxx_alloc_sysfs_attr(ha);

	printk(KERN_INFO
	    " QLogic iSCSI HBA Driver version: %s\n"
	    "  QLogic %s%04x @ %s hdma%c, host#=%d, fw=%02d.%02d.%02d.%02d\n",
	    QLA4XXX_DRIVER_VERSION, IS_QLA8022(ha) ? "" : "ISP",
	    ha->pdev->device, pci_name(ha->pdev),
	    test_bit(AF_64BIT_PCI_ADDR, &ha->flags) ? '+': '-', ha->host_no,
	    ha->firmware_version[0], ha->firmware_version[1],
	    ha->patch_number, ha->build_number);

	scsi_scan_host(host);

	if (qla4xxx_ioctl_probe_init(ha)) {
		dev_warn(&ha->pdev->dev, "ioctl init failed\n");
		goto probe_failed;
	}

	return 0;

probe_failed:
	qla4xxx_free_adapter(ha);
probe_failed_ioconfig:
	pci_disable_pcie_error_reporting(pdev);
	scsi_host_put(ha->host);

probe_disable_device:
	pci_disable_device(pdev);

	return ret;
}

/**
 * qla4xxx_prevent_other_port_reinit - Mark the other ISP-4xxx port to indicate
 * that the driver is being removed, so that the other port will not
 * re-initialize while in the process of removing the ha due to driver unload
 * or hba hotplug.
 * @ha: pointer to adapter structure
 **/
static void qla4xxx_prevent_other_port_reinit(struct scsi_qla_host *ha)
{
	scsi_qla_host_t *ha_listp;

	read_lock(&qla4xxx_hostlist_lock);
	list_for_each_entry(ha_listp, &qla4xxx_hostlist, list) {
		if (ha == ha_listp)
			continue;

		if ((pci_domain_nr(ha->pdev->bus) ==
		     pci_domain_nr(ha_listp->pdev->bus)) &&
		    (ha->pdev->bus->number ==
		     ha_listp->pdev->bus->number) &&
		    (PCI_SLOT(ha->pdev->devfn) ==
		     PCI_SLOT(ha_listp->pdev->devfn))) {
			set_bit(AF_HA_REMOVAL, &ha_listp->flags);
			DEBUG2(ql4_printk(KERN_INFO, ha, "%s: Prevent %s reinit\n",
				__func__, pci_name((ha_listp)->pdev)));
		}
	}
	read_unlock(&qla4xxx_hostlist_lock);
}

/**************************************************************************
 * qla4xxx_remove_adapter
 *
 * Input:
 *	pci_dev - PCI device pointer
 *
 * Returns:
 *
 * Context:
 *	Kernel context.
 **************************************************************************/
void __devexit qla4xxx_remove_adapter(struct pci_dev *pdev)
{
	scsi_qla_host_t *ha;

	/*
	 * If PCI device is disabled then it means probe_adapter had failed
	 * and resources already cleaned up on probe_adapter exit
	 */
	if (!pdev->is_enabled)
		return;

	ha = pci_get_drvdata(pdev);

	if (IS_QLA40XX(ha))
		qla4xxx_prevent_other_port_reinit(ha);

	write_lock(&qla4xxx_hostlist_lock);
	list_del_init(&ha->list);
	write_unlock(&qla4xxx_hostlist_lock);

	qla4_8xxx_free_sysfs_attr(ha);

	scsi_remove_host(ha->host);

	qla4xxx_free_adapter(ha);

	scsi_host_put(ha->host);

	pci_disable_pcie_error_reporting(pdev);
	pci_disable_device(ha->pdev);
	pci_set_drvdata(pdev, NULL);
}

void qla4xxx_free_adapter(scsi_qla_host_t *ha)
{
	int ret;

	/* NOTE: On 4xxx dual port adapters, if one port unloads and resets the
	 *       chip, the other port will no longer be able to communicate
	 *       with the chip.  Thus, iSNS will not deregister or get
	 *       disabled on the second port.  The iSNS server will deregister
	 *       the second port (no ESI notification) in approximately 15
	 *       minutes.  But, if driver re-loads within that time, the driver
	 *       will first de-register with the iSNS server, then register
	 *       with it to prevent multiple registration errors.
	 */
	if (test_bit(AF_ONLINE, &ha->flags) &&
	    test_bit(ISNS_FLAG_ISNS_ENABLED_IN_ISP, &ha->isns.flags)) {
		QL4PRINT(QLP7, printk("scsi%d: %s: Stop iSNS service\n",
		    ha->host_no, __func__));
		ql4_isns_stop_service(ha);
	}

	if (test_bit(AF_INTERRUPTS_ON, &ha->flags)) {
		/* Turn-off interrupts on the card. */
		ha->isp_ops->disable_intrs(ha);
	}

	/* clear the interrupt */
	if (IS_QLA8032(ha)) {
		writel(0, &ha->qla4_83xx_reg->risc_intr);
		readl(&ha->qla4_83xx_reg->risc_intr);
	} else if (IS_QLA8022(ha)) {
		writel(0, &ha->qla4_82xx_reg->host_int);
		readl(&ha->qla4_82xx_reg->host_int);
	} else if (IS_QLA40XX(ha)) {
		WRT_REG_DWORD(&ha->reg->ctrl_status,
				SET_RMASK(CSR_SCSI_PROCESSOR_INTR));
		PCI_POSTING(&ha->reg->ctrl_status);
	}

	/* Remove timer thread, if present */
	if (ha->timer_active) {
		QL4PRINT(QLP7, printk("scsi%d: %s: Removing timer thread for "
		    "adapter %d\n", ha->host_no, __func__, ha->instance));

		del_timer_sync(&ha->timer);
		ha->timer_active = 0;
	}

	/* Kill the kernel thread for this host */
	if (ha->dpc_pid >= 0) {
		ha->dpc_should_die = 1;
		wmb();
		ret = kill_proc(ha->dpc_pid, SIGHUP, 1);
		if (ret) {
			ql4_printk(KERN_ERR, ha,
			    "Unable to signal DPC thread -- (%d)\n", ret);
		} else
			wait_for_completion(&ha->dpc_exited);
	}

	if (ha->pt_thread)
		destroy_workqueue(ha->pt_thread);

	/* Put firmware in a known state */
	ha->isp_ops->reset_firmware(ha);

	/* Detach interrupts */
	if (test_and_clear_bit(AF_IRQ_ATTACHED, &ha->flags))
		qla4xxx_free_irqs(ha);

	if (IS_QLA80XX(ha)) {
		ha->isp_ops->idc_lock(ha);
		qla4_8xxx_clear_drv_active(ha);
		ha->isp_ops->idc_unlock(ha);
	}

	/* free extra memory */
	qla4xxx_mem_free(ha);

#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,10))
        #if ENABLE_MSI
	if (test_and_clear_bit(AF_MSI_ENABLED, &ha->flags))
		pci_disable_msi(ha->pdev);
	#endif
#endif

	/* Free I/O Region */
	if (ha->io_addr) {
		release_region(ha->io_addr, ha->io_len);
		ha->io_addr = 0;
	}

	if (IS_QLA80XX(ha))
		pci_release_regions(ha->pdev);

}

int qla4_8xxx_iospace_config(struct scsi_qla_host *ha)
{
	int status = 0;
	uint8_t revision_id;
	unsigned long mem_base, mem_len, db_base, db_len;
	struct pci_dev *pdev = ha->pdev;

	if ((status = pci_request_regions(pdev, DRIVER_NAME))) {
		printk(KERN_WARNING
				"scsi(%d) Failed to reserve PIO regions (%s) status=%d\n",
				ha->host_no, pci_name(pdev), status);
		goto iospace_error_exit;
	}

	pci_read_config_byte(pdev, PCI_REVISION_ID, &revision_id);
	DEBUG2(printk(KERN_INFO "%s: revision-id=%d\n",
				__func__, revision_id));
	ha->revision_id = revision_id;

	/* remap phys address */
	mem_base = pci_resource_start(pdev, 0); /* 0 is for BAR 0 */
	mem_len = pci_resource_len(pdev, 0);
	DEBUG2(printk(KERN_INFO "%s: ioremap from %lx a size of %lx\n",
				__func__, mem_base, mem_len));

	/* mapping of pcibase pointer */
	ha->nx_pcibase = (unsigned long)ioremap(mem_base, mem_len);
	if (!ha->nx_pcibase) {
		printk(KERN_ERR
				"cannot remap MMIO (%s), aborting\n", pci_name(pdev));
		pci_release_regions(ha->pdev);
		goto iospace_error_exit;
	}

	/* Mapping of IO base pointer, door bell read and write pointer */

	/* mapping of IO base pointer */
	if (IS_QLA8022(ha)) {
		ha->qla4_82xx_reg = (struct device_reg_82xx  __iomem *)
				    ((uint8_t *)ha->nx_pcibase +
				     0xbc000 + (ha->pdev->devfn << 11));

		/* faster CAMRAM doorbell mechanism */
		ha->nx_db_wr_ptr = (ha->pdev->devfn == 4 ? QLA82XX_CAM_RAM_DB1 :
					QLA82XX_CAM_RAM_DB2);
	} else if (IS_QLA8032(ha)) {
		ha->qla4_83xx_reg = (struct device_reg_83xx __iomem *)
				    ((uint8_t *)ha->nx_pcibase);
	}

	db_base = pci_resource_start(pdev, 4);  /* doorbell is on bar 4 */
	db_len = pci_resource_len(pdev, 4);

	return 0;

iospace_error_exit:
	return (-ENOMEM);
}

/**************************************************************************
 * qla4xxx_iospace_config
 *    This routine
 *
 * Input:
 *
 * Returns:
 *
 * Context:
 *	Kernel context.
 **************************************************************************/
int qla4xxx_iospace_config(scsi_qla_host_t *ha)
{
	int bar;

	/* Search for I/O register. */
	for (bar = 0; bar <= 5; bar++) {
		unsigned long pci_base_address;

		pci_base_address = pci_resource_start(ha->pdev, bar);
		ha->pci_resource_flags = pci_resource_flags(ha->pdev, bar);

#if MEMORY_MAPPED_IO
		if (ha->pci_resource_flags & IORESOURCE_MEM) {
			QL4PRINT(QLP7, printk("scsi%d: Assigned to Memory I/O "
			    "0x%lx in PCI BAR%d\n", ha->host_no,
			    pci_base_address, bar));

			ha->mem_addr = pci_base_address;
			ha->mem_len = pci_resource_len(ha->pdev, bar);
			break;
		}
#else
		if (ha->pci_resource_flags IORESOURCE_IO) {
			QL4PRINT(QLP7, printk("scsi%d: Assigned to I/O Port "
			    "0x%lx in PCI BAR%d\n", ha->host_no,
			    pci_base_address, bar));

			ha->io_addr = pci_base_address;
			ha->io_len = pci_resource_len(ha->pdev, bar);
			break;
		}
#endif
	}

	/* Map the Memory I/O register. */
	if (ha->mem_addr) {
		unsigned long  page_offset, base;

		if (!request_mem_region(ha->mem_addr, ha->mem_len,
		    DRIVER_NAME)) {
			printk(KERN_WARNING
			    "Could not allocate IO Memory space %lx len %ld.\n",
			    ha->mem_addr, ha->mem_len);
			return -1;
		}

		QL4PRINT(QLP7, printk("scsi%d: %s: base memory address = "
		    "0x%lx\n", ha->host_no, __func__, ha->mem_addr));

		/* Find proper memory chunk for memory map I/O reg. */
		base = ha->mem_addr & PAGE_MASK;
		page_offset = ha->mem_addr - base;

		/* Get virtual address for I/O registers. */
		ha->virt_mmapbase = ioremap(base, page_offset +
		    sizeof(*ha->reg));
		if (ha->virt_mmapbase == NULL) {
			QL4PRINT(QLP2, printk("scsi%d: %s: I/O Remap Failed\n",
			    ha->host_no, __func__));
			return -1;
		}

		QL4PRINT(QLP7, printk("scsi%d: %s: virt memory_mapped_address "
		    "= 0x%p\n", ha->host_no, __func__, ha->virt_mmapbase));

		ha->reg = (isp_reg_t __iomem *)(ha->virt_mmapbase + page_offset);
		QL4PRINT(QLP7, printk("scsi%d: %s: registers = 0x%p\n",
		    ha->host_no, __func__, ha->reg));
	}

	if (ha->io_addr) {
		if (!request_region(ha->io_addr, ha->io_len, DRIVER_NAME)) {
			printk(KERN_WARNING
			    "Could not allocate IO space %lx len %ld.\n",
			    ha->io_addr, ha->io_len);

			return -1;
		}
	}

	return QLA_SUCCESS;
}

/**************************************************************************
 * qla4xxx_display_config
 *      This routine  displays the configuration information to be used in
 *      modules.conf.
 *
 * Input:
 *      ha - Pointer to host adapter structure
 *
 * Output:
 *      None
 *
 * Returns:
 *      None
 *
 * Context:
 *      Kernel context.
 **************************************************************************/
void
qla4xxx_display_config(void)
{
	scsi_qla_host_t *ha, *htemp;

	read_lock(&qla4xxx_hostlist_lock);
	list_for_each_entry_safe(ha, htemp, &qla4xxx_hostlist, list) {
		/* Display the M.A.C. Address for adapter */
		printk(KERN_INFO
		    "scsi-qla%d-mac=%02x%02x%02x%02x%02x%02x\\;\n",
		    ha->instance,
		    ha->my_mac[0], ha->my_mac[1], ha->my_mac[2],
		    ha->my_mac[3], ha->my_mac[4], ha->my_mac[5]);
	}
	read_unlock(&qla4xxx_hostlist_lock);

}

int qla4xxx_ioctl_probe_init(struct scsi_qla_host *ha)
{
	ha->ql4_isns_send_dev_get_next = ql4_isns_send_dev_get_next;
	ha->ql4_isns_send_dev_attr_qry = ql4_isns_send_dev_attr_qry;
	ha->ql4_is_isns_active = ql4_is_isns_active;
	ha->ql4_isns_start_svc = ql4_isns_start_service;
	ha->ql4_isns_stop_svc = ql4_isns_stop_service;
	ha->ql4_isns_populate_server_ip = ql4_isns_populate_server_ip;
	ha->ql4_mbx = qla4xxx_mailbox_command;

	return 0;
}

/****************************************************************************/
/*  LINUX -  Loadable Module Functions.                                     */
/****************************************************************************/

/**
 * qla4xxx_config_dma_addressing() - Configure OS DMA addressing method.
 * @ha: HA context
 *
 * At exit, the @ha's flags.enable_64bit_addressing set to indicated
 * supported addressing method.
 */
void
qla4xxx_config_dma_addressing(scsi_qla_host_t *ha)
{
	int retval;

	/* Assume 32bit DMA address. */
	clear_bit(AF_64BIT_PCI_ADDR, &ha->flags);

	/*
	 * Given the two variants pci_set_dma_mask(), allow the compiler to
	 * assist in setting the proper dma mask.
	 */
	if (sizeof(dma_addr_t) > 4) {
		/* Update our PCI device dma_mask for full 64 bit mask */
		if (pci_set_dma_mask(ha->pdev, DMA_64BIT_MASK) == 0) {
			set_bit(AF_64BIT_PCI_ADDR, &ha->flags);

			if (pci_set_consistent_dma_mask(ha->pdev,
			    DMA_64BIT_MASK)) {
				ql4_printk(KERN_DEBUG, ha,
				    "Failed to set 64 bit PCI consistent mask; "
				    "using 32 bit.\n");

				retval = pci_set_consistent_dma_mask(ha->pdev,
				    DMA_32BIT_MASK);
			}
		} else {
			ql4_printk(KERN_DEBUG, ha,
			    "Failed to set 64 bit PCI DMA mask, falling back "
			    "to 32 bit MASK.\n");

			retval = pci_set_dma_mask(ha->pdev, DMA_32BIT_MASK);
		}
	} else {
		retval = pci_set_dma_mask(ha->pdev, DMA_32BIT_MASK);
	}
}

/**************************************************************************
 * qla4xxx_alloc_srb_pool
 *      This routine is called during driver initialization to allocate
 *      memory for the local srb pool.
 *
 * Input:
 *      ha - Pointer to host adapter structure
 *
 * Returns:
 *      QLA_SUCCESS - Successfully allocated srbs
 *      QLA_ERROR   - Failed to allocate any srbs
 *
 * Context:
 *      Kernel context.
 **************************************************************************/
uint8_t qla4xxx_alloc_srb_pool(scsi_qla_host_t *ha)
{
	srb_t *srb;
	int i;
	uint8_t status = QLA_ERROR;

	ha->num_srbs_allocated = 0;
	ha->free_srb_q_count = 0; /* incremented in add_to_free_srb_q routine */

	/*
	 * NOTE: Need to allocate each SRB separately, as Kernel 2.4.4 seems to
	 * have an error when allocating a large amount of memory.
	 */
	for (i=0; i < MAX_SRBS; i++) {
		srb = (srb_t *) kmalloc(sizeof(srb_t), GFP_KERNEL);
		if (srb == NULL) {
			QL4PRINT(QLP2, printk(
			    "scsi%d: %s: failed to allocate memory, count = "
			    "%d\n", ha->host_no, __func__, i));
		} else {
			ha->num_srbs_allocated++;
			__add_to_free_srb_q(ha, srb);
		}
	}

	if (ha->free_srb_q_count)
		status = QLA_SUCCESS;

	DEBUG2(printk("scsi%d: %s: Allocated %d SRB(s)\n",
	    ha->host_no, __func__, ha->free_srb_q_count));

	return (status);
}

/**************************************************************************
 * qla4xxx_free_srb_pool
 *      This routine is called during driver unload to deallocate the srb
 *      pool.
 *
 * Input:
 *      ha - Pointer to host adapter structure
 *
 * Returns:
 *      None
 *
 * Context:
 *      Kernel context.
 **************************************************************************/
void qla4xxx_free_srb_pool(scsi_qla_host_t *ha)
{
	srb_t *srb, *stemp;
	int cnt_free_srbs = 0;
	unsigned long flags;

	spin_lock_irqsave(&ha->list_lock, flags);
	list_for_each_entry_safe(srb, stemp, &ha->free_srb_q, list_entry) {
		__del_from_free_srb_q(ha, srb);
		kfree(srb);
		cnt_free_srbs++;
	}
	spin_unlock_irqrestore(&ha->list_lock, flags);

	if (cnt_free_srbs != ha->num_srbs_allocated) {
		QL4PRINT(QLP2, printk(KERN_WARNING
		    "scsi%d: Did not free all srbs, Free'd srb count = %d, "
		    "Alloc'd srb count %d\n", ha->host_no, cnt_free_srbs,
		    ha->num_srbs_allocated));
	}
}

/**************************************************************************
 * qla4xxx_mem_alloc
 *      This routine allocates memory use by the adapter.
 *
 * Input:
 *      ha - Pointer to host adapter structure
 *
 * Returns:
 *      QLA_SUCCESS - Successfully allocated adapter memory
 *      QLA_ERROR   - Failed to allocate adapter memory
 *
 * Context:
 *      Kernel context.
 **************************************************************************/
uint8_t
qla4xxx_mem_alloc(scsi_qla_host_t *ha)
{
	unsigned long	align;
	__u8 name[24];

	/* Allocate contiguous block of DMA memory for queues. */
	ha->queues_len = ((ha->maxcmds * QUEUE_SIZE) +
	    (ha->response_qdepth * QUEUE_SIZE) + sizeof(shadow_regs_t) +
	    MEM_ALIGN_VALUE + (PAGE_SIZE - 1)) & ~(PAGE_SIZE - 1);
	ha->queues = pci_alloc_consistent(ha->pdev, ha->queues_len,
	    &ha->queues_dma);
	if (ha->queues == NULL) {
		ql4_printk(KERN_WARNING, ha,
		    "Memory Allocation failed - queues.\n");

		goto mem_alloc_error_exit;
	}
	memset(ha->queues, 0, ha->queues_len);

	/*
	 * As per RISC alignment requirements -- the bus-address must be a
	 * multiple of the request-ring size (in bytes).
	 */
	align = 0;
	if (IS_QLA40XX(ha)) {
		if ((unsigned long)ha->queues_dma & (MEM_ALIGN_VALUE - 1)) {
			align = MEM_ALIGN_VALUE -
			    ((unsigned long)ha->queues_dma & (MEM_ALIGN_VALUE - 1));
		}
	}

	/* Update request and response queue pointers. */
	ha->request_dma = ha->queues_dma + align;
	ha->request_ring = (QUEUE_ENTRY *)(ha->queues + align);
	ha->response_dma = ha->queues_dma + align +
	    (ha->maxcmds * QUEUE_SIZE);
	ha->response_ring = (QUEUE_ENTRY *)(ha->queues + align +
	    (ha->maxcmds * QUEUE_SIZE));
	ha->shadow_regs_dma = ha->queues_dma + align +
	    (ha->maxcmds * QUEUE_SIZE) +
	    (ha->response_qdepth * QUEUE_SIZE);
	ha->shadow_regs = (shadow_regs_t *)(ha->queues + align +
	    (ha->maxcmds * QUEUE_SIZE) +
	    (ha->response_qdepth * QUEUE_SIZE));

	QL4PRINT(QLP7, printk("scsi%d: %s: queues 0x%lx (%p) %lx\n",
	    ha->host_no, __func__, (unsigned long)ha->queues_dma,
	    ha->queues, ha->queues_len));
	QL4PRINT(QLP7, printk("scsi%d: %s: request ring 0x%lx (%p)\n",
	    ha->host_no, __func__, (unsigned long)ha->request_dma,
	    ha->request_ring));
	QL4PRINT(QLP7, printk("scsi%d: %s: response ring 0x%lx (%p)\n",
	    ha->host_no, __func__, (unsigned long)ha->response_dma,
	    ha->response_ring));
	QL4PRINT(QLP7, printk("scsi%d: %s: shadow regs 0x%lx (%p)\n",
	    ha->host_no, __func__, (unsigned long)ha->shadow_regs_dma,
	    ha->shadow_regs));

        ha->gen_req_rsp_iocb = pci_alloc_consistent(ha->pdev, PAGE_SIZE,
            &ha->gen_req_rsp_iocb_dma);
        if (ha->gen_req_rsp_iocb == NULL) {
                ql4_printk(KERN_WARNING, ha,
                    "Memory Allocation failed - IOCB Mem.\n");
                goto mem_alloc_error_exit;
        }
        memset(ha->gen_req_rsp_iocb, 0, PAGE_SIZE);

	/* Create DMA pool for Passthru IOCBs */
	snprintf(name, sizeof(name), "%s_iocb_%d", DRIVER_NAME, ha->pdev->device);
	ha->pt_iocb_dmapool = dma_pool_create(name, &ha->pdev->dev, PAGE_SIZE,
				sizeof(PASSTHRU0_ENTRY), 0);

	if (ha->pt_iocb_dmapool == NULL) {
		ql4_printk(KERN_WARNING, ha,
			"Memory Allocation failed - Passthru IOCB dmapool.\n");
		goto mem_alloc_error_exit;
	}

	/*
	 * Allocate memory for srb pool
	 *-----------------------------*/
	if (qla4xxx_alloc_srb_pool(ha) == QLA_ERROR)
		goto mem_alloc_error_exit;

	return (QLA_SUCCESS);

mem_alloc_error_exit:
	qla4xxx_mem_free(ha);
	return (QLA_ERROR);
}

/**************************************************************************
 * qla4xxx_mem_free
 *      This routine frees adapter allocated memory
 *
 * Input:
 *      ha - Pointer to host adapter structure.
 *
 * Returns:
 *      None
 *
 * Context:
 *      Kernel context.
 **************************************************************************/
void qla4xxx_mem_free(scsi_qla_host_t *ha)
{
	struct list_head *list, *temp;

	if (ha->queues) {
		QL4PRINT(QLP7, printk("scsi%d: %s: free queues.\n", ha->host_no,
		    __func__));

		pci_free_consistent(ha->pdev, ha->queues_len, ha->queues,
		    ha->queues_dma);
	}

	if(ha->fw_dump)
		vfree(ha->fw_dump);
	ha->queues_len = 0;
	ha->queues = NULL;
	ha->queues_dma = 0;
	ha->request_ring = NULL;
	ha->request_dma = 0;
	ha->response_ring = NULL;
	ha->response_dma = 0;
	ha->shadow_regs = NULL;
	ha->shadow_regs_dma = 0;
	ha->fw_dump = NULL;
	ha->fw_dump_size = 0;

        if (ha->gen_req_rsp_iocb) {
                QL4PRINT(QLP7, printk("scsi%d: %s: free iocb mem.\n", ha->host_no,
                    __func__));
                pci_free_consistent(ha->pdev, PAGE_SIZE, ha->gen_req_rsp_iocb,
                    ha->gen_req_rsp_iocb_dma);
        }

        list_for_each_safe(list, temp, &ha->async_iocb_list) {
                async_pdu_iocb_t *apdu_iocb;

                apdu_iocb = list_entry(list, async_pdu_iocb_t, list_entry);
                list_del_init(&apdu_iocb->list_entry);
                vfree(apdu_iocb);
        }

	/* Free srb pool */
	if (ha->num_srbs_allocated) {
		QL4PRINT(QLP7, printk("scsi%d: %s: free srb pool\n",
		    ha->host_no, __func__));

		qla4xxx_free_srb_pool(ha);
	}

	/* Free ddb list */
	if (!list_empty(&ha->ddb_list)) {
		QL4PRINT(QLP7, printk("scsi%d: %s: free ddb list\n",
		    ha->host_no, __func__));

		qla4xxx_free_ddb_list(ha);
	}

	/* Free DMA Pool for Passthru IOCBs */
	if (ha->pt_iocb_dmapool) {
		dma_pool_destroy(ha->pt_iocb_dmapool);
		ha->pt_iocb_dmapool = NULL;
	}

	/* Unmap Memory Mapped I/O region */
	if (IS_QLA8032(ha)) {
		if(ha->nx_pcibase)
                        iounmap(
			   (struct device_reg_83xx __iomem *)ha->nx_pcibase);
	} else if (IS_QLA8022(ha)) {
		if (ha->nx_pcibase)
			iounmap(
			   (struct device_reg_82xx __iomem *)ha->nx_pcibase);
	} else if (ha->virt_mmapbase) {
		QL4PRINT(QLP7, printk("scsi%d: %s: unmap mem io region\n",
		    ha->host_no, __func__));

		iounmap(ha->virt_mmapbase);
		ha->virt_mmapbase = NULL;
	}

	if (ha->mem_addr)
		release_mem_region(ha->mem_addr, ha->mem_len);
	ha->mem_addr = 0;

	if (ha->reset_tmplt.buff) {
		vfree(ha->reset_tmplt.buff);
		ha->reset_tmplt.buff = NULL;
	}

	qla4xxx_free_other_mem(ha);
}

/*
 *  qla4_8xxx_check_temp
 *      Check the ISP82XX temperature.
 *
 * Input:
 *      ha  = adapter block pointer.
 * Note: The caller should not hold the idc lock.
 */
int
qla4_8xxx_check_temp(struct scsi_qla_host *ha)
{
	uint32_t temp, temp_state, temp_val;
	int status = 0;

	temp = qla4_8xxx_rd_direct(ha, QLA8XXX_CRB_TEMP_STATE);

	temp_state = qla82xx_get_temp_state(temp);
	temp_val = qla82xx_get_temp_val(temp);

	if (temp_state == QLA82XX_TEMP_PANIC) {
		printk("scsi(%d): "
			"Device temperature %d degrees C exceeds"
			" maximum allowed. Hardware has been shut down.\n",
			ha->host_no, temp_val);
		status = 1;
	} else if (temp_state == QLA82XX_TEMP_WARN) {
		if (ha->temperature == QLA82XX_TEMP_NORMAL) {
			printk("scsi(%d): "
				"Device temperature %d degrees C "
				"exceeds operating range."
				" Immediate action needed.\n",
				ha->host_no, temp_val);
		}
	} else {
		if (ha->temperature == QLA82XX_TEMP_WARN) {
			printk("scsi(%d): "
				"Device temperature is now %d degrees C"
				" in normal range.\n",
				ha->host_no, temp_val);
		}
	}
	ha->temperature = temp_state;

	return status;
}

/**************************************************************************
* qla2xxx_slave_configure
*
* Description:
**************************************************************************/
int
qla4xxx_slave_configure(struct scsi_device *sdev)
{
	scsi_qla_host_t *ha = to_qla_host(sdev->host);
	int queue_depth;
	os_tgt_t     *tgt_entry;
	os_lun_t     *lun_entry;

	queue_depth = 32;

	/* Enable TCQ. */
	if (sdev->tagged_supported) {
		if (ql4xmaxqdepth != 0 && ql4xmaxqdepth <= 0xffffU)
			queue_depth = ql4xmaxqdepth;

		ql4xmaxqdepth = queue_depth;

		scsi_activate_tcq(sdev, queue_depth);

		ql4_printk(KERN_INFO, ha,
		    "scsi(%d:%d:%d:%d): Enabled tagged queuing, queue "
		    "depth %d.\n", sdev->host->host_no, sdev->channel,
		    sdev->id, sdev->lun, sdev->queue_depth);
	} else {
		 scsi_adjust_queue_depth(sdev, 0 /* TCQ off */,
		     sdev->host->hostt->cmd_per_lun /* 3 */);
	}

	/* Save misc. information. */
	tgt_entry = qla4xxx_lookup_target_by_SCSIID(ha, sdev->channel,
	    sdev->id);
	if (tgt_entry != NULL) {
		lun_entry = qla4xxx_lookup_lun_handle(ha, tgt_entry,
		    sdev->lun);
	        if (lun_entry != NULL) {
			lun_entry->sdev = sdev;
			if (sdev->type == TYPE_TAPE) {
				tgt_entry->fcport->flags |= FCF_TAPE_PRESENT;
			}
		}
	}

	return (0);
}


/**************************************************************************
 * del_from_active_array
 *      This routine removes and returns the srb at the specified index
 *
 * Input:
 *      ha - Pointer to host adapter structure.
 *      index - index into to the active_array
 *
 * Returns:
 *      Pointer to corresponding SCSI Request Block
 *
 * Context:
 *      Kernel/Interrupt context.
 **************************************************************************/
srb_t *
del_from_active_array(scsi_qla_host_t *ha, uint32_t index)
{
	srb_t *srb = NULL;

	/* validate handle and remove from active array */
	if (index < MAX_SRBS) {
		srb = ha->active_srb_array[index];
		ha->active_srb_array[index] = NULL;

		if (srb) {
			os_lun_t *lun_entry = srb->lun_queue;

			/* update counters */
			if ((srb->flags & SRB_DMA_VALID) != 0) {
				ha->iocb_cnt -= srb->entry_count;
				if (ha->active_srb_count)
					ha->active_srb_count--;
				if (lun_entry)
					lun_entry->out_count--;
				if (srb->cmd)
					srb->cmd->host_scribble = NULL;
			}
		}
		else
			QL4PRINT(QLP2,
				 printk("scsi%d: %s: array_index=%d "
					"already completed.\n",
					ha->host_no, __func__, index));
	}
	else
		QL4PRINT(QLP2, printk("scsi%d: %s: array_index=%d "
				      "exceeded max index of %d\n",
				      ha->host_no, __func__, index, MAX_SRBS));

	return(srb);
}

uint16_t qla4xxx_calc_request_entries(uint16_t dsds)
{
	uint16_t iocbs;/* number of request queue entries */
				/* (commamd + continue) */
	iocbs = 1;
	if (dsds > COMMAND_SEG) {
		iocbs += (dsds - COMMAND_SEG) / CONTINUE_SEG;
		if ((dsds - COMMAND_SEG) % CONTINUE_SEG)
			iocbs++;
	}
	return (iocbs);
}

void qla4xxx_build_scsi_iocbs(srb_t *srb, COMMAND_ENTRY *cmd_entry, uint16_t tot_dsds)
{
	scsi_qla_host_t	*ha;
	uint16_t	avail_dsds;
	DATA_SEG_A64 *cur_dsd;
	struct scsi_cmnd *cmd;

	cmd = srb->cmd;
	ha = srb->ha;

	if (cmd->request_bufflen == 0 ||
	    cmd->sc_data_direction == DMA_NONE) {
		/* No data being transferred */
		QL4PRINT(QLP5, printk("scsi%d:%d:%d:%d: %s: No data xfer\n",
		    ha->host_no, cmd->device->channel, cmd->device->id,
		    cmd->device->lun, __func__));

		cmd_entry->ttlByteCnt = __constant_cpu_to_le32(0);
		return;
	}

	avail_dsds = COMMAND_SEG;
	cur_dsd = (DATA_SEG_A64 *) &(cmd_entry->dataseg[0]);

	/* Load data segments */
	if (cmd->use_sg) {
		struct  scatterlist *cur_seg;
		struct  scatterlist *end_seg;

		/* Data transfer with Scatter/Gather
		 *
		 * We must build an SG list in adapter format, as the kernel's
		 * SG list cannot be used directly because of data field size
		 * (__alpha__) differences and the kernel SG list uses virtual
		 * addresses where we need physical addresses.
		 */
		cur_seg = (struct scatterlist *) cmd->request_buffer;
		end_seg = cur_seg + tot_dsds;

		while (cur_seg < end_seg) {
			dma_addr_t sle_dma;

			/* Allocate additional continuation packets? */
			if (avail_dsds == 0) {
				CONTINUE_ENTRY *cont_entry;

				cont_entry = qla4xxx_alloc_cont_entry(ha);
				cur_dsd = (DATA_SEG_A64 *) &cont_entry->dataseg[0];
				avail_dsds = CONTINUE_SEG;
			}

			sle_dma = sg_dma_address(cur_seg);
			cur_dsd->base.addrLow = cpu_to_le32(LSDW(sle_dma));
			cur_dsd->base.addrHigh = cpu_to_le32(MSDW(sle_dma));
			cur_dsd->count = cpu_to_le32(sg_dma_len(cur_seg));
			avail_dsds--;

			DEBUG(printk("scsi%d:%d:%d:%d: %s: S/G "
			    "DSD %p phys_addr=%x:%08x, len=0x%x, tot_dsd=0x%x, "
			    "avail_dsd=0x%x\n", ha->host_no,
			    cmd->device->channel, cmd->device->id,
			    cmd->device->lun, __func__, cur_dsd,
			    cur_dsd->base.addrHigh, cur_dsd->base.addrLow,
			    cur_dsd->count, tot_dsds, avail_dsds));

			cur_dsd++;
			cur_seg++;
		}
	} else {
		/* Data transfer without SG entries. */
		dma_addr_t	req_dma;
		struct page	*page;
		unsigned long	offset;

		page = virt_to_page(cmd->request_buffer);
		offset = ((unsigned long) cmd->request_buffer & ~PAGE_MASK);
		req_dma = pci_map_page(ha->pdev, page, offset,
		    cmd->request_bufflen, cmd->sc_data_direction);
		srb->saved_dma_handle = req_dma;

		cur_dsd->base.addrLow = cpu_to_le32(LSDW(req_dma));
		cur_dsd->base.addrHigh = cpu_to_le32(MSDW(req_dma));
		cur_dsd->count = cpu_to_le32(cmd->request_bufflen);

		QL4PRINT(QLP5, printk("scsi%d:%d:%d:%d: %s: No S/G transfer, "
		    "DSD=%p cmd=%p dma_addr=%x:%08x, len=%x, tot_dsd=0x%x, "
		    "avail_dsd=0x%x\n", ha->host_no, cmd->device->channel,
		    cmd->device->id, cmd->device->lun, __func__, cur_dsd, cmd,
		    cur_dsd->base.addrHigh, cur_dsd->base.addrLow,
		    cur_dsd->count, tot_dsds, avail_dsds));

		cur_dsd++;
	}
}

CONTINUE_ENTRY *
qla4xxx_alloc_cont_entry(scsi_qla_host_t *ha)
{
        CONTINUE_ENTRY *cont_entry;

        cont_entry = (CONTINUE_ENTRY *)ha->request_ptr;

        /* Advance request queue pointer */
        if (ha->request_in == (ha->maxcmds - 1)) {
                ha->request_in = 0;
                ha->request_ptr = ha->request_ring;
                QL4PRINT(QLP10, printk("scsi%d: %s: wraparound -- new "
                    "request_in = %04x, new request_ptr = %p\n", ha->host_no,
                    __func__, ha->request_in, ha->request_ptr));
        } else {
		ha->request_in++;
                ha->request_ptr++;
                QL4PRINT(QLP10, printk("scsi%d: %s: new request_in = %04x, new "
                    "request_ptr = %p\n", ha->host_no, __func__, ha->request_in,
                    ha->request_ptr));
        }

        /* Load packet defaults */
        cont_entry->hdr.entryType = ET_CONTINUE;
        cont_entry->hdr.entryCount = 1;
        cont_entry->hdr.systemDefined =
        (uint8_t) cpu_to_le16(ha->request_in);

        return(cont_entry);
}

/**************************************************************************
 * qla4xxx_send_command_to_isp
 *      This routine is called by qla4xxx_queuecommand to build an ISP
 *      command and pass it to the ISP for execution.
 *
 * Input:
 *      ha - Pointer to host adapter structure.
 *      srb - pointer to SCSI Request Block to be sent to ISP
 *
 * Output:
 *      None
 *
 * Remarks:
 *      None
 *
 * Returns:
 *      QLA_SUCCESS - Successfully sent command to ISP
 *      QLA_ERROR   - Failed to send command to ISP
 *
 * Context:
 *      Kernel context.
 **************************************************************************/
uint8_t
qla4xxx_send_command_to_isp(scsi_qla_host_t *os_ha, srb_t *srb)
{
	struct scsi_cmnd     *cmd = srb->cmd;
	ddb_entry_t   *ddb_entry;
	os_lun_t   *lun_entry;
	COMMAND_ENTRY *cmd_entry;
	struct scatterlist *sg = NULL;

	uint16_t      tot_dsds;	/* number of data segments */
				/* (sg entries, if sg request) */
	uint16_t	req_cnt;	/* number of request queue entries */

	unsigned long flags;
	uint16_t	cnt;
	uint16_t      i;
	uint32_t      index;
	fc_lun_t                *fclun;
	scsi_qla_host_t *ha;
	char		tag[2];
	uint32_t	timeout;

	/* Get real lun and adapter */
	fclun = srb->lun_queue->fclun;
	ha = fclun->fcport->ha;

	cmd = srb->cmd;
	ddb_entry = fclun->fcport->ddbptr;
	lun_entry = srb->lun_queue;

	/* Acquire hardware specific lock */
	spin_lock_irqsave(&ha->hardware_lock, flags);

	/* Check for room in active srb array */
	index = ha->current_active_index;
	for (i = 0; i < MAX_SRBS; i++) {
		index++;
		if (index == MAX_SRBS)
			index = 1;
		if (ha->active_srb_array[index] == 0) {
			ha->current_active_index = index;
			break;
		}
	}
	if (i >= MAX_SRBS) {
		printk(KERN_INFO "scsi%d: %s: NO more SRB entries "
		    "used iocbs=%d, reqs remaining=%d\n", ha->host_no,
		    __func__, ha->iocb_cnt, ha->req_q_count);
		goto queuing_error;
	}


	/* Calculate the number of request entries needed. */
	tot_dsds = 0;
	if (cmd->use_sg) {
		sg = (struct scatterlist *) cmd->request_buffer;
		tot_dsds = pci_map_sg(ha->pdev, sg, cmd->use_sg,
		    cmd->sc_data_direction);
		if (tot_dsds == 0)
			goto queuing_error;
	} else if (cmd->request_bufflen) {
	    tot_dsds++;
	}
	req_cnt = qla4xxx_calc_request_entries(tot_dsds);

	/* Check if room for request in request ring. */
	if (ha->req_q_count < (req_cnt + 2)) {
	    	cnt = ha->isp_ops->rd_shdw_req_q_out(ha);
		if (ha->request_in < cnt) {
			ha->req_q_count = cnt - ha->request_in;
		}
		else {
			ha->req_q_count = ha->maxcmds - (ha->request_in - cnt);
		}

		DEBUG(printk(KERN_INFO "scsi%d: %s: new request queue count -"
		    "calc req_q_count=%d, reqs needed=%d, used_iocbs=%d,"
		    "shadow reqs=%d, req_in =%d\n", ha->host_no,
		    __func__, ha->req_q_count, req_cnt, ha->iocb_cnt,
		    cnt, ha->request_in));
	}

	if (ha->req_q_count < (req_cnt + 2)) {
		/* free the resources */
		if (cmd->use_sg)
			pci_unmap_sg(ha->pdev, sg, cmd->use_sg, cmd->sc_data_direction);

		DEBUG2(printk(KERN_INFO "scsi%d: %s: No more request queues, "
		    "used iocbs=%d, avail_iocbs=%d, reqs needed=%d\n",
		    ha->host_no, __func__, ha->iocb_cnt, ha->req_q_count,
		    req_cnt));
		goto queuing_error;
	}

	/* total iocbs active */
	if ((ha->iocb_cnt + req_cnt)  >= ha->maxcmds) {
		if (cmd->use_sg)
			pci_unmap_sg(ha->pdev, sg, cmd->use_sg, cmd->sc_data_direction);
		goto queuing_error;
	}

	/* Build command packet */
	cmd_entry = (COMMAND_ENTRY *) ha->request_ptr;
	memset(cmd_entry, 0, sizeof(COMMAND_ENTRY));
	cmd_entry->hdr.entryType = ET_COMMAND;
	cmd_entry->handle = cpu_to_le32(index);
	cmd_entry->target = cpu_to_le16(ddb_entry->fw_ddb_index);
	cmd_entry->connection_id = cpu_to_le16(ddb_entry->connection_id);

	cmd_entry->lun[1] = LSB(cmd->device->lun);	/* SAMII compliant. */
	cmd_entry->lun[2] = MSB(cmd->device->lun);
	cmd_entry->cmdSeqNum = cpu_to_le32(ddb_entry->CmdSn);
	cmd_entry->ttlByteCnt = cpu_to_le32(cmd->request_bufflen);
	memcpy(cmd_entry->cdb, cmd->cmnd, MIN(MAX_COMMAND_SIZE, cmd->cmd_len));
	cmd_entry->dataSegCnt = cpu_to_le16(tot_dsds);
	cmd_entry->hdr.entryCount = req_cnt;

	/* We want the firmware to time out the command first.*/
	timeout = (uint32_t)(cmd->timeout_per_command / HZ);
	if (timeout > 65535)
                cmd_entry->timeout = __constant_cpu_to_le16(0);
	else if (timeout > MIN_CMD_TOV)
		cmd_entry->timeout = cpu_to_le16((uint16_t) timeout -
					 (QLA_CMD_TIMER_DELTA+1));
	else
		cmd_entry->timeout = cpu_to_le16((uint16_t)timeout);

	srb->iocb_tov = le16_to_cpu(cmd_entry->timeout);
	srb->os_tov = timeout;

	/* Set data transfer direction control flags
	 * NOTE: Look at data_direction bits iff there is data to be
	 *       transferred, as the data direction bit is sometimed filled
	 *       in when there is no data to be transferred */
	cmd_entry->control_flags = CF_NO_DATA;
	if (cmd->request_bufflen) {
		if (cmd->sc_data_direction == DMA_TO_DEVICE)
			cmd_entry->control_flags = CF_WRITE;
		else if (cmd->sc_data_direction == DMA_FROM_DEVICE)
			cmd_entry->control_flags = CF_READ;
	}

	/* Set tagged queueing control flags */
	cmd_entry->control_flags |= CF_SIMPLE_TAG;
	if (scsi_populate_tag_msg(cmd, tag)) {
		switch (tag[0]) {
		case MSG_HEAD_TAG:
			cmd_entry->control_flags |= CF_HEAD_TAG;
			break;
		case MSG_ORDERED_TAG:
			cmd_entry->control_flags |= CF_ORDERED_TAG;
			break;
		}
	}

	/* Advance request queue pointer */
	ha->request_in++;
        if (ha->request_in == ha->maxcmds) {
                ha->request_in = 0;
                ha->request_ptr = ha->request_ring;
        } else {
                ha->request_ptr++;
        }

	qla4xxx_build_scsi_iocbs(srb, cmd_entry, tot_dsds);

	wmb();

	/*
	 * Check to see if adapter is online before placing request on
	 * request queue.  If a reset occurs and a request is in the queue,
	 * the firmware will still attempt to process the request, retrieving
	 * garbage for pointers.
	 */
	if (!test_bit(AF_ONLINE, &ha->flags)) {
		QL4PRINT(QLP2, printk("scsi%d: %s: Adapter OFFLINE! "
				      "Do not issue command.\n",
                                      ha->host_no, __func__));
		if (cmd->use_sg)
			pci_unmap_sg(ha->pdev, sg, cmd->use_sg, cmd->sc_data_direction);
		goto queuing_error;
	}

	/* put command in active array */
	ha->active_srb_array[index] = srb;
	srb->cmd->host_scribble = (unsigned char *)(unsigned long)index;

	/* update counters */
	ha->active_srb_count++;
	lun_entry->out_count++;
	lun_entry->tot_io_count++;
	srb->state = SRB_ACTIVE_STATE;
	srb->flags |= SRB_DMA_VALID;

	/* Track IOCB used */
	ha->iocb_cnt += req_cnt;
	srb->entry_count = req_cnt;
	ha->req_q_count -= req_cnt;

	/* Debug print statements */
#ifdef QL_DEBUG_LEVEL_3
	printk("scsi%d:%d:%d:%d: %s: CDB = ", ha->host_no,
	    cmd->device->channel, cmd->device->target, cmd->device->lun,
	    __func__);
	for (i = 0; i < cmd->cmd_len; i++)
		printk("%02x ", cmd->cmnd[i]);
	printk("\n");
#endif
	srb->u_start = jiffies;

	/* Tell ISP it's got a new I/O request */
	ha->isp_ops->queue_iocb(ha);

	spin_unlock_irqrestore(&ha->hardware_lock, flags);
	return(QLA_SUCCESS);

queuing_error:
	/* Release hardware specific lock */
	spin_unlock_irqrestore(&ha->hardware_lock, flags);
	return(QLA_ERROR);
}

inline uint8_t
qla4xxx_free_srb_dma(scsi_qla_host_t *ha, srb_t *srb)
{
	struct scsi_cmnd *cmd;

	cmd = srb->cmd;
	if (cmd == NULL) {
		QL4PRINT(QLP2,
			 printk("scsi%d: %s: ERROR: NULL cmd pointer in "
				"srb=%p, srb->state=%d, srb->flags=0x%x, ref=%d\n",
				ha->host_no, __func__, srb, srb->state, srb->flags,
				atomic_read(&srb->ref_count)));
		return QLA_ERROR;
	}
	/* Release memory used for this I/O */
	if ( (srb->flags & SRB_DMA_VALID) ) {
		srb->flags &= ~SRB_DMA_VALID;

		/* Release memory used for this I/O */
		if (cmd->use_sg) {
			pci_unmap_sg(ha->pdev,
				     cmd->request_buffer,
				     cmd->use_sg,
				     cmd->sc_data_direction);
		}
		else if (cmd->request_bufflen) {
			pci_unmap_page(ha->pdev,
				       srb->saved_dma_handle,
				       cmd->request_bufflen,
				       srb->cmd->sc_data_direction);
		}
		ha->total_mbytes_xferred += cmd->request_bufflen / 1024;
	}
	return QLA_SUCCESS;
}

/**************************************************************************
 * qla4xxx_request_cleanup
 *      This routine frees resources for a command that
 *      didn't get completed.
 *
 * Input:
 *      ha - Pointer to host adapter structure.
 *      srb - Pointer to SCSI Request Block
 *
 * Remarks:
 *    The srb pointer should be guaranteed to be nonzero before calling
 *    this function.  The caller should also ensure that the list_lock is
 *    released before calling this function.
 *
 * Returns:
 *
 * Context:
 *      Kernel/Interrupt context.
 **************************************************************************/
inline void
qla4xxx_request_cleanup(scsi_qla_host_t *ha, srb_t *srb)
{
	qla4xxx_delete_timer_from_cmd(srb);
	qla4xxx_free_srb_dma(ha, srb);

	/*  Let abort handler know we are completing the command */
 	CMD_SP(srb->cmd) = NULL;

 	atomic_dec(&srb->ref_count);
	srb->cmd = NULL;
	add_to_free_srb_q(ha, srb);

}

/**************************************************************************
* qla4xxx_done
*      Process completed commands.
*
* Input:
*      old_ha           = adapter block pointer.
*
* Returns:
* int
**************************************************************************/
int
qla4xxx_done(scsi_qla_host_t *ha)
{
 	unsigned long   flags = 0;
 	int             cnt;
 	srb_t           *srb, *stemp;
 	struct  list_head local_sp_list;

 	cnt = 0;

 	INIT_LIST_HEAD(&local_sp_list);

 	/*
 	 * Get into local queue such that we do not wind up calling done queue
 	 * takslet for the same IOs from DPC or any other place.
 	 */
 	spin_lock_irqsave(&ha->list_lock,flags);
 	list_splice_init(&ha->done_srb_q, &local_sp_list);
 	spin_unlock_irqrestore(&ha->list_lock, flags);

 	list_for_each_entry_safe(srb, stemp, &local_sp_list, list_entry) {
		__del_from_done_srb_q(ha, srb);

 		cnt++;
 		sp_put(ha, srb);

 	} /* end of while */
 	return(cnt);
 }

/**************************************************************************
 * qla4xxx_complete_request
 *      This routine returns a command to the caller via the done_fn
 *      specified in the cmd structure.
 *
 * Input:
 *      ha - Pointer to host adapter structure.
 *      srb - Pointer to SCSI Request Block
 *
 * Remarks:
 *    The srb pointer should be guaranteed to be nonzero before calling
 *    this function.  The caller should also ensure that the list_lock is
 *    released before calling this function.
 *
 * Returns:
 *      QLA_SUCCESS - Successfully completed request
 *      QLA_ERROR   - Failed to complete request
 *
 * Context:
 *      Kernel/Interrupt context.
 **************************************************************************/
uint8_t
qla4xxx_complete_request(scsi_qla_host_t *ha, srb_t *srb)
{
	uint8_t status = QLA_ERROR;
	struct scsi_cmnd *cmd;

	/* Make sure the cmd pointer is valid */
	if (srb == NULL) {
		QL4PRINT(QLP2, printk("scsi%d: %s: ERROR: NULL srb \n",
				      ha->host_no, __func__));
		goto exit_complete_request;
	}

	cmd = srb->cmd;
	if (cmd == NULL) {
		QL4PRINT(QLP2, printk("scsi%d: %s: ERROR: command already "
		    "completed for srb %p, state=%s, ref=%d\n",
		    ha->host_no, __func__, srb, srb_state_msg[srb->state],
			atomic_read(&srb->ref_count)));
		goto exit_complete_request;
	}

	if (qla4xxx_free_srb_dma(ha, srb) == QLA_ERROR) {
		goto exit_complete_request;
	}

	/*  Let abort handler know we are completing the command */
	CMD_SP(cmd) = NULL;

	if (host_byte(cmd->result) == DID_OK) {
		if (!(srb->flags & SRB_GOT_SENSE)) {
			os_lun_t *lun_entry = srb->lun_queue;
			unsigned long flags;

			if (lun_entry) {
				/*
				 * If lun was not ready (suspended or timeout)
				 * then change state to "READY".
				 */
				spin_lock_irqsave(&lun_entry->lun_lock, flags);
				if (lun_entry->lun_state != LS_LUN_READY) {
					lun_entry->lun_state = LS_LUN_READY;
				}
				spin_unlock_irqrestore(&lun_entry->lun_lock, flags);
			}
		}
	}

#if 0
	if (host_byte(cmd->result) != DID_OK ) {
			    DEBUG2(printk("scsi%d:%d:%d:%d: %s: "
				"did_error=%d, cmd=%p pid=%ld\n",
				ha->host_no, cmd->device->channel, cmd->device->id,
				cmd->device->lun,
				__func__,
				host_byte(cmd->result),
				cmd,
    				cmd->serial_number));
	}
#endif
	/*
	 * WORKAROUND
	 * A backdoor device-reset (via eh_resets) requires different
	 * error handling.  This code differentiates between normal
	 * error handling and the backdoor method
	 */
	if (host_byte(cmd->result) == DID_RESET) {
		if (qla4xxx_is_eh_active(ha->host))
			srb->cmd->result = DID_BUS_BUSY << 16;
	}

#ifdef QL_DEBUG_LEVEL_3
	if (cmd->result & 0xff) {
		QL4PRINT(QLP13,
			 printk("REQUEST_SENSE data:  "
				"(MAX 0x20 bytes displayed)\n"));

		qla4xxx_dump_bytes(QLP13, cmd->sense_buffer,
				   MIN(0x20, sizeof(cmd->sense_buffer)));
	}
#endif

	srb->cmd = NULL;
	add_to_free_srb_q(ha, srb);


	/* Call the mid-level driver interrupt handler */
	(*(cmd)->scsi_done)(cmd);

	exit_complete_request:
	return status;
}

/**************************************************************************
 * qla4xxx_queuecommand
 *      This routine is invoked by Linux to send a SCSI command to the driver.
 *
 * Input:
 *      cmd - Pointer to Linux's SCSI command structure
 *      done_fn - Function that the driver calls to notify the SCSI mid-layer
 *                that the command has been processed.
 *
 * Remarks:
 *    The mid-level driver tries to ensure that queuecommand never gets
 *    invoked concurrently with itself or the interrupt handler (although
 *    the interrupt handler may call this routine as part of request-
 *    completion handling).   Unfortunely, it sometimes calls the scheduler
 *    in interrupt context which is a big NO! NO!.
 *
 * Returns:
 *      0 - cmd accepted, must call done() function before returning cmd
 *      SCSI_MLQUEUE_DEVICE_BUSY -
 *      SCSI_MLQUEUE_HOST_BUSY & other non-zero -
 *
 * Context:
 *      Kernel context.
 **************************************************************************/
int
qla4xxx_queuecommand(struct scsi_cmnd *cmd, void (*done_fn)(struct scsi_cmnd *))
{
	scsi_qla_host_t *ha;
	ddb_entry_t     *ddb_entry;
	os_lun_t     *lun_entry;
	os_tgt_t     *tgt_entry;
	uint32_t        b, t, l;
	int             return_status = 0;
	srb_t           *srb;
	fc_port_t       *fcport;
	fc_lun_t       *fclun;

	b = cmd->device->channel;
	t = cmd->device->id;
	l = cmd->device->lun;
	ha = (scsi_qla_host_t *) cmd->device->host->hostdata;

	cmd->scsi_done = done_fn;

	if (test_bit(DPC_RESET_HA_INTR, &ha->dpc_flags) ||
	    test_bit(DPC_RESET_HA, &ha->dpc_flags) ||
	    test_bit(DPC_HA_UNRECOVERABLE, &ha->dpc_flags) ||
	    test_bit(DPC_HA_NEED_QUIESCENT, &ha->dpc_flags) ||
	    test_bit(DPC_RESET_QUIESCENT, &ha->dpc_flags) ||
	    test_bit(DPC_RESET_ACTIVE, &ha->dpc_flags) ||
	    !test_bit(AF_ONLINE, &ha->flags) ||
	    test_bit(DPC_RESET_HA_FW_CONTEXT, &ha->dpc_flags)){
		return SCSI_MLQUEUE_HOST_BUSY;
	}

	spin_unlock_irq(ha->host->host_lock);

	/*
	 * Allocate a comand packet
	 */
	srb = del_from_free_srb_q_head(ha);
	if (srb == NULL) {
		DEBUG2(printk("scsi%d: %s: srb not available. "
		       "Free srbs = (%d/%d)\n",
		       ha->host_no, __func__,
		       ha->free_srb_q_count, ha->num_srbs_allocated);)

		spin_lock_irq(ha->host->host_lock);
		return SCSI_MLQUEUE_HOST_BUSY;
	}

	/* Link the srb with cmd */
	srb->cmd = cmd;
	CMD_SP(cmd) = (void *)srb;
	srb->r_start = jiffies;
	srb->flags = 0;
	srb->ha = ha;

	/* Start command timer. */
	if ((cmd->timeout_per_command/HZ) > MIN_CMD_TOV)
		qla4xxx_add_timer_to_cmd(ha, srb, (cmd->timeout_per_command / HZ) -
		    QLA_CMD_TIMER_DELTA);
	else
		srb->flags |= SRB_NO_TIMER;

	if (test_bit(AF_HA_REMOVAL, &ha->flags)) {
                cmd->result = DID_NO_CONNECT << 16;
                DEBUG2(printk("scsi%d: %s: Module unloading, return cmd= %p pid=%ld\n",
                        ha->host_no, __func__, cmd, cmd->serial_number));
                goto qc_complete;
        }

	if (test_bit(AF_EEH_BUSY, &ha->flags)) {
		if (test_bit(AF_PCI_CHANNEL_IO_PERM_FAILURE, &ha->flags))
			cmd->result = DID_NO_CONNECT << 16;
		else
			cmd->result = DID_REQUEUE << 16;
		goto qc_complete;
	}

	/* retrieve device and lun handles */
	tgt_entry = qla4xxx_lookup_target_by_SCSIID(ha, b, t);
	if (tgt_entry ==  NULL) {
		cmd->result = DID_NO_CONNECT << 16;
		goto qc_complete;
	}

	lun_entry = qla4xxx_lookup_lun_handle(ha, tgt_entry, l);
	if (lun_entry == NULL) {

		/* If no target, don't create lun */
		if (tgt_entry->fcport == NULL) {
			cmd->result = DID_NO_CONNECT << 16;
			goto qc_complete;
		}

		/* Only allocate luns for ddb device state SESSION_ACTIVE */
		if (tgt_entry->fcport->ddbptr->fw_ddb_device_state != DDB_DS_SESSION_ACTIVE) {
			cmd->result = DID_NO_CONNECT << 16;
			goto qc_complete;
		}

		/*
		 * Allocate a LUN queue for this request if we haven't
		 * already did it on a previous command.
		 */
		fcport = tgt_entry->fcport;
		fclun = qla4xxx_add_fclun(fcport, l);
		if (fclun == NULL) {
			DEBUG2(printk("%s: Can't get FCLUN queue.\n",
			    __func__);)
			cmd->result = DID_ERROR << 16;
			goto qc_complete;
		}

		/* Assume this type right now and fixup after command completes */
		fclun->device_type = TYPE_DISK;
		lun_entry = qla4xxx_fclun_bind(ha, fcport, fclun);
		if( lun_entry  == NULL ) {
			DEBUG2(printk("%s: Can't Bind or allocate LUN queue.\n",
		    __func__);)
			cmd->result = DID_ERROR << 16;
			goto qc_complete;
		}
	}

	srb->tgt_queue = tgt_entry;
	srb->lun_queue = lun_entry;
	srb->fclun = lun_entry->fclun;
	if (lun_entry->fclun == NULL) {
		cmd->result = DID_NO_CONNECT << 16;

		DEBUG2(printk(
		    "scsi%d: (lq->fclun == NULL) pid=%ld,lq=%p\n",
    		ha->host_no, srb->cmd->serial_number, lun_entry));
		goto qc_complete;
	}
	fcport = lun_entry->fclun->fcport;
	if (fcport ==  NULL) {
		cmd->result = DID_NO_CONNECT << 16;
		DEBUG2(printk(
		 "scsi%d: (lq->fclun->fcport == NULL) pid=%ld, lq=%p,"
		 "lq->fclun=%p\n",
    		ha->host_no, srb->cmd->serial_number,
		lun_entry, lun_entry->fclun));
		goto qc_complete;
	}

	ddb_entry = fcport->ddbptr;
	if (ddb_entry ==  NULL) {
		cmd->result = DID_NO_CONNECT << 16;
		DEBUG2(printk("scsi%d: (ddbptr == NULL) pid=%ld, ddb entry=%p\n",
    		ha->host_no, srb->cmd->serial_number, ddb_entry));
		goto qc_complete;
	}
	srb->ha = fcport->ha;

	if (cmd->allowed < ql4xcmdretrycount)
		cmd->allowed = ql4xcmdretrycount;

	if (atomic_read(&ddb_entry->state) == DEV_STATE_DEAD ||
            atomic_read(&ddb_entry->state) == DEV_STATE_DELETED) {
		cmd->result = DID_NO_CONNECT << 16;
		DEBUG2(printk(
			"scsi%d:%d:%d:%d: PORT DEAD or DELETED cmd=%p cmd_err_flag=0x%x "
			"did_error=%x, pid=%ld\n",
                     	ha->host_no, b, t, l, srb->cmd, srb->cmd->eh_eflags,
			host_byte(cmd->result), srb->cmd->serial_number));
		add_to_done_srb_q(ha, srb);
		qla4xxx_done(ha);
		spin_lock_irq(ha->host->host_lock);
		return 0;
	}

	/*
	 * If the device is missing or the adapter is OFFLINE,
	 * put the request on the retry queue.
	 */
	if (atomic_read(&ddb_entry->state) == DEV_STATE_MISSING ||
	    !ADAPTER_UP(ha)) {
		DEBUG2(printk("scsi%d: PORT MISSING or HBA link-down"
		    "- ddb state=0x%x, hba flags=0x%lx, pid=%ld"
		    "\n", ha->host_no,
		    atomic_read(&ddb_entry->state),
		    ha->flags, srb->cmd->serial_number));

		qla4xxx_device_suspend(ha, srb);
		spin_lock_irq(ha->host->host_lock);
		return 0;
	}

	/*
	 * If this request's lun is suspended then put the request on
	 * the  scsi_retry queue.
	 */
	if (lun_entry->lun_state == LS_LUN_SUSPENDED) {
		DEBUG2(printk("scsi%d: Lun suspended - pid=%ld - "
		    "retry_q\n", ha->host_no,
		    srb->cmd->serial_number));

		qla4xxx_device_suspend(ha, srb);
		spin_lock_irq(ha->host->host_lock);
		return 0;
	}

	DEBUG(printk(
		    "scsi%d: %s pid=%ld, sp->flags=0x%x\n",
    		ha->host_no, __func__, srb->cmd->serial_number, srb->flags));

	/* If target suspended put incoming I/O in retry_q. */
	if (test_bit(TQF_SUSPENDED, &tgt_entry->flags) &&
	    (srb->flags & SRB_TAPE) == 0) {
		qla4xxx_device_suspend(ha, srb);
		spin_lock_irq(ha->host->host_lock);
		return 0;
	}

	if (qla4xxx_send_command_to_isp(ha, srb) != QLA_SUCCESS) {
		/*
		 * Unable to send command to the ISP at this time.
		 * Notify the OS to queue commands.
		 */
		DEBUG(printk("scsi%d: %s: unable to send cmd "
		    "to ISP, retry later\n", ha->host_no, __func__));
		qla4xxx_request_cleanup(ha, srb);
		return_status = SCSI_MLQUEUE_HOST_BUSY;

	}
	spin_lock_irq(ha->host->host_lock);
	return(return_status);

qc_complete:
	qla4xxx_delete_timer_from_cmd(srb);
 	sp_put(ha, srb);

	spin_lock_irq(ha->host->host_lock);
	return(0);
}

/**************************************************************************
 * qla4xxx_device_suspend
 *      This routine is invoked by driver to stall the request queue
 *
 * Input:
 *
 * Returns:
 *      None
 *
 * Context:
 *      Kernel context.
 **************************************************************************/
void
qla4xxx_device_suspend( scsi_qla_host_t *ha, srb_t *srb )
{
	QL4PRINT(QLP2, printk("scsi%d:%d:%d:%d: %s:(+) cmd=%p pid=%ld state=%d, extend to, retry_q\n",
			      ha->host_no, srb->cmd->device->channel,
			      srb->cmd->device->id, srb->cmd->device->lun,
			      __func__, srb->cmd, srb->cmd->serial_number,
			      srb->state));

	qla4xxx_extend_timeout(srb->cmd, EXTEND_CMD_TOV);
	add_to_retry_srb_q(ha, srb);
}

static inline void
qla4xxx_extend_scsi_ml_timeout(struct scsi_cmnd *cmd, int timeout)
{
	u_long our_jiffies;

	if (del_timer(&cmd->eh_timeout)) {

		our_jiffies = (timeout * HZ) + cmd->eh_timeout.expires;
		mod_timer(&cmd->eh_timeout, our_jiffies);
	}
}

/**************************************************************************
 * qla4xxx_extend_timeout
 *      This routine will extend the timeout to the specified value.
 *
 * Input:
 *      cmd - Pointer to Linux's SCSI command structure
 *      timeout - Amount of time to extend the OS timeout
 *
 * Returns:
 *      None
 *
 * Context:
 *      Kernel/Interrupt context.
 **************************************************************************/
void
qla4xxx_extend_timeout(struct scsi_cmnd *cmd, int timeout)
{
	srb_t *srb = (srb_t *) CMD_SP(cmd);

	if (del_timer(&cmd->eh_timeout)) {
		u_long our_jiffies;

		our_jiffies = (timeout * HZ) + cmd->eh_timeout.expires;
		mod_timer(&cmd->eh_timeout, our_jiffies);
	}
	if (srb->flags & SRB_NO_TIMER) {
		return;
	}
	if (del_timer(&srb->timer)) {
		u_long our_jiffies;
		/*
		 * Our internal timer should timeout before the midlayer has a
		 * chance begin the abort process
		 */
		our_jiffies = (timeout * HZ) + srb->timer.expires;
		mod_timer(&srb->timer, our_jiffies - (QLA_CMD_TIMER_DELTA * HZ));
	}
}


/**************************************************************************
 * qla4xxx_os_cmd_timeout
 *
 * Description:
 *       Handles the command if it times out in any state.
 *
 * Input:
 *     sp - pointer to validate
 *
 * Returns:
 * None.
 **************************************************************************/
void qla4xxx_os_cmd_timeout(srb_t *sp)
{
	int t, l;
	int processed;
	int timer_extended = 0;
	scsi_qla_host_t *ha;
	struct scsi_cmnd *cmd;
	ulong      flags;
	ulong      cpu_flags;
	fc_port_t       *fcport;
	u_long our_jiffies;

	cmd = sp->cmd;
	ha = sp->ha;

	DEBUG2(printk("scsi%d: %s: sp=%p sp->state = %x\n",
		      ha->host_no, __func__, sp, sp->state);)

	t = cmd->device->id;
	l = cmd->device->lun;
	fcport = sp->fclun->fcport;
	processed = 0;

	/*
 	 * If IO is found either in retry Queue OR Lun Queue
	 * Return this IO back to host
	 */
	spin_lock_irqsave(&ha->list_lock, flags);
 	DEBUG2(printk("scsi%d: %s: sp=%p, sp->state = %x, sp->flags=%x, cmd=%p, ref=%d\n",
 		      ha->host_no, __func__, sp, sp->state, sp->flags, cmd,
 		      atomic_read(&sp->ref_count)));

	if ((sp->state == SRB_RETRY_STATE)) {

		DEBUG2(printk("scsi%d: Found in (Scsi) Retry queue "
			      "pid %ld, State = %x., "
			      "ddb state=%d jiffies=%lx retried=%d\n",
			      ha->host_no,
			      sp->cmd->serial_number, sp->state,
			      atomic_read(&fcport->ddbptr->state),
			      jiffies, sp->cmd->retries);)

		__del_from_retry_srb_q(ha, sp);

		/*
 		 * If device is marked as dead return the cmd with
 		 * DID_NO_CONNECT status.  Otherwise set the host_byte to
 		 * DID_BUS_BUSY to let the OS  retry this cmd.
		 */
		if (atomic_read(&fcport->ddbptr->state) == DEV_STATE_DEAD ||
		    atomic_read(&fcport->ddbptr->state) == DEV_STATE_DELETED) {
			DEBUG2(printk("scsi%d: %s: Extending timeout\n",
				      ha->host_no, __func__));
			qla4xxx_extend_scsi_ml_timeout(cmd, EXTEND_CMD_TOV);
			cmd->result = DID_NO_CONNECT << 16;
		}
		else {
			if (host_byte(cmd->result) != DID_NO_CONNECT)
				cmd->result = DID_IMM_RETRY << 16;
		}

		sp_put(ha, sp);     /* release timer reference as expired */
		__add_to_done_srb_q(ha, sp);
		processed++;
	}
	spin_unlock_irqrestore(&ha->list_lock, flags);

	if (processed) {
		qla4xxx_done(ha);
		DEBUG2(printk("scsi%d: %s: Leaving 1, sp=%p, cmd=%p, pid=%ld\n",
			ha->host_no, __func__, sp, cmd, cmd->serial_number);)
		return;
	}

	spin_lock_irqsave(&ha->list_lock, cpu_flags);
	if (sp->state == SRB_DONE_STATE) {
		/* IO in done_q  -- leave it */
		DEBUG2(printk("scsi%d: %s: Found in Done queue pid %ld sp=%p.\n",
			      ha->host_no, __func__, sp->cmd->serial_number, sp);)
	}
	else if (sp->state == SRB_ACTIVE_STATE) {
		/*
		 * IO is with ISP find the command in our active list.
		 */
		spin_unlock_irqrestore(&ha->list_lock, cpu_flags);
		spin_lock_irqsave(&ha->hardware_lock, flags);
		if (sp == ha->active_srb_array
		    [(unsigned long)sp->cmd->host_scribble]) {

			DEBUG2(printk("scsi%d: %s: Found in ISP pid=%ld "
			    "hdl=%ld\n", ha->host_no, __func__,
			    cmd->serial_number,
			    (unsigned long)sp->cmd->host_scribble));

			if  (!(sp->flags & SRB_IOCTL_CMD)) {
				/*
				 * We cannot allow the midlayer error handler
				 * to wakeup and begin the abort process.
				 * Extend the timer so that the firmware can
				 * properly return the IOCB.
				 */
				DEBUG2(printk("scsi%d: %s: Extending timeout\n",
					      ha->host_no, __func__));
				qla4xxx_extend_scsi_ml_timeout(cmd, EXTEND_CMD_TOV);
			}

			sp->state = SRB_ACTIVE_TIMEOUT_STATE;
			spin_unlock_irqrestore(&ha->hardware_lock, flags);
		}
		else {
			spin_unlock_irqrestore(&ha->hardware_lock, flags);

			/* Command could have come back during the time the
			 * list_lock was released.  If not, display error */
			if (sp->state == SRB_ACTIVE_STATE) {
				DEBUG2(printk(
				    "scsi%d: %s: State indicates it is with "
				    "ISP, But not in active array\n",
				    ha->host_no, __func__));
				DEBUG2(qla4xxx_print_srb_info(QLP2, sp);)
			}
		}
		spin_lock_irqsave(&ha->list_lock, cpu_flags);
	}
	else if (sp->state == SRB_ACTIVE_TIMEOUT_STATE) {
		/* double timeout */
		DEBUG2(printk("scsi%d: %s: Found in Active timeout state "
				"pid %ld, State = %x.\n",
				ha->host_no, __func__,
				sp->cmd->serial_number, sp->state));
		DEBUG2(qla4xxx_print_srb_info(QLP2, sp);)
	}
	else {
		/* EMPTY */
		DEBUG3(printk("scsi%d: %s: LOST command state = "
			      "0x%x, sp=%p\n",
			      ha->host_no, __func__, sp->state, sp);)
		printk(KERN_INFO
		       "scsi%d: %s: LOST command state = 0x%x\n",
			ha->host_no, __func__, sp->state);

		DEBUG2(qla4xxx_print_srb_info(QLP2, sp);)

		printk(KERN_INFO
			"scsi%d: %s: Extending SCSI-ML timeout and "
			"srb timeout.\n",
			ha->host_no, __func__);
		qla4xxx_extend_scsi_ml_timeout(cmd, EXTEND_CMD_TOV);
		our_jiffies = (EXTEND_CMD_TOV * HZ) + sp->timer.expires;
		mod_timer(&sp->timer, our_jiffies - (QLA_CMD_TIMER_DELTA * HZ));

		timer_extended = 1;
	}
	spin_unlock_irqrestore(&ha->list_lock, cpu_flags);

	if (!timer_extended) {
		DEBUG2(printk(KERN_INFO "scsi%d: %s: timer extended, "
			"delete timer ref_count %d srb %p, "
			"cmd=%p, srb->state=%d, srb->flags=%x\n",
			ha->host_no, __func__, atomic_read(&sp->ref_count), sp,
			sp->cmd, sp->state, sp->flags));
		sp_put(ha, sp);     /* release timer reference as expired */
	}

       DEBUG2(printk("scsi%d: %s: Leaving 2 sp=%p, cmd=%p, pid=%ld\n",
		ha->host_no, __func__, sp, cmd, cmd->serial_number);)
}


/**************************************************************************
 * qla4xxx_add_timer_to_cmd
 *      This routine creates a timer for the specified command. The timeout
 *      is usually the command time from kernel minus 2 secs.
 *
 * Input:
 *      srb - Pointer to SCSI Request Block
 *      timeout - Number of seconds to extend command timeout.
 *
 * Returns:
 *      None
 *
 * Context:
 *      Kernel context.
 **************************************************************************/
void
qla4xxx_add_timer_to_cmd(scsi_qla_host_t *ha, srb_t *srb, int timeout)
{
	init_timer(&srb->timer);
	srb->timer.expires = jiffies + timeout * HZ;
	srb->timer.data = (unsigned long) srb;
	srb->timer.function = (void (*) (unsigned long))qla4xxx_os_cmd_timeout;
	add_timer(&srb->timer);
	sp_get(ha, srb);
	QL4PRINT(QLP3, printk("%s: srb %p, timeout %d\n",
			      __func__, srb, timeout));
}

/**************************************************************************
 * qla4xxx_delete_timer_from_cmd
 *      This routine deletes the timer for the specified command.
 *
 * Input:
 *      srb - Pointer to SCSI Request Block
 *
 * Returns:
 *      None
 *
 * Context:
 *      Kernel/Interrupt context.
 **************************************************************************/
void
qla4xxx_delete_timer_from_cmd(srb_t *srb )
{
	if (srb->flags & SRB_NO_TIMER)
		return;

	if (del_timer(&srb->timer)) {
		atomic_dec(&srb->ref_count);
		srb->timer.function =  NULL;
		srb->timer.data = (unsigned long) NULL;
	}
}


/****************************************************************************/
/*                        Interrupt Service Routine.                        */
/****************************************************************************/

/**
 * qla4_8xxx_check_fw_alive - Check firmware health
 * @ha: Pointer to host adapter structure.
 *
 * Returns: 1 if the firmware is hung.
 * Context: Interrupt
**/
static int
qla4_8xxx_check_fw_alive(struct scsi_qla_host *ha)
{
	uint32_t fw_heartbeat_counter;
	int status = 0;

	fw_heartbeat_counter = qla4_8xxx_rd_direct(ha,
					QLA8XXX_PEG_ALIVE_COUNTER);

	/* If PEG_ALIVE_COUNTER is 0xffffffff, AER/EEH is in progress, ignore */
	if (fw_heartbeat_counter == 0xffffffff) {
		DEBUG2(printk("scsi%d: %s: Device in frozen state, "
			"QLA8XXX_PEG_ALIVE_COUNTER is 0xffffffff\n",
			ha->host_no, __func__));
		status = 1;
		return status;
	}

	if (ha->fw_heartbeat_counter == fw_heartbeat_counter) {
		ha->seconds_since_last_heartbeat++;
		/* FW not alive after 2 seconds */
		if (ha->seconds_since_last_heartbeat == 2) {
			ha->seconds_since_last_heartbeat = 0;
			ql4_printk(KERN_INFO, ha, "scsi%d: %s: FW HANG detected !!\n",
					 ha->host_no, __func__);
			status = 1;
		}
	} else {
		ha->seconds_since_last_heartbeat = 0;
	}

	ha->fw_heartbeat_counter = fw_heartbeat_counter;

	return status;
}

/**
 * qla4_8xxx_watchdog - Poll dev state
 * @ha: Pointer to host adapter structure.
 *
 * Context: Interrupt
 **/
void qla4_8xxx_watchdog(struct scsi_qla_host *ha)
{
	uint32_t dev_state, halt_status;
	int halt_status_unrecoverable = 0;

	/* don't poll if reset is going on or FW hang in quiescent state */
	if (!(test_bit(DPC_RETRY_RESET_HA, &ha->dpc_flags) ||
	      test_bit(DPC_RESET_HA, &ha->dpc_flags) ||
	      test_bit(DPC_RESET_ACTIVE, &ha->dpc_flags) ||
	      test_bit(DPC_RESET_HA_FW_CONTEXT, &ha->dpc_flags))) {
		dev_state = qla4_8xxx_rd_direct(ha, QLA8XXX_CRB_DEV_STATE);
		if (qla4_8xxx_check_temp(ha)) {
			/* Note: For ISP8324 the reset sequence takes care of
			 * disabling the pause frames */
			if (IS_QLA8022(ha)) {
				ql4_printk(KERN_INFO, ha, "scsi%d: %s: "
					"disabling pause transmit on port 0 "
					"and 1\n", ha->host_no, __func__);
				qla4_82xx_wr_32(ha, QLA82XX_CRB_NIU + 0x98,
					CRB_NIU_XG_PAUSE_CTL_P0 |
					CRB_NIU_XG_PAUSE_CTL_P1);
			}
			set_bit(DPC_HA_UNRECOVERABLE, &ha->dpc_flags);
			qla4xxx_wake_dpc(ha);
		} else if ((dev_state == QLA8XXX_DEV_NEED_RESET) &&
			   !test_bit(DPC_RESET_HA, &ha->dpc_flags)) {
			if (IS_QLA8032(ha) ||
			    (IS_QLA8022(ha) && !ql4xdontresethba)) {
				ql4_printk(KERN_INFO, ha, "scsi%d: %s: "
					"HW State: NEED_RESET detected!\n",
					ha->host_no, __func__);
				set_bit(DPC_RESET_HA, &ha->dpc_flags);
				qla4xxx_wake_dpc(ha);
			}
		} else if ((dev_state == QLA8XXX_DEV_NEED_QUIESCENT) &&
			   !test_bit(DPC_HA_NEED_QUIESCENT, &ha->dpc_flags)) {
			ql4_printk(KERN_INFO, ha, "scsi%d: %s: "
					"HW State: NEED QUIESCENT "
					"detected\n", ha->host_no, __func__);
			set_bit(DPC_HA_NEED_QUIESCENT, &ha->dpc_flags);
			qla4xxx_wake_dpc(ha);
		} else if ((dev_state == QLA8XXX_DEV_QUIESCENT) &&
			   test_bit(DPC_RESET_QUIESCENT, &ha->dpc_flags)) {
			qla4xxx_wake_dpc(ha);
		} else {
			/* Check firmware health */
			if (qla4_8xxx_check_fw_alive(ha)) {
				halt_status = qla4_8xxx_rd_direct(ha,
						QLA8XXX_PEG_HALT_STATUS1);
				if (IS_QLA8022(ha)) {
					ql4_printk(KERN_INFO, ha, "scsi%d: %s: "
						"disabling pause transmit on "
						"port 0 and 1\n", ha->host_no,
						__func__);
					qla4_82xx_wr_32(ha,
						QLA82XX_CRB_NIU + 0x98,
						CRB_NIU_XG_PAUSE_CTL_P0 |
						CRB_NIU_XG_PAUSE_CTL_P1);
					ql4_printk(KERN_INFO, ha, "scsi%d: %s: "
						"Dumping hw/fw registers:\n "
						" PEG_HALT_STATUS1: 0x%x, "
						"PEG_HALT_STATUS2: 0x%x,\n "
						" PEG_NET_0_PC: 0x%x, "
						"PEG_NET_1_PC: 0x%x,\n "
						" PEG_NET_2_PC: 0x%x, "
						"PEG_NET_3_PC: 0x%x,\n "
						" PEG_NET_4_PC: 0x%x\n",
						ha->host_no, __func__,
						halt_status,
						qla4_8xxx_rd_direct(ha,
						QLA8XXX_PEG_HALT_STATUS2),
						qla4_82xx_rd_32(ha,
						QLA82XX_CRB_PEG_NET_0 + 0x3c),
						qla4_82xx_rd_32(ha,
						QLA82XX_CRB_PEG_NET_1 + 0x3c),
						qla4_82xx_rd_32(ha,
						QLA82XX_CRB_PEG_NET_2 + 0x3c),
						qla4_82xx_rd_32(ha,
						QLA82XX_CRB_PEG_NET_3 + 0x3c),
						qla4_82xx_rd_32(ha,
						QLA82XX_CRB_PEG_NET_4 + 0x3c));
					if (QLA82XX_FWERROR_CODE(halt_status) ==
						0x67)
						ql4_printk(KERN_ERR, ha,
							"scsi%d: %s: "
							"Firmware aborted with "
							"error code 0x00006700."
							" Device is being reset"
							"\n", ha->host_no,
							__func__);

					if (halt_status &
					    HALT_STATUS_UNRECOVERABLE)
						halt_status_unrecoverable = 1;
				} else if (IS_QLA8032(ha)) {
					ql4_printk(KERN_INFO, ha, "scsi(%d): "
						"%s, ISP8324 Dumping hw/fw "
						"registers:\n "
						" PEG_HALT_STATUS1: 0x%x, "
						"PEG_HALT_STATUS2: 0x%x,\n",
						ha->host_no, __func__,
						halt_status,
						qla4_8xxx_rd_direct(ha,
						QLA8XXX_PEG_HALT_STATUS2));

					if (halt_status &
					    QLA83XX_HALT_STATUS_UNRECOVERABLE)
						halt_status_unrecoverable = 1;
					else if (halt_status &
						 QLA83XX_HALT_STATUS_FW_RESET)
						ql4_printk(KERN_INFO, ha,
							"%s: Firmware Error "
							"detected device is "
							"being reset\n",
							__func__);
				}

				/* Since we cannot change dev_state in interrupt
				 * context, set appropriate DPC flag then
				 * wakeup DPC */
				if (halt_status_unrecoverable)
					set_bit(DPC_HA_UNRECOVERABLE,
						&ha->dpc_flags);
				else {
					if (dev_state ==
					    QLA8XXX_DEV_QUIESCENT) {
						set_bit(DPC_RESET_HA_FW_CONTEXT,
							&ha->dpc_flags);
						ql4_printk(KERN_INFO, ha,
							"scsi%d: %s: "
							"FW CONTEXT "
							"Reset needed!\n",
							ha->host_no, __func__);
					} else {
						ql4_printk(KERN_INFO, ha,
							"scsi%d: %s: detect HA "
							"Reset needed!\n",
							ha->host_no, __func__);
						set_bit(DPC_RESET_HA,
							&ha->dpc_flags);
						qla4xxx_mailbox_premature_completion(ha);
					}
				}

				qla4xxx_wake_dpc(ha);
			}
		}
	}
}

/**************************************************************************
 * qla4xxx_timer
 *      This routine is scheduled to be invoked every second to search for
 *      work to do.
 *
 * Input:
 *      p - Pointer to host adapter structure.
 *
 * Returns:
 *      None
 *
 * Context:
 *      Interrupt context.
 **************************************************************************/
void
qla4xxx_timer(unsigned long p)
{
	scsi_qla_host_t *ha = (scsi_qla_host_t *) p;
	ddb_entry_t *ddb_entry, *dtemp;
	int             start_dpc = 0;
	os_lun_t *lun_entry;
	unsigned long cpu_flags;
	int             t, l;

	/* If we are in the middle of AER/EEH processing
	 * skip any processing and reschedule the timer
	 */
	if (test_bit(AF_EEH_BUSY, &ha->flags)) {
		mod_timer(&ha->timer, jiffies + HZ);
		return;
	}

#if ISP_RESET_TEST
	if (ha->isp_reset_timer++ == (60 *3)) {
		printk("scsi%d: %s going to schedule BIG HAMMER\n",
		       ha->host_no, __func__);

		set_bit(DPC_RESET_HA, &ha->dpc_flags);
		ha->isp_reset_timer = 0;
	}
#endif

	DEBUG3(printk("scsi%d: %s: Host%d=%d/%d flags=[%lx,%lx,%lx] <%d,%d> "
	    "AEN in/out={%d/%d}, counters={%d,%d} isns=(%d,%d) %d\n",
	    ha->host_no, __func__, ha->instance,
	    ha->spurious_int_count, (uint32_t)ha->isr_count, ha->flags,
	    ha->dpc_flags, ha->isns.flags, ha->aborted_io_count,
	    ha->mailbox_timeout_count, ha->aen_in, ha->aen_out,
	    ha->retry_srb_q_count, ha->active_srb_count,
	    atomic_read(&ha->isns.state), atomic_read(&ha->isns.restart_timer),
	    ha->seconds_since_last_intr));

	/* Do we need to process the retry queue? */
	if (!list_empty(&ha->retry_srb_q)) {
		start_dpc++;
	}

	/* LUN suspension */
	for (t = 0; t < MAX_TARGETS; t++) {
		for (l = 0; l < MAX_LUNS ; l++) {
			lun_entry = GET_LU_Q(ha, t, l);
			if (lun_entry == NULL)
				continue;

			spin_lock_irqsave(&lun_entry->lun_lock, cpu_flags);
			if (lun_entry->lun_state != LS_LUN_SUSPENDED ||
			    !atomic_read(&lun_entry->suspend_timer)) {
				spin_unlock_irqrestore(&lun_entry->lun_lock,
				    cpu_flags);
				continue;
			}

			DEBUG2(printk("scsi%d: %s:"
			    "suspended lun_q - lun=%d, timer=%d "
			    "retry_count=%d\n", ha->host_no, __func__,
			    lun_entry->lun,
			    atomic_read(&lun_entry->suspend_timer),
			    lun_entry->retry_count));

			if (!atomic_dec_and_test(&lun_entry->suspend_timer)) {
				spin_unlock_irqrestore(&lun_entry->lun_lock,
				    cpu_flags);
				continue;
			}


			if (test_and_clear_bit(LF_LUN_DELAYED,
			    &lun_entry->flags)) {
				lun_entry->lun_state = LS_LUN_READY;
			} else {
				lun_entry->retry_count++;
				if (lun_entry->retry_count ==
				    lun_entry->max_retry_count) {
					DEBUG2(printk("scsi%d: %s: LUN "
					    "%d TIMEOUT RETRY_CNT:%d\n",
					    ha->host_no, __func__,
					    lun_entry->lun,
					    lun_entry->retry_count));

					lun_entry->lun_state = LS_LUN_TIMEOUT;
				} else {
					DEBUG2(printk("scsi%d: %s: LUN "
					    "%d RETRY\n", ha->host_no, __func__,
					    lun_entry->lun));

					lun_entry->lun_state = LS_LUN_RETRY;
				}
			}
			spin_unlock_irqrestore(&lun_entry->lun_lock, cpu_flags);
		}
	}

	if (IS_QLA80XX(ha))
		qla4_8xxx_watchdog(ha);

	/*
	 * Search for relogin's to time-out and port down retry.
	 */
	list_for_each_entry_safe(ddb_entry, dtemp, &ha->ddb_list, list_entry) {
		/* First check to see if the device has exhausted the
		 * port down retry count */
		if (atomic_read(&ddb_entry->state) == DEV_STATE_MISSING) {
			if (atomic_read(&ddb_entry->port_down_timer) == 0)
				continue;

			if (atomic_dec_and_test(&ddb_entry->port_down_timer)) {
				DEBUG2(printk("scsi%d: %s: index [%d] "
				    "port down retry count of (%d) secs "
				    "exhausted.\n",
				    ha->host_no, __func__,
				    ddb_entry->fw_ddb_index,
				    ha->port_down_retry_count);)

				qla4xxx_set_device_state(ha, ddb_entry, DEV_STATE_DEAD);
				start_dpc++;
			}
		}

		/* Count down time between sending relogins */
		if (ADAPTER_UP(ha) && (!test_bit(DF_RELOGIN, &ddb_entry->flags) &&
		    (atomic_read(&ddb_entry->state) != DEV_STATE_ONLINE))) {
			if (atomic_read(&ddb_entry->retry_relogin_timer) !=
			    INVALID_ENTRY) {
				if (atomic_read(&ddb_entry->retry_relogin_timer) == 0) {
					atomic_set(&ddb_entry->retry_relogin_timer, INVALID_ENTRY);
					set_bit(DF_RELOGIN,
					    &ddb_entry->flags);
					set_bit(DPC_RELOGIN_DEVICE,
					    &ha->dpc_flags);
					DEBUG2(printk("scsi%d:%d:%d: "
				    "%s: ddb [%d] login device\n", ha->host_no,
				    ddb_entry->bus, ddb_entry->target, __func__,
				    ddb_entry->fw_ddb_index);)
				} else
					atomic_dec(&ddb_entry->retry_relogin_timer);
			}
		}

		/* Wait for relogin to timeout */
		if (atomic_read(&ddb_entry->relogin_timer)  &&
		    (atomic_dec_and_test(&ddb_entry->relogin_timer) != 0)) {
			/*
			 * If the relogin times out and the device is
			 * still NOT ONLINE then try and relogin again.
			 */
			if (atomic_read(&ddb_entry->state) != DEV_STATE_ONLINE &&
			    ddb_entry->fw_ddb_device_state == DDB_DS_SESSION_FAILED &&
			    !test_bit(DF_NO_RELOGIN, &ddb_entry->flags)) {
				/* Reset retry relogin timer */
				atomic_inc(&ddb_entry->relogin_retry_count);
				QL4PRINT(QLP2, printk(
				    "scsi%d:%d:%d: ddb [%d] relogin timed "
				    "out-retrying relogin (%d)\n", ha->host_no,
				    ddb_entry->bus, ddb_entry->target,
				    ddb_entry->fw_ddb_index,
				    atomic_read(&ddb_entry->relogin_retry_count)));
				start_dpc++;
				QL4PRINT(QLP2, printk(
				    "scsi%d:%d:%d: ddb [%d] initate relogin "
				    "after %d seconds\n", ha->host_no,
				    ddb_entry->bus, ddb_entry->target,
				    ddb_entry->fw_ddb_index,
				    ddb_entry->default_time2wait+4));

				atomic_set(&ddb_entry->retry_relogin_timer,
				    ddb_entry->default_time2wait + 4);
			}
		}
	}

	if (!list_empty(&ha->done_srb_q)) {
		DEBUG2(printk("scsi%d: %s:"
			   "Pending done_q requests, "
			    "=%d\n", ha->host_no, __func__,
			    ha->done_srb_q_count));
		start_dpc++;
	}

	/*
	 * Check for heartbeat interval
	 */
	if (IS_QLA40XX(ha)) {
		if ((ha->firmware_options & FWOPT_HEARTBEAT_ENABLE) &&
		    (ha->heartbeat_interval != 0)) {
			ha->seconds_since_last_heartbeat ++;

			if (ha->seconds_since_last_heartbeat >
			    ha->heartbeat_interval+2) {
				QL4PRINT(QLP2, printk(
				    "scsi%d: Heartbeat not received for %d seconds. "
				    "HeartbeatInterval = %d seconds. Scheduling SOFT "
				    "RESET.\n", ha->host_no,
				    ha->seconds_since_last_heartbeat,
				    ha->heartbeat_interval));

				set_bit(DPC_RESET_HA, &ha->dpc_flags);
			}
		}
	}

	if (ADAPTER_UP(ha)) {
		/*
		 * Check for iSNS actions
		 */
		if (test_bit(ISNS_FLAG_ISNS_SRV_REGISTERED, &ha->isns.flags) &&
		    atomic_read(&ha->isns.esi_timer)) {
			if (atomic_dec_and_test(&ha->isns.esi_timer)) {
				/* Re-register with the  iSNS server if two times
				 * the esi interval has elapsed-- to prevent iSNS
				 * server from de-registering us. */
				ql4_printk(KERN_INFO, ha, "ESI timer expired. "
					"Re-Register with iSNS server\n");
				set_bit(DPC_ISNS_REREGISTER, &ha->dpc_flags);
			}
		}

		/* Decrement the restart timer. When it has elapsed, start the iSNS server */
		if ((atomic_read(&ha->isns.state) == ISNS_STATE_RESTART_SRV_WAIT) &&
		    (atomic_read(&ha->isns.restart_timer) != 0)) {
			if (atomic_dec_and_test(&ha->isns.restart_timer))
				set_bit(DPC_ISNS_START, &ha->dpc_flags);
		}

		if (test_bit(ISNS_FLAG_ISNS_ENABLED_IN_ISP, &ha->isns.flags) &&
		    !IPAddrIsZero(ha, ha->isns_server_ip_addr) &&
		    atomic_read(&ha->isns.state) == ISNS_STATE_TCP_DISCONNECTED &&
		    !test_bit(DPC_ISNS_RESTART, &ha->dpc_flags) &&
		    !test_bit(DPC_ISNS_START, &ha->dpc_flags)) {
			/* If iSNS is enabled in ISP, but no TCP connection
			 * with an iSNS server has been established,
			 * periodically poll for an iSNS server connection.*/
			ql4_isns_restart_timer(ha, ISNS_POLL_SVR_TOV);
		}
	}

	/* Wakeup the dpc routine for this adapter, if needed */
	if ((start_dpc ||
	     test_bit(DPC_RESET_HA, &ha->dpc_flags) ||
	     test_bit(DPC_RETRY_RESET_HA, &ha->dpc_flags) ||
	     test_bit(DPC_RELOGIN_DEVICE, &ha->dpc_flags) ||
	     test_bit(DPC_RESET_HA_FW_CONTEXT, &ha->dpc_flags) ||
	     test_bit(DPC_RESET_HA_INTR, &ha->dpc_flags) ||
	     test_bit(DPC_RESET_HA_INTR_COMPLETION, &ha->dpc_flags) ||
	     test_bit(DPC_IOCTL_ERROR_RECOVERY, &ha->dpc_flags) ||
	     test_bit(DPC_ISNS_RESTART, &ha->dpc_flags) ||
	     test_bit(DPC_ISNS_START, &ha->dpc_flags) ||
	     test_bit(DPC_ISNS_REREGISTER, &ha->dpc_flags) ||
	     test_bit(DPC_ISNS_STOP, &ha->dpc_flags) ||
	     test_bit(DPC_ISNS_DEREGISTER, &ha->dpc_flags) ||
	     test_bit(DPC_GET_DHCP_IP_ADDR, &ha->dpc_flags) ||
	     test_bit(DPC_ASYNC_PDU, &ha->dpc_flags) ||
	     test_bit(DPC_LUN_SCAN, &ha->dpc_flags) ||
	     test_bit(DPC_AEN, &ha->dpc_flags) ||
	     test_bit(DPC_HA_UNRECOVERABLE, &ha->dpc_flags) ||
	     test_bit(DPC_HA_NEED_QUIESCENT, &ha->dpc_flags) ||
	     test_bit(DPC_RESET_QUIESCENT, &ha->dpc_flags) ||
	     test_bit(DPC_DYNAMIC_LUN_SCAN, &ha->dpc_flags)) &&
	    !test_bit(AF_DPC_SCHEDULED, &ha->flags) &&
	    !ha->dpc_active && ha->dpc_wait) {
		qla4xxx_wake_dpc(ha);
	}

	/* Reschedule timer thread to call us back in one second */
	mod_timer(&ha->timer, jiffies + HZ);

	DEBUG2(ha->seconds_since_last_intr++;)
}

int IS_OTHER_PORT_RESETTING(scsi_qla_host_t *ha){
	scsi_qla_host_t *ha_listp;

	read_lock(&qla4xxx_hostlist_lock);
	list_for_each_entry(ha_listp, &qla4xxx_hostlist, list) {
		if (ha == ha_listp)
			continue;

		if ((pci_domain_nr(ha->pdev->bus) ==
		     pci_domain_nr(ha_listp->pdev->bus)) &&
		    (ha->pdev->bus->number ==
		     ha_listp->pdev->bus->number) &&
		    (PCI_SLOT(ha->pdev->devfn) ==
		     PCI_SLOT(ha_listp->pdev->devfn))) {

			if (test_bit(AF_INIT_ACTIVE, &ha_listp->flags)) {
				read_unlock(&qla4xxx_hostlist_lock);
				return (1);
			}
		}
	}
	read_unlock(&qla4xxx_hostlist_lock);
	return (0);
}

void qla4xxx_dynamic_lun_scan(scsi_qla_host_t *ha, struct device *dev, uint16_t target_id)
{
	#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,12)
	scsi_scan_target(dev, 0,
		target_id, SCAN_WILD_CARD, 0);
	#else
	QL4PRINT(QLP2,
		 printk("scsi%d: reported luns data has changed"
			"for SCSI target ID %d.  "
			"Execute ql-dynamic-tgt-lun-disc.sh "
			"via command line to probe luns.\n",
		ha->host_no, target_id));
	#endif
}

void qla4xxx_process_async_pdu_iocb(scsi_qla_host_t *ha,
				       async_pdu_iocb_t *apdu_iocb)
{
	ASYNC_PDU_ENTRY *apdu;
	ISCSI_PDU_HEADER *hdr;
	PASSTHRU0_ENTRY *pt_iocb;
        uint32_t len, offset;
	void *buff_addr;
	dma_addr_t buff_addr_dma;
	uint8_t using_prealloc = 1;

	apdu = &apdu_iocb->iocb;
	hdr = (ISCSI_PDU_HEADER *) apdu->iscsi_pdu_header;
	len = hdr->addl_hdr_seg_len + hdr->data_seg_len[2] +
		(hdr->data_seg_len[1]<<8) + (hdr->data_seg_len[0]<<16);

	offset = sizeof(PASSTHRU0_ENTRY) + sizeof(PASSTHRU_STATUS_ENTRY);
	if (len <= (PAGE_SIZE - offset)) {
		buff_addr_dma = ha->gen_req_rsp_iocb_dma + offset;
		buff_addr = (uint8_t *)ha->gen_req_rsp_iocb + offset;
	} else {
		using_prealloc = 0;
		buff_addr = pci_alloc_consistent(ha->pdev, len, &buff_addr_dma);
		if (buff_addr == NULL) {
			ql4_printk(KERN_WARNING, ha,
			    "Memory Allocation failed - APDU IOCB Mem.\n");
			return;
		}
	}

	pt_iocb = ha->gen_req_rsp_iocb;
	memset(pt_iocb, 0, offset);

	pt_iocb->hdr.entryType = ET_PASSTHRU0;
	pt_iocb->hdr.entryCount = 1;
	pt_iocb->async_pdu_handle = cpu_to_le32(apdu->handle);
	pt_iocb->handle = cpu_to_le32(apdu->handle);
	pt_iocb->target = cpu_to_le16(apdu->target_id);
	pt_iocb->controlFlags =
		cpu_to_le16(PT_FLAG_iSCSI_PDU | PT_FLAG_WAIT_4_RESPONSE);
	pt_iocb->timeout = cpu_to_le16(PT_DEFAULT_TIMEOUT);
	pt_iocb->inDataSeg64.base.addrHigh =
		cpu_to_le32(MSDW(buff_addr_dma));
	pt_iocb->inDataSeg64.base.addrLow =
		cpu_to_le32(LSDW(buff_addr_dma));
	pt_iocb->inDataSeg64.count = cpu_to_le16(len);

	if (qla4xxx_issue_iocb(ha, sizeof(PASSTHRU0_ENTRY),
			       ha->gen_req_rsp_iocb_dma) != QLA_SUCCESS) {
		QL4PRINT(QLP2, printk("scsi%d: %s: Issue IOCB failed\n",
			ha->host_no, __func__));
		goto exit_async_pdu_iocb;
	}

	QL4PRINT(QLP19,
		 printk("scsi%d: %s: PT IOCB (0x%p):\n",
			ha->host_no, __func__, pt_iocb));
	qla4xxx_dump_bytes(QLP19, pt_iocb, offset+len);

	switch (hdr->async_event) {
	case ISCSI_PDU_ASYNC_EVENT_SCSI_SENSE_DATA:
	{
		ASYNC_PDU_SENSE	*pdu_sense = buff_addr;

		if (pdu_sense->sense_data[12] == 0x3F) {
			if (pdu_sense->sense_data[13] == 0x0E) {
				/* reported luns data has changed */
				ddb_entry_t *ddb_entry;
				uint16_t fw_index = apdu->target_id;

				ddb_entry = qla4xxx_lookup_ddb_by_fw_index(ha, fw_index);
				if (ddb_entry == NULL) {
					QL4PRINT(QLP2,
						 printk("scsi%d: %s: No DDB entry for index [%d]\n",
						ha->host_no, __func__, fw_index));
					break;
				}
				if (ddb_entry->fw_ddb_device_state != DDB_DS_SESSION_ACTIVE) {
					QL4PRINT(QLP2,
						 printk("scsi%d: %s: No Active Session for index [%d]\n",
						ha->host_no, __func__, fw_index));
					break;
				}
				if (ddb_entry->target == (uint16_t) (-1)) {
					QL4PRINT(QLP2,
						 printk("scsi%d: %s: No valid Target ID for index [%d]\n",
						ha->host_no, __func__, fw_index));
					break;
				}

				/* report new lun to kernel */
				qla4xxx_dynamic_lun_scan(ha,
					&ha->host->shost_gendev,
					ddb_entry->target);
				/* report new lun to GUI */
				qla4xxx_queue_lun_change_aen(ha,
					ddb_entry->fw_ddb_index);
			}
		}


		break;
	}
	case ISCSI_PDU_ASYNC_EVENT_LOGOUT_REQ:
	case ISCSI_PDU_ASYNC_EVENT_CONNECTION_DROPPED:
	case ISCSI_PDU_ASYNC_EVENT_ALL_CONNECTIONS_DROPPED:
	case ISCSI_PDU_ASYNC_EVENT_PARAM_NEG_REQ:
		qla4xxx_conn_close_sess_logout(ha, apdu->target_id,0,0);
		break;
	default:
		QL4PRINT(QLP2, printk("scsi%d: %s: Async Event 0x%x"
				      " not processed\n",
                                      ha->host_no, __func__, hdr->async_event));
	}

exit_async_pdu_iocb:
	if (!using_prealloc)
		pci_free_consistent(ha->pdev, len, buff_addr, buff_addr_dma);

	return;
}

void qla4xxx_wake_dpc(struct scsi_qla_host *ha)
{
	if (!test_bit(AF_DPC_SCHEDULED, &ha->flags) &&
	    !ha->dpc_active && ha->dpc_wait) {
		DEBUG2(printk("scsi%d: %s: scheduling dpc "
				"routine - dpc flags = 0x%lx\n"
				, ha->host_no, __func__,
				ha->dpc_flags));
		set_bit(AF_DPC_SCHEDULED, &ha->flags);
		up(ha->dpc_wait);
	}
}

/**************************************************************************
 * qla4xxx_do_dpc
 *      This routine is a task that is schedule by the interrupt handler
 *      to perform the background processing for interrupts. We put it
 *      on a task queue that is consumed whenever the scheduler runs; that's
 *      so you can do anything (i.e. put the process to sleep etc).  In fact,
 *      the mid-level tries to sleep when it reaches the driver threshold
 *      "host->can_queue". This can cause a panic if we were in our interrupt
 *      code.
 *
 * Input:
 *      p - Pointer to host adapter structure.
 *
 * Returns:
 *      None
 *
 * Context:
 *      Kernel context.
 **************************************************************************/
int qla4xxx_do_dpc(void *data)
{
	DECLARE_MUTEX_LOCKED(sem);
	scsi_qla_host_t *ha = (scsi_qla_host_t *) data;
	fc_port_t       *fcport;

	lock_kernel();

	daemonize("qla4xxx_%d_dpc", ha->host_no);
	allow_signal(SIGHUP);

	ha->dpc_wait = &sem;

	set_user_nice(current, -20);

	unlock_kernel();

	complete(&ha->dpc_inited);

	while (1) {
		DEBUG2(printk("scsi%d: %s: DPC handler sleeping "
		    "*****************\n", ha->host_no, __func__));

		if (down_interruptible(&sem))
			break;

		if (ha->dpc_should_die)
			break;

		DEBUG2(printk("scsi%d: %s: DPC handler waking up "
		    "****************\n", ha->host_no, __func__));

		DEBUG2(printk("scsi%d: %s: ha->flags = 0x%08lx\n",
		    ha->host_no, __func__, ha->flags));
		DEBUG2(printk("scsi%d: %s: ha->dpc_flags = 0x%08lx\n",
		    ha->host_no, __func__, ha->dpc_flags));

		/* Initialization not yet finished. Don't do anything yet. */
		if (!test_bit(AF_INIT_DONE, &ha->flags) || ha->dpc_active)
			continue;

		if (test_bit(AF_EEH_BUSY, &ha->flags)) {
			DEBUG2(printk(KERN_INFO "scsi%d: %s: flags = 0x%lx\n",
				ha->host_no, __func__, ha->flags));
			continue;
		}

		if (IS_QLA80XX(ha)) {
			if (test_bit(DPC_HA_UNRECOVERABLE, &ha->dpc_flags)) {
				ha->isp_ops->idc_lock(ha);
				qla4_8xxx_wr_direct(ha, QLA8XXX_CRB_DEV_STATE,
						QLA8XXX_DEV_FAILED);
				ha->isp_ops->idc_unlock(ha);
				ql4_printk(KERN_INFO, ha,
					   "%s: HW State: FAILED\n", __func__);
				qla4_8xxx_device_state_handler(ha);
			}

			if (test_and_clear_bit(DPC_POST_IDC_ACK,
						&ha->dpc_flags))
				qla4_83xx_post_idc_ack(ha);

			if (test_bit(DPC_HA_NEED_QUIESCENT, &ha->dpc_flags)) {
				ha->isp_ops->idc_lock(ha);
				qla4_8xxx_wr_direct(ha, QLA8XXX_CRB_DEV_STATE,
					       QLA8XXX_DEV_NEED_QUIESCENT);
				ha->isp_ops->idc_unlock(ha);
				qla4_8xxx_device_state_handler(ha);

				scsi_unblock_requests(ha->host);
				clear_bit(DPC_HA_NEED_QUIESCENT, &ha->dpc_flags);

				/* Clear quiescent state of all functions except
				* quiesce owner quiescent state is cleared for owner
				* during reset qsnt
				*/
				if (!test_bit(AF_QUIESCE_OWNER, &ha->flags)) {
					ha->isp_ops->idc_lock(ha);
					qla4_8xxx_clear_qsnt_ready(ha);
					clear_bit(DPC_QUIESCE_ACTIVE, &ha->dpc_flags);
					ha->isp_ops->idc_unlock(ha);
				}
			}

			if (test_bit(DPC_RESET_QUIESCENT, &ha->dpc_flags)) {
				if (test_bit(AF_QUIESCE_OWNER, &ha->flags)) {
					ha->isp_ops->idc_lock(ha);
					qla4_8xxx_wr_direct(ha, QLA8XXX_CRB_DEV_STATE,
							QLA8XXX_DEV_READY);
					qla4_8xxx_clear_qsnt_ready(ha);
					clear_bit(DPC_QUIESCE_ACTIVE, &ha->dpc_flags);
					ha->isp_ops->idc_lock(ha);
				}
				clear_bit(DPC_RESET_QUIESCENT, &ha->dpc_flags);
			}
		}

		ha->dpc_active = 1;
		clear_bit(AF_DPC_SCHEDULED, &ha->flags);

		if (!list_empty(&ha->done_srb_q))
			qla4xxx_done(ha);

		/* ---- return cmds on retry_q? --- */
		if (!list_empty(&ha->retry_srb_q)) {
			srb_t    *srb, *stemp;
			unsigned long flags;

			spin_lock_irqsave(&ha->list_lock, flags);

			DEBUG2(printk("scsi%d: %s: found %d srbs in "
			    "retry_srb_q \n", ha->host_no, __func__,
			    ha->retry_srb_q_count));

			list_for_each_entry_safe(srb, stemp, &ha->retry_srb_q,
			    list_entry) {
				ddb_entry_t *ddb_entry;
				os_lun_t *lun_entry;
				struct scsi_cmnd  *cmd;

				lun_entry = srb->lun_queue;

				if (lun_entry == NULL)
					continue;

				if (lun_entry->lun_state ==
				    LS_LUN_SUSPENDED)
					continue;
				fcport = lun_entry->fclun->fcport;
				ddb_entry = fcport->ddbptr;

				if (ddb_entry == NULL)
					continue;

				cmd = srb->cmd;
				if (cmd == NULL)
					continue;

				DEBUG2(printk(
				    "scsi%d:%d:%d:%d %s: "
				    "found srb=%p cmd=%p pid=%ld "
				    "in retry_srb_q ",
				    ha->host_no,
				    cmd->device->channel,
				    cmd->device->id,
				    cmd->device->lun,
				    __func__, srb, cmd,
				    cmd->serial_number);)

				if (atomic_read(&ddb_entry->state) ==
						DEV_STATE_DEAD ||
				    atomic_read(&ddb_entry->state) ==
						DEV_STATE_DELETED) {
					DEBUG2(printk("Device DEAD or DELETED, "
					    "returning\n"));

					__del_from_retry_srb_q(ha, srb);
					srb->cmd->result = DID_NO_CONNECT << 16;
					__add_to_done_srb_q(ha,srb);

				} else if (ADAPTER_UP(ha) &&
				    (atomic_read(&ddb_entry->state)
				    == DEV_STATE_ONLINE)) {
					/*
					 * Send requests to OS when device goes ONLINE
					 * so that the OS will retry them via I/O thread.
					 * We don't want to issue I/O via recovery thread.
					 */

					DEBUG2(printk("Device ONLINE, "
						"returning\n"));
					__del_from_retry_srb_q(ha, srb);
					if (host_byte(srb->cmd->result) !=
					    DID_NO_CONNECT) {
						srb->cmd->result =
						    DID_IMM_RETRY << 16;
					}
						__add_to_done_srb_q(ha, srb);
				} else {
					DEBUG2(printk("srb_state=%d\n",
					    srb->state);)
				}
			}
			spin_unlock_irqrestore(&ha->list_lock, flags);

			if (!list_empty(&ha->done_srb_q))
				qla4xxx_done(ha);

		}

		/* ---- Async PDU IOCB ? --- */
		if (ADAPTER_UP(ha) &&
		    test_bit(DPC_ASYNC_PDU, &ha->dpc_flags)) {
			struct list_head *list, *temp;
			async_pdu_iocb_t *apdu_iocb;

			list_for_each_safe(list, temp, &ha->async_iocb_list) {
				apdu_iocb = list_entry(list, async_pdu_iocb_t,
						       list_entry);

				qla4xxx_process_async_pdu_iocb(ha, apdu_iocb);
				list_del_init(&apdu_iocb->list_entry);
				vfree(apdu_iocb);
			}
			clear_bit(DPC_ASYNC_PDU, &ha->dpc_flags);
		}

		/*
		 * Determine what action is necessary
		 */

		/* ---- recover adapter? --- */
		if (!test_bit(DPC_RESET_ACTIVE, &ha->dpc_flags) &&
		    (test_bit(DPC_RESET_HA, &ha->dpc_flags) ||
		     test_bit(DPC_RESET_HA_INTR, &ha->dpc_flags) ||
		     test_bit(DPC_RESET_HA_INTR_COMPLETION, &ha->dpc_flags) ||
		     test_bit(DPC_RESET_HA_FW_CONTEXT, &ha->dpc_flags))) {

			if (IS_QLA8022(ha) && ql4xdontresethba) {
				QL4PRINT(QLP2, printk("scsi: %s: ignoring RESET_HA, "
				    "rebootdisable=1 \n", __func__));
				clear_bit(DPC_RESET_HA, &ha->dpc_flags);
				clear_bit(DPC_RESET_HA_INTR, &ha->dpc_flags);
				clear_bit(DPC_RESET_HA_FW_CONTEXT, &ha->dpc_flags);
				goto dpc_post_reset_ha;
			}

			/* ISP-4xxx Specific */
			if (test_bit(DPC_RESET_HA_INTR, &ha->dpc_flags)) {

				uint8_t wait_time = RESET_INTR_TOV*2;

				qla4xxx_abort_active_cmds(ha, DID_RESET << 16);

				while ((RD_REG_DWORD(&ha->reg->ctrl_status) &
                                        CSR_SOFT_RESET) != 0) {
					if (IS_QLA40XX(ha) &&
					   (qla4xxx_poll_and_ack_scsi_reset(ha) == QLA_SUCCESS)) {
						wait_time = RESET_INTR_TOV*2;
					}

					if (--wait_time == 0)
						break;

					msleep(1000);
				}

				if (wait_time == 0) {
					QL4PRINT(QLP2,
						 printk("scsi%d: %s: SR|FSR "
							"bit not cleared-- "
							"resetting\n",
							ha->host_no,
							__func__));

					set_bit(DPC_RESET_HA, &ha->dpc_flags);
				}
				else if (!test_bit(AF_HA_REMOVAL,
							&ha->flags)) {
					set_bit(DPC_RESET_HA_INTR_COMPLETION,
							&ha->dpc_flags);
					clear_bit(DPC_RESET_HA_INTR,
						&ha->dpc_flags);
					clear_bit(DPC_RESET_HA, &ha->dpc_flags);
					clear_bit(DPC_RETRY_RESET_HA,
						&ha->dpc_flags);
					clear_bit(DPC_RESET_HA_FW_CONTEXT,
						&ha->dpc_flags);
				}
			}

			if (!test_bit(AF_HA_REMOVAL, &ha->flags) &&
			    test_bit(DPC_RESET_HA_INTR_COMPLETION,
							&ha->dpc_flags)) {
				if (IS_QLA40XX(ha)) {
					if (IS_OTHER_PORT_RESETTING(ha)) {
						DEBUG2(printk("scsi%d: %s - Other port"
							" is resetting.Retry later..\n"
							, ha->host_no,__func__));
					} else {
						if (qla4xxx_lock_drvr_wait(ha) ==
							QLA_SUCCESS) {
							clear_bit(DPC_RESET_HA_INTR_COMPLETION,
								&ha->dpc_flags);
							qla4xxx_process_aen(ha,
								FLUSH_DDB_CHANGED_AENS);
							qla4xxx_recover_adapter(ha,
								PRESERVE_DDB_LIST);
						}
					}
				}
			}
			if (!test_bit(DPC_RESET_ACTIVE, &ha->dpc_flags) &&
			    (test_bit(DPC_RESET_HA_FW_CONTEXT, &ha->dpc_flags) ||
			    test_bit(DPC_RESET_HA, &ha->dpc_flags))) {
				if (IS_QLA8022(ha) && ql4xdontresethba) {
					DEBUG2(printk("scsi%d: %s: Don't Reset HBA\n",
						ha->host_no, __func__));
					goto dpc_post_reset_ha;
				}
				if (IS_QLA80XX(ha)) {
					qla4xxx_recover_adapter(ha,
							PRESERVE_DDB_LIST);
				} else {
					if (IS_OTHER_PORT_RESETTING(ha)) {
						DEBUG2(printk("scsi%d: %s - Other port"
							"is resetting... "
							"Reattempt later..\n",
							ha->host_no,__func__));
						/* NOTE: If the other port is
						 * resetting, DPC_RESET_HA_INTR may
						 * clear out DPC_RESET */
					} else {
						qla4xxx_recover_adapter(ha,
								PRESERVE_DDB_LIST);
					}
				}
				
			}
		}

dpc_post_reset_ha:
		/* ---- process AEN? --- */
		if (test_and_clear_bit(DPC_AEN, &ha->dpc_flags))
			qla4xxx_process_aen(ha, PROCESS_ALL_AENS);

		/* ---- Get DHCP IP Address? --- */
		if (test_and_clear_bit(DPC_GET_DHCP_IP_ADDR, &ha->dpc_flags))
			qla4xxx_get_dhcp_ip_address(ha);

		if (!ADAPTER_UP(ha))
		    goto exit_dpc;

		/* ---- relogin device? --- */
		if (test_and_clear_bit(DPC_RELOGIN_DEVICE, &ha->dpc_flags)) {
		ddb_entry_t *ddb_entry, *dtemp;

			list_for_each_entry_safe(ddb_entry, dtemp,
			&ha->ddb_list, list_entry) {
				if (test_and_clear_bit(DF_RELOGIN,
				    &ddb_entry->flags) &&
				    atomic_read(&ddb_entry->state)
				    != DEV_STATE_ONLINE) {
					qla4xxx_relogin_device(ha, ddb_entry);
				}
				/* If mbx cmd times out there is no point
				 * in continuing further.
				 * With large no of targets this can hang
				 * the system.
				 */
				if (test_bit(DPC_RESET_HA, &ha->dpc_flags)) {
				    QL4PRINT(QLP2, printk("scsi%d: %s: "
					"need to reset hba\n",
					ha->host_no, __func__));
				    break;
				}
			}
		}

		/* ---- dynamic lun scan? --- */
		if ((test_bit(AF_INTERRUPTS_ON, &ha->flags)) &&
		   (test_and_clear_bit(DPC_LUN_SCAN, &ha->dpc_flags) ||
		    test_and_clear_bit(DPC_DYNAMIC_LUN_SCAN, &ha->dpc_flags))) {
			ddb_entry_t *ddb_entry, *dtemp;
			list_for_each_entry_safe(ddb_entry, dtemp,
			&ha->ddb_list, list_entry) {
				if ((test_and_clear_bit(DF_LUN_SCAN,
					&ddb_entry->flags)  ||
				    test_and_clear_bit(DF_DYNAMIC_LUN_SCAN_NEEDED,
					&ddb_entry->flags)) &&
				    ddb_entry->fw_ddb_device_state ==
					DDB_DS_SESSION_ACTIVE &&
				    ddb_entry->target != (uint16_t) (-1)) {
					DEBUG2(printk("scsi%d:%d:%d: "
					    "%s: index [%d] perform"
					    " dynamic lun scan\n",
					    ha->host_no, ddb_entry->bus,
					    ddb_entry->target, __func__,
					    ddb_entry->fw_ddb_index);)
					/* report new lun to kernel */
					qla4xxx_dynamic_lun_scan(ha,
						&ha->host->shost_gendev,
						ddb_entry->target);
					/* report new lun to GUI */
					qla4xxx_queue_lun_change_aen(ha,
						ddb_entry->fw_ddb_index);
				}
			}
		}

		/* ---- restart iSNS server? --- */
		if (ADAPTER_UP(ha)) {
			/* Error Recovery Case:
			 * The driver detected some sort of iSNS error.
			 * Stop the TCP connection then reschedule */
			if (test_and_clear_bit(DPC_ISNS_RESTART, &ha->dpc_flags))
				ql4_isns_restart_service(ha);

			/* IP Address Change Case:
			 * If the initiator's IP address changes, we
			 * stop the TCP connection using the old source IP address. */
			if (test_and_clear_bit(DPC_ISNS_STOP, &ha->dpc_flags))
				ql4_isns_stop_service(ha);

			/* AEN 8021 iSNS Service Connection FAILED Case:
			 * We already have a TCP connection to iSNS server, but the
			 * iSNS Server closes the connection, so de-register.
			 * A future attempt will be made to re-register with iSNS
			 * server after the ISNS_STATE_RESTART_SRV_WAIT wait-time
			 * has expired. */
			if (test_and_clear_bit(DPC_ISNS_DEREGISTER, &ha->dpc_flags)) {
				ql4_isns_deregister_isns_server(ha);
			}

			/* ESI Timeout Case:
			 * We already have a TCP connection to iSNS server.
			 * De-register then attempt to re-register with iSNS server. */
			if (test_bit(DPC_ISNS_REREGISTER, &ha->dpc_flags) &&
			    ql4_is_isns_active(ha) == ISNS_STATUS_ACTIVE ) {
				ql4_isns_deregister_isns_server(ha);
			}

			/* No Current TCP Connection --
			 * Polling, IP Address Change and Rescheduled Start Cases: */
			if (test_and_clear_bit(DPC_ISNS_START, &ha->dpc_flags))
				ql4_isns_start_service(ha);
		}

exit_dpc:
		ha->dpc_active = 0;
	}

	/*
	* Make sure that nobody tries to wake us up again.
	*/
	ha->dpc_wait = NULL;
	ha->dpc_active = 0;

	complete_and_exit(&ha->dpc_exited, 0);
}

int qla4xxx_eh_wait_for_sp_put(scsi_qla_host_t *ha, srb_t *srb)
{
	unsigned long wait_cnt = jiffies + (EH_WAIT_CMD_TOV * HZ);

	while (time_before(jiffies, wait_cnt)) {
		if (atomic_read(&srb->ref_count) == 0) {
			DEBUG2(printk("%s: EXIT found cmd=%p\n", __func__, srb->cmd));
			return (1);
		}

		set_current_state(TASK_INTERRUPTIBLE);
		schedule_timeout(HZ);
	}

	DEBUG2(printk("%s: EXIT not found cmd=%p\n", __func__, srb->cmd));
	return (0);
}

/**************************************************************************
 * qla4xxx_eh_wait_on_command
 *      This routine waits for the command to be returned by the Firmware
 *      for some max time.
 *
 * Input:
 *    ha = actual ha whose done queue will contain the command
 *            returned by firmware.
 *    cmd = Scsi Command to wait on.
 *
 * Returns:
 *    Not Found : 0
 *    Found : 1
 *
 * Context:
 *      Kernel context.
 **************************************************************************/
int qla4xxx_eh_wait_on_command(scsi_qla_host_t *ha, struct scsi_cmnd *cmd)
{
#define ABORT_POLLING_PERIOD      HZ
#define ABORT_WAIT_TIME		((10 * HZ) / (ABORT_POLLING_PERIOD))

	int		found = 0;
	int 		done = 0;
	srb_t 		*rp = NULL;
	struct list_head *list, *temp;
	uint32_t max_wait_time = ABORT_WAIT_TIME;
	unsigned long	flags;
	int ret = SUCCESS;

	DEBUG2(printk("%s: ENTER,  cmd=%p\n", __func__, cmd);)

	/* Dont wait on command if PCI error is being handled
	 * by PCI AER driver
	 */
	if ((test_bit(AF_EEH_BUSY, &ha->flags))) {
		ql4_printk(KERN_WARNING, ha, "scsi%d: Return from %s\n",
				ha->host_no, __func__);
		return ret;
	}

	do {
		/* Check on done queue */
		spin_lock_irqsave(&ha->list_lock, flags);
		list_for_each_safe(list, temp, &ha->done_srb_q) {
			rp = list_entry(list, srb_t, list_entry);

			/*
			 * Found command. Just exit and wait for the cmd sent
			 * to OS.
			*/
			if (cmd == rp->cmd) {
				DEBUG2(printk("%s: found in done queue.\n",
				    __func__);)
				__del_from_done_srb_q(ha, rp);
				found++;
				break;
			}
		}
		spin_unlock_irqrestore(&ha->list_lock, flags);

		/* Checking to see if its returned to OS */
		rp = (srb_t *) CMD_SP(cmd);
		if (rp == NULL ) {
			done++;
			break;
		}

		/* Complete the cmd right away. */
		if (found) {
			sp_put(ha, rp);
			done++;
			break;
		}

		set_current_state(TASK_UNINTERRUPTIBLE);
		schedule_timeout(ABORT_POLLING_PERIOD);

	} while (max_wait_time--);

	DEBUG2(printk("%s: EXIT done=%d found=%d cmd=%p\n",
		      __func__, done, found, cmd));
	return done;
}

/**************************************************************************
 * qla4xxx_wait_for_hba_online
 *      This routine
 *
 * Input:
 *      ha - Pointer to host adapter structure
 *
 * Remarks:
 *
 * Returns:
 *      SUCCESS - Adapter is ONLINE
 *      FAILED  - Adapter is DEAD
 *
 * Context:
 *      Kernel context.  Assume io_request_lock LOCKED upon entry
 **************************************************************************/
inline uint8_t
qla4xxx_wait_for_hba_online(scsi_qla_host_t *ha)
{
	unsigned long wait_online;

	wait_online = jiffies + (ADAPTER_ONLINE_TOV * HZ);
	while (time_before(jiffies, wait_online)) {
		if ((test_bit(AF_ONLINE, &ha->flags) != 0))
			return QLA_SUCCESS;

		QL4PRINT(QLP3, printk("scsi%d: %s: adapter down, "
		    "delay 2 sec.\n", ha->host_no,
		    __func__));

		set_current_state(TASK_UNINTERRUPTIBLE);
		schedule_timeout(2 * HZ);
	}

	return QLA_ERROR;
}

inline uint8_t
qla4xxx_wait_for_link_up(scsi_qla_host_t *ha)
{
	unsigned long wait_online;

	wait_online = jiffies + (ADAPTER_ONLINE_TOV * HZ);
	while (time_before(jiffies, wait_online)) {
		if (ADAPTER_UP(ha))
			return QLA_SUCCESS;

		QL4PRINT(QLP3, printk("scsi%d: %s: link down, "
				"delay 2 sec.\n", ha->host_no,
				__func__));

		set_current_state(TASK_UNINTERRUPTIBLE);
		schedule_timeout(2 * HZ);
	}

	return QLA_ERROR;
}

/***********************************************************
 * qla4xxx_wait_for_reset_ready
 *      This routine waits until adapter reset is complete
 *
 * Input:
 *      ha - Pointer to host adapter structure
 *
 * Remarks:
 *
 * Returns:
 *      QLA_SUCCESS	- Adapter is ONLINE
 *      QLA_ERROR	- Adapter is DEAD
 ***********************************************************/
inline uint8_t
qla4xxx_wait_for_reset_ready(scsi_qla_host_t *ha)
{
	unsigned long wait_online = 0;

	wait_online = jiffies + (ADAPTER_ONLINE_TOV * HZ);
	while (time_before(jiffies, wait_online) &&
		test_bit(DPC_RESET_HA, &ha->dpc_flags)) {
		ql4_printk(KERN_INFO, ha, "Waiting. Adapter reset is "
				"in progress.\n");
		msleep(1000);
	}

	if (test_bit(AF_ONLINE, &ha->flags))
		return QLA_SUCCESS;
	else
		return QLA_ERROR;
}

/**************************************************************************
 * qla4xxx_eh_abort
 *      This routine aborts commands that currently held in the adapter's
 *      internal queues.  Commands that are active are NOT aborted.
 *
 * Input:
 *      cmd - Pointer to Linux's SCSI command structure
 *
 * Remarks:
 *      Aborts get translated to "device resets" by the scsi switch
 *      which will return a RESET status and not ABORT. Since the
 *      mid-level is expecting an ABORT status during an abort(),
 *      we always elevate to device reset.
 *
 * Returns:
 *      SUCCESS - Successfully aborted non-active command
 *      FAILED  - Command not found, or command currently active
 *
 * Context:
 *      Kernel context.  io_request_lock LOCKED
 **************************************************************************/
int
qla4xxx_eh_abort(struct scsi_cmnd *cmd)
{
	int return_status = FAILED;
	scsi_qla_host_t *ha;
	srb_t *srb;
	srb_t *stemp;
	unsigned long	flags;
	uint8_t		found = 0;

	srb = (srb_t *) CMD_SP(cmd);
	if (!srb) {
		/* Already returned to upper-layers. */
		ql4_printk(KERN_INFO, to_qla_host(cmd->device->host),
		    "Command already completed cmd=%p, pid=%ld.\n",
			   cmd, cmd->serial_number);
		DEBUG2(printk("%s: Command already completed cmd=%p, pid=%ld.\n",
			   __func__, cmd, cmd->serial_number);)
		return SUCCESS;
	}

	ha = (scsi_qla_host_t *) cmd->device->host->hostdata;

	ha->aborted_io_count++;

	/* Print statements
	 * ---------------- */
	QL4PRINT(QLP2, printk(
	    "scsi%d:%d:%d:%d: abort srb=%p, cmd=%p, state=%s, r_start=%ld , "
	    "u_start=%ld\n", ha->host_no, cmd->device->channel,
	    cmd->device->id, cmd->device->lun, srb, cmd,
	    srb_state_msg[srb->state], srb->r_start, srb->u_start));
	DEBUG2(printk("scsi%d: %s: os_tov = %d, iocb_tov = %d, pid=%ld, "
	    "ref_count = %d\n", ha->host_no, __func__,
	    srb->os_tov, srb->iocb_tov, cmd->serial_number,
	    atomic_read(&srb->ref_count));)

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,12)
	spin_unlock_irq(ha->host->host_lock);
#endif

	if (qla4xxx_wait_for_hba_online(ha) != QLA_SUCCESS) {
		DEBUG2(printk("scsi%d: %s: Board Disabled\n",
			     ha->host_no, __func__));
		goto eh_abort_complete;
	}

	if (qla4xxx_wait_for_link_up(ha) != QLA_SUCCESS) {
		DEBUG2(printk("scsi%d: %s: Link Down\n",
				ha->host_no, __func__));
		goto eh_abort_complete;
	}

	/* If srb found in done_q, return the cmd with existing status */
	spin_lock_irqsave(&ha->list_lock, flags);
	list_for_each_entry_safe(srb, stemp, &ha->done_srb_q, list_entry) {
		if (srb->cmd != cmd)
			continue;

		QL4PRINT(QLP2, printk("scsi%d: %s: srb %p found on done "
		    "queue\n", ha->host_no, __func__, srb));

		__del_from_done_srb_q(ha, srb);
		return_status = SUCCESS;
		found++;
		break;
	}
	spin_unlock_irqrestore(&ha->list_lock, flags);

	if (found) {
		sp_put(ha, srb);
		goto eh_abort_complete;
	}

	/*
         * If srb found in retry_q, return the cmd with ABORTED status
         */
	spin_lock_irqsave(&ha->list_lock, flags);
	list_for_each_entry_safe(srb, stemp, &ha->retry_srb_q, list_entry) {
		if (srb->cmd != cmd)
			continue;

		QL4PRINT(QLP2,
			 printk("scsi%d: %s: srb %p found on retry queue\n",
				ha->host_no, __func__, srb));

		__del_from_retry_srb_q(ha, srb);
		found++;
		break;
	}
	spin_unlock_irqrestore(&ha->list_lock, flags);

	if (found) {
		qla4xxx_delete_timer_from_cmd(srb);
		sp_put(ha, srb);

		if (atomic_read(&srb->ref_count) != 0) {
			/* Possibly another thread hasn't completed processing
			 * os_cmd_timeout, so yield control to the OS to
			 * give that thread a chance to complete */
			if (!qla4xxx_eh_wait_for_sp_put(ha, srb)) {
				return_status = FAILED;
				goto eh_abort_complete;
			}
		}
		cmd->result = DID_ABORT << 16;
		return_status = SUCCESS;
		goto eh_abort_complete;
	}

	if (qla4xxx_eh_wait_on_command(ha, cmd)) {
		QL4PRINT(QLP2, printk("scsi%d: %s: cmd returned back to OS\n",
		    ha->host_no, __func__));
		return_status =  SUCCESS;
	}

	/*
	 * Aborts get translated to "device resets" by the scsi switch which
	 * will return a RESET status and not ABORT. Since the mid-level is
	 * expecting an ABORT status during an abort(), we always elevate to
	 * device reset.
	 */

eh_abort_complete:
	QL4PRINT(QLP2, printk("scsi%d: %s: return with status = %x\n",
	    ha->host_no, __func__, return_status));
	QL4PRINT(QLP2, printk("scsi%d: %s: cmd=%p cmd_err_flags=0x%x\n",
			      ha->host_no, __func__, cmd, cmd->eh_eflags));
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,12)
	spin_lock_irq(ha->host->host_lock);
#endif
	return return_status;
}

int qla4xxx_hw_reset(scsi_qla_host_t *ha) {
 	uint32_t ctrl_status;
	unsigned long flags = 0;

	QL4PRINT(QLP2,
		 printk(KERN_ERR "scsi%d: SOFT RESET issued\n", ha->host_no));

	if (qla4xxx_lock_drvr_wait(ha) != QLA_SUCCESS)
		return QLA_ERROR;

	/*
	 * If the SCSI Reset Interrupt bit is set, clear it.
	 * Otherwise, the Soft Reset won't work.
	 */
	spin_lock_irqsave(&ha->hardware_lock, flags);
	ctrl_status = RD_REG_DWORD(&ha->reg->ctrl_status);
	if ((ctrl_status & CSR_SCSI_RESET_INTR) != 0)
		WRT_REG_DWORD(&ha->reg->ctrl_status, SET_RMASK(CSR_SCSI_RESET_INTR));

	/* Issue Soft Reset */
	WRT_REG_DWORD(&ha->reg->ctrl_status, SET_RMASK(CSR_SOFT_RESET));
	PCI_POSTING(&ha->reg->ctrl_status);

	spin_unlock_irqrestore(&ha->hardware_lock, flags);

	return QLA_SUCCESS;
}

/**************************************************************************
 * qla4xxx_soft_reset
 *      This routine performs a SOFT RESET.
 *
 * Input:
 *      ha - Pointer to host adapter structure.
 *
 * Prerequisite:
 *	Make sure that the DRVR semaphore is taken prior to calling this
 *	routine.  It's neccessary for the coexistance of the qla3xxx and
 *	qla4xxx cards.
 *
 * Returns:
 *      QLA_SUCCESS - Successfully reset the firmware
 *      QLA_ERROR   - Failed to reset the firmware
 *
 * Context:
 *      Kernel context.
 **************************************************************************/
int
qla4xxx_soft_reset(scsi_qla_host_t *ha){
	uint32_t  max_wait_time;
	unsigned long flags = 0;
	uint8_t status = QLA_ERROR;
	uint32_t ctrl_status;

	status = qla4xxx_hw_reset(ha);
	if (status != QLA_SUCCESS)
		return QLA_ERROR;

	/* Wait until the Network Reset Intr bit is cleared */
	max_wait_time = RESET_INTR_TOV;
	do {
		spin_lock_irqsave(&ha->hardware_lock, flags);
		ctrl_status = RD_REG_DWORD(&ha->reg->ctrl_status);
		spin_unlock_irqrestore(&ha->hardware_lock, flags);

		if ((ctrl_status & CSR_NET_RESET_INTR) == 0)
			break;

		msleep(1000);
	} while ((--max_wait_time));

	spin_lock_irqsave(&ha->hardware_lock, flags);
	ctrl_status = RD_REG_DWORD(&ha->reg->ctrl_status);
	if ((ctrl_status & CSR_NET_RESET_INTR) != 0) {
		QL4PRINT(QLP2,
			 printk(KERN_WARNING "scsi%d: Network Reset Intr not cleared "
				"by Network function, clearing it now!\n", ha->host_no));
		WRT_REG_DWORD(&ha->reg->ctrl_status, SET_RMASK(CSR_NET_RESET_INTR));
		PCI_POSTING(&ha->reg->ctrl_status);
	}
	spin_unlock_irqrestore(&ha->hardware_lock, flags);

	/* Wait until the firmware tells us the Soft Reset is done */
	max_wait_time = SOFT_RESET_TOV;
	do {
		spin_lock_irqsave(&ha->hardware_lock, flags);
		ctrl_status = RD_REG_DWORD(&ha->reg->ctrl_status);
		spin_unlock_irqrestore(&ha->hardware_lock, flags);

		if ((ctrl_status & CSR_SOFT_RESET) == 0) {
			status = QLA_SUCCESS;
			break;
		}

		msleep(1000);
	} while ((--max_wait_time));

	/*
	 * Also, make sure that the SCSI Reset Interrupt bit has been cleared
	 * after the soft reset has taken place.
	 */
	spin_lock_irqsave(&ha->hardware_lock, flags);
	ctrl_status = RD_REG_DWORD(&ha->reg->ctrl_status);
	if ((ctrl_status & CSR_SCSI_RESET_INTR) != 0) {
		WRT_REG_DWORD(&ha->reg->ctrl_status, SET_RMASK(CSR_SCSI_RESET_INTR));
		PCI_POSTING(&ha->reg->ctrl_status);
	}
	spin_unlock_irqrestore(&ha->hardware_lock, flags);

	/* If soft reset fails then most probably the bios on other
	 * function is also enabled.
	 * Since the initialization is sequential the other fn
	 * wont be able to acknowledge the soft reset.
	 * Issue a force soft reset to workaround this scenario.
	 */
	if (max_wait_time == 0) {
		QL4PRINT(QLP2, printk(KERN_ERR "scsi%d: FORCED SOFT RESET issued\n",
				      ha->host_no));

		/* Issue Force Soft Reset */
		spin_lock_irqsave(&ha->hardware_lock, flags);
		WRT_REG_DWORD(&ha->reg->ctrl_status,
			SET_RMASK(CSR_FORCE_SOFT_RESET));
		PCI_POSTING(&ha->reg->ctrl_status);
		spin_unlock_irqrestore(&ha->hardware_lock, flags);

		/* Wait until the firmware tells us the Soft Reset is done */
		max_wait_time = SOFT_RESET_TOV;
		do {
			spin_lock_irqsave(&ha->hardware_lock, flags);
			ctrl_status = RD_REG_DWORD(&ha->reg->ctrl_status);
			spin_unlock_irqrestore(&ha->hardware_lock, flags);

			if ((ctrl_status & CSR_SOFT_RESET) == 0) {
				status = QLA_SUCCESS;
				break;
			}

			msleep(1000);
		} while ((--max_wait_time));
	}

	QL4PRINT(QLP2, printk("scsi%d: %s status=%d\n",
			      ha->host_no, __func__, status));
	return status;
}

/**************************************************************************
 * qla4xxx_cmd_wait
 *      This routine stalls the driver until all outstanding commands are
 *      returned.
 *
 * Input:
 *      ha - Pointer to host adapter structure.
 *
 * Remarks:
 *       Caller must release the Hardware Lock prior to calling this routine.
 *
 * Returns:
 *      QLA_SUCCESS - All outstanding commands completed
 *      QLA_ERROR   - All outstanding commands did not complete
 *
 * Context:
 *      Kernel context.
 **************************************************************************/
uint8_t qla4xxx_cmd_wait(scsi_qla_host_t *ha){
	uint32_t index;
	unsigned long flags;

	/* Initialized for 30 seconds as we expect all commands
	 * to be returned ASAP.
	 */
	unsigned long wtime = jiffies + (WAIT_CMD_TOV * HZ);

	DEBUG2(ql4_printk(KERN_INFO, ha, "Wait up to %d seconds for cmds "
				"to complete\n", WAIT_CMD_TOV));
	while (!time_after_eq(jiffies, wtime)) {
		/* Find a command that hasn't completed. */
		for (index = 0; index < MAX_SRBS; index++) {
			spin_lock_irqsave(&ha->hardware_lock, flags);
			if (ha->active_srb_array[index] != NULL) {
				spin_unlock_irqrestore(&ha->hardware_lock, flags);
				QL4PRINT(QLP2,printk("scsi%d: %s: Done waiting "
					 "on commands - array_index=%d\n",
					 ha->host_no, __func__, index));
				break;
			}
			spin_unlock_irqrestore(&ha->hardware_lock, flags);
		}

		/* If No Commands are pending, wait is complete */
		if (index == MAX_SRBS) {
			QL4PRINT(QLP2,printk("scsi%d: %s: Done waiting "
				 "on commands - array_index=%d\n",
				 ha->host_no, __func__, index));
			return QLA_SUCCESS;
		}

		msleep(1000);
	} /* End of While (wtime) */

	/* If we timed out on waiting for commands to come back return ERROR.*/
	return QLA_ERROR;
}

/**
 * qla4xxx_dead_adapter_cleanup - Permanently disable HA
 * @ha: pointer to adapter structure
 **/
void
qla4xxx_dead_adapter_cleanup(struct scsi_qla_host *ha)
{
        struct ddb_entry *ddb_entry, *ddbtemp;

        /* Disable the board */
        ql4_printk(KERN_INFO, ha, "Disabling the board\n");

        qla4xxx_abort_active_cmds(ha, DID_NO_CONNECT << 16);

        /* Mark all device MISSING */
        list_for_each_entry_safe(ddb_entry, ddbtemp, &ha->ddb_list, list_entry)
                qla4xxx_set_device_state(ha, ddb_entry, DEV_STATE_MISSING);

        clear_bit(AF_ONLINE, &ha->flags);
        clear_bit(AF_INIT_DONE, &ha->flags);
}

/**************************************************************************
 * qla4xxx_recover_adapter
 *      This routine recovers that adapter from a fatal state.
 *
 * Input:
 *      ha - Pointer to host adapter structure.
 *      renew_ddb_list - Indicates what to do with the adapter's ddb list
 *                      after adapter recovery has completed.
 *                      0=preserve ddb list, 1=destroy and rebuild ddb list
 *
 * Returns:
 *      QLA_SUCCESS - Successfully recovered adapter
 *      QLA_ERROR   - Failed to recover adapter
 *
 * Context:
 *      Kernel context.
 **************************************************************************/
uint8_t
qla4xxx_recover_adapter(scsi_qla_host_t *ha, uint8_t renew_ddb_list)
{
	uint8_t status = QLA_ERROR;
	uint8_t reset_chip = 0;
	unsigned long wait;

	QL4PRINT(QLP2,
		 printk("scsi%d: recover adapter (begin)\n",
			ha->host_no));

	scsi_block_requests(ha->host);

	/* Stall incoming I/O until we are done */
	clear_bit(AF_ONLINE, &ha->flags);
	QL4PRINT(QLP2, printk("scsi%d: %s: adapter OFFLINE !!\n",
				ha->host_no, __func__));

	set_bit(DPC_RESET_ACTIVE, &ha->dpc_flags);

	if (test_bit(DPC_RESET_HA, &ha->dpc_flags))
		reset_chip = 1;

	/* For the DPC_RESET_HA_INTR case (ISP-4xxx specific)
	 * do not reset adapter, jump to initialize_adapter */
	if (test_bit(DPC_RESET_HA_INTR, &ha->dpc_flags)) {
		status = QLA_SUCCESS;
		goto recover_ha_init_adapter;
	}

	/* For the ISP-8xxx adapter, issue a stop_firmware if invoked
	 * from eh_host_reset or ioctl module */
	if(IS_QLA80XX(ha) && !reset_chip &&
	   test_bit(DPC_RESET_HA_FW_CONTEXT, &ha->dpc_flags)) {
		DEBUG2(dev_info(&ha->pdev->dev,
		       "scsi%d: %s - Performing stop_firmware..\n",
		       ha->host_no, __func__));
		status = ha->isp_ops->reset_firmware(ha);
		if (status == QLA_SUCCESS) {
			/* Wait for outstanding commands to complete.
			 * Stalls the driver for max 30 secs */
			(void) qla4xxx_cmd_wait(ha);

			ha->isp_ops->disable_intrs(ha);

			/* Flush any pending ddb changed AENs */
			qla4xxx_process_aen(ha, FLUSH_DDB_CHANGED_AENS);

			qla4xxx_abort_active_cmds(ha, DID_RESET << 16);
		} else {
			/* If the stop firmware failed, then
			* reset the entire chip */
			reset_chip = 1;
			clear_bit(DPC_RESET_HA_FW_CONTEXT, &ha->dpc_flags);
			set_bit(DPC_RESET_HA, &ha->dpc_flags);
		}
	}

	/* Issue full chip reset if recovering from a catastrophic error,
	 * or if stop_firmware fails for ISP-8xxx.
	 * This is the default case for ISP-4xxx */
	if (IS_QLA40XX(ha) || reset_chip) {
		/* Check if 8XXX firmware is alive or not
		 * We may have arrived here from NEED_RESET
		 * detection only */
		if (IS_QLA80XX(ha)) {
			wait = jiffies + (FW_ALIVE_WAIT_TOV * HZ);
			if (!test_bit(AF_FW_RECOVERY, &ha->flags)) {
				while (time_before(jiffies, wait)) {
					if (qla4_8xxx_check_fw_alive(ha)) {
						qla4xxx_mailbox_premature_completion(ha);
						break;
					}

					set_current_state(TASK_UNINTERRUPTIBLE);
					schedule_timeout(HZ);
				}
			}

			/* Wait for outstanding commands to complete.
			 * Stalls the driver for max 30 secs
			 */
			if (!test_bit(AF_FW_RECOVERY, &ha->flags))
				(void) qla4xxx_cmd_wait(ha);
		} else
			(void) qla4xxx_cmd_wait(ha);

		/* Flush any pending ddb changed AENs */
		qla4xxx_process_aen(ha, FLUSH_DDB_CHANGED_AENS);

		qla4xxx_abort_active_cmds(ha, DID_RESET << 16);

		if (IS_QLA40XX(ha) &&
		   (qla4xxx_poll_and_ack_scsi_reset(ha) == QLA_SUCCESS)) {
			status = QLA_ERROR;
			goto exit_recover_adapter;
		}

		DEBUG2(printk("scsi%d: %s - Performing chip reset..\n",
				ha->host_no,__func__));

		status = ha->isp_ops->reset_chip(ha);
	}

	/* Flush any pending ddb changed AENs, again */
	qla4xxx_process_aen(ha, FLUSH_DDB_CHANGED_AENS);

recover_ha_init_adapter:
	/* Upon successful firmware/chip reset, re-initialize the adapter */
	if (status == QLA_SUCCESS) {
		DEBUG2(printk(
		    "scsi%d: %s - Initializing adapter..\n",
				ha->host_no, __func__));

		/* NOTE: AF_ONLINE flag set upon successful completion of
		 *       qla4xxx_initialize_adapter */
		status = qla4xxx_initialize_adapter(ha, renew_ddb_list);
	}

	/* Retry failed adapter initialization, if necessary
	 * Do not retry initialize_adapter for RESET_HA_INTR
	 * (ISP-4xxx specific) case to prevent ping-pong resets between
	 * functions */
	if (!test_bit(DPC_RESET_HA_INTR, &ha->dpc_flags) &&
	    !test_bit(AF_ONLINE, &ha->flags)) {
		/* Adapter initialization failed, see if we can retry
		 * resetting the ha.
		 * Since we don't want to block the DPC for too long
		 * with multiple resets in the same thread,
		 * utilize DPC to retry */
		if (!test_bit(DPC_RETRY_RESET_HA, &ha->dpc_flags)) {
			ha->retry_reset_ha_cnt = MAX_RESET_HA_RETRIES;
			DEBUG2(printk("scsi%d: recover adapter - "
					"retrying (%d) more times\n",
					ha->host_no, ha->retry_reset_ha_cnt));
			set_bit(DPC_RETRY_RESET_HA, &ha->dpc_flags);
			status = QLA_ERROR;
		}
		else {
			if (ha->retry_reset_ha_cnt > 0) {
				/* Schedule another Reset HA -- DPC will retry */
				ha->retry_reset_ha_cnt--;
				DEBUG2(printk(
					"scsi%d: recover adapter - "
					"retry remaining %d\n", ha->host_no,
						ha->retry_reset_ha_cnt));
				status = QLA_ERROR;
			}

			if (ha->retry_reset_ha_cnt == 0) {
				/* Recover adapter retries have been exhausted.
				 * Adapter DEAD */
				DEBUG2( printk(
					"scsi%d: recover adapter failed - "
					"board disabled\n", ha->host_no));
				qla4xxx_dead_adapter_cleanup(ha);
				clear_bit(DPC_RETRY_RESET_HA, &ha->dpc_flags);
				clear_bit(DPC_RESET_HA, &ha->dpc_flags);
				clear_bit(DPC_RESET_HA_FW_CONTEXT,
					  &ha->dpc_flags);
				status = QLA_ERROR;
			}
		}
	} else {
		clear_bit(DPC_RESET_HA, &ha->dpc_flags);
		clear_bit(DPC_RESET_HA_FW_CONTEXT, &ha->dpc_flags);
		clear_bit(DPC_RETRY_RESET_HA, &ha->dpc_flags);
	}

exit_recover_adapter:

	ha->adapter_error_count++;

	if (test_bit(AF_ONLINE, &ha->flags))
		ha->isp_ops->enable_intrs(ha);

	DEBUG2(ql4_printk(KERN_INFO, ha, "%s: scsi_unblock_requests\n", __func__));
	scsi_unblock_requests(ha->host);

	clear_bit(DPC_RESET_ACTIVE, &ha->dpc_flags);
	DEBUG2(printk("scsi%d: recover adapter: %s\n", ha->host_no,
			status == QLA_ERROR ? "FAILED" : "SUCCEDED"));

	return status;
}

/**************************************************************************
 * qla4xxx_eh_wait_for_active_target_commands
 *      This routine
 *
 * Input:
 *      ha - Pointer to host adapter structure.
 *      target - SCSI target ID
 *
 * Returns:
 *      0 - All pending commands returned
 *      non-zero - All pending commands did not return
 *
 * Context:
 *      Kernel context.
 **************************************************************************/
int qla4xxx_eh_wait_for_active_target_commands(scsi_qla_host_t *ha, int t, int l)
{
	int cnt;
	int status;
	srb_t *sp;
	struct scsi_cmnd *cmd;
	unsigned long flags;

	/*
	 * Waiting for all commands for the designated target in the active
	 * array
	 */
	status = 0;
	for (cnt = 1; cnt < MAX_SRBS; cnt++) {
		spin_lock_irqsave(&ha->hardware_lock, flags);
		sp = ha->active_srb_array[cnt];
		if (sp) {
			cmd = sp->cmd;
			spin_unlock_irqrestore(&ha->hardware_lock, flags);
			if (cmd->device->id == t && cmd->device->lun == l) {
				if (!qla4xxx_eh_wait_on_command(ha, cmd)) {
					status++;
					break;
				}
			}
		} else {
			spin_unlock_irqrestore(&ha->hardware_lock, flags);
		}
	}
	return status;
}

/**************************************************************************
 * qla4xxx_eh_device_reset
 *      This routine is called by the Linux OS to reset all luns on the
 * 	specified target.
 *
 * Input:
 *      cmd - Pointer to Linux's SCSI command structure
 *
 * Output:
 *      None
 *
 * Remarks:
 *      None
 *
 * Returns:
 *      SUCCESS - Successfully reset target/lun
 *      FAILED  - Failed to reset target/lun
 *
 * Context:
 *      Kernel context.  io_request_lock LOCKED
 **************************************************************************/
int
qla4xxx_eh_device_reset(struct scsi_cmnd *cmd)
{
	int return_status = FAILED;
	scsi_qla_host_t *ha;
	ddb_entry_t *ddb_entry;
	os_lun_t *lun_entry;
	os_tgt_t *tgt_entry;
	fc_lun_t *fclun;
	uint8_t stat;
	srb_t           *srb;
	struct list_head *list, *temp;
	unsigned long flags;
	struct Scsi_Host *h;
	unsigned int b, t, l;

	if (cmd == NULL) {
		printk(KERN_INFO
		    "%s(): **** SCSI mid-layer passing in NULL cmd\n",
		    __func__);
		return (return_status);
	}

	h = cmd->device->host;
	b = cmd->device->channel;
	t = cmd->device->id;
	l = cmd->device->lun;
	ha = (scsi_qla_host_t *) h->hostdata;

	/* Retrieve device and lun handles */
	tgt_entry = qla4xxx_lookup_target_by_SCSIID(ha, b, t);

	if (!tgt_entry) {
		DEBUG2(printk("scsi%d: %s: **** CMD derives a NULL TGT_Q\n",
		   ha->host_no, __func__);)
		return FAILED;
	}
	lun_entry = qla4xxx_lookup_lun_handle(ha, tgt_entry, cmd->device->lun);
	if (!lun_entry) {
		DEBUG2(printk("scsi%d: %s: **** CMD derives a NULL LUN_Q\n",
		    ha->host_no, __func__);)
		return FAILED;
	}

	fclun = lun_entry->fclun;
	if (!fclun) {
		DEBUG2(printk("scsi%d: %s: **** CMD derives a NULL FC_LUN_Q\n",
		       ha->host_no, __func__);)
		return FAILED;
	}

	if (!(fclun->fcport)) {
		DEBUG2(printk("scsi%d: %s: **** CMD derives a NULL FCPORT\n",
		    ha->host_no, __func__);)
	}
	ddb_entry = fclun->fcport->ddbptr;

	if (ddb_entry == NULL)  {
		DEBUG2(printk("scsi%d: %s: **** CMD derives a NULL DDB\n",
			ha->host_no, __func__);)
		return FAILED;
	}

	/* Wait until adapter reset is complete */
	if (qla4xxx_wait_for_reset_ready(ha) != QLA_SUCCESS) {
		QL4PRINT(QLP2, printk("scsi%d:%d: %s: Unable to reset "
			 "device. Adapter reset in progress.\n", ha->host_no,
			 cmd->device->channel, __func__));
		goto eh_dev_reset_done;
	}

	ql4_printk(KERN_INFO, ha,
	    "scsi%d:%d:%d:%d: DEVICE RESET ISSUED.\n", ha->host_no, b, t, l);

	DEBUG2(printk(
	    "scsi%d: DEVICE_RESET cmd=%p jiffies = 0x%lx, timeout=%x, "
	    "dpc_flags=%lx, status=%x allowed=%d\n",
	    ha->host_no, cmd, jiffies, cmd->timeout_per_command / HZ,
	    ha->dpc_flags, cmd->result, cmd->allowed));

	/* If we are coming in from the back-door, stall I/O until complete. */
	if (!qla4xxx_is_eh_active(h))
		set_bit(TQF_SUSPENDED, &tgt_entry->flags);

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,12)
	spin_unlock_irq(ha->host->host_lock);
#endif

	/* Clear commands from the retry queue. */
	spin_lock_irqsave(&ha->list_lock, flags);
	list_for_each_safe(list, temp, &ha->retry_srb_q) {
		srb = list_entry(list, srb_t, list_entry);

		if (tgt_entry->id != srb->cmd->device->id)
			continue;

		DEBUG2(printk(
		    "scsi%d: %s: found in retry queue. SP=%p\n",
		    ha->host_no, __func__, srb));

		__del_from_retry_srb_q(ha, srb);
		srb->cmd->result = DID_RESET << 16;
		__add_to_done_srb_q(ha, srb);
	}
	spin_unlock_irqrestore(&ha->list_lock, flags);

	/* If the adapter is not up fail the request. */
	if (!ADAPTER_UP(ha)) {
		dev_info(&ha->pdev->dev,
		    "%s: %d:%d:%d: DEVICE RESET - ADAPTER DOWN\n",
		    __func__, cmd->device->channel, cmd->device->id,
		    cmd->device->lun);
		goto eh_dev_reset_done;
	}

	/* If the ddb is not connected, fail the request. */
	if ((ddb_entry->fw_ddb_device_state == DDB_DS_UNASSIGNED) ||
	   (ddb_entry->fw_ddb_device_state == DDB_DS_NO_CONNECTION_ACTIVE)) {
		DEBUG2(printk(
		    "scsi%d:%d:%d:%d: DEVICE RESET - DDB NOT-CONNECTED\n",
		    ha->host_no, cmd->device->channel, cmd->device->id,
		    cmd->device->lun));
		goto eh_dev_reset_done;
	}

	stat = qla4xxx_reset_lun(ha, fclun->fcport->ddbptr, fclun->lun);
	if (stat != QLA_SUCCESS) {
		ql4_printk(KERN_INFO, ha, "DEVICE RESET FAILED. %d\n", stat);
		goto eh_dev_reset_done;
	}

	/*
	 * If we are coming down the EH path, wait for all commands to complete
	 * for the device.
	 */
	if (qla4xxx_is_eh_active(h)) {
		if (qla4xxx_eh_wait_for_active_target_commands(ha, t, l)) {
			ql4_printk(KERN_INFO, ha, "DEVICE RESET FAILED - "
			    "waiting for commands.\n");

			goto eh_dev_reset_done;
		}
	}

	ql4_printk(KERN_INFO, ha,
	    "scsi(%d:%d:%d:%d): DEVICE RESET SUCCEEDED.\n", ha->host_no,
	    b, t, l);

	return_status = SUCCESS;

eh_dev_reset_done:

	if (!qla4xxx_is_eh_active(h))
		clear_bit(TQF_SUSPENDED, &tgt_entry->flags);

	QL4PRINT(QLP2, printk("scsi%d: %s: return with status %s\n",
	    ha->host_no, __func__,
	    (return_status == FAILED) ? "FAILED" : "SUCCEDED"));

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,12)
	spin_lock_irq(ha->host->host_lock);
#endif
	return return_status;
}


/**************************************************************************
 * qla4xxx_eh_bus_reset
 *      This routine is called by the Linux OS to reset the specified
 *      adapter/bus.
 *
 * Input:
 *      cmd - Pointer to Linux's SCSI command structure
 *
 * Returns:
 *      SUCCESS - Successfully reset adapter/bus
 *      FAILED  - Failed to reset adapter/bus
 *
 * Context:
 *      Kernel context.  io_request_lock LOCKED
 **************************************************************************/
int
qla4xxx_eh_bus_reset(struct scsi_cmnd *cmd)
{
	uint8_t status = QLA_SUCCESS;
	int     return_status = FAILED;
	scsi_qla_host_t *ha;
	ddb_entry_t *ddb_entry, *dtemp;
	ha = (scsi_qla_host_t *) cmd->device->host->hostdata;

	ql4_printk(KERN_INFO, ha,
	    "scsi(%d:%d:%d:%d): BUS RESET ISSUED.\n", ha->host_no,
	    cmd->device->channel, cmd->device->id, cmd->device->lun);

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,12)
 	spin_unlock_irq(ha->host->host_lock);
#endif

	if (qla4xxx_wait_for_hba_online(ha) != QLA_SUCCESS) {
		QL4PRINT(QLP2, printk("scsi%d:%d: %s: Unable to reset "
		    "bus.  Adapter DEAD.\n", ha->host_no,
		    cmd->device->channel, __func__));

		goto eh_bus_reset_done;
	}

	if (qla4xxx_wait_for_link_up(ha) != QLA_SUCCESS) {
		DEBUG2(printk("scsi%d: %s: Link Down\n",
				ha->host_no, __func__));
		goto eh_bus_reset_done;
	}

	/* Wait until adapter reset is complete */
	if (qla4xxx_wait_for_reset_ready(ha) != QLA_SUCCESS) {
		QL4PRINT(QLP2, printk("scsi%d:%d: %s: Unable to reset "
			 "bus. Adapter reset in progress.\n", ha->host_no,
			 cmd->device->channel, __func__));
		goto eh_bus_reset_done;
	}

	/* Attempt to reset all valid targets with outstanding commands */
	list_for_each_entry_safe(ddb_entry, dtemp, &ha->ddb_list, list_entry) {
		int eh_active = qla4xxx_is_eh_active(cmd->device->host);

		if ((ddb_entry->fw_ddb_device_state == DDB_DS_UNASSIGNED) ||
		    (ddb_entry->fw_ddb_device_state ==
		     DDB_DS_NO_CONNECTION_ACTIVE)) {
			continue;
		}

		QL4PRINT(QLP2, printk("scsi%d: %s: reset target b%d, t%x, "
		    "index [%d]\n", ha->host_no, __func__, ddb_entry->bus,
		    ddb_entry->target, ddb_entry->fw_ddb_index));

		/* Issue a reset */
		status |= qla4xxx_reset_target(ha, ddb_entry, eh_active);
	}

	/*
	 * Status is QLA_SUCCESS if target resets for ALL devices completed
	 * successfully.  Otherwise the status is QLA_ERROR.
	 */
	if (status == QLA_SUCCESS)
		return_status = SUCCESS;

eh_bus_reset_done:
	ql4_printk(KERN_INFO, ha, "BUS RESET %s.\n",
	    (return_status == FAILED) ? "FAILED" : "SUCCEDED");
	DEBUG2(printk("%s: EXIT (%s)\n", __func__,
		    (return_status == FAILED) ? "FAILED" : "SUCCEDED"));

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,12)
	spin_lock_irq(ha->host->host_lock);
#endif
	return return_status;
}

/**************************************************************************
 * qla4xxx_reset_target
 *      This routine issues either a warm or cold target reset to the
 *      specified device.
 *
 * Input:
 *      ha - Pointer to host adapter structure.
 *      ddb_entry - Pointer to device database entry
 *
 * Remarks:
 *      The caller must ensure that the ddb_entry pointer is valid before
 *      calling this routine.
 *
 * Returns:
 *      QLA_SUCCESS - Successfully reset target
 *      QLA_ERROR   - Failed to reset target
 *      eh_active - 1 = mid-layer error handler is active
 *
 * Context:
 *      Kernel context.
 **************************************************************************/
uint8_t
qla4xxx_reset_target(scsi_qla_host_t *ha, ddb_entry_t *ddb_entry, int eh_active)
{
	uint8_t status = QLA_SUCCESS;
	fc_lun_t *fclun;
	fc_port_t *fcport;

	/* Reset all LUNs on this target */
	/* NOTE: We do not issue target reset because it is an optional task
	 * management command and all targets may not have implemented it.
	 * The LUN reset task management command, however, is mandatory.
	 */
	fcport = ddb_entry->fcport;
	list_for_each_entry(fclun, &fcport->fcluns, list) {
		status = qla4xxx_reset_lun(ha, ddb_entry, fclun->lun);
		if (status == QLA_SUCCESS && eh_active) {
			/*
			 * Waiting for all active commands to complete for the
			 * device.
			 */
			status |= qla4xxx_eh_wait_for_active_target_commands(
			    ha, ddb_entry->target, fclun->lun);
		}
	}

	if (status == QLA_SUCCESS) {
		QL4PRINT(QLP2, printk("scsi%d:%d:%d: device reset SUCCEEDED.\n",
		    ha->host_no, ddb_entry->bus, fcport->os_target_id));
	} else {
		QL4PRINT(QLP2, printk("scsi%d:%d:%d: device reset FAILED.\n",
		    ha->host_no, ddb_entry->bus, fcport->os_target_id));

		status = QLA_ERROR;
	}

	return status;
}

/**************************************************************************
 * qla4xxx_abort_active_cmds
 *      This routine is called just prior to a HARD RESET to return all
 *      outstanding commands back to the Operating System.
 *
 * Input:
 *      ha - Pointer to host adapter structure.
 * 	@res: Command Return Code.
 *
 * Remarks:
 *      Caller should make sure that the following locks are released
 *      before this calling routine:
 *              Hardware lock, io_request_lock, list_lock, and lun_lock.
 *
 * Returns:
 *      None
 *
 * Context:
 *      Kernel context.
 **************************************************************************/
void qla4xxx_abort_active_cmds(scsi_qla_host_t *ha, int res) {
	srb_t    *srb;
	int      i;
	unsigned long flags;

	spin_lock_irqsave(&ha->hardware_lock, flags);
	for (i = 1; i < MAX_SRBS; i++) {
		if ((srb = ha->active_srb_array[i]) != NULL) {
			QL4PRINT(QLP5,
				 printk("scsi%d: %s: found srb %p in active array, "
					"returning\n", ha->host_no, __func__, srb));
			del_from_active_array(ha, i);
			srb->cmd->result = res;
			add_to_done_srb_q(ha,srb);
		}
	}
	spin_unlock_irqrestore(&ha->hardware_lock, flags);
}

/**************************************************************************
 * qla4xxx_eh_host_reset
 *      This routine is invoked by the Linux kernel to perform fatal error
 *      recovery on the specified adapter.
 *
 * Input:
 *      cmd - Pointer to Linux's SCSI command structure
 *
 * Returns:
 *      SUCCESS - Successfully recovered host adapter
 *      FAILED  - Failed to recover host adapter
 *
 * Context:
 *      Kernel context.  io_request_lock LOCKED
 **************************************************************************/
int
qla4xxx_eh_host_reset(struct scsi_cmnd *cmd)
{
	int return_status = FAILED;
	scsi_qla_host_t *ha;

	ha = (scsi_qla_host_t *) cmd->device->host->hostdata;

	if (IS_QLA8032(ha) && ql4xdontresethba)
		qla4_83xx_set_idc_dontreset(ha);

	/* For ISP8324, if IDC_CTRL DONTRESET_BIT0 is set by other
	 * protocol drivers, we should not set device_state to
	 * NEED_RESET */
	if ((ql4xdontresethba == 1) ||
	    (IS_QLA8032(ha) && qla4_83xx_idc_dontreset(ha))) {
		DEBUG2(printk("scsi%d: %s: Don't Reset HBA\n",
			ha->host_no, __func__));
		return FAILED;
	}

	ql4_printk(KERN_INFO, ha,
	    "scsi(%d:%d:%d:%d): HOST RESET ISSUED.\n", ha->host_no,
	    cmd->device->channel, cmd->device->id, cmd->device->lun);
	DEBUG2(printk("scsi(%d:%d:%d:%d): HOST RESET ISSUED.\n",
		      ha->host_no, cmd->device->channel,
		      cmd->device->id, cmd->device->lun));

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,12)
 	spin_unlock_irq(ha->host->host_lock);
#endif

	if (qla4xxx_wait_for_hba_online(ha) != QLA_SUCCESS) {
		QL4PRINT(QLP2, printk("scsi%d:%d: %s: Unable to reset "
		    "host.  Adapter DEAD.\n", ha->host_no,
		    cmd->device->channel, __func__));
		goto eh_host_reset_done;
	}

	/* Wait until adapter reset is complete */
	if (qla4xxx_wait_for_reset_ready(ha) != QLA_SUCCESS) {
		QL4PRINT(QLP2, printk("scsi%d:%d: %s: Unable to reset "
			"host. Adapter reset in progress.\n", ha->host_no,
			cmd->device->channel, __func__));
		goto eh_host_reset_done;
	}

	if (!test_bit(DPC_RESET_HA, &ha->dpc_flags)) {
		if (IS_QLA80XX(ha))
			set_bit(DPC_RESET_HA_FW_CONTEXT, &ha->dpc_flags);
		else
			set_bit(DPC_RESET_HA, &ha->dpc_flags);
	}

	if (qla4xxx_recover_adapter(ha, PRESERVE_DDB_LIST) == QLA_SUCCESS) {
		return_status = SUCCESS;
	}

eh_host_reset_done:
	ql4_printk(KERN_INFO, ha, "HOST RESET %s.\n",
	    (return_status == FAILED) ? "FAILED" : "SUCCEDED");
	DEBUG2(printk("HOST RESET %s.\n",
	    (return_status == FAILED) ? "FAILED" : "SUCCEDED"));

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,12)
 	spin_lock_irq(ha->host->host_lock);
#endif

	return return_status;
}

/*
* qla4xxx_free_other_mem
*      Frees all adapter allocated memory.
*
* Input:
*      ha = adapter block pointer.
*/
void qla4xxx_free_other_mem(scsi_qla_host_t *ha)
{
	uint32_t        t;
	fc_port_t       *fcport, *fptemp;
	fc_lun_t        *fclun, *fltemp;

	if (ha == NULL) {
		/* error */
		DEBUG2(printk("%s: ERROR invalid ha pointer.\n", __func__));
		return;
	}

	/* Free the target and lun queues */
	for (t = 0; t < MAX_TARGETS; t++) {
		qla4xxx_tgt_free(ha, t);
	}

	/* Free fcport and fcluns */
	list_for_each_entry_safe(fcport, fptemp, &ha->fcports, list) {
		list_for_each_entry_safe(fclun, fltemp, &fcport->fcluns, list) {
			list_del_init(&fclun->list);
			kfree(fclun);
		}
		list_del_init(&fcport->list);
		kfree(fcport);
	}
	INIT_LIST_HEAD(&ha->fcports);
}

/* PCI AER driver recovers from all correctable errors w/o
 * driver intervention. For uncorrectable errors PCI AER
 * driver calls the following device driver's callbacks
 *
 * - Fatal Errors - link_reset
 * - Non-Fatal Errors - driver's pci_error_detected() which
 * returns CAN_RECOVER, NEED_RESET or DISCONNECT.
 *
 * PCI AER driver calls
 * CAN_RECOVER - driver's pci_mmio_enabled(), mmio_enabled
 *               returns RECOVERED or NEED_RESET if fw_hung
 * NEED_RESET - driver's slot_reset()
 * DISCONNECT - device is dead & cannot recover
 * RECOVERED - driver's pci_resume()
 */
static pci_ers_result_t
qla4xxx_pci_error_detected(struct pci_dev *pdev, pci_channel_state_t state)
{
	struct scsi_qla_host *ha = pci_get_drvdata(pdev);

	ql4_printk(KERN_WARNING, ha, "scsi%d: %s: error detected:state %x\n",
	    ha->host_no, __func__, state);

	if (!IS_AER_SUPPORTED(ha))
		return PCI_ERS_RESULT_NONE;

	switch (state) {
	case pci_channel_io_normal:
		clear_bit(AF_EEH_BUSY, &ha->flags);
		return PCI_ERS_RESULT_CAN_RECOVER;
	case pci_channel_io_frozen:
		set_bit(AF_EEH_BUSY, &ha->flags);
		/* If the firmware is in a hung state and a mbox
		 * command has been issued, complete the mbox, so
		 * slot reset can be done
		 */
		set_bit(AF_FW_RECOVERY, &ha->flags);
		if (test_bit(AF_MBOX_COMMAND, &ha->flags)) {
			if (test_bit(AF_MBOX_COMMAND_NOPOLL, &ha->flags)) {
				complete(&ha->mbx_intr_comp);
				ql4_printk(KERN_WARNING, ha, "scsi%d: %s: "
				    "In nopoll mode doing premature completion "
				    "of mbox command due to FW_HUNG\n",
				    ha->host_no, __func__);
			} else {
				set_bit(AF_MBOX_COMMAND_DONE, &ha->flags);
				ql4_printk(KERN_WARNING, ha, "scsi%d: %s: FW_HUNG "
				    "in polling mode, set AF_MBOX_COMMAND_DONE "
				    "to stop polling\n", ha->host_no, __func__);
			}
		}
		qla4xxx_free_irqs(ha);
		pci_disable_device(pdev);
		/* Abort all active commands */
		qla4xxx_abort_active_cmds(ha, DID_RESET << 16);
		return PCI_ERS_RESULT_NEED_RESET;
	case pci_channel_io_perm_failure:
		set_bit(AF_EEH_BUSY, &ha->flags);
		set_bit(AF_PCI_CHANNEL_IO_PERM_FAILURE, &ha->flags);
		qla4xxx_abort_active_cmds(ha, DID_NO_CONNECT << 16);
		return PCI_ERS_RESULT_DISCONNECT;
	}
	return PCI_ERS_RESULT_NEED_RESET;
}

/**
 * qla4xxx_pci_mmio_enabled() gets called if
 * qla4xxx_pci_error_detected() returns PCI_ERS_RESULT_CAN_RECOVER
 * and read/write to the device still works.
 **/
static pci_ers_result_t
qla4xxx_pci_mmio_enabled(struct pci_dev *pdev)
{
	struct scsi_qla_host *ha = pci_get_drvdata(pdev);

	if (!IS_AER_SUPPORTED(ha))
		return PCI_ERS_RESULT_NONE;

	return PCI_ERS_RESULT_RECOVERED;
}

uint32_t qla4_8xxx_error_recovery(struct scsi_qla_host *ha)
{
	uint32_t rval = QLA_ERROR;
	int fn;
	struct pci_dev *other_pdev = NULL;
	struct ddb_entry *ddb_entry, *ddbtemp;

	printk(KERN_WARNING "scsi%d: In %s\n",
		ha->host_no, __func__);

	set_bit(DPC_RESET_ACTIVE, &ha->dpc_flags);

	if (test_bit(AF_ONLINE, &ha->flags)) {
		clear_bit(AF_ONLINE, &ha->flags);
		/* Mark all devices MISSING */
		list_for_each_entry_safe(ddb_entry, ddbtemp, &ha->ddb_list, list_entry)
			qla4xxx_set_device_state(ha, ddb_entry, DEV_STATE_MISSING);
		qla4xxx_process_aen(ha, FLUSH_DDB_CHANGED_AENS);
		if (!test_bit(AF_EEH_BUSY, &ha->flags))
			qla4xxx_abort_active_cmds(ha, DID_RESET << 16);
	}

	fn = PCI_FUNC(ha->pdev->devfn);
	while (fn > 0) {
		fn--;
		printk(KERN_INFO "scsi%d: %s: Finding PCI device at func %x\n",
		    ha->host_no, __func__, fn);
		/* Get the pci device given the domain, bus, slot/function number */
		other_pdev =
		    pci_get_domain_bus_and_slot(pci_domain_nr(ha->pdev->bus),
		    ha->pdev->bus->number, PCI_DEVFN(PCI_SLOT(ha->pdev->devfn),
		    fn));

		if (!other_pdev)
			continue;

		if (other_pdev && other_pdev->is_enabled) {
			printk(KERN_INFO "scsi%d: %s: Found PCI func in "
			    "enabled state%x\n", ha->host_no, __func__, fn);
			pci_dev_put(other_pdev);
			break;
		}
		pci_dev_put(other_pdev);
	}

	/* The first function on the card, the reset owner willF_FW_RECOVERY
	 * start & initialize the firmware. The other functions
	 * on the card will reset the firmware context
	 */
	if (!fn) {
		printk(KERN_INFO "scsi%d: %s: devfn being reset 0x%x is the "
			"owner\n", ha->host_no, __func__, ha->pdev->devfn);

		ha->isp_ops->idc_lock(ha);
		qla4_8xxx_wr_direct(ha, QLA8XXX_CRB_DEV_STATE,
			QLA8XXX_DEV_COLD);
		rval = qla4_8xxx_update_idc_reg(ha);
		ha->isp_ops->idc_unlock(ha);

		if (rval == QLA_ERROR) {
			printk(KERN_INFO "scsi%d: %s: HW State: FAILED \n",
					ha->host_no, __func__);
			ha->isp_ops->idc_lock(ha);
			qla4_8xxx_wr_direct(ha, QLA8XXX_CRB_DEV_STATE,
					QLA8XXX_DEV_FAILED);
			ha->isp_ops->idc_unlock(ha);
			goto exit_error_recovery;
		}

		clear_bit(AF_FW_RECOVERY, &ha->flags);

		rval = qla4xxx_initialize_adapter(ha, PRESERVE_DDB_LIST);

		ha->isp_ops->idc_lock(ha);
		if (rval != QLA_SUCCESS) {
			printk(KERN_INFO "scsi%d: %s: HW State: FAILED \n",
			    ha->host_no, __func__);
			qla4_8xxx_clear_drv_active(ha);
			qla4_8xxx_wr_direct(ha, QLA8XXX_CRB_DEV_STATE,
			    QLA8XXX_DEV_FAILED);
		} else {
			printk(KERN_INFO "scsi%d: %s: HW State: READY \n",
			    ha->host_no, __func__);
			qla4_8xxx_wr_direct(ha, QLA8XXX_CRB_DEV_STATE,
			    QLA8XXX_DEV_READY);
			qla4_8xxx_set_drv_active(ha);
			/* Clear driver state register */
			qla4_8xxx_wr_direct(ha, QLA8XXX_CRB_DRV_STATE, 0);
			ha->isp_ops->enable_intrs(ha);
		}
		ha->isp_ops->idc_unlock(ha);
	} else {
		ql4_printk(KERN_INFO, ha, "scsi%d: %s: devfn 0x%x is not the reset "
			"owner\n", ha->host_no, __func__, ha->pdev->devfn);
		if ((qla4_8xxx_rd_direct(ha, QLA8XXX_CRB_DEV_STATE) ==
		     QLA8XXX_DEV_READY)) {
			clear_bit(AF_FW_RECOVERY, &ha->flags);
			rval = qla4xxx_initialize_adapter(ha, PRESERVE_DDB_LIST);
			if (rval == QLA_SUCCESS)
				ha->isp_ops->enable_intrs(ha);
			ha->isp_ops->idc_lock(ha);
			qla4_8xxx_set_drv_active(ha);
			ha->isp_ops->idc_unlock(ha);
		}
	}

exit_error_recovery:
	clear_bit(DPC_RESET_ACTIVE, &ha->dpc_flags);
	return rval;
}

static pci_ers_result_t
qla4xxx_pci_slot_reset(struct pci_dev *pdev)
{
	pci_ers_result_t ret = PCI_ERS_RESULT_DISCONNECT;
	struct scsi_qla_host *ha = pci_get_drvdata(pdev);
	int rc;

	ql4_printk(KERN_WARNING, ha, "scsi%d: %s: slot_reset \n",
	    ha->host_no, __func__);

	if (!IS_AER_SUPPORTED(ha))
		return PCI_ERS_RESULT_NONE;

	/* Workaround to properly reflect error_state as
	 * pci_channel_io_online as result of eeh_reset_device()
	 * called just prior to this callback.
	 * eeh_driver.c delays updating error_state until
	 * report_resume callback which is too late for
	 * qla4xxx driver which access hardware earlier.
	 * Can be removed when fixed in mainline.
	 */
	pdev->error_state = pci_channel_io_normal;

	/* Restore the saved state of PCIe device -
	 * BAR registers, PCI Config space, PCIX, MSI,
	 * IOV states
	 */
	pci_restore_state(pdev);

	/* pci_restore_state() clears the saved_state flag of the device
	 * save restored state which resets saved_state flag
	 */
	pci_save_state(pdev);

	/* Initialize device or resume if in suspended state */
	rc = pci_enable_device(pdev);
	if (rc) {
		printk(KERN_WARNING "scsi%d: %s: Cant re-enable "
		    "device after reset\n", ha->host_no, __func__);
		goto exit_slot_reset;
	}

	ret = qla4xxx_request_irqs(ha);
	if (ret) {
		dev_warn(&ha->pdev->dev, "Failed to reserve interrupt %d"
		    " already in use.\n", pdev->irq);
		goto exit_slot_reset;
	}

	if (IS_QLA80XX(ha)) {
		if (qla4_8xxx_error_recovery(ha) == QLA_SUCCESS) {
			ret = PCI_ERS_RESULT_RECOVERED;
			goto exit_slot_reset;
		} else
			goto exit_slot_reset;
	}

exit_slot_reset:
	printk(KERN_WARNING "scsi%d: %s: Return=%x \n"
		"device after reset\n", ha->host_no, __func__, ret);
	return ret;
}

static void
qla4xxx_pci_resume(struct pci_dev *pdev)
{
	struct scsi_qla_host *ha = pci_get_drvdata(pdev);
	int ret;

	printk(KERN_WARNING "scsi%d: %s: pci_resume \n",
	    ha->host_no, __func__);

	ret = qla4xxx_wait_for_hba_online(ha);
	if (ret != QLA_SUCCESS) {
		printk(KERN_ERR "scsi%d: %s: "
		    "the device failed to resume I/O "
		    "from slot/link_reset\n",
		    ha->host_no, __func__);
	}

	pci_cleanup_aer_uncorrect_error_status(pdev);
	clear_bit(AF_EEH_BUSY, &ha->flags);
}
/*
 * Overrides for Emacs so that we almost follow Linus's tabbing style.
 * Emacs will notice this stuff at the end of the file and automatically
 * adjust the settings for this buffer only.  This must remain at the end
 * of the file.
 * ---------------------------------------------------------------------------
 * Local variables:
 * c-indent-level: 2
 * c-brace-imaginary-offset: 0
 * c-brace-offset: -2
 * c-argdecl-indent: 2
 * c-label-offset: -2
 * c-continued-statement-offset: 2
 * c-continued-brace-offset: 0
 * indent-tabs-mode: nil
 * tab-width: 8
 * End:
 */



