/*
 * QLogic iSCSI IOCTL Module
 * Copyright (c)  2003-2012 QLogic Corporation
 *
 * See LICENSE.qla4xxx for copyright and licensing details.
 */

/******************************************************************************
 *             Please see release.txt for revision history.                   *
 *                                                                            *
 ******************************************************************************
 * Function Table of Contents:
 *	qla4extioctl_query_hba_iscsi_node
 *	qla4extioctl_query_hba_iscsi_portal
 *	qla4extioctl_query_disc_iscsi_node
 *	qla4extioctl_query_disc_iscsi_portal
 *	qla4extioctl_query_driver
 *	qla4extioctl_query_fw
 *	qla4extioctl_query_chip
 *	qla4extioctl_query_ip_state
 *	qla4extioctl_query_device_current_ip
 *	qla4extioctl_query
 *	qla4extioctl_reg_aen
 *	qla4extioctl_get_aen
 *	qla4extioctl_get_statistics_gen
 *	qla4extioctl_get_statistics_iscsi
 *	qla4extioctl_get_device_entry_iscsi
 *	qla4extioctl_get_init_fw_iscsi
 *	qla4extioctl_get_isns_server
 *	qla4extioctl_get_isns_disc_targets
 *	qla4extioctl_get_acb
 *	qla4extioctl_get_neighbor_cache
 *	qla4extioctl_get_destination_cache
 *	qla4extioctl_get_default_router_list
 *	qla4extioctl_get_local_prefix_list
 *	qla4extioctl_get_isns_targets_buffer
 *	qla4extioctl_get_data
 *	qla4extioctl_rst_statistics_gen
 *	qla4extioctl_rst_statistics_iscsi
 *	qla4extioctl_set_device_entry_iscsi
 *	qla4extioctl_set_init_fw_iscsi
 *	qla4extioctl_set_isns_server
 *	qla4extioctl_set_acb
 *	qla4extioctl_set_data
 *	qla4xxx_ioctl_sleep_done
 *	qla4xxx_ioctl_sem_init
 *	qla4xxx_scsi_pass_done
 *	qla4extioctl_scsi_passthru
 *	qla4extioctl_iscsi_passthru
 *	qla4extioctl_disable_acb
 *	qla4extioctl_send_router_sol
 *	qla4extioctl_get_hbacnt
 *	qla4xxx_send_ioctl
 ****************************************************************************/

#include <linux/version.h>
#include <linux/smp_lock.h>
#include <linux/vmalloc.h>
#include "ql4im_def.h"
#include "ql4im_ioctl.h"
#include "ql4_isns.h"

/*
 * Externs from ql4_inioct.c
 */
extern int qla4intioctl_logout_iscsi(ql4im_scsi_qla_host_t *, EXT_IOCTL_ISCSI *);
extern int qla4intioctl_ping(ql4im_scsi_qla_host_t *, EXT_IOCTL_ISCSI *);
extern int qla4intioctl_get_data(ql4im_scsi_qla_host_t *, EXT_IOCTL_ISCSI *);
extern int qla4intioctl_set_data(ql4im_scsi_qla_host_t *, EXT_IOCTL_ISCSI *);
extern int qla4intioctl_hba_reset(ql4im_scsi_qla_host_t *, EXT_IOCTL_ISCSI *);
extern int qla4intioctl_copy_fw_flash(ql4im_scsi_qla_host_t *, EXT_IOCTL_ISCSI *);
extern int qla4intioctl_restore_factory_defaults(ql4im_scsi_qla_host_t *, EXT_IOCTL_ISCSI *);

/* local function prototypes */
int qla4xxx_send_ioctl(struct scsi_device *, int, void __user *);
extern void ql4im_ioctl_sem_init (ql4im_scsi_qla_host_t *ql4im_ha);
extern uint8_t ql4im_resize_ioctl_dma_buf(ql4im_scsi_qla_host_t *ql4im_ha, uint32_t size);

/*
 * ioctl initialization
 */
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,13)
static struct class *apidev_class;
#else
static struct class_simple *apidev_class;
#endif
static int apidev_major;

static int
apidev_ioctl(struct inode *inode, struct file *fp, unsigned int cmd,
    unsigned long arg)
{
	return (qla4xxx_send_ioctl(NULL, (int)cmd, (void __user *)arg));
}

#if LINUX_VERSION_CODE > KERNEL_VERSION(2,6,14)
static long
qla4xxx_ioctl32(struct file *file, unsigned int cmd, unsigned long arg)
{
    int rval = -ENOIOCTLCMD;

    lock_kernel();
    rval = apidev_ioctl(file->f_dentry->d_inode, file, cmd, arg);
    unlock_kernel();

    return rval;
}
#endif


#if defined(QL_DEBUG_LEVEL_4)
/**
 * ql4_prn_str - retrieve associated print string
 * @val: Value of string
 * @tbl: Table of strings
 * @return: String corresponding to value in specified table
 *
 * Iterate through specified table searching for @val.  If found,
 * returns associated string.  Otherwise, return default "UNKNOWN" string.
 * Last element in table MUST be {-1, "UNKNOWN"}.
 **/
static const char * ql4_prn_str(int val, struct prn_str_tbl *tbl)
{
	for (; tbl->val != (-1); tbl++)
		if (tbl->val == val)
			break;

	return (tbl->s);
}
#endif

static struct file_operations apidev_fops = {
	.owner = THIS_MODULE,
	.ioctl = apidev_ioctl,
#if LINUX_VERSION_CODE > KERNEL_VERSION(2,6,14)
	.compat_ioctl = qla4xxx_ioctl32,
#endif
};

inline void *
ql4_kzmalloc(int siz, gfp_t code)
{
	void *		bp;

	if ((bp = kmalloc(siz, code)) != NULL) {
		memset(bp, 0, siz);
	}

	return (bp);
}


/**************************************************************************
 * ql4im_resize_ioctl_dma_buf
 *	This routine deallocates the dma_buf of the previous size and re-
 *	allocates the dma_buf with the given size.
 *
 * Input:
 *	ql4im_ha      - Pointer to host adapter structure
 *	size    - Size of dma buffer to allocate
 *
 * Output:
 *	dma_buf - virt_addr, phys_addr, and buf_len values filled in
 *
 * Returns:
 *	QLA_SUCCESS - Successfully re-allocates memory
 *	QLA_ERROR   - Failed to re-allocate memory
 *
 * Context:
 *	Kernel context.
 **************************************************************************/
uint8_t
ql4im_resize_ioctl_dma_buf(ql4im_scsi_qla_host_t *ql4im_ha, uint32_t size)
{
	uint8_t status = 0;

	if (ql4im_ha->ioctl_dma_buf_len) {
		DEBUG4(printk("qisioctl%d: %s: deallocate old dma_buf, size=0x%x\n",
		    ql4im_ha->host_no, __func__, ql4im_ha->ioctl_dma_buf_len));

		pci_free_consistent(ql4im_ha->drvr_ha->pdev,
				    ql4im_ha->ioctl_dma_buf_len,
                                    ql4im_ha->ioctl_dma_bufv,
				    ql4im_ha->ioctl_dma_bufp);
		ql4im_ha->ioctl_dma_buf_len = 0;
		ql4im_ha->ioctl_dma_bufv = NULL;
		ql4im_ha->ioctl_dma_bufp = 0;
	}

	DEBUG4(printk("qisioctl%d: %s: allocate new ioctl_dma_buf, size=0x%x\n",
	    ql4im_ha->host_no, __func__, size));
	
	ql4im_ha->ioctl_dma_bufv = pci_alloc_consistent(ql4im_ha->drvr_ha->pdev,
							size,
                                                        &ql4im_ha->ioctl_dma_bufp);
	if (ql4im_ha->ioctl_dma_bufv == NULL) {
		DEBUG2(printk("qisioctl%d: %s: ERROR allocating new ioctl_dma_buf, "
		    "size=0x%x\n", ql4im_ha->host_no, __func__, size));
	} else {
		ql4im_ha->ioctl_dma_buf_len = size;
	}

	return(status);
}

/*
 * ql4im_alloc_ioctl_mem
 *	Allocates memory needed by IOCTL code.
 *
 * Input:
 *	ql4im_ha = adapter state pointer.
 *
 * Returns:
 *	ql4xxx local function return status code.
 *
 * Context:
 *	Kernel context.
 */
int
ql4im_alloc_ioctl_mem(ql4im_scsi_qla_host_t *ql4im_ha)
{
	/* Allocate IOCTL DMA Buffer
	 * ------------------------- */
	ql4im_ha->ioctl_dma_buf_len = DMA_BUFFER_SIZE;
	ql4im_ha->ioctl_dma_bufv = pci_alloc_consistent(ql4im_ha->drvr_ha->pdev,
	    ql4im_ha->ioctl_dma_buf_len, &ql4im_ha->ioctl_dma_bufp);
	if (ql4im_ha->ioctl_dma_bufv == NULL) {
		printk(KERN_WARNING "qisioctl%d: Memory Allocation failed - "
		    "IOCTL DMA buffer.\n", ql4im_ha->host_no);

		return QLA_ERROR;
	}

	memset(ql4im_ha->ioctl_dma_bufv, 0, ql4im_ha->ioctl_dma_buf_len);
	DEBUG4(printk("qisioctl%d: %s: IOCTL DMAv = 0x%p\n",
	    ql4im_ha->host_no, __func__, ql4im_ha->ioctl_dma_bufv));
	DEBUG4(printk("qisioctl%d: %s: IOCTL DMAp = 0x%lx\n",
	    ql4im_ha->host_no, __func__, (unsigned long)ql4im_ha->ioctl_dma_bufp));

	/* Allocate context memory buffer */
	ql4im_ha->ioctl = QL_KMEM_ZALLOC(sizeof(hba_ioctl_context));
	if (ql4im_ha->ioctl == NULL) {
		/* error */
		printk(KERN_WARNING "qisioctl%d: ERROR in ioctl context allocation.\n",
		    ql4im_ha->host_no);
		return QLA_ERROR;
	}

	/* Allocate AEN tracking buffer */
	ql4im_ha->ioctl->aen_tracking_queue =
	    QL_KMEM_ZALLOC(EXT_DEF_MAX_AEN_QUEUE * sizeof(EXT_ASYNC_EVENT));
	if (ql4im_ha->ioctl->aen_tracking_queue == NULL) {
		printk(KERN_WARNING "qisioctl%d: ERROR in ioctl aen_queue allocation.\n",
		    ql4im_ha->host_no);
		return QLA_ERROR;
	}

	/* Pick the largest size we'll need per ql4im_ha of all ioctl cmds.
	 * Use this size when freeing.
	 */
	ql4im_ha->ioctl->scrap_mem = QL_KMEM_ZALLOC(QLA_IOCTL_SCRAP_SIZE);
	if (ql4im_ha->ioctl->scrap_mem == NULL) {
		printk(KERN_WARNING "qisioctl%d: ERROR in ioctl scrap_mem allocation.\n",
		    ql4im_ha->host_no);
		return QLA_ERROR;
	}
	ql4im_ha->ioctl->scrap_mem_size = QLA_IOCTL_SCRAP_SIZE;
	ql4im_ha->ioctl->scrap_mem_used = 0;

	DEBUG4(printk("qisioctl%d: %s: scrap_mem_size=%d.\n",
	    ql4im_ha->host_no, __func__, ql4im_ha->ioctl->scrap_mem_size));

        /*
         * Allocate core dump buffer
         */
	if (IS_QLA80XX(ql4im_ha->drvr_ha)) {
		if (ql4im_ha->drvr_ha->fw_dump_size) {
			ql4im_ha->ioctl->core_dump_mem =
				vmalloc(ql4im_ha->drvr_ha->fw_dump_size);
			if (ql4im_ha->ioctl->core_dump_mem == NULL) {
				printk(KERN_WARNING
					"scsi%d: ERROR in ioctl core dump "
					"memory allocation.\n",
					ql4im_ha->host_no);
				return QLA_ERROR;
			}
		}
	} else {
		ql4im_ha->ioctl->core_dump_mem = vmalloc(QLA_IOCTL_DUMP_IMAGE_SIZE);
		if (ql4im_ha->ioctl->core_dump_mem == NULL) {
		       printk(KERN_WARNING
			       "scsi%d: ERROR in ioctl core dump memory allocation.\n",
			       ql4im_ha->host_no);
		       return QLA_ERROR;
		}
	}
	
	DEBUG4(printk("scsi%d: %s: core_dump_mem_size=%ld, addr=%p.\n",
	    ql4im_ha->host_no, __func__, QLA_IOCTL_DUMP_IMAGE_SIZE,
	    ql4im_ha->ioctl->core_dump_mem));

	/* Initialize ioctl semaphore */
	init_MUTEX(&ql4im_ha->ioctl->ioctl_sem);
	init_MUTEX(&ql4im_ha->ioctl->ioctl_cmpl_sem);

	return QLA_SUCCESS;
}

/*
 * ql4im_free_ioctl_mem
 *	Frees memory used by IOCTL code for the specified ha.
 *
 * Input:
 *	ql4im_ha = adapter state pointer.
 *
 * Context:
 *	Kernel context.
 */
void
ql4im_free_ioctl_mem(ql4im_scsi_qla_host_t *ql4im_ha)
{
	if (ql4im_ha->ioctl != NULL) {

		if (ql4im_ha->ioctl->scrap_mem != NULL) {
			/* The size here must match up to what we
			 * allocated before.
			 */
			QL_KMEM_FREE(ql4im_ha->ioctl->scrap_mem);
			ql4im_ha->ioctl->scrap_mem = NULL;
			ql4im_ha->ioctl->scrap_mem_size = 0;
		}

		if (ql4im_ha->ioctl->aen_tracking_queue != NULL) {
			QL_KMEM_FREE(ql4im_ha->ioctl->aen_tracking_queue);
			ql4im_ha->ioctl->aen_tracking_queue = NULL;
		}

		if (ql4im_ha->ioctl->core_dump_mem) {
		        DEBUG4(printk("scsi%d: %s: freeing IOCTL Core Dump Buffer %p\n",
			    ql4im_ha->host_no, __func__, ql4im_ha->ioctl->core_dump_mem));
		
		        vfree(ql4im_ha->ioctl->core_dump_mem);
		        ql4im_ha->ioctl->core_dump_mem = NULL;
		}

		QL_KMEM_FREE(ql4im_ha->ioctl);
		ql4im_ha->ioctl = NULL;
	}

	if (ql4im_ha->ioctl_dma_bufv) {
		DEBUG4(printk("qisioctl%d: %s: freeing IOCTL DMA Buffers\n",
		    ql4im_ha->host_no, __func__));
		pci_free_consistent(ql4im_ha->drvr_ha->pdev, ql4im_ha->ioctl_dma_buf_len,
		    ql4im_ha->ioctl_dma_bufv, ql4im_ha->ioctl_dma_bufp);
	}
	ql4im_ha->ioctl_dma_buf_len = 0;
	ql4im_ha->ioctl_dma_bufv = NULL;
	ql4im_ha->ioctl_dma_bufp = 0;
}

/*
 * ql4im_get_ioctl_scrap_mem
 *	Returns pointer to memory of the specified size from the scrap buffer.
 *	This can be called multiple times before the free call as long
 *	as the memory is to be used by the same ioctl command and
 *	there's still memory left in the scrap buffer.
 *
 * Input:
 *	ql4im_ha = adapter state pointer.
 *	ppmem = pointer to return a buffer pointer.
 *	size = size of buffer to return.
 *
 * Returns:
 *	local function return status code.
 *
 * Context:
 *	Kernel context.
 */
int
ql4im_get_ioctl_scrap_mem(ql4im_scsi_qla_host_t *ql4im_ha, void **ppmem, uint32_t size)
{
	int		ret = QLA_SUCCESS;
	uint32_t	free_mem;

	free_mem = ql4im_ha->ioctl->scrap_mem_size - ql4im_ha->ioctl->scrap_mem_used;

	/* Round memory allocation up to a long aligned size, if needed */
	if (size % sizeof(long)) {
		size = ((size / sizeof(long)) + 1) * sizeof(long);
	}

	if (free_mem >= size) {
		*ppmem = ql4im_ha->ioctl->scrap_mem + ql4im_ha->ioctl->scrap_mem_used;
		ql4im_ha->ioctl->scrap_mem_used += size;
	} else {
		DEBUG2(printk("qisioctl%d: %s: no more scrap memory.\n",
		    ql4im_ha->host_no, __func__));

		ret = QLA_ERROR;
	}

	return (ret);
}

/*
 * ql4im_free_ioctl_scrap_mem
 *	Makes the entire scrap buffer free for use.
 *
 * Input:
 *	ql4im_ha = adapter state pointer.
 *
 * Returns:
 *	qla4xxx local function return status code.
 *
 */
void
ql4im_free_ioctl_scrap_mem(ql4im_scsi_qla_host_t *ql4im_ha)
{
	memset(ql4im_ha->ioctl->scrap_mem, 0, ql4im_ha->ioctl->scrap_mem_size);
	ql4im_ha->ioctl->scrap_mem_used = 0;
}

int
qla4xxx_ioctl_init(void)
{
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,13)
	apidev_class = class_create(THIS_MODULE, "qla4xxx");
#else
	apidev_class = class_simple_create(THIS_MODULE, "qla4xxx");
#endif
	if (IS_ERR(apidev_class)) {
		DEBUG2(printk("qisioctl: %s: Unable to create sysfs class for qla4xxx.\n",
		    __func__));

		apidev_class = NULL;
		return 1;
	}
	DEBUG4(printk("qisioctl: %s: apidev_class=%p.\n", __func__, apidev_class));

	apidev_major = register_chrdev(0, "qla4xxx", &apidev_fops);
	if (apidev_major < 0) {
		DEBUG2(printk("qisioctl: %s: Unable to register CHAR device (%d)\n",
		    __func__, apidev_major));

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,13)
		class_destroy(apidev_class);
#else
		class_simple_destroy(apidev_class);
#endif
		apidev_class = NULL;

		return apidev_major;
	}
	DEBUG4(printk("qisioctl: %s: apidev_major=%d.\n", __func__, apidev_major));

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,13)
        class_device_create(apidev_class, NULL, MKDEV(apidev_major, 0), NULL,
            "qla4xxx");
#else
	class_simple_device_add(apidev_class, MKDEV(apidev_major, 0), NULL,
	    "qla4xxx");
#endif

#if defined(CONFIG_COMPAT) && !defined(CONFIG_IA64)
	ql4_apidev_init_32ioctl();
#endif

	return 0;
}

int
qla4xxx_ioctl_exit(void)
{
	if (!apidev_class)
		return 1;

#if defined(CONFIG_COMPAT) && !defined(CONFIG_IA64)
	ql4_apidev_cleanup_32ioctl();
#endif

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,13)
	class_device_destroy(apidev_class, MKDEV(apidev_major, 0));
#else
	class_simple_device_remove(MKDEV(apidev_major, 0));
#endif

	unregister_chrdev(apidev_major, "qla4xxx");

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,13)
	class_destroy(apidev_class);
#else
	class_simple_destroy(apidev_class);
#endif

	apidev_class = NULL;
	return 0;
}

/*
 * ioctl support functions
 */

__user void *
Q64BIT_TO_PTR(uint64_t buf_addr, uint16_t addr_mode)
{
#if defined(CONFIG_COMPAT) || !defined(CONFIG_IA64) || !defined(CONFIG_64BIT)
	union ql_doublelong {
		struct {
			uint32_t        lsl;
			uint32_t        msl;
		} longs;
		uint64_t        dl;
	};

	union ql_doublelong tmpval;

	tmpval.dl = buf_addr;

#if defined(CONFIG_COMPAT) && !defined(CONFIG_IA64)
	/* 32bit user - 64bit kernel */
	if (addr_mode == EXT_DEF_ADDR_MODE_32) {
		DEBUG9(printk("%s: got 32bit user address.\n", __func__);)
		return((void *)(uint64_t)(tmpval.longs.lsl));
	} else {
		DEBUG9(printk("%s: got 64bit user address.\n", __func__);)
		return((void *)buf_addr);
	}
#else
	return((void __user *)(tmpval.longs.lsl));
#endif
#else
	return((void __user *)buf_addr);
#endif
}

/**************************************************************************
 * qla4extioctl_query_hba_iscsi_node
 *	This routine retrieves the HBA node properties
 *
 * Input:
 *	ql4im_ha    = adapter structure pointer.
 *	ioctl = IOCTL structure pointer.
 *
 * Output:
 *	None
 *
 * Returns:
 *	QLA_SUCCESS = success
 *	QLA_ERROR   = error
 *
 * Context:
 *	Kernel context.
 **************************************************************************/
static int
qla4extioctl_query_hba_iscsi_node(ql4im_scsi_qla_host_t *ql4im_ha,
				  EXT_IOCTL_ISCSI *ioctl)
{
	int			status = 0;
	uint32_t		mbox_cmd[MBOX_REG_COUNT];
	uint32_t		mbox_sts[MBOX_REG_COUNT];
	EXT_HBA_ISCSI_NODE	*phba_node = NULL;
	ADDRESS_CTRL_BLK	*init_fw_cb;

	if (ql4im_get_ioctl_scrap_mem(ql4im_ha, (void **)&phba_node,
	    sizeof(EXT_HBA_ISCSI_NODE))) {
		/* not enough memory */
		ioctl->Status = EXT_STATUS_NO_MEMORY;
		DEBUG2(printk("qisioctl%d: %s: inst=%d scrap not big enough. "
		    "size requested=%ld.\n",
		    ql4im_ha->host_no, __func__, ql4im_ha->instance,
		    (ulong)sizeof(EXT_HBA_ISCSI_NODE)));
		goto exit_query_hba_node;
	}

	if (!ql4im_ha->ioctl_dma_bufv ||
	    !ql4im_ha->ioctl_dma_bufp ||
	    !ioctl->ResponseAdr) {
		DEBUG2(printk("qisioctl%d: %s: memory allocation problem\n",
		    ql4im_ha->host_no, __func__));

		ioctl->Status = EXT_STATUS_NO_MEMORY;
		goto exit_query_hba_node;
	}

	if (ioctl->ResponseLen < sizeof(EXT_HBA_ISCSI_NODE) ||
	    ql4im_ha->ioctl_dma_buf_len < sizeof(EXT_HBA_ISCSI_NODE)) {
		DEBUG2(printk("qisioctl%d: %s: memory area too small\n",
		    ql4im_ha->host_no, __func__));

		ioctl->Status = EXT_STATUS_BUFFER_TOO_SMALL;
		goto exit_query_hba_node;
	}

	/*
	 * Send mailbox command
	 */
	memset(mbox_cmd, 0, sizeof(mbox_cmd));
	memset(mbox_sts, 0, sizeof(mbox_sts));
	mbox_cmd[0] = MBOX_CMD_GET_INIT_FW_CTRL_BLOCK;
	mbox_cmd[2] = LSDW(ql4im_ha->ioctl_dma_bufp);
	mbox_cmd[3] = MSDW(ql4im_ha->ioctl_dma_bufp);

	if (ql4im_ha->drvr_ha->ql4_mbx(ql4im_ha->drvr_ha, 5, 1, &mbox_cmd[0],
				  &mbox_sts[0]) == QLA_ERROR) {
		DEBUG2(printk("qisioctl%d: %s: command failed \n",
		    ql4im_ha->host_no, __func__));

		ioctl->Status = EXT_STATUS_MAILBOX;
		ioctl->VendorSpecificStatus[0] = mbox_sts[0];
		ioctl->ResponseLen = 0;

		goto exit_query_hba_node;
	}

	/*
	 * Transfer data from Fw's DEV_DB_ENTRY buffer to IOCTL's
	 * EXT_HBA_ISCSI_NODE buffer
	 */
	init_fw_cb = (ADDRESS_CTRL_BLK *) ql4im_ha->ioctl_dma_bufv;

	memset(phba_node, 0, sizeof(EXT_HBA_ISCSI_NODE));
	phba_node->PortNumber = le16_to_cpu(init_fw_cb->PortNumber);
	phba_node->NodeInfo.PortalCount = 1;

	memcpy(phba_node->NodeInfo.IPAddr.IPAddress, init_fw_cb->IPAddr,
	    min(sizeof(phba_node->NodeInfo.IPAddr.IPAddress),
		sizeof(init_fw_cb->IPAddr)));
	memcpy(phba_node->NodeInfo.iSCSIName, init_fw_cb->iSCSINameString,
	    min(sizeof(phba_node->NodeInfo.iSCSIName),
		sizeof(init_fw_cb->iSCSINameString)));
	memcpy(phba_node->NodeInfo.Alias, init_fw_cb->iSCSIAlias,
	    min(sizeof(phba_node->NodeInfo.Alias),
		sizeof(init_fw_cb->iSCSIAlias)));

	sprintf(phba_node->DeviceName, "/proc/scsi/qla4xxx/%d",
	    ql4im_ha->host_no);

	/*
	 * Copy the IOCTL EXT_HBA_ISCSI_NODE buffer to the user's data space
	 */
	if (copy_to_user(Q64BIT_TO_PTR(ioctl->ResponseAdr, ioctl->AddrMode),
				   phba_node, ioctl->ResponseLen) != 0) {
		DEBUG2(printk("qisioctl%d: %s: copy failed\n",
			ql4im_ha->host_no, __func__));

		status = (-EFAULT);
		ioctl->Status = EXT_STATUS_COPY_ERR;
		ioctl->ResponseLen = 0;
		goto exit_query_hba_node;
	}

	ioctl->Status = EXT_STATUS_OK;

exit_query_hba_node:
	ql4im_free_ioctl_scrap_mem(ql4im_ha);
	return(status);
}

static int
ql4_get_flash_sys_info(struct ql4im_scsi_qla_host *ql4im_ha,
                   FLASH_SYS_INFO *sys_info,
                   dma_addr_t sys_info_dma,
                   uint32_t *mbox_status)
{
        uint32_t mbox_cmd[MBOX_REG_COUNT];
        uint32_t mbox_sts[MBOX_REG_COUNT];
        memset(&mbox_cmd, 0, sizeof(mbox_cmd));
        memset(&mbox_sts, 0, sizeof(mbox_sts));
        mbox_cmd[0] = MBOX_CMD_READ_FLASH;
        mbox_cmd[1] = LSDW(sys_info_dma);
        mbox_cmd[2] = MSDW(sys_info_dma);
        mbox_cmd[3] = INT_ISCSI_SYSINFO_FLASH_OFFSET;
        mbox_cmd[4] = sizeof(*sys_info);

        if (ql4im_ha->drvr_ha->ql4_mbx(ql4im_ha->drvr_ha, MBOX_REG_COUNT, 2, &mbox_cmd[0], &mbox_sts[0]) != QLA_SUCCESS) {
                *mbox_status = mbox_sts[0];
                return QLA_ERROR;
        }
        return QLA_SUCCESS;
}

static int
ql4_get_mbx_sys_info(struct ql4im_scsi_qla_host *ql4im_ha,
                 MBX_SYS_INFO *sys_info,
                 dma_addr_t sys_info_dma,
                 uint32_t *mbox_status)
{
        uint32_t mbox_cmd[MBOX_REG_COUNT];
        uint32_t mbox_sts[MBOX_REG_COUNT];
        memset(&mbox_cmd, 0, sizeof(mbox_cmd));
        memset(&mbox_sts, 0, sizeof(mbox_sts));
        mbox_cmd[0] = MBOX_CMD_GET_SYS_INFO;
        mbox_cmd[1] = LSDW(sys_info_dma);
        mbox_cmd[2] = MSDW(sys_info_dma);
        mbox_cmd[4] = sizeof(*sys_info);

        if (ql4im_ha->drvr_ha->ql4_mbx(ql4im_ha->drvr_ha, MBOX_REG_COUNT, 6, &mbox_cmd[0], &mbox_sts[0]) != QLA_SUCCESS) {
                *mbox_status = mbox_sts[0];
                return QLA_ERROR;
        }
	/* Make sure we receive the minimum required data to cache internally */
	if (mbox_sts[4] < offsetof(MBX_SYS_INFO, reserved)) {
                DEBUG2(printk("scsi%d: %s: GET_SYS_INFO data receive error (%x)\n",
                        ql4im_ha->drvr_ha->host_no, __func__, mbox_sts[4]));
                *mbox_status = mbox_sts[0];
                return QLA_ERROR;
        }

        return QLA_SUCCESS;
}

/**************************************************************************
 * qla4extioctl_query_hba_iscsi_portal
 *	This routine retrieves the HBA iSCSI portal properties
 *
 * Input:
 *	ql4im_ha    = adapter structure pointer.
 *	ioctl = IOCTL structure pointer.
 *
 * Output:
 *	None
 *
 * Returns:
 *	QLA_SUCCESS = success
 *	QLA_ERROR   = error
 *
 * Context:
 *	Kernel context.
 **************************************************************************/
static int
qla4extioctl_query_hba_iscsi_portal(ql4im_scsi_qla_host_t *ql4im_ha,
				    EXT_IOCTL_ISCSI *ioctl)
{
	int			status = 0;
	uint32_t		mbox_cmd[MBOX_REG_COUNT];
	uint32_t		mbox_sts[MBOX_REG_COUNT];
	EXT_HBA_ISCSI_PORTAL	*phba_portal;
	uint32_t 		num_valid_ddb_entries;


	if (!ioctl->ResponseAdr) {
		DEBUG2(printk("qisioctl%d: %s: no response buffer found.\n",
		    ql4im_ha->host_no, __func__));

		status = (-ENOMEM);
		ioctl->Status = EXT_STATUS_NO_MEMORY;
		ioctl->ResponseLen = 0;
		goto exit_query_hba_portal;
	}

	if (ql4im_get_ioctl_scrap_mem(ql4im_ha, (void **)&phba_portal,
	    sizeof(EXT_HBA_ISCSI_PORTAL))) {
		/* not enough memory */
		ioctl->Status = EXT_STATUS_NO_MEMORY;
		DEBUG2(printk("qisioctl%d: %s: inst=%d scrap not big enough. "
		    "size requested=%ld.\n",
		    ql4im_ha->host_no, __func__, ql4im_ha->instance,
		    (ulong)sizeof(EXT_HBA_ISCSI_PORTAL)));

		status = (-ENOMEM);
		ioctl->Status = EXT_STATUS_NO_MEMORY;
		ioctl->ResponseLen = 0;
		goto exit_query_hba_portal;
	}

	if (ioctl->ResponseLen < sizeof(*phba_portal)) {
		DEBUG2(printk("qisioctl%d: %s: memory area too small\n",
		    ql4im_ha->host_no, __func__));

		status = (-ENOMEM);
		ioctl->Status = EXT_STATUS_BUFFER_TOO_SMALL;
		ioctl->ResponseLen = 0;
		goto exit_query_hba_portal;
	}

	/*
	 * Fill in EXT_HBA_ISCSI_PORTAL buffer
	 */
	memset(phba_portal, 0, sizeof(EXT_HBA_ISCSI_PORTAL));

	strcpy(phba_portal->DriverVersion, ql4im_ha->drv_verstr);
	sprintf(phba_portal->FWVersion, "%02d.%02d Patch %02d Build %02d",
		ql4im_ha->firmware_version[0], ql4im_ha->firmware_version[1],
		ql4im_ha->patch_number, ql4im_ha->build_number);

	/* ----- Get firmware state information ---- */
	memset(mbox_cmd, 0, sizeof(mbox_cmd));
	memset(mbox_sts, 0, sizeof(mbox_sts));
	mbox_cmd[0] = MBOX_CMD_GET_FW_STATE;
	if (ql4im_ha->drvr_ha->ql4_mbx(ql4im_ha->drvr_ha, 1, 4, &mbox_cmd[0], &mbox_sts[0]) !=
	    QLA_SUCCESS) {
		DEBUG2(printk("qisioctl%d: %s: MBOX_CMD_GET_FW_STATE "
		    "failed w/ status %04X\n",
		    ql4im_ha->host_no, __func__, mbox_sts[0]));

		ioctl->Status = EXT_STATUS_MAILBOX;
		ioctl->VendorSpecificStatus[0] = mbox_sts[0];
		ioctl->VendorSpecificStatus[1] = mbox_sts[1];
		ioctl->VendorSpecificStatus[2] = mbox_sts[2];
		ioctl->VendorSpecificStatus[3] = mbox_sts[3];
		ioctl->ResponseLen = 0;
		goto exit_query_hba_portal;
	}

	switch (mbox_sts[1]) {
	case FW_STATE_READY:
		phba_portal->State = EXT_DEF_CARD_STATE_READY;
		break;
	case FW_STATE_CONFIG_WAIT:
		phba_portal->State = EXT_DEF_CARD_STATE_CONFIG_WAIT;
		break;
	case FW_STATE_WAIT_AUTOCONNECT:
		phba_portal->State = EXT_DEF_CARD_STATE_LOGIN;
		break;
	case FW_STATE_ERROR:
		phba_portal->State = EXT_DEF_CARD_STATE_ERROR;
		break;
	}

	switch (mbox_sts[3] & 0x0001) {
	case FW_ADDSTATE_COPPER_MEDIA:
		phba_portal->Type = EXT_DEF_TYPE_COPPER;
		break;
	case FW_ADDSTATE_OPTICAL_MEDIA:
		phba_portal->Type = EXT_DEF_TYPE_OPTICAL;
		break;
	}

	/* ----- Get ddb entry information ---- */
	if (ql4im_mbx_get_fwddb_entry(ql4im_ha->drvr_ha, 0, NULL, 0, &num_valid_ddb_entries,
	    NULL, NULL, NULL, NULL, NULL) == QLA_ERROR) {
		DEBUG2(printk("qisioctl%d: %s: ql4im_mbx_get_ddb_entry failed!\n",
		    ql4im_ha->host_no, __func__));

		status = (-EFAULT);
		ioctl->Status = EXT_STATUS_ERR;
		ioctl->RequestLen = 0;

		goto exit_query_hba_portal;
	}

	phba_portal->DiscTargetCount = (uint16_t) num_valid_ddb_entries;

	if (IS_QLA80XX(ql4im_ha->drvr_ha)) {
		/* Get Sys Info from Mailbox */
		MBX_SYS_INFO *sys_info;
		sys_info = (MBX_SYS_INFO *) ql4im_ha->ioctl_dma_bufv;

		status = ql4_get_mbx_sys_info(ql4im_ha, sys_info, ql4im_ha->ioctl_dma_bufp, &mbox_sts[0]);
		if (status == QLA_SUCCESS) {
			phba_portal->SerialNum = le32_to_cpu(sys_info->serial_number);
			memcpy(phba_portal->IPAddr.IPAddress, ql4im_ha->ip_address,
				MIN(sizeof(phba_portal->IPAddr.IPAddress),
				sizeof(ql4im_ha->ip_address)));
			memcpy(phba_portal->MacAddr, sys_info->mac_addr,
				sizeof(phba_portal->MacAddr));
			memcpy(phba_portal->Model, sys_info->board_id_str,
				sizeof(phba_portal->Model));

			/*memcpy(phba_portal->OptRomVersion, ?,
				sizeof(phba_portal->OptRomVersion)); */
		}
	} else {
		/* Get Sys Info from Flash */
		FLASH_SYS_INFO *sys_info;
		sys_info = (FLASH_SYS_INFO *) ql4im_ha->ioctl_dma_bufv;

		status = ql4_get_flash_sys_info(ql4im_ha, sys_info, ql4im_ha->ioctl_dma_bufp, &mbox_sts[0]);
		if (status == QLA_SUCCESS) {
			phba_portal->SerialNum = le32_to_cpu(sys_info->serialNumber);
			memcpy(phba_portal->IPAddr.IPAddress, ql4im_ha->ip_address,
			   MIN(sizeof(phba_portal->IPAddr.IPAddress),
			       sizeof(ql4im_ha->ip_address)));
			memcpy(phba_portal->MacAddr, sys_info->physAddr[0].address,
			   sizeof(phba_portal->MacAddr));
			memcpy(phba_portal->Manufacturer, sys_info->vendorId,
			   sizeof(phba_portal->Manufacturer));
			memcpy(phba_portal->Model, sys_info->productId,
			   sizeof(phba_portal->Model));
		}
	}



	/*
	 * Copy the IOCTL EXT_HBA_ISCSI_PORTAL buffer to the user's data space
	 */
	if (copy_to_user(Q64BIT_TO_PTR(ioctl->ResponseAdr,
						 ioctl->AddrMode),
                                   phba_portal, ioctl->ResponseLen) != 0) {
		DEBUG2(printk("qisioctl%d: %s: memory area too small\n",
		    ql4im_ha->host_no, __func__));

		status = (-EFAULT);
		ioctl->Status = EXT_STATUS_COPY_ERR;
		ioctl->ResponseLen = 0;
		goto exit_query_hba_portal;
	}

	ioctl->Status = EXT_STATUS_OK;

exit_query_hba_portal:
	ql4im_free_ioctl_scrap_mem(ql4im_ha);
	return(status);
}

/**************************************************************************
 * qla4extioctl_query_disc_iscsi_node
 *	This routine retrieves the properties of the attached devices
 *	registered as iSCSI nodes discovered by the HBA driver.
 *
 * Input:
 *	ql4im_ha    = adapter structure pointer.
 *	ioctl = IOCTL structure pointer.
 *
 * Output:
 *	None
 *
 * Returns:
 *	QLA_SUCCESS = success
 *	QLA_ERROR   = error
 *
 * Context:
 *	Kernel context.
 **************************************************************************/
static int
qla4extioctl_query_disc_iscsi_node(ql4im_scsi_qla_host_t *ql4im_ha, EXT_IOCTL_ISCSI *ioctl)
{
	int		status = 0;
	DEV_DB_ENTRY 	*fw_ddb_entry = (DEV_DB_ENTRY *) ql4im_ha->ioctl_dma_bufv;
	EXT_DISC_ISCSI_NODE *pdisc_node = NULL;
	ddb_entry_t 	*ddb_entry;

	if (ioctl->ResponseLen < sizeof(EXT_DISC_ISCSI_NODE)) {
		DEBUG2(printk("qisioctl%d: %s: response buffer too small\n",
		    ql4im_ha->host_no, __func__));

		status = (-ENOMEM);
		ioctl->Status = EXT_STATUS_BUFFER_TOO_SMALL;
		ioctl->ResponseLen = 0;
		goto exit_disc_node;
	}

	if (ql4im_ha->ioctl_dma_buf_len < sizeof(DEV_DB_ENTRY)) {
		if (ql4im_resize_ioctl_dma_buf(ql4im_ha, sizeof(DEV_DB_ENTRY)) !=
		    QLA_SUCCESS) {
			DEBUG2(printk("qisioctl%d: %s: unable to allocate memory "
			    "for dma buffer.\n",
			    ql4im_ha->host_no, __func__));

			status = (-ENOMEM);
			ioctl->Status = EXT_STATUS_NO_MEMORY;
			ioctl->ResponseLen = 0;
			goto exit_disc_node;
		}
	}

	if (ql4im_get_ioctl_scrap_mem(ql4im_ha, (void **)&pdisc_node,
	    sizeof(EXT_DISC_ISCSI_NODE))) {
		/* not enough memory */
		DEBUG2(printk("qisioctl%d: %s: inst=%d scrap not big enough. "
		    "size requested=%ld.\n",
		    ql4im_ha->host_no, __func__, ql4im_ha->instance,
		    (ulong)sizeof(EXT_DISC_ISCSI_NODE)));

		status = (-ENOMEM);
		ioctl->Status = EXT_STATUS_NO_MEMORY;
		ioctl->ResponseLen = 0;
		goto exit_disc_node;
	}

	/* ----- get device database entry info from firmware ---- */
	if (ql4im_mbx_get_fwddb_entry(ql4im_ha->drvr_ha, ioctl->Instance, fw_ddb_entry,
	    ql4im_ha->ioctl_dma_bufp, NULL, NULL, NULL, NULL, NULL, NULL) !=
	    QLA_SUCCESS) {
		DEBUG2(printk("qisioctl%d: %s: failed to get DEV_DB_ENTRY "
		    "info.\n", ql4im_ha->host_no, __func__));

		status = (-EFAULT);
		ioctl->Status = EXT_STATUS_ERR;
		ioctl->RequestLen = 0;
		goto exit_disc_node;
	}

	/* --- Transfer data from Fw's DEV_DB_ENTRY buffer to
	*      IOCTL's EXT_DISC_ISCSI_PORTAL buffer --- */
	memset(pdisc_node, 0, sizeof(EXT_DISC_ISCSI_NODE));
	pdisc_node->NodeInfo.PortalCount = 1;
	pdisc_node->NodeInfo.IPAddr.Type = EXT_DEF_TYPE_ISCSI_IP;
	memcpy(pdisc_node->NodeInfo.IPAddr.IPAddress, fw_ddb_entry->RemoteIPAddr,
	    MIN(sizeof(pdisc_node->NodeInfo.IPAddr.IPAddress),
	    sizeof(fw_ddb_entry->RemoteIPAddr)));
	strncpy(pdisc_node->NodeInfo.Alias, fw_ddb_entry->iSCSIAlias,
	    MIN(sizeof(pdisc_node->NodeInfo.Alias),
	    sizeof(fw_ddb_entry->iSCSIAlias)));
	strncpy(pdisc_node->NodeInfo.iSCSIName, fw_ddb_entry->iscsiName,
	    MIN(sizeof(pdisc_node->NodeInfo.iSCSIName),
	    sizeof(fw_ddb_entry->iscsiName)));

	ddb_entry = ql4im_lookup_ddb_by_fw_index(ql4im_ha->drvr_ha, ioctl->Instance);

	/* WORKAROUND: Sometimes another procesor is processing the AEN
	 * to add this DDB, so yield control for other processors to
	 * execute and check again */
	if (ddb_entry == NULL) {
		set_current_state(TASK_INTERRUPTIBLE);
		schedule_timeout(1 * HZ);
		ddb_entry = ql4im_lookup_ddb_by_fw_index(ql4im_ha->drvr_ha, ioctl->Instance);
	}

	if (ddb_entry == NULL) {
		DEBUG2(printk("qisioctl%d: %s: device index [%d] not logged in. "
		    "Dummy target info returned.\n",
		    ql4im_ha->host_no, __func__, ioctl->Instance));

		pdisc_node->SessionID       = 0xDEAD;
		pdisc_node->ConnectionID    = 0xDEAD;
		pdisc_node->PortalGroupID   = 0xDEAD;
		pdisc_node->ScsiAddr.Bus    = 0xFF;
		pdisc_node->ScsiAddr.Target = 0xFF;
		pdisc_node->ScsiAddr.Lun    = 0xFF;
	}
	else {
		pdisc_node->SessionID       = ddb_entry->target_session_id;
		pdisc_node->ConnectionID    = ddb_entry->connection_id;
		pdisc_node->PortalGroupID   = 0;
		pdisc_node->ScsiAddr.Bus    = 0;
		pdisc_node->ScsiAddr.Target = ddb_entry->fcport->os_target_id;
		pdisc_node->ScsiAddr.Lun    = 0;
	}

	/* --- Copy Results to user space --- */
	if (copy_to_user(Q64BIT_TO_PTR(ioctl->ResponseAdr,
						 ioctl->AddrMode),
                                   pdisc_node,
				   sizeof(EXT_DISC_ISCSI_NODE)) != 0) {
		DEBUG2(printk("qisioctl%d: %s: copy error to user space.\n",
		    ql4im_ha->host_no, __func__));

		status = (-EFAULT);
		ioctl->Status = EXT_STATUS_COPY_ERR;
		ioctl->ResponseLen = 0;
		goto exit_disc_node;
	}

	ioctl->Status = EXT_STATUS_OK;

exit_disc_node:
	ql4im_free_ioctl_scrap_mem(ql4im_ha);
	return(status);
}

/**************************************************************************
 * qla4extioctl_query_disc_iscsi_portal
 *	This routine retrieves the properties of the iSCSI portal
 *	discovered by the HBA driver.
 *
 * Input:
 *	ql4im_ha    = adapter structure pointer.
 *	ioctl = IOCTL structure pointer.
 *
 * Output:
 *	None
 *
 * Returns:
 *	QLA_SUCCESS = success
 *	QLA_ERROR   = error
 *
 * Context:
 *	Kernel context.
 **************************************************************************/
static int
qla4extioctl_query_disc_iscsi_portal(ql4im_scsi_qla_host_t *ql4im_ha,
    EXT_IOCTL_ISCSI *ioctl)
{
	int		status = 0;
	DEV_DB_ENTRY 	*fw_ddb_entry = (DEV_DB_ENTRY *) ql4im_ha->ioctl_dma_bufv;
	EXT_DISC_ISCSI_PORTAL *pdisc_portal = NULL;

	if (ql4im_get_ioctl_scrap_mem(ql4im_ha, (void **)&pdisc_portal,
	    sizeof(EXT_DISC_ISCSI_PORTAL))) {
		/* not enough memory */
		DEBUG2(printk("qisioctl%d: %s: inst=%d scrap not big enough. "
		    "size requested=%ld.\n",
		    ql4im_ha->host_no, __func__, ql4im_ha->instance,
		    (ulong)sizeof(EXT_DISC_ISCSI_PORTAL)));

		status = (-ENOMEM);
		ioctl->Status = EXT_STATUS_NO_MEMORY;
		ioctl->ResponseLen = 0;
		goto exit_disc_portal;
	}

	if (ioctl->ResponseLen < sizeof(EXT_DISC_ISCSI_PORTAL)) {
		DEBUG2(printk("qisioctl%d: %s: response buffer too small\n",
		    ql4im_ha->host_no, __func__));

		status = (-ENOMEM);
		ioctl->Status = EXT_STATUS_BUFFER_TOO_SMALL;
		ioctl->ResponseLen = 0;
		goto exit_disc_portal;
	}

	if (ql4im_ha->ioctl_dma_buf_len < sizeof(DEV_DB_ENTRY)) {
		if (ql4im_resize_ioctl_dma_buf(ql4im_ha, sizeof(DEV_DB_ENTRY)) !=
		    QLA_SUCCESS) {
			DEBUG2(printk("qisioctl%d: %s: unable to allocate memory "
			    "for dma buffer.\n",
			    ql4im_ha->host_no, __func__));

			status = (-ENOMEM);
			ioctl->Status = EXT_STATUS_NO_MEMORY;
			ioctl->ResponseLen = 0;
			goto exit_disc_portal;
		}
	}

	/* ----- get device database entry info from firmware ---- */
	if (ql4im_mbx_get_fwddb_entry(ql4im_ha->drvr_ha, ioctl->Instance, fw_ddb_entry,
	    ql4im_ha->ioctl_dma_bufp, NULL, NULL, NULL, NULL, NULL, NULL) !=
	    QLA_SUCCESS) {
		DEBUG2(printk("qisioctl%d: %s: failed to get DEV_DB_ENTRY info.\n",
		    ql4im_ha->host_no, __func__));

		status = (-EFAULT);
		ioctl->Status = EXT_STATUS_ERR;
		ioctl->RequestLen = 0;
		goto exit_disc_portal;
	}

	/* --- Transfer data from Fw's DEV_DB_ENTRY buffer to IOCTL's
	*      EXT_DISC_ISCSI_PORTAL buffer --- */
	memset(pdisc_portal, 0, sizeof(EXT_DISC_ISCSI_PORTAL));
	memcpy(pdisc_portal->IPAddr.IPAddress, fw_ddb_entry->RemoteIPAddr,
	    MIN(sizeof(pdisc_portal->IPAddr.IPAddress),
	    sizeof(fw_ddb_entry->RemoteIPAddr)));

	pdisc_portal->PortNumber = le16_to_cpu(fw_ddb_entry->RemoteTCPPortNumber);
	pdisc_portal->IPAddr.Type = EXT_DEF_TYPE_ISCSI_IP;
	pdisc_portal->NodeCount = 0;

	strncpy(pdisc_portal->HostName, fw_ddb_entry->iscsiName,
	    MIN(sizeof(pdisc_portal->HostName),
	    sizeof(fw_ddb_entry->iscsiName)));

	/* --- Copy Results to user space --- */
	if (copy_to_user(Q64BIT_TO_PTR(ioctl->ResponseAdr,
						 ioctl->AddrMode),
                                   pdisc_portal,
				   sizeof(EXT_DISC_ISCSI_PORTAL)) != 0) {
		DEBUG2(printk("qisioctl%d: %s: copy error to user space.\n",
		    ql4im_ha->host_no, __func__));

		status = (-EFAULT);
		ioctl->Status = EXT_STATUS_COPY_ERR;
		ioctl->ResponseLen = 0;
		goto exit_disc_portal;
	}

	ioctl->Status = EXT_STATUS_OK;

exit_disc_portal:
	ql4im_free_ioctl_scrap_mem(ql4im_ha);
	return(status);
}

/**************************************************************************
 * qla4extioctl_query_driver
 *	This routine retrieves the driver properties.
 *
 * Input:
 *	ql4im_ha    = adapter structure pointer.
 *	ioctl = IOCTL structure pointer.
 *
 * Output:
 *	None
 *
 * Returns:
 *	QLA_SUCCESS = success
 *	QLA_ERROR   = error
 *
 * Context:
 *	Kernel context.
 **************************************************************************/
static int
qla4extioctl_query_driver(ql4im_scsi_qla_host_t *ql4im_ha, EXT_IOCTL_ISCSI *ioctl)
{
	EXT_DRIVER_INFO	*pdinfo = NULL;
	int		status = 0;

	if (ql4im_get_ioctl_scrap_mem(ql4im_ha, (void **)&pdinfo,
	    sizeof(EXT_DRIVER_INFO))) {
		/* not enough memory */
		DEBUG2(printk("qisioctl%d: %s: inst=%d scrap not big enough. "
		    "size requested=%ld.\n",
		    ql4im_ha->host_no, __func__, ql4im_ha->instance,
		    (ulong)sizeof(EXT_DRIVER_INFO)));

		status = (-ENOMEM);
		ioctl->Status = EXT_STATUS_NO_MEMORY;
		ioctl->ResponseLen = 0;
		goto exit_query_driver;
	}

	if (ioctl->ResponseLen < sizeof(EXT_DRIVER_INFO)) {
		DEBUG2(printk("qisioctl%d: %s: response buffer too small\n",
		    ql4im_ha->host_no, __func__));

		status = (-ENOMEM);
		ioctl->Status = EXT_STATUS_BUFFER_TOO_SMALL;
		ioctl->ResponseLen = 0;
		goto exit_query_driver;
	}

	memset(pdinfo, 0, sizeof(EXT_DRIVER_INFO));
	memcpy(pdinfo->Version, ql4im_ha->drv_verstr,
	       MIN(sizeof(pdinfo->Version),
		   sizeof(ql4im_ha->drv_verstr)));

	pdinfo->NumOfBus        = EXT_DEF_MAX_HBA;
	pdinfo->TargetsPerBus   = EXT_DEF_MAX_TARGET;
	pdinfo->LunPerTarget    = EXT_DEF_MAX_LUN;
	pdinfo->LunPerTargetOS  = EXT_DEF_MAX_BUS;

	if (sizeof(dma_addr_t) > 4)
		pdinfo->DmaBitAddresses = 1;  /* 64-bit */
	else
		pdinfo->DmaBitAddresses = 0;  /* 32-bit */

	if (ql4im_ha->drvr_ha->mem_addr)
		pdinfo->IoMapType       = 1;  /* Memory Mapped I/O */
	else
		pdinfo->IoMapType       = 0;  /* I/O Mapped I/O */

	if (copy_to_user(Q64BIT_TO_PTR(ioctl->ResponseAdr,
						 ioctl->AddrMode), pdinfo,
                                   sizeof(EXT_DRIVER_INFO)) != 0) {
		status = (-EFAULT);
		DEBUG2(printk("qisioctl%d: %s: error copy to response buffer.\n",
		    ql4im_ha->host_no, __func__));

		status = (-EFAULT);
		ioctl->Status = EXT_STATUS_COPY_ERR;
		ioctl->ResponseLen = 0;
		goto exit_query_driver;
	}

	ioctl->Status = EXT_STATUS_OK;

exit_query_driver:
	ql4im_free_ioctl_scrap_mem(ql4im_ha);
	return(status);
}

/**************************************************************************
 * qla4extioctl_query_fw
 *	This routine retrieves the firmware properties.
 *
 * Input:
 *	ql4im_ha    = adapter structure pointer.
 *	ioctl = IOCTL structure pointer.
 *
 * Output:
 *	None
 *
 * Returns:
 *	QLA_SUCCESS = success
 *	QLA_ERROR   = error
 *
 * Context:
 *	Kernel context.
 **************************************************************************/
static int
qla4extioctl_query_fw(ql4im_scsi_qla_host_t *ql4im_ha, EXT_IOCTL_ISCSI *ioctl)
{
	EXT_FW_INFO	*pfw_info = NULL;
	uint32_t	mbox_cmd[MBOX_REG_COUNT];
	uint32_t	mbox_sts[MBOX_REG_COUNT];
	int		status = 0;

	if (ql4im_get_ioctl_scrap_mem(ql4im_ha, (void **)&pfw_info,
	    sizeof(EXT_FW_INFO))) {
		/* not enough memory */
		DEBUG2(printk("qisioctl%d: %s: inst=%d scrap not big enough. "
		    "size requested=%ld.\n",
		    ql4im_ha->host_no, __func__, ql4im_ha->instance,
		    (ulong)sizeof(EXT_FW_INFO)));

		status = (-ENOMEM);
		ioctl->Status = EXT_STATUS_NO_MEMORY;
		ioctl->ResponseLen = 0;
		goto exit_query_fw;
	}

	if (ioctl->ResponseLen < sizeof(EXT_FW_INFO)) {
		DEBUG2(printk("qisioctl%d: %s: response buffer too small\n",
		    ql4im_ha->host_no, __func__));

		status = (-ENOMEM);
		ioctl->Status = EXT_STATUS_BUFFER_TOO_SMALL;
		ioctl->ResponseLen = 0;
		goto exit_query_fw;
	}

	/* Fill in structure */
	memset(pfw_info, 0, sizeof(EXT_FW_INFO));

	/* ----- Get firmware version information ---- */
	memset(mbox_cmd, 0, sizeof(mbox_cmd));
	memset(mbox_sts, 0, sizeof(mbox_sts));
	mbox_cmd[0] = MBOX_CMD_ABOUT_FW;

	/*
	 * NOTE: In QLA4010, mailboxes 2 & 3 may hold an address for data.
	 * Make sure that we write 0 to those mailboxes, if unused.
	 */
	if (ql4im_ha->drvr_ha->ql4_mbx(ql4im_ha->drvr_ha, 4, 5, &mbox_cmd[0], &mbox_sts[0]) !=
	    QLA_SUCCESS) {
		DEBUG2(printk("qisioctl%d: %s: MBOX_CMD_ABOUT_FW failed w/ "
		    "status %04X\n",
		    ql4im_ha->host_no, __func__, mbox_sts[0]));

		ioctl->Status = EXT_STATUS_MAILBOX;
		ioctl->VendorSpecificStatus[0] = mbox_sts[0];
		ioctl->VendorSpecificStatus[1] = mbox_sts[1];
		ioctl->VendorSpecificStatus[2] = mbox_sts[2];
		ioctl->VendorSpecificStatus[3] = mbox_sts[3];
		ioctl->VendorSpecificStatus[4] = mbox_sts[4];
		ioctl->ResponseLen = 0;
		goto exit_query_fw;
	}

	sprintf(pfw_info->Version, "FW Version %d.%d Patch %d Build %d",
	    mbox_sts[1], mbox_sts[2], mbox_sts[3], mbox_sts[4]);

	/* Copy info to caller */
	if (copy_to_user(Q64BIT_TO_PTR(ioctl->ResponseAdr,
						 ioctl->AddrMode), pfw_info,
                                   sizeof(EXT_FW_INFO)) != 0) {
		DEBUG2(printk("qisioctl%d: %s: response copy error.\n",
		    ql4im_ha->host_no, __func__));

		status = (-EFAULT);
		ioctl->Status = EXT_STATUS_COPY_ERR;
		ioctl->ResponseLen = 0;
		goto exit_query_fw;
	}

	ioctl->Status = EXT_STATUS_OK;

exit_query_fw:
	ql4im_free_ioctl_scrap_mem(ql4im_ha);
	return(status);
}

/**************************************************************************
 * qla4extioctl_query_chip
 *	This routine retrieves the chip properties.
 *
 * Input:
 *	ql4im_ha    = adapter structure pointer.
 *	ioctl = IOCTL structure pointer.
 *
 * Output:
 *	None
 *
 * Returns:
 *	QLA_SUCCESS = success
 *	QLA_ERROR   = error
 *
 * Context:
 *	Kernel context.
 **************************************************************************/
static int
qla4extioctl_query_chip(ql4im_scsi_qla_host_t *ql4im_ha, EXT_IOCTL_ISCSI *ioctl)
{
	int		status = 0;
	uint32_t	mbox_cmd[MBOX_REG_COUNT];
	uint32_t	mbox_sts[MBOX_REG_COUNT];
	EXT_CHIP_INFO	*pchip_info = NULL;


	if (ql4im_get_ioctl_scrap_mem(ql4im_ha, (void **)&pchip_info,
	    sizeof(EXT_CHIP_INFO))) {
		/* not enough memory */
		DEBUG2(printk("qisioctl%d: %s: inst=%d scrap not big enough. "
		    "size requested=%ld.\n",
		    ql4im_ha->host_no, __func__, ql4im_ha->instance,
		    (ulong)sizeof(EXT_CHIP_INFO)));

		status = (-ENOMEM);
		ioctl->Status = EXT_STATUS_NO_MEMORY;
		ioctl->ResponseLen = 0;
		goto exit_query_chip;
	}

	if (!ioctl->ResponseAdr || ioctl->ResponseLen < sizeof(EXT_CHIP_INFO)) {
		DEBUG2(printk("qisioctl%d: %s: response buffer too small\n",
		    ql4im_ha->host_no, __func__));

		status = (-ENOMEM);
		ioctl->Status = EXT_STATUS_BUFFER_TOO_SMALL;
		ioctl->ResponseLen = 0;
		goto exit_query_chip;
	}

	/* Fill in structure */
	memset(pchip_info, 0, sizeof(EXT_CHIP_INFO));

	pchip_info->VendorId    = ql4im_ha->drvr_ha->pdev->vendor;
	pchip_info->DeviceId    = ql4im_ha->drvr_ha->pdev->device;
	pchip_info->SubVendorId = ql4im_ha->drvr_ha->pdev->subsystem_vendor;
	pchip_info->SubSystemId = ql4im_ha->drvr_ha->pdev->subsystem_device;
	if (IS_QLA80XX(ql4im_ha->drvr_ha))
		pchip_info->ChipRevision = CHIP_REVISION_VALID |
					   ql4im_ha->drvr_ha->revision_id;

	/* ----- Get firmware state information ---- */
	memset(mbox_cmd, 0, sizeof(mbox_cmd));
	memset(mbox_sts, 0, sizeof(mbox_sts));
	mbox_cmd[0] = MBOX_CMD_GET_FW_STATE;
	if (ql4im_ha->drvr_ha->ql4_mbx(ql4im_ha->drvr_ha, 1, 4, &mbox_cmd[0], &mbox_sts[0]) !=
	    QLA_SUCCESS) {
		DEBUG2(printk("qisioctl%d: %s: MBOX_CMD_GET_FW_STATE failed "
		    "w/ status %04X\n",
		    ql4im_ha->host_no, __func__, mbox_sts[0]));

		ioctl->Status = EXT_STATUS_MAILBOX;
		ioctl->VendorSpecificStatus[0] = mbox_sts[0];
		ioctl->VendorSpecificStatus[1] = mbox_sts[1];
		ioctl->VendorSpecificStatus[2] = mbox_sts[2];
		ioctl->VendorSpecificStatus[3] = mbox_sts[3];
		ioctl->ResponseLen = 0;
		goto exit_query_chip;
	}

	pchip_info->BoardID     = mbox_sts[2];

	/* Copy info to caller */
	if (copy_to_user(Q64BIT_TO_PTR(ioctl->ResponseAdr,
						 ioctl->AddrMode),
                                   pchip_info, sizeof(EXT_CHIP_INFO)) != 0) {
		DEBUG2(printk("qisioctl%d: %s: response copy error.\n",
		    ql4im_ha->host_no, __func__));

		status = (-EFAULT);
		ioctl->Status = EXT_STATUS_COPY_ERR;
		ioctl->ResponseLen = 0;
		goto exit_query_chip;
	}

	ioctl->Status = EXT_STATUS_OK;

exit_query_chip:
	ql4im_free_ioctl_scrap_mem(ql4im_ha);
	return(status);
}

/**************************************************************************
 * qla4extioctl_query_ip_state
 *	This routine retrieves operational state of the address control
 *	block and IPv6.
 *
 * Input:
 *	ql4im_ha    = adapter structure pointer.
 *	ioctl = IOCTL structure pointer.
 *
 * Output:
 *	None
 *
 * Returns:
 *	QLA_SUCCESS = success
 *	QLA_ERROR   = error
 *
 * Context:
 *	Kernel context.
 **************************************************************************/
static int
qla4extioctl_query_ip_state(ql4im_scsi_qla_host_t *ql4im_ha, EXT_IOCTL_ISCSI *ioctl)
{
	int	 status = 0;
	EXT_QUERY_IP_STATE *pquery_ip_state = NULL;
	uint32_t mbox_cmd[MBOX_REG_COUNT];
	uint32_t mbox_sts[MBOX_REG_COUNT];

	if (ql4im_get_ioctl_scrap_mem(ql4im_ha, (void **)&pquery_ip_state,
	    sizeof(EXT_QUERY_IP_STATE))) {
		/* not enough memory */
		DEBUG2(
		    printk("scsi%d: %s: inst=%d scrap not big enough. "
		    "size requested=%ld.\n",
		    ql4im_ha->host_no, __func__, ql4im_ha->instance,
		    (ulong)sizeof(EXT_QUERY_IP_STATE)));

		status = (-ENOMEM);
		ioctl->Status = EXT_STATUS_NO_MEMORY;
		ioctl->ResponseLen = 0;
		goto exit_query_ip_state;
	}

	memset(mbox_cmd, 0, sizeof(mbox_cmd));
	memset(mbox_sts, 0, sizeof(mbox_sts));
	mbox_cmd[0] = MBOX_CMD_GET_IP_ADDR_STATE;
	mbox_cmd[1] = ioctl->Instance;
	mbox_cmd[2] = ioctl->Reserved1;

	if (ql4im_ha->drvr_ha->ql4_mbx(ql4im_ha->drvr_ha, 3, 8, &mbox_cmd[0], &mbox_sts[0])
	    != QLA_SUCCESS) {
		DEBUG2( printk("scsi%d: %s: MBOX_CMD_GET_IP_ADDR_STATE "
				       "FAILED w/ status %04X\n",
		    ql4im_ha->host_no, __func__, mbox_sts[0]));

		ioctl->Status = EXT_STATUS_MAILBOX;
		ioctl->VendorSpecificStatus[0] = mbox_sts[0];
		ioctl->VendorSpecificStatus[1] = mbox_sts[1];
		ioctl->VendorSpecificStatus[2] = mbox_sts[2];
		ioctl->VendorSpecificStatus[3] = mbox_sts[3];
		ioctl->VendorSpecificStatus[4] = mbox_sts[4];
		ioctl->VendorSpecificStatus[5] = mbox_sts[5];
		ioctl->VendorSpecificStatus[6] = mbox_sts[6];
		ioctl->VendorSpecificStatus[7] = mbox_sts[7];
		ioctl->ResponseLen = 0;
		goto exit_query_ip_state;
	}

	DEBUG4(printk("scsi%d: %s: Succeeded\n",
	    ql4im_ha->host_no, __func__));

	memcpy(pquery_ip_state->IP_ACBState, &mbox_sts[1],
	       MIN(sizeof(pquery_ip_state->IP_ACBState), sizeof(mbox_sts[1])));
	pquery_ip_state->ValidLifetime     = le32_to_cpu(mbox_sts[2]);
	pquery_ip_state->PreferredLifetime = le32_to_cpu(mbox_sts[3]);
	memcpy(&pquery_ip_state->IPAddressInfo1, &mbox_sts[4],
	       MIN(sizeof(pquery_ip_state->IPAddressInfo2), sizeof(mbox_sts[4])));
	memcpy(&pquery_ip_state->IPAddressInfo2, &mbox_sts[5],
	       MIN(sizeof(pquery_ip_state->IPAddressInfo3), sizeof(mbox_sts[5])));
	memcpy(&pquery_ip_state->IPAddressInfo3, &mbox_sts[6],
	       MIN(sizeof(pquery_ip_state->IPAddressInfo4), sizeof(mbox_sts[6])));
	memcpy(&pquery_ip_state->IPAddressInfo4, &mbox_sts[7],
	       MIN(sizeof(pquery_ip_state->IPAddressInfo1), sizeof(mbox_sts[7])));

	/*
	 * Copy the IOCTL EXT_QUERY_IP_STATE buffer to the user's data space
	 */
	if (copy_to_user(Q64BIT_TO_PTR(ioctl->ResponseAdr, ioctl->AddrMode),
	    pquery_ip_state, ioctl->ResponseLen) != 0) {
		DEBUG2( printk("scsi%d: %s: copy failed\n",
				      ql4im_ha->host_no, __func__));

		status = (-EFAULT);
		ioctl->Status = EXT_STATUS_COPY_ERR;
		ioctl->ResponseLen = 0;
		goto exit_query_ip_state;
	}

	ioctl->Status = EXT_STATUS_OK;

exit_query_ip_state:
	ql4im_free_ioctl_scrap_mem(ql4im_ha);
	return(status);
}

/**************************************************************************
 * qla4extioctl_query_device_current_ip
 *	This routine retrieves the current IP Address, TCP port nember
 * 	and other information for the specified DDB.
 *
 * Input:
 *	ql4im_ha    = adapter structure pointer.
 *	ioctl = IOCTL structure pointer.
 *
 * Output:
 *	None
 *
 * Returns:
 *	QLA_SUCCESS = success
 *	QLA_ERROR   = error
 *
 * Context:
 *	Kernel context.
 **************************************************************************/
static int
qla4extioctl_query_device_current_ip(ql4im_scsi_qla_host_t *ql4im_ha, EXT_IOCTL_ISCSI *ioctl)
{
	int	 status = 0;
	EXT_QUERY_DEVICE_CURRENT_IP *pquery_ddb_curr_ip = NULL;
	uint32_t mbox_cmd[MBOX_REG_COUNT];
	uint32_t mbox_sts[MBOX_REG_COUNT];

	if (ql4im_get_ioctl_scrap_mem(ql4im_ha, (void **)&pquery_ddb_curr_ip,
	    sizeof(EXT_QUERY_DEVICE_CURRENT_IP))) {
		/* not enough memory */
		DEBUG2(
		    printk("scsi%d: %s: inst=%d scrap not big enough. "
		    "size requested=%ld.\n",
		    ql4im_ha->host_no, __func__, ql4im_ha->instance,
		    (ulong)sizeof(EXT_QUERY_DEVICE_CURRENT_IP)));

		status = (-ENOMEM);
		ioctl->Status = EXT_STATUS_NO_MEMORY;
		ioctl->ResponseLen = 0;
		goto exit_query_ddb_curr_ip;
	}

	memset(mbox_cmd, 0, sizeof(mbox_cmd));
	memset(mbox_sts, 0, sizeof(mbox_sts));
	mbox_cmd[0] = MBOX_CMD_GET_DATABASE_ENTRY_CURRENT_IP_ADDR;
	mbox_cmd[1] = ioctl->Instance;

	if (ql4im_ha->drvr_ha->ql4_mbx(ql4im_ha->drvr_ha, 2, 7, &mbox_cmd[0], &mbox_sts[0])
	    != QLA_SUCCESS) {
		DEBUG2( printk("scsi%d: %s: "
			"MBOX_CMD_GET_DATABASE_ENTRY_CURRENT_IP_ADDR "
			"FAILED w/ status %04X\n",
		        ql4im_ha->host_no, __func__, mbox_sts[0]));

		ioctl->Status = EXT_STATUS_MAILBOX;
		ioctl->VendorSpecificStatus[0] = mbox_sts[0];
		ioctl->VendorSpecificStatus[1] = mbox_sts[1];
		ioctl->VendorSpecificStatus[2] = mbox_sts[2];
		ioctl->VendorSpecificStatus[3] = mbox_sts[3];
		ioctl->VendorSpecificStatus[4] = mbox_sts[4];
		ioctl->VendorSpecificStatus[5] = mbox_sts[5];
		ioctl->VendorSpecificStatus[6] = mbox_sts[6];
		ioctl->ResponseLen = 0;
		goto exit_query_ddb_curr_ip;
	}

	DEBUG4(printk("scsi%d: %s: Succeeded\n",
	    ql4im_ha->host_no, __func__));


	pquery_ddb_curr_ip->DeviceState = (uint32_t) __le16_to_cpu(MSW(mbox_sts[1]));
	pquery_ddb_curr_ip->TCPPort = __le16_to_cpu(MSW(mbox_sts[2]));
	memcpy(&pquery_ddb_curr_ip->Flags[0], &mbox_sts[2],
	       sizeof(pquery_ddb_curr_ip->Flags));
	memcpy(&pquery_ddb_curr_ip->Addr.IPAddress[0], &mbox_sts[3], 4);
	memcpy(&pquery_ddb_curr_ip->Addr.IPAddress[4], &mbox_sts[4], 4);
	memcpy(&pquery_ddb_curr_ip->Addr.IPAddress[8], &mbox_sts[5], 4);
	memcpy(&pquery_ddb_curr_ip->Addr.IPAddress[12], &mbox_sts[6], 4);
	if ((ql4im_ha->drvr_ha->ipv6_tcp_options & IPV6_TCPOPT_ISNSv6_ENABLE) != 0) {
		pquery_ddb_curr_ip->Addr.Type = EXT_DEF_TYPE_ISCSI_IPV6;
	}

	/*
	 * Copy the IOCTL EXT_QUERY_IP_STATE buffer to the user's data space
	 */
	if (copy_to_user(Q64BIT_TO_PTR(ioctl->ResponseAdr, ioctl->AddrMode),
	    pquery_ddb_curr_ip, ioctl->ResponseLen) != 0) {
		DEBUG2( printk("scsi%d: %s: copy failed\n",
				      ql4im_ha->host_no, __func__));

		status = (-EFAULT);
		ioctl->Status = EXT_STATUS_COPY_ERR;
		ioctl->ResponseLen = 0;
		goto exit_query_ddb_curr_ip;
	}

	ioctl->Status = EXT_STATUS_OK;

exit_query_ddb_curr_ip:
	ql4im_free_ioctl_scrap_mem(ql4im_ha);
	return(status);
}

/**************************************************************************
 * qla4extioctl_query
 *	This routine calls query IOCTLs based on the IOCTL Sub Code.
 *
 * Input:
 *	ql4im_ha    = adapter structure pointer.
 *	ioctl = IOCTL structure pointer.
 *
 * Output:
 *	None
 *
 * Returns:
 *	QLA_SUCCESS = success
 *	QLA_ERROR   = error
 *    	-EINVAL     = if the command is invalid
 *
 * Context:
 *	Kernel context.
 **************************************************************************/
static int
qla4extioctl_query(ql4im_scsi_qla_host_t *ql4im_ha, EXT_IOCTL_ISCSI *ioctl)
{
	switch (ioctl->SubCode) {
	case EXT_SC_QUERY_HBA_ISCSI_NODE:
		return(qla4extioctl_query_hba_iscsi_node(ql4im_ha, ioctl));

	case EXT_SC_QUERY_HBA_ISCSI_PORTAL:
		return(qla4extioctl_query_hba_iscsi_portal(ql4im_ha, ioctl));

	case EXT_SC_QUERY_DISC_ISCSI_NODE:
		return(qla4extioctl_query_disc_iscsi_node(ql4im_ha, ioctl));

	case EXT_SC_QUERY_DISC_ISCSI_PORTAL:
		return(qla4extioctl_query_disc_iscsi_portal(ql4im_ha, ioctl));

	case EXT_SC_QUERY_DRIVER:
		return(qla4extioctl_query_driver(ql4im_ha, ioctl));

	case EXT_SC_QUERY_FW:
		return(qla4extioctl_query_fw(ql4im_ha, ioctl));

	case EXT_SC_QUERY_CHIP:
		return(qla4extioctl_query_chip(ql4im_ha, ioctl));

	case EXT_SC_QUERY_IP_STATE:
		return(qla4extioctl_query_ip_state(ql4im_ha, ioctl));

	case EXT_SC_QUERY_DEVICE_CURRENT_IP:
		return(qla4extioctl_query_device_current_ip(ql4im_ha, ioctl));

	default:
		DEBUG2(printk("qisioctl%d: %s: unsupported query sub-command "
		    "code (%x)\n",
		    ql4im_ha->host_no, __func__, ioctl->SubCode));

		ioctl->Status = EXT_STATUS_UNSUPPORTED_SUBCODE;
		return(-EINVAL);
	}
}

/**************************************************************************
 * qla4extioctl_reg_aen
 *	This routine enables/disables storing of asynchronous events
 *	from the ISP into the driver's internal buffer.
 *
 * Input:
 *	ql4im_ha    = adapter structure pointer.
 *	ioctl = IOCTL structure pointer.
 *
 * Output:
 *	None
 *
 * Returns:
 *	QLA_SUCCESS = success
 *	QLA_ERROR   = error
 *
 * Context:
 *	Kernel context.
 **************************************************************************/
static int
qla4extioctl_reg_aen(ql4im_scsi_qla_host_t *ql4im_ha, EXT_IOCTL_ISCSI *ioctl)
{
	EXT_REG_AEN_ISCSI reg_aen;
	int status = 0;

	if (ioctl->RequestLen > sizeof(EXT_REG_AEN_ISCSI)) {
		DEBUG2(printk("qisioctl%d: %s: memory area too small\n",
		    ql4im_ha->host_no, __func__));

		status = (-ENOMEM);
		ioctl->Status = EXT_STATUS_BUFFER_TOO_SMALL;
		ioctl->ResponseLen = 0;
		goto exit_reg_aen;
	}

	/* --- Copy input structure from user space --- */
	if ((status = copy_from_user((void *)&reg_aen,
	    Q64BIT_TO_PTR(ioctl->RequestAdr, ioctl->AddrMode),
				     sizeof(EXT_REG_AEN_ISCSI))) != 0) {
		DEBUG2(printk("qisioctl%d: %s: unable to copy data from "
		    "user's memory area\n", ql4im_ha->host_no, __func__));

		status = (-EFAULT);
		ioctl->Status = EXT_STATUS_COPY_ERR;
		ioctl->ResponseLen = 0;
		goto exit_reg_aen;
	}

	ql4im_ha->drvr_ha->aen_reg_mask = reg_aen.Enable;
	DEBUG4(printk("qisioctl%d: %s: mask = 0x%x\n",
	    ql4im_ha->host_no, __func__, ql4im_ha->drvr_ha->aen_reg_mask));
	ioctl->Status = EXT_STATUS_OK;

exit_reg_aen:
	return(status);
}

/**************************************************************************
 * qla4extioctl_get_aen
 *	This routine retrieves the contents of the driver's internal
 *	asynchronous event tracking queue.
 *
 * Input:
 *	ql4im_ha    = adapter structure pointer.
 *	ioctl = IOCTL structure pointer.
 *
 * Output:
 *	None
 *
 * Returns:
 *	QLA_SUCCESS = success
 *	QLA_ERROR   = error
 *
 * Context:
 *	Kernel context.
 **************************************************************************/
static int
qla4extioctl_get_aen(ql4im_scsi_qla_host_t *ql4im_ha, EXT_IOCTL_ISCSI *ioctl)
{
	EXT_ASYNC_EVENT *async_event;
	uint32_t 	num_aens_returned;
	uint32_t 	drvr_aen_index;
	scsi_qla_host_t *drvr_ha;
	int 		i, j;
	int		status = 0;

	drvr_ha = ql4im_ha->drvr_ha;

	DEBUG4(printk("qisioctl%d: %s: mask = 0x%x\n",
	    ql4im_ha->host_no, __func__, drvr_ha->aen_reg_mask));

	if (drvr_ha->aen_reg_mask == EXT_DEF_ENABLE_NO_AENS) {
		DEBUG2(printk("qisioctl%d: %s: AEN mask not enabled\n",
		    ql4im_ha->host_no, __func__));
		ioctl->Status = EXT_STATUS_OK;
		return 0;
	}

	if (ql4im_get_ioctl_scrap_mem(ql4im_ha, (void **)&async_event,
	    sizeof(EXT_ASYNC_EVENT) * EXT_DEF_MAX_AEN_QUEUE)) {
		/* not enough memory */
		DEBUG2(printk("qisioctl%d: %s: inst=%d scrap not big enough. "
		    "size requested=%ld.\n",
		    ql4im_ha->host_no, __func__, ql4im_ha->instance,
		    (ulong)(sizeof(EXT_ASYNC_EVENT) * EXT_DEF_MAX_AEN_QUEUE)));

		status = (-ENOMEM);
		ioctl->Status = EXT_STATUS_NO_MEMORY;
		ioctl->ResponseLen = 0;
		goto exit_get_gen;
	}

	if (ioctl->ResponseLen < sizeof(EXT_ASYNC_EVENT) * EXT_DEF_MAX_AEN_QUEUE) {
		DEBUG2(printk("qisioctl%d: %s: memory area too small\n",
		    ql4im_ha->host_no, __func__));

		status = (-ENOMEM);
		ioctl->Status = EXT_STATUS_BUFFER_TOO_SMALL;
		ioctl->ResponseLen = 0;
		goto exit_get_gen;
	}

	memset(async_event, 0, sizeof(EXT_ASYNC_EVENT) * EXT_DEF_MAX_AEN_QUEUE);

	if (drvr_ha->aen_count - drvr_ha->aen_report < MAX_AEN_ENTRIES)
		num_aens_returned = drvr_ha->aen_count - drvr_ha->aen_report;
	else
		num_aens_returned = MAX_AEN_ENTRIES;

	drvr_aen_index = (drvr_ha->aen_report % MAX_AEN_ENTRIES) + 1;

	for (i=0; i<num_aens_returned; i++) {
		async_event[i].AsyncEventCode =
			drvr_ha->aen_q[drvr_aen_index].mbox_sts[0];

		for (j=0; j<EXT_DEF_MAX_AEN_PAYLOAD; j++) {
			async_event[i].Payload[j] =
				drvr_ha->aen_q[drvr_aen_index].mbox_sts[j+1];
		}

		if (drvr_aen_index < MAX_AEN_ENTRIES-1)
			drvr_aen_index++;
		else
			drvr_aen_index = 0;
	}
	drvr_ha->aen_report += num_aens_returned;

	ioctl->ResponseLen = sizeof(EXT_ASYNC_EVENT) * num_aens_returned;
	ioctl->Status = EXT_STATUS_OK;

	/*
	 * Copy the IOCTL EXT_ASYNC_EVENT buffer to the user's data space
	 */
	if ((status = copy_to_user(Q64BIT_TO_PTR(ioctl->ResponseAdr,
						 ioctl->AddrMode), async_event,
                                   ioctl->ResponseLen)) != 0) {
		DEBUG2(printk("qisioctl%d: %s: memory area too small\n",
		    ql4im_ha->host_no, __func__));

		status = (-EFAULT);
		ioctl->Status = EXT_STATUS_COPY_ERR;
		ioctl->ResponseLen = 0;
	}

exit_get_gen:
        ql4im_free_ioctl_scrap_mem(ql4im_ha);
	return(status);
}

/**************************************************************************
 * qla4extioctl_get_statistics_gen
 *	This routine retrieves the HBA general statistical information.
 *
 * Input:
 *	ql4im_ha    = adapter structure pointer.
 *	ioctl = IOCTL structure pointer.
 *
 * Output:
 *	None
 *
 * Returns:
 *	QLA_SUCCESS = success
 *	QLA_ERROR   = error
 *
 * Context:
 *	Kernel context.
 **************************************************************************/
static int
qla4extioctl_get_statistics_gen(ql4im_scsi_qla_host_t *ql4im_ha,
				EXT_IOCTL_ISCSI *ioctl)
{
	int			status = 0;
	EXT_HBA_PORT_STAT_GEN	*pstat_gen = NULL;
	scsi_qla_host_t		*drvr_ha = ql4im_ha->drvr_ha;

	if (ql4im_get_ioctl_scrap_mem(ql4im_ha, (void **)&pstat_gen,
	    sizeof(EXT_HBA_PORT_STAT_GEN))) {
		/* not enough memory */
		DEBUG2(printk("qisioctl%d: %s: inst=%d scrap not big enough. "
		    "size requested=%ld.\n",
		    ql4im_ha->host_no, __func__, ql4im_ha->instance,
		    (ulong)sizeof(EXT_HBA_PORT_STAT_GEN)));

		status = (-ENOMEM);
		ioctl->Status = EXT_STATUS_NO_MEMORY;
		ioctl->ResponseLen = 0;
		goto exit_get_stat_gen;
	}

	if (ioctl->ResponseLen < sizeof(EXT_HBA_PORT_STAT_GEN)) {
		DEBUG2(printk("qisioctl%d: %s: memory area too small\n",
		    ql4im_ha->host_no, __func__));

		status = (-ENOMEM);
		ioctl->Status = EXT_STATUS_BUFFER_TOO_SMALL;
		ioctl->ResponseLen = 0;
		goto exit_get_stat_gen;
	}

	/*
	 * Fill in the data
	 */
	memset(pstat_gen, 0, sizeof(EXT_HBA_PORT_STAT_GEN));
	pstat_gen->HBAPortErrorCount     = drvr_ha->adapter_error_count;
	pstat_gen->DevicePortErrorCount  = drvr_ha->device_error_count;
	pstat_gen->IoCount               = drvr_ha->total_io_count;
	pstat_gen->MBytesCount           = drvr_ha->total_mbytes_xferred;
	pstat_gen->InterruptCount        = drvr_ha->isr_count;
	pstat_gen->LinkFailureCount      = drvr_ha->link_failure_count;
	pstat_gen->InvalidCrcCount       = drvr_ha->invalid_crc_count;

	/*
	 * Copy the IOCTL EXT_HBA_PORT_STAT_GEN buffer to the user's data space
	 */
	if (copy_to_user(Q64BIT_TO_PTR(ioctl->ResponseAdr,
						 ioctl->AddrMode), pstat_gen,
                                   ioctl->ResponseLen) != 0) {
		DEBUG2(printk("qisioctl%d: %s: memory area too small\n",
		    ql4im_ha->host_no, __func__));

		status = (-EFAULT);
		ioctl->Status = EXT_STATUS_COPY_ERR;
		ioctl->ResponseLen = 0;
		goto exit_get_stat_gen;
	}

	ioctl->Status = EXT_STATUS_OK;

exit_get_stat_gen:
	ql4im_free_ioctl_scrap_mem(ql4im_ha);
	return(status);
}

/**************************************************************************
 * qla4extioctl_get_statistics_iscsi
 *	This routine retrieves the HBA iSCSI statistical information.
 *
 * Input:
 *	ql4im_ha    = adapter structure pointer.
 *	ioctl = IOCTL structure pointer.
 *
 * Output:
 *	None
 *
 * Returns:
 *	QLA_SUCCESS = success
 *	QLA_ERROR   = error
 *
 * Context:
 *	Kernel context.
 **************************************************************************/
static int
qla4extioctl_get_statistics_iscsi(ql4im_scsi_qla_host_t *ql4im_ha,
				  EXT_IOCTL_ISCSI *ioctl)
{
	int		status = 0;
	uint32_t	mbox_cmd[MBOX_REG_COUNT];
	uint32_t	mbox_sts[MBOX_REG_COUNT];
	EXT_HBA_PORT_STAT_ISCSI* pstat_local;
	EXT_HBA_PORT_STAT_ISCSI* pstat_user;

	pstat_user = kmalloc(sizeof(EXT_HBA_PORT_STAT_ISCSI), GFP_ATOMIC);

	if (ql4im_get_ioctl_scrap_mem(ql4im_ha, (void **)&pstat_user,
	    sizeof(EXT_HBA_PORT_STAT_ISCSI))) {
		/* not enough memory */
		DEBUG2(printk("qisioctl%d: %s: inst=%d scrap not big enough. "
		    "size requested=%ld.\n",
		    ql4im_ha->host_no, __func__, ql4im_ha->instance,
		    (ulong)sizeof(EXT_HBA_PORT_STAT_ISCSI)));

		status = (-ENOMEM);
		ioctl->Status = EXT_STATUS_NO_MEMORY;
		ioctl->ResponseLen = 0;
		goto exit_get_stats_iscsi;
	}

	if (!ioctl->ResponseAdr || !ioctl->ResponseLen) {
		DEBUG2(printk("qisioctl%d: %s: invalid parameter\n",
		    ql4im_ha->host_no, __func__));

		status = (-EINVAL);
		ioctl->Status = EXT_STATUS_INVALID_PARAM;
		ioctl->ResponseLen = 0;
		goto exit_get_stats_iscsi;
	}

	if (ioctl->ResponseLen < sizeof(EXT_HBA_PORT_STAT_ISCSI)) {
		DEBUG2(printk("qisioctl%d: %s: RespLen too small (0x%x),  "
		    "need (0x%x).\n",
		    ql4im_ha->host_no, __func__, ioctl->ResponseLen,
		    (unsigned int) sizeof(EXT_HBA_PORT_STAT_ISCSI)));

		status = (-ENOMEM);
		ioctl->Status = EXT_STATUS_BUFFER_TOO_SMALL;
		ioctl->ResponseLen = 0;
		goto exit_get_stats_iscsi;
	}

	if ((ql4im_ha->ioctl_dma_buf_len < sizeof(EXT_HBA_PORT_STAT_ISCSI)) &&
	    (ql4im_resize_ioctl_dma_buf(ql4im_ha, sizeof(EXT_HBA_PORT_STAT_ISCSI))
	     != QLA_SUCCESS)) {
		DEBUG2(printk("qisioctl%d: %s: unable to allocate memory "
		    "for dma buffer.\n",
		    ql4im_ha->host_no, __func__));

		status = (-ENOMEM);
		ioctl->Status = EXT_STATUS_NO_MEMORY;
		ioctl->ResponseLen = 0;
		goto exit_get_stats_iscsi;
	}

	/*
	 * Make the mailbox call
	 */
	memset(mbox_cmd, 0, sizeof(mbox_cmd));
	memset(mbox_sts, 0, sizeof(mbox_sts));
	mbox_cmd[0] = MBOX_CMD_GET_MANAGEMENT_DATA;
	mbox_cmd[1] = ioctl->Instance;
	mbox_cmd[2] = LSDW(ql4im_ha->ioctl_dma_bufp);
	mbox_cmd[3] = MSDW(ql4im_ha->ioctl_dma_bufp);

	if (ql4im_ha->drvr_ha->ql4_mbx(ql4im_ha->drvr_ha, 4, 1,
				  &mbox_cmd[0], &mbox_sts[0]) != QLA_SUCCESS) {
		DEBUG2(printk("qisioctl%d: %s: get mngmt data for index [%d] failed "
		    "w/ mailbox ststus 0x%x\n",
		    ql4im_ha->host_no, __func__, ioctl->Instance, mbox_sts[0]));

		ioctl->Status = EXT_STATUS_MAILBOX;
		ioctl->VendorSpecificStatus[0] = mbox_sts[0];
		ioctl->ResponseLen = 0;
		goto exit_get_stats_iscsi;
	}

	pstat_local = (EXT_HBA_PORT_STAT_ISCSI *) ql4im_ha->ioctl_dma_bufv;
	memset(pstat_user, 0, sizeof(EXT_HBA_PORT_STAT_ISCSI));
	pstat_user->MACTxFramesCount          =
	    le64_to_cpu(pstat_local->MACTxFramesCount);
	pstat_user->MACTxBytesCount           =
	    le64_to_cpu(pstat_local->MACTxBytesCount);
	pstat_user->MACRxFramesCount          =
	    le64_to_cpu(pstat_local->MACRxFramesCount);
	pstat_user->MACRxBytesCount           =
	    le64_to_cpu(pstat_local->MACRxBytesCount);
	pstat_user->MACCRCErrorCount          =
	    le64_to_cpu(pstat_local->MACCRCErrorCount);
	pstat_user->MACEncodingErrorCount     =
	    le64_to_cpu(pstat_local->MACEncodingErrorCount);
	pstat_user->IPTxPacketsCount          =
	    le64_to_cpu(pstat_local->IPTxPacketsCount);
	pstat_user->IPTxBytesCount            =
	    le64_to_cpu(pstat_local->IPTxBytesCount);
	pstat_user->IPTxFragmentsCount        =
	    le64_to_cpu(pstat_local->IPTxFragmentsCount);
	pstat_user->IPRxPacketsCount          =
	    le64_to_cpu(pstat_local->IPRxPacketsCount);
	pstat_user->IPRxBytesCount            =
	    le64_to_cpu(pstat_local->IPRxBytesCount);
	pstat_user->IPRxFragmentsCount        =
	    le64_to_cpu(pstat_local->IPRxFragmentsCount);
	pstat_user->IPDatagramReassemblyCount =
	    le64_to_cpu(pstat_local->IPDatagramReassemblyCount);
	pstat_user->IPv6RxPacketsCount        =
	    le64_to_cpu(pstat_local->IPv6RxPacketsCount);
	pstat_user->IPRxPacketErrorCount      =
	    le64_to_cpu(pstat_local->IPRxPacketErrorCount);
	pstat_user->IPReassemblyErrorCount    =
	    le64_to_cpu(pstat_local->IPReassemblyErrorCount);
	pstat_user->TCPTxSegmentsCount        =
	    le64_to_cpu(pstat_local->TCPTxSegmentsCount);
	pstat_user->TCPTxBytesCount           =
	    le64_to_cpu(pstat_local->TCPTxBytesCount);
	pstat_user->TCPRxSegmentsCount        =
	    le64_to_cpu(pstat_local->TCPRxSegmentsCount);
	pstat_user->TCPRxBytesCount           =
	    le64_to_cpu(pstat_local->TCPRxBytesCount);
	pstat_user->TCPTimerExpiredCount      =
	    le64_to_cpu(pstat_local->TCPTimerExpiredCount);
	pstat_user->TCPRxACKCount             =
	    le64_to_cpu(pstat_local->TCPRxACKCount);
	pstat_user->TCPTxACKCount             =
	    le64_to_cpu(pstat_local->TCPTxACKCount);
	pstat_user->TCPRxErrorSegmentCount    =
	    le64_to_cpu(pstat_local->TCPRxErrorSegmentCount);
	pstat_user->TCPWindowProbeUpdateCount =
	    le64_to_cpu(pstat_local->TCPWindowProbeUpdateCount);
	pstat_user->iSCSITxPDUCount           =
	    le64_to_cpu(pstat_local->iSCSITxPDUCount);
	pstat_user->iSCSITxBytesCount         =
	    le64_to_cpu(pstat_local->iSCSITxBytesCount);
	pstat_user->iSCSIRxPDUCount           =
	    le64_to_cpu(pstat_local->iSCSIRxPDUCount);
	pstat_user->iSCSIRxBytesCount         =
	    le64_to_cpu(pstat_local->iSCSIRxBytesCount);
	pstat_user->iSCSICompleteIOsCount     =
	    le64_to_cpu(pstat_local->iSCSICompleteIOsCount);
	pstat_user->iSCSIUnexpectedIORxCount  =
	    le64_to_cpu(pstat_local->iSCSIUnexpectedIORxCount);
	pstat_user->iSCSIFormatErrorCount     =
	    le64_to_cpu(pstat_local->iSCSIFormatErrorCount);
	pstat_user->iSCSIHeaderDigestCount    =
	    le64_to_cpu(pstat_local->iSCSIHeaderDigestCount);
	pstat_user->iSCSIDataDigestErrorCount =
	    le64_to_cpu(pstat_local->iSCSIDataDigestErrorCount);
	pstat_user->iSCSISeqErrorCount        =
	    le64_to_cpu(pstat_local->iSCSISeqErrorCount);

	/*
	 * Copy the data from the dma buffer to the user's data space
	 */
	if (copy_to_user(Q64BIT_TO_PTR(ioctl->ResponseAdr,
						 ioctl->AddrMode),
                                   pstat_user, ioctl->ResponseLen) != 0) {
		DEBUG2(printk("qisioctl%d: %s: memory area too small\n",
		    ql4im_ha->host_no, __func__));

		status = (-EFAULT);
		ioctl->Status = EXT_STATUS_COPY_ERR;
		ioctl->ResponseLen = 0;
		goto exit_get_stats_iscsi;
	}

	ioctl->Status = EXT_STATUS_OK;

exit_get_stats_iscsi:
	ql4im_free_ioctl_scrap_mem(ql4im_ha);
	return(status);
}

/**************************************************************************
 * qla4extioctl_get_device_entry_iscsi
 *	This routine retrieves the database entry for the specified device.
 *
 * Input:
 *	ql4im_ha    = adapter structure pointer.
 *	ioctl = IOCTL structure pointer.
 *
 * Output:
 *	None
 *
 * Returns:
 *	QLA_SUCCESS = success
 *	QLA_ERROR   = error
 *
 * Context:
 *	Kernel context.
 **************************************************************************/
static int
qla4extioctl_get_device_entry_iscsi(ql4im_scsi_qla_host_t *ql4im_ha,
				    EXT_IOCTL_ISCSI *ioctl)
{
	int                     status = 0;
	uint32_t		mbox_cmd[MBOX_REG_COUNT];
	uint32_t		mbox_sts[MBOX_REG_COUNT];
	DEV_DB_ENTRY		*pfw_ddb_entry;
	EXT_DEVICE_ENTRY_ISCSI	*pdev_entry;

	if (ql4im_get_ioctl_scrap_mem(ql4im_ha, (void **)&pdev_entry,
	    sizeof(EXT_DEVICE_ENTRY_ISCSI))) {
		/* not enough memory */
		DEBUG2(printk("qisioctl%d: %s: inst=%d scrap not big enough. "
		    "size requested=%ld.\n",
		    ql4im_ha->host_no, __func__, ql4im_ha->instance,
		    (ulong)sizeof(EXT_DEVICE_ENTRY_ISCSI)));

		status = (-ENOMEM);
		ioctl->Status = EXT_STATUS_NO_MEMORY;
		ioctl->ResponseLen = 0;
		goto exit_get_dev_entry;
	}

	if (ql4im_ha->ioctl_dma_buf_len < sizeof(DEV_DB_ENTRY)) {
		if (ql4im_resize_ioctl_dma_buf(ql4im_ha, sizeof(DEV_DB_ENTRY)) !=
		    QLA_SUCCESS) {
			DEBUG2(printk("qisioctl%d: %s: unable to allocate memory "
			    "for dma buffer.\n",
			    ql4im_ha->host_no, __func__));

			status = (-ENOMEM);
			ioctl->Status = EXT_STATUS_NO_MEMORY;
			ioctl->ResponseLen = 0;
			goto exit_get_dev_entry;
		}
	}

	if (ioctl->ResponseLen < sizeof(EXT_DEVICE_ENTRY_ISCSI)) {
		DEBUG2(printk("qisioctl%d: %s: memory area too small\n",
		    ql4im_ha->host_no, __func__));

		status = (-ENOMEM);
		ioctl->Status = EXT_STATUS_BUFFER_TOO_SMALL;
		ioctl->ResponseLen = 0;
		goto exit_get_dev_entry;
	}

	/*
	 * Make the mailbox call
	 */
	memset(mbox_cmd, 0, sizeof(mbox_cmd));
	memset(mbox_sts, 0, sizeof(mbox_sts));
	memset(pdev_entry, 0, sizeof(EXT_DEVICE_ENTRY_ISCSI));

	if (ioctl->SubCode == EXT_SC_GET_DEVICE_ENTRY_ISCSI)
		mbox_cmd[0] = MBOX_CMD_GET_DATABASE_ENTRY;
	else
		mbox_cmd[0] = MBOX_CMD_GET_DATABASE_ENTRY_DEFAULTS;

	mbox_cmd[1] = ioctl->Instance;
	mbox_cmd[2] = LSDW(ql4im_ha->ioctl_dma_bufp);
	mbox_cmd[3] = MSDW(ql4im_ha->ioctl_dma_bufp);

	if (ql4im_ha->drvr_ha->ql4_mbx(ql4im_ha->drvr_ha, 4, 5,
				  &mbox_cmd[0], &mbox_sts[0]) != QLA_SUCCESS) {
		DEBUG2(printk("qisioctl%d: %s: get ddb entry for index [%d] failed "
		    "w/ mailbox ststus 0x%x\n",
		    ql4im_ha->host_no, __func__, ioctl->Instance, mbox_sts[0]));

		ioctl->Status = EXT_STATUS_MAILBOX;
		ioctl->VendorSpecificStatus[0] = mbox_sts[0];
		ioctl->VendorSpecificStatus[1] = mbox_sts[1];
		ioctl->VendorSpecificStatus[2] = mbox_sts[2];
		ioctl->VendorSpecificStatus[3] = mbox_sts[3];
		ioctl->VendorSpecificStatus[4] = mbox_sts[4];
		ioctl->ResponseLen = 0;
		goto exit_get_dev_entry;
	}

	/*
	 * Transfer data from Fw's DEV_DB_ENTRY buffer to IOCTL's
	 * EXT_DEVICE_ENTRY_ISCSI buffer
	 */
	pfw_ddb_entry = ql4im_ha->ioctl_dma_bufv;

	pdev_entry->NumValid     = mbox_sts[2];
	pdev_entry->NextValid    = mbox_sts[3];
	pdev_entry->DeviceState  = mbox_sts[4];
	pdev_entry->Options      = pfw_ddb_entry->options;
	pdev_entry->TargetSessID = le16_to_cpu(pfw_ddb_entry->TSID);
	memcpy(pdev_entry->InitiatorSessID, pfw_ddb_entry->ISID,
	    sizeof(pfw_ddb_entry->ISID));

	pdev_entry->DeviceInfo.DeviceType = le16_to_cpu(EXT_DEF_ISCSI_REMOTE);
	pdev_entry->DeviceInfo.ExeThrottle =
	    le16_to_cpu(pfw_ddb_entry->exeThrottle);
	pdev_entry->DeviceInfo.InitMarkerlessInt =
	    le16_to_cpu(pfw_ddb_entry->iSCSIMaxSndDataSegLen);
	pdev_entry->DeviceInfo.RetryCount = pfw_ddb_entry->retryCount;
	pdev_entry->DeviceInfo.RetryDelay = pfw_ddb_entry->retryDelay;
	pdev_entry->DeviceInfo.iSCSIOptions =
	    le16_to_cpu(pfw_ddb_entry->iSCSIOptions);
	pdev_entry->DeviceInfo.TCPOptions =
	    le16_to_cpu(pfw_ddb_entry->TCPOptions);
	pdev_entry->DeviceInfo.IPOptions =
	    le16_to_cpu(pfw_ddb_entry->IPOptions);
	pdev_entry->DeviceInfo.MaxPDUSize =
	    le16_to_cpu(pfw_ddb_entry->maxPDUSize);
	pdev_entry->DeviceInfo.FirstBurstSize =
	    le16_to_cpu(pfw_ddb_entry->firstBurstSize);
	pdev_entry->DeviceInfo.LogoutMinTime =
	    le16_to_cpu(pfw_ddb_entry->DefaultTime2Wait);
	pdev_entry->DeviceInfo.LogoutMaxTime =
	    le16_to_cpu(pfw_ddb_entry->DefaultTime2Retain);
	pdev_entry->DeviceInfo.MaxOutstandingR2T =
	    le16_to_cpu(pfw_ddb_entry->maxOutstndngR2T);
	pdev_entry->DeviceInfo.KeepAliveTimeout =
	    le16_to_cpu(pfw_ddb_entry->keepAliveTimeout);
	pdev_entry->DeviceInfo.PortNumber =
	    le16_to_cpu(pfw_ddb_entry->RemoteTCPPortNumber);
	pdev_entry->DeviceInfo.MaxBurstSize =
	    le16_to_cpu(pfw_ddb_entry->maxBurstSize);
	pdev_entry->DeviceInfo.TaskMgmtTimeout =
	    le16_to_cpu(pfw_ddb_entry->taskMngmntTimeout);
	pdev_entry->EntryInfo.PortalCount = mbox_sts[2];
	pdev_entry->ExeCount = le16_to_cpu(pfw_ddb_entry->exeCount);
	pdev_entry->DDBLink = le16_to_cpu(pfw_ddb_entry->ddbLink);

	memcpy(pdev_entry->DeviceInfo.TargetAddr, pfw_ddb_entry->targetAddr,
	    sizeof(pdev_entry->DeviceInfo.TargetAddr));
	memcpy(pdev_entry->EntryInfo.IPAddr.IPAddress, pfw_ddb_entry->RemoteIPAddr,
	    sizeof(pdev_entry->EntryInfo.IPAddr.IPAddress));
	memcpy(pdev_entry->EntryInfo.iSCSIName, pfw_ddb_entry->iscsiName,
	    sizeof(pdev_entry->EntryInfo.iSCSIName));
	memcpy(pdev_entry->EntryInfo.Alias, pfw_ddb_entry->iSCSIAlias,
	    sizeof(pdev_entry->EntryInfo.Alias));

	DEBUG10(printk("qisioctl%d: DEV_DB_ENTRY structure:\n", ql4im_ha->host_no));
	DEBUG10(ql4im_dump_bytes(pfw_ddb_entry, sizeof(DEV_DB_ENTRY));)
	DEBUG10(printk("qisioctl%d: EXT_DEVICE_ENTRY_ISCSI structure:\n",
	    ql4im_ha->host_no));
	DEBUG10(ql4im_dump_bytes(pdev_entry, sizeof(EXT_DEVICE_ENTRY_ISCSI));)

	/*
	 * Copy the IOCTL EXT_DEVICE_ENTRY_ISCSI buffer to the user's data space
	 */
	if (copy_to_user(Q64BIT_TO_PTR(ioctl->ResponseAdr,
						 ioctl->AddrMode),
                                   pdev_entry, ioctl->ResponseLen) != 0) {
		DEBUG2(printk("qisioctl%d: %s: memory area too small\n",
		    ql4im_ha->host_no, __func__));

		status = (-EFAULT);
		ioctl->Status = EXT_STATUS_COPY_ERR;
		ioctl->ResponseLen = 0;
		goto exit_get_dev_entry;
	}

	ioctl->Status = EXT_STATUS_OK;

exit_get_dev_entry:
	ql4im_free_ioctl_scrap_mem(ql4im_ha);
	return(status);
}


/**************************************************************************
 * qla4extioctl_get_init_fw_iscsi
 *	This routine retrieves the initialize firmware control block for
 *	the specified HBA.
 *
 * Input:
 *	ql4im_ha    = adapter structure pointer.
 *	ioctl = IOCTL structure pointer.
 *
 * Output:
 *	None
 *
 * Returns:
 *	QLA_SUCCESS = success
 *	QLA_ERROR   = error
 *
 * Context:
 *	Kernel context.
 **************************************************************************/
static int
qla4extioctl_get_init_fw_iscsi(ql4im_scsi_qla_host_t *ql4im_ha,
			       EXT_IOCTL_ISCSI *ioctl)
{
	int			status = 0;
	uint32_t		mbox_cmd[MBOX_REG_COUNT];
	uint32_t		mbox_sts[MBOX_REG_COUNT];
	EXT_INIT_FW_ISCSI	*pinit_fw = NULL;
	ADDRESS_CTRL_BLK	*pinit_fw_cb;
	scsi_qla_host_t *drvr_ha = ql4im_ha->drvr_ha;

	if (ql4im_get_ioctl_scrap_mem(ql4im_ha, (void **)&pinit_fw,
	    sizeof(EXT_INIT_FW_ISCSI))) {
		/* not enough memory */
		DEBUG2(printk("qisioctl%d: %s: inst=%d scrap not big enough. "
		    "size requested=%ld.\n",
		    ql4im_ha->host_no, __func__, ql4im_ha->instance,
		    (ulong)sizeof(EXT_INIT_FW_ISCSI)));

		status = (-ENOMEM);
		ioctl->Status = EXT_STATUS_NO_MEMORY;
		ioctl->ResponseLen = 0;
		goto exit_get_init_fw;
	}

	if (!ql4im_ha->ioctl_dma_bufv || !ql4im_ha->ioctl_dma_bufp ||
	    (ql4im_ha->ioctl_dma_buf_len < drvr_ha->ifcb_size) ||
	    (ioctl->ResponseLen < sizeof(EXT_INIT_FW_ISCSI))) {
		DEBUG2(printk("qisioctl%d: %s: response buffer too small\n",
		    ql4im_ha->host_no, __func__));

		status = (-ENOMEM);
		ioctl->Status = EXT_STATUS_BUFFER_TOO_SMALL;
		ioctl->ResponseLen = 0;
		goto exit_get_init_fw;
	}

	/*
	 * Send mailbox command
	 */
	memset(mbox_cmd, 0, sizeof(mbox_cmd));
	memset(mbox_sts, 0, sizeof(mbox_sts));
	switch (ioctl->SubCode) {
	case EXT_SC_GET_INIT_FW_ISCSI:
		mbox_cmd[0] = MBOX_CMD_GET_INIT_FW_CTRL_BLOCK;
		break;
	case EXT_SC_GET_INIT_FW_DEFAULTS_ISCSI:
		mbox_cmd[0] = MBOX_CMD_GET_INIT_FW_CTRL_BLOCK_DEFAULTS;
		break;
	default:
		DEBUG2(printk("qisioctl%d: %s: invalid subcode (0x%04X) speficied\n",
		    ql4im_ha->host_no, __func__, ioctl->SubCode));

		status = (-EINVAL);
		ioctl->Status = EXT_STATUS_INVALID_PARAM;
		ioctl->ResponseLen = 0;
		goto exit_get_init_fw;
	}

	mbox_cmd[1] = 0;
	mbox_cmd[2] = LSDW(ql4im_ha->ioctl_dma_bufp);
	mbox_cmd[3] = MSDW(ql4im_ha->ioctl_dma_bufp);
	if (ql4im_ha->acb_version == ACB_SUPPORTED) {
		mbox_cmd[4] = ql4im_ha->ifcb_size;
	}

	if (ql4im_ha->drvr_ha->ql4_mbx(ql4im_ha->drvr_ha, 4, 1,
				  &mbox_cmd[0], &mbox_sts[0]) == QLA_ERROR) {
		DEBUG2(printk("qisioctl%d: %s: command failed \n",
		    ql4im_ha->host_no, __func__));

		ioctl->Status = EXT_STATUS_MAILBOX;
		ioctl->VendorSpecificStatus[0] = mbox_sts[0];
		goto exit_get_init_fw;
	}

	/*
	 * Transfer Data from DMA buffer to Local buffer
	 */
	pinit_fw_cb = (ADDRESS_CTRL_BLK *)ql4im_ha->ioctl_dma_bufv;
	memset(pinit_fw, 0, sizeof(EXT_INIT_FW_ISCSI));

	pinit_fw->Version         = pinit_fw_cb->Version;
	pinit_fw->FWOptions       = le16_to_cpu(pinit_fw_cb->FwOptions);
	pinit_fw->AddFWOptions    = le16_to_cpu(pinit_fw_cb->AddFwOptions);
	memcpy(pinit_fw->IPAddr.IPAddress, pinit_fw_cb->IPAddr,
	    MIN(sizeof(pinit_fw->IPAddr.IPAddress),
	    sizeof(pinit_fw_cb->IPAddr)));
	memcpy(pinit_fw->SubnetMask.IPAddress, pinit_fw_cb->SubnetMask,
	    MIN(sizeof(pinit_fw->SubnetMask.IPAddress),
	    sizeof(pinit_fw_cb->SubnetMask)));
	memcpy(pinit_fw->Gateway.IPAddress, pinit_fw_cb->GatewayIPAddr,
	    MIN(sizeof(pinit_fw->Gateway.IPAddress),
	    sizeof(pinit_fw_cb->GatewayIPAddr)));
	memcpy(pinit_fw->DNSConfig.IPAddr.IPAddress,
	    pinit_fw_cb->PriDNSIPAddr,
	    MIN(sizeof(pinit_fw->DNSConfig.IPAddr.IPAddress),
	    sizeof(pinit_fw_cb->PriDNSIPAddr)));
	memcpy(pinit_fw->Alias, pinit_fw_cb->iSCSIAlias,
	    MIN(sizeof(pinit_fw->Alias), sizeof(pinit_fw_cb->iSCSIAlias)));
	memcpy(pinit_fw->iSCSIName, pinit_fw_cb->iSCSINameString,
	    MIN(sizeof(pinit_fw->iSCSIName),
	    sizeof(pinit_fw_cb->iSCSINameString)));

	pinit_fw->DeviceInfo.DeviceType = le16_to_cpu(EXT_DEF_ISCSI_LOCAL);
	pinit_fw->DeviceInfo.ExeThrottle =
	    le16_to_cpu(pinit_fw_cb->ExecThrottle);
	pinit_fw->DeviceInfo.RetryCount = pinit_fw_cb->ZIOCount;
	pinit_fw->DeviceInfo.iSCSIOptions =
	    le16_to_cpu(pinit_fw_cb->iSCSIOptions);
	pinit_fw->DeviceInfo.TCPOptions = le16_to_cpu(pinit_fw_cb->TCPOptions);
	pinit_fw->DeviceInfo.IPOptions = le16_to_cpu(pinit_fw_cb->IPOptions);
	pinit_fw->DeviceInfo.MaxPDUSize = le16_to_cpu(pinit_fw_cb->MaxPDUSize);
	pinit_fw->DeviceInfo.FirstBurstSize =
	    le16_to_cpu(pinit_fw_cb->FirstBurstSize);
	pinit_fw->DeviceInfo.LogoutMinTime =
	    le16_to_cpu(pinit_fw_cb->DefaultTime2Wait);
	pinit_fw->DeviceInfo.LogoutMaxTime =
	    le16_to_cpu(pinit_fw_cb->DefaultTime2Retain);
	pinit_fw->DeviceInfo.LogoutMaxTime =
	    le16_to_cpu(pinit_fw_cb->DefaultTime2Retain);
	pinit_fw->DeviceInfo.MaxOutstandingR2T =
	    le16_to_cpu(pinit_fw_cb->MaxOutStndngR2T);
	pinit_fw->DeviceInfo.KeepAliveTimeout =
	    le16_to_cpu(pinit_fw_cb->KeepAliveTimeout);
	pinit_fw->DeviceInfo.PortNumber = le16_to_cpu(pinit_fw_cb->PortNumber);
	pinit_fw->DeviceInfo.MaxBurstSize =
	    le16_to_cpu(pinit_fw_cb->MaxBurstSize);

	/*
	 * Copy the local data to the user's buffer
	 */
	if (copy_to_user(Q64BIT_TO_PTR(ioctl->ResponseAdr,
						 ioctl->AddrMode), pinit_fw,
                                   sizeof(EXT_INIT_FW_ISCSI)) != 0) {
		DEBUG2(printk("qisioctl%d: %s: unable to copy data to user's "
		    "memory area\n", ql4im_ha->host_no, __func__));

		status = (-EFAULT);
		ioctl->Status = EXT_STATUS_COPY_ERR;
		ioctl->ResponseLen = 0;
		goto exit_get_init_fw;
	}

	ioctl->Status = EXT_STATUS_OK;

	DEBUG10(printk("qisioctl%d: EXT_INIT_FW_ISCSI structure:\n", ql4im_ha->host_no));
	DEBUG10(ql4im_dump_bytes(pinit_fw, sizeof(EXT_INIT_FW_ISCSI));)

exit_get_init_fw:
	ql4im_free_ioctl_scrap_mem(ql4im_ha);
	return(status);
}

/**************************************************************************
 * qla4extioctl_get_isns_server
 *	This routine retrieves the iSNS server information.
 *
 * Input:
 *	ql4im_ha    = adapter structure pointer.
 *	ioctl = IOCTL structure pointer.
 *
 * Output:
 *	None
 *
 * Returns:
 *	QLA_SUCCESS = success
 *	QLA_ERROR   = error
 *
 * Context:
 *	Kernel context.
 **************************************************************************/
static int
qla4extioctl_get_isns_server(ql4im_scsi_qla_host_t *ql4im_ha,
			     EXT_IOCTL_ISCSI *ioctl)
{
	int			status = 0;
	EXT_ISNS_SERVER		*pisns_server = NULL;
#if ISNS_SUPPORTED
	uint32_t		mbox_cmd[MBOX_REG_COUNT];
	uint32_t		mbox_sts[MBOX_REG_COUNT];
	FLASH_INIT_FW_CTRL_BLK	*pflash_init_fw_cb = NULL;
	uint16_t		tcp_options = 0;
	uint16_t		ipv6_tcp_options = 0;
#endif

	if (ql4im_get_ioctl_scrap_mem(ql4im_ha, (void **)&pisns_server,
	    sizeof(EXT_ISNS_SERVER))) {
		/* not enough memory */
		DEBUG2(printk("qisioctl%d: %s: inst=%d scrap not big enough. "
		    "size requested=%ld.\n",
		    ql4im_ha->host_no, __func__, ql4im_ha->instance,
		    (ulong)sizeof(EXT_ISNS_SERVER)));

		status = (-ENOMEM);
		ioctl->Status = EXT_STATUS_NO_MEMORY;
		ioctl->ResponseLen = 0;
		goto exit_get_isns_server;
	}

	if (ioctl->ResponseLen < sizeof(EXT_ISNS_SERVER)) {
		DEBUG2(printk("qisioctl%d: %s: response buffer too small\n",
		    ql4im_ha->host_no, __func__));

		status = (-ENOMEM);
		ioctl->Status = EXT_STATUS_BUFFER_TOO_SMALL;
		ioctl->ResponseLen = 0;
		goto exit_get_isns_server;
	}

#if ISNS_SUPPORTED
	if (!ql4im_ha->ioctl_dma_bufv || !ql4im_ha->ioctl_dma_bufp ||
	    (ql4im_ha->ioctl_dma_buf_len < sizeof(FLASH_INIT_FW_CTRL_BLK))) {
		if (ql4im_resize_ioctl_dma_buf(ql4im_ha,
		    sizeof(FLASH_INIT_FW_CTRL_BLK)) != QLA_SUCCESS) {
			DEBUG2(printk("qisioctl%d: %s: unable to allocate memory "
			    "for dma buffer.\n",
			    ql4im_ha->host_no, __func__));

			status = (-ENOMEM);
			ioctl->Status = EXT_STATUS_NO_MEMORY;
			ioctl->ResponseLen = 0;
			goto exit_get_isns_server;
		}
	}

	/*
	 * Copy the IOCTL EXT_ISNS_SERVER buffer from the user's
	 * data space
	 */
	memset(pisns_server, 0, sizeof(EXT_ISNS_SERVER));
	if (copy_from_user((uint8_t *)pisns_server,
		Q64BIT_TO_PTR(ioctl->RequestAdr, ioctl->AddrMode),
			      ioctl->RequestLen) != 0) {
		DEBUG2(printk("scsi%d: %s: unable to copy data from user's "
		       "memory area\n", ql4im_ha->host_no, __func__));

		status = (-EFAULT);
		ioctl->Status = EXT_STATUS_COPY_ERR;
		ioctl->ResponseLen = 0;
		goto exit_get_isns_server;
	}

	/*
	 * First get Flash Initialize Firmware Control Block, so as not to
	 * destroy unaffected data
	 *----------------------------------------------------------------*/
	pflash_init_fw_cb = (FLASH_INIT_FW_CTRL_BLK *)ql4im_ha->ioctl_dma_bufv;

	memset(mbox_cmd, 0, sizeof(mbox_cmd));
	memset(mbox_sts, 0, sizeof(mbox_sts));
	mbox_cmd[0] = MBOX_CMD_READ_FLASH;
	mbox_cmd[1] = LSDW(ql4im_ha->ioctl_dma_bufp);
	mbox_cmd[2] = MSDW(ql4im_ha->ioctl_dma_bufp);
	mbox_cmd[3] = INT_ISCSI_INITFW_FLASH_OFFSET;
	mbox_cmd[4] = sizeof(FLASH_INIT_FW_CTRL_BLK);

	if (ql4im_ha->drvr_ha->ql4_mbx(ql4im_ha->drvr_ha, 5, 2,
				  &mbox_cmd[0], &mbox_sts[0]) == QLA_ERROR) {
		DEBUG2(printk("qisioctl%d: %s: READ_FLASH command failed \n",
		    ql4im_ha->host_no, __func__));

		ioctl->Status = EXT_STATUS_MAILBOX;
		ioctl->VendorSpecificStatus[0] = mbox_sts[0];
		ioctl->VendorSpecificStatus[1] = mbox_sts[1];
		ioctl->ResponseLen = 0;
		goto exit_get_isns_server;
	}

	DEBUG4( printk("qisioctl%d: %s: READ_FLASH command successful \n",
	    ql4im_ha->host_no, __func__));

	/*
	 * Copy iSNS Server info to the isns_server structure
	 *---------------------------------------------------*/
	tcp_options = cpu_to_le16(pflash_init_fw_cb->init_fw_cb.TCPOptions);
	ipv6_tcp_options = cpu_to_le16(pflash_init_fw_cb->init_fw_cb.IPv6TCPOptions);

	if (tcp_options & TOPT_ISNSv4_ENABLE) {
		pisns_server->PerformiSNSDiscovery = 1;
		pisns_server->AutomaticiSNSDiscovery = 0;
		pisns_server->PortNumber =
		    cpu_to_le16(pflash_init_fw_cb->init_fw_cb.isns_svr_port);
		pisns_server->IPAddr.Type = EXT_DEF_TYPE_ISCSI_IP;
		memcpy(pisns_server->IPAddr.IPAddress,
		       pflash_init_fw_cb->init_fw_cb.ipv4_isns_svr_ip,
		       MIN(sizeof(pisns_server->IPAddr.IPAddress),
			   sizeof(pflash_init_fw_cb->init_fw_cb.ipv4_isns_svr_ip)));
		memcpy(pisns_server->InitiatorName,
		       pflash_init_fw_cb->init_fw_cb.iSCSINameString,
		       MIN(sizeof(pisns_server->InitiatorName),
			   sizeof(pflash_init_fw_cb->init_fw_cb.iSCSINameString)));
	} else if (ipv6_tcp_options & IPV6_TCPOPT_ISNSv6_ENABLE) {
		pisns_server->PerformiSNSDiscovery = 1;
		pisns_server->AutomaticiSNSDiscovery = 0;
		pisns_server->PortNumber =
			cpu_to_le16(pflash_init_fw_cb->init_fw_cb.isns_svr_port);
		pisns_server->IPAddr.Type = EXT_DEF_TYPE_ISCSI_IPV6;
		memcpy(pisns_server->IPAddr.IPAddress,
			pflash_init_fw_cb->init_fw_cb.ipv6_isns_svr_ip,
			MIN(sizeof(pisns_server->IPAddr.IPAddress),
			    sizeof(pflash_init_fw_cb->init_fw_cb.ipv6_isns_svr_ip)));
		memcpy(pisns_server->InitiatorName,
			pflash_init_fw_cb->init_fw_cb.iSCSINameString,
			MIN(sizeof(pisns_server->InitiatorName),
			    sizeof(pflash_init_fw_cb->init_fw_cb.iSCSINameString)));
	}

	/*
         * Notify application layer which features bits are supported.
         *-----------------------------------------------------------*/
	pisns_server->FeaturesResponse |=
		EXT_DEF_RETURN_ISNS_CONN_STATUS | EXT_DEF_IMMEDIATE_ISNS_CONFIG;

	/*
	 * Display the iSNS Server State
	 *-----------------------------------------------*/
	#if defined(QL_DEBUG_LEVEL_2) || defined(QL_DEBUG_LEVEL_4)
	{
		memset(mbox_cmd, 0, sizeof(mbox_cmd));
		memset(mbox_sts, 0, sizeof(mbox_sts));
		mbox_cmd[0] = MBOX_CMD_GET_FW_STATE;

		if (ql4im_ha->drvr_ha->ql4_mbx(ql4im_ha->drvr_ha, 1, 4, &mbox_cmd[0], &mbox_sts[0]) ==
		    QLA_SUCCESS) {
			if ((pisns_server->FeaturesRequest & EXT_DEF_RETURN_ISNS_CONN_STATUS) != 0) {
				pisns_server->iSNSServerConnOpen =
						ql4im_ha->drvr_ha->addl_fw_state & FW_ADDSTATE_ISNSv4_SVC_ENABLED;
				DEBUG4(printk("scsi%d: %s: GET ISNS SERVICE CONNECTION STATUS = \"%s\"\n",
						ql4im_ha->host_no, __func__,
						((mbox_sts[3] & FW_ADDSTATE_ISNSv4_SVC_ENABLED) != 0)
						? "ACTIVE" : "INACTIVE"));
			}
		} else {
			DEBUG2(printk("scsi%d: %s: MBOX_CMD_GET_FW_STATE command "
			    "failed \n", ql4im_ha->host_no, __func__));
		}

	}
	#endif
#else
	/*
	 * If iSNS is NOT supported in the driver,
	 * report this info to the application layer here...
	 */
	memset(pisns_server, 0, sizeof(EXT_ISNS_SERVER));
	pisns_server->ISNSNotSupported = 1;
#endif

	/*
	 * Copy the local data to the user's buffer
	 *-----------------------------------------*/
	if (copy_to_user(Q64BIT_TO_PTR(ioctl->ResponseAdr,
						 ioctl->AddrMode),
                                   pisns_server, sizeof(EXT_ISNS_SERVER)) != 0) {
		DEBUG2(printk("qisioctl%d: %s: unable to copy data to user's "
		    "memory area\n", ql4im_ha->host_no, __func__));

		status = (-EFAULT);
		ioctl->Status = EXT_STATUS_COPY_ERR;
		ioctl->ResponseLen = 0;
		goto exit_get_isns_server;
	}

	ioctl->Status = EXT_STATUS_OK;
	ioctl->ResponseLen = sizeof(EXT_ISNS_SERVER);
	ioctl->ResponseLen = 0;

	DEBUG10(printk("qisioctl%d: EXT_ISNS_SERVER structure:\n", ql4im_ha->host_no));
	DEBUG10(ql4im_dump_bytes(pisns_server, sizeof(EXT_ISNS_SERVER));)

exit_get_isns_server:
	ql4im_free_ioctl_scrap_mem(ql4im_ha);
	return(status);
}

/**************************************************************************
 * qla4extioctl_get_acb
 *	This routine retrieves the address control block.
 *
 * Input:
 *	ql4im_ha    = adapter structure pointer.
 *	ioctl = IOCTL structure pointer.
 *
 * Output:
 *	None
 *
 * Returns:
 *	QLA_SUCCESS = success
 *	QLA_ERROR   = error
 *
 * Context:
 *	Kernel context.
 **************************************************************************/
static int
qla4extioctl_get_acb(ql4im_scsi_qla_host_t *ql4im_ha, EXT_IOCTL_ISCSI *ioctl)
{
	int	status = 0;
	uint32_t mbox_cmd[MBOX_REG_COUNT];
	uint32_t mbox_sts[MBOX_REG_COUNT];

	if (ql4im_ha->firmware_version[0] == 2) {
		DEBUG2(
		    printk("scsi%d: %s: Not supported by 2-series firmware.\n",
		    ql4im_ha->host_no, __func__));

		ioctl->Status = EXT_STATUS_UNSUPPORTED_SUBCODE;
		ioctl->ResponseLen = 0;
		goto exit_get_acb;
	}

	if (ioctl->ResponseLen < EXT_DEF_ACB_SIZE) {
		DEBUG2(
		    printk("scsi%d: %s: response buffer too small (%d/%xh)\n",
		    ql4im_ha->host_no, __func__, ioctl->ResponseLen,
		    ioctl->ResponseLen));

		status = (-ENOMEM);
		ioctl->Status = EXT_STATUS_BUFFER_TOO_SMALL;
		ioctl->ResponseLen = 0;
		goto exit_get_acb;
	}

	if (!ql4im_ha->ioctl_dma_bufv || !ql4im_ha->ioctl_dma_bufp ||
	    (ql4im_ha->ioctl_dma_buf_len < EXT_DEF_ACB_SIZE)) {
		if (ql4im_resize_ioctl_dma_buf(ql4im_ha, EXT_DEF_ACB_SIZE) !=
		    QLA_SUCCESS) {
			DEBUG2(
			    printk("scsi%d: %s: unable to allocate memory "
			    "for dma buffer.\n",
			    ql4im_ha->host_no, __func__));

			status = (-ENOMEM);
			ioctl->Status = EXT_STATUS_NO_MEMORY;
			ioctl->ResponseLen = 0;
			goto exit_get_acb;
		}
	}

	memset(mbox_cmd, 0, sizeof(mbox_cmd));
	memset(mbox_sts, 0, sizeof(mbox_sts));
	mbox_cmd[0] = MBOX_CMD_GET_ACB;
	mbox_cmd[1] = ioctl->Instance;  /* Primary/Secondary */
	mbox_cmd[2] = LSDW(ql4im_ha->ioctl_dma_bufp);
	mbox_cmd[3] = MSDW(ql4im_ha->ioctl_dma_bufp);
	mbox_cmd[4] = ioctl->ResponseLen;

	if (ql4im_ha->drvr_ha->ql4_mbx(ql4im_ha->drvr_ha, 5, 5, &mbox_cmd[0], &mbox_sts[0])
	    == QLA_ERROR) {
		DEBUG2( printk("scsi%d: %s: MBOX_CMD_GET_ACB FAILED "
				       " w/ status %04X\n",
				       ql4im_ha->host_no, __func__, mbox_sts[0]));
		DEBUG2( printk("mbox[1] = 0x%04x\n"
					   "mbox[2] = 0x%04x\n"
					   "mbox[3] = 0x%04x\n"
					   "mbox[4] = 0x%04x\n",
                                           mbox_sts[1], mbox_sts[2],
					   mbox_sts[3], mbox_sts[4]));

		ioctl->Status = EXT_STATUS_MAILBOX;
		ioctl->VendorSpecificStatus[0] = mbox_sts[0];
		ioctl->VendorSpecificStatus[1] = mbox_sts[1];
		ioctl->VendorSpecificStatus[2] = mbox_sts[2];
		ioctl->VendorSpecificStatus[3] = mbox_sts[3];
		ioctl->VendorSpecificStatus[4] = mbox_sts[4];
		ioctl->ResponseLen = 0;
		goto exit_get_acb;
	}

	/*
	 * Transfer Data from DMA buffer to the user's buffer
	 */
	if (copy_to_user(Q64BIT_TO_PTR(ioctl->ResponseAdr, ioctl->AddrMode),
	    ql4im_ha->ioctl_dma_bufv, EXT_DEF_ACB_SIZE) != 0) {
		DEBUG2(
		    printk("scsi%d: %s: unable to copy data to user's "
		    "memory area\n", ql4im_ha->host_no, __func__));

		status = (-EFAULT);
		ioctl->Status = EXT_STATUS_COPY_ERR;
		ioctl->ResponseLen = 0;
		goto exit_get_acb;
	}

	ioctl->Status = EXT_STATUS_OK;
	ioctl->ResponseLen = mbox_sts[4];

	DEBUG4(printk("scsi%d: %s: Succeeded\n",
	    ql4im_ha->host_no, __func__));
	DEBUG10(printk("scsi%d: ACB dump:\n", ql4im_ha->host_no));
	DEBUG10(ql4im_dump_bytes(ql4im_ha->ioctl_dma_bufv, EXT_DEF_ACB_SIZE);)

exit_get_acb:
	return(status);
}

/**************************************************************************
 * qla4extioctl_get_neighbor_cache
 *	This routine retrieves the neighbor cache or neighbor cache size.
 *
 * Input:
 *	ql4im_ha    = adapter structure pointer.
 *	ioctl = IOCTL structure pointer.
 *
 * Output:
 *	None
 *
 * Returns:
 *	QLA_SUCCESS = success
 *	QLA_ERROR   = error
 *
 * Context:
 *	Kernel context.
 **************************************************************************/
static int
qla4extioctl_get_neighbor_cache(ql4im_scsi_qla_host_t *ql4im_ha, EXT_IOCTL_ISCSI *ioctl)
{
	int	status = 0;
	EXT_NEIGHBOR_CACHE *pncache = NULL;
	uint32_t mbox_cmd[MBOX_REG_COUNT];
	uint32_t mbox_sts[MBOX_REG_COUNT];

	/*
	 * Allocate local EXT_NEIGHBOR_CACHE buffer
	 */
	 if (ioctl->ResponseLen >= sizeof(EXT_NEIGHBOR_CACHE) &&
	     ql4im_get_ioctl_scrap_mem(ql4im_ha, (void **)&pncache,
	     ioctl->ResponseLen)) {
		 /* not enough memory */
		 DEBUG2(
		     printk("scsi%d: %s: inst=%d scrap not big enough. "
		     "size requested=%ld.\n",
		     ql4im_ha->host_no, __func__, ql4im_ha->instance,
		     (ulong)ioctl->ResponseLen));

		 status = (-ENOMEM);
		 ioctl->Status = EXT_STATUS_NO_MEMORY;
		 ioctl->ResponseLen = 0;
		 goto exit_get_ncache;
	 }

	 /*
	  * Copy user's data to local buffer
	  */
	 if ((status = copy_from_user((uint8_t *)pncache,
	     Q64BIT_TO_PTR(ioctl->RequestAdr, ioctl->AddrMode), ioctl->RequestLen)) != 0) {
		 DEBUG2(
		     printk("scsi%d: %s: unable to copy data from user's "
		     "memory area\n", ql4im_ha->host_no, __func__));

		 status = (-EFAULT);
		 ioctl->Status = EXT_STATUS_COPY_ERR;
		 ioctl->ResponseLen = 0;
		 goto exit_get_ncache;
	 }

	 if (pncache->CacheBufferSize) {
		 /*
		  * Resize pre-allocated IOCTL DMA memory, if necesary
		  */
		if (!ql4im_ha->ioctl_dma_bufv || !ql4im_ha->ioctl_dma_bufp ||
		    (ql4im_ha->ioctl_dma_buf_len < pncache->CacheBufferSize)) {
			if (ql4im_resize_ioctl_dma_buf(ql4im_ha,
			    pncache->CacheBufferSize) != QLA_SUCCESS) {
				DEBUG2(
				    printk("scsi%d: %s: unable to allocate memory "
				    "for dma buffer.\n",
				    ql4im_ha->host_no, __func__));
	
				status = (-ENOMEM);
				ioctl->Status = EXT_STATUS_NO_MEMORY;
				ioctl->ResponseLen = 0;
				goto exit_get_ncache;
			}
		}
	 }

	memset(mbox_cmd, 0, sizeof(mbox_cmd));
	memset(mbox_sts, 0, sizeof(mbox_sts));
	mbox_cmd[0] = MBOX_CMD_GET_IPV6_NEIGHBOR_CACHE;
	mbox_cmd[1] = ioctl->Instance;  /* Primary/Secondary */
	mbox_cmd[2] = LSDW(ql4im_ha->ioctl_dma_bufp);
	mbox_cmd[3] = MSDW(ql4im_ha->ioctl_dma_bufp);
	mbox_cmd[4] = ioctl->Reserved1;
	mbox_cmd[5] = pncache->CacheBufferSize;

	status = ql4im_ha->drvr_ha->ql4_mbx(ql4im_ha->drvr_ha, 6, 7, &mbox_cmd[0], &mbox_sts[0]);

	ioctl->VendorSpecificStatus[0] = mbox_sts[0];
	ioctl->VendorSpecificStatus[1] = mbox_sts[1];
	ioctl->VendorSpecificStatus[2] = mbox_sts[2];
	ioctl->VendorSpecificStatus[3] = mbox_sts[3];
	ioctl->VendorSpecificStatus[4] = mbox_sts[4];
	ioctl->VendorSpecificStatus[5] = mbox_sts[5];
	ioctl->VendorSpecificStatus[6] = mbox_sts[6];

	if (status == QLA_ERROR) {
		DEBUG2( printk("scsi%d: %s: "
					   "MBOX_CMD_GET_IPV6_NEIGHBOR_CACHE "
					   "FAILED w/ status %04X\n",
                                           ql4im_ha->host_no, __func__, mbox_sts[0]));
		DEBUG2( printk("mbox[1] = 0x%04x\n"
					   "mbox[2] = 0x%04x\n"
					   "mbox[3] = 0x%04x\n"
					   "mbox[4] = 0x%04x\n"
					   "mbox[5] = 0x%04x\n"
					   "mbox[6] = 0x%04x\n",
                                           mbox_sts[1], mbox_sts[2],
                                           mbox_sts[3], mbox_sts[4],
					   mbox_sts[5], mbox_sts[6]));

		ioctl->Status = EXT_STATUS_MAILBOX;
		ioctl->ResponseLen = 0;
		goto exit_get_ncache;
	}

	if (pncache->CacheBufferSize) {
		/*
		 * Copy DMA data to local IOCTL buffer
		 */
		memcpy(&pncache->Buffer[0], ql4im_ha->ioctl_dma_bufv,
		    MIN(pncache->CacheBufferSize,
			ioctl->ResponseLen - offsetof(EXT_NEIGHBOR_CACHE, Buffer)));
	}
	else {
		pncache->CacheBufferSize = mbox_sts[5];
	}

	/*
	 * Transfer Data from local IOCTL buffer to the user's buffer
	 */
	if (copy_to_user(Q64BIT_TO_PTR(ioctl->ResponseAdr, ioctl->AddrMode),
	    pncache, ioctl->ResponseLen) != 0) {
		DEBUG2(
		    printk("scsi%d: %s: unable to copy data to user's "
		    "memory area\n", ql4im_ha->host_no, __func__));

		status = (-EFAULT);
		ioctl->Status = EXT_STATUS_COPY_ERR;
		ioctl->ResponseLen = 0;
		goto exit_get_ncache;
	}

	ioctl->Status = EXT_STATUS_OK;

	DEBUG4(printk("scsi%d: %s: Succeeded\n",
	    ql4im_ha->host_no, __func__));
	DEBUG10(printk("scsi%d: NEIGHBOR CACHE dump:\n",
	    ql4im_ha->host_no));
	DEBUG10(ql4im_dump_bytes(pncache, ioctl->ResponseLen);)

exit_get_ncache:
	ql4im_free_ioctl_scrap_mem(ql4im_ha);
	return(status);
}

/**************************************************************************
 * qla4extioctl_get_destination_cache
 *	This routine retrieves the destination cache or destination cache size
 *
 * Input:
 *	ql4im_ha    = adapter structure pointer.
 *	ioctl = IOCTL structure pointer.
 *
 * Output:
 *	None
 *
 * Returns:
 *	QLA_SUCCESS = success
 *	QLA_ERROR   = error
 *
 * Context:
 *	Kernel context.
 **************************************************************************/
static int
qla4extioctl_get_destination_cache(ql4im_scsi_qla_host_t *ql4im_ha, EXT_IOCTL_ISCSI *ioctl)
{
	int	status = 0;
	EXT_DESTINATION_CACHE *pdcache = NULL;
	uint32_t mbox_cmd[MBOX_REG_COUNT];
	uint32_t mbox_sts[MBOX_REG_COUNT];

	/*
	 * Allocate local EXT_DESTINATION_CACHE buffer
	 */
	 if (ioctl->ResponseLen >= sizeof(EXT_DESTINATION_CACHE) &&
	     ql4im_get_ioctl_scrap_mem(ql4im_ha, (void **)&pdcache,
	     ioctl->ResponseLen)) {
		 /* not enough memory */
		 DEBUG2(
		     printk("scsi%d: %s: inst=%d scrap not big enough. "
		     "size requested=%ld.\n",
		     ql4im_ha->host_no, __func__, ql4im_ha->instance,
		     (ulong)ioctl->ResponseLen));

		 status = (-ENOMEM);
		 ioctl->Status = EXT_STATUS_NO_MEMORY;
		 ioctl->ResponseLen = 0;
		 goto exit_get_dcache;
	 }

	 /*
	  * Copy user's data to local buffer
	  */
	 if ((status = copy_from_user((uint8_t *)pdcache,
	     Q64BIT_TO_PTR(ioctl->RequestAdr, ioctl->AddrMode), ioctl->RequestLen)) != 0) {
		 DEBUG2(
		     printk("scsi%d: %s: unable to copy data from user's "
		     "memory area\n", ql4im_ha->host_no, __func__));

		 status = (-EFAULT);
		 ioctl->Status = EXT_STATUS_COPY_ERR;
		 ioctl->ResponseLen = 0;
		 goto exit_get_dcache;
	 }

	 if (pdcache->CacheBufferSize) {
		 /*
		  * Resize pre-allocated IOCTL DMA memory, if necesary
		  */
		if (!ql4im_ha->ioctl_dma_bufv || !ql4im_ha->ioctl_dma_bufp ||
		    (ql4im_ha->ioctl_dma_buf_len < pdcache->CacheBufferSize)) {
			if (ql4im_resize_ioctl_dma_buf(ql4im_ha,
			    pdcache->CacheBufferSize) != QLA_SUCCESS) {
				DEBUG2(
				    printk("scsi%d: %s: unable to allocate memory "
				    "for dma buffer.\n",
				    ql4im_ha->host_no, __func__));
	
				status = (-ENOMEM);
				ioctl->Status = EXT_STATUS_NO_MEMORY;
				ioctl->ResponseLen = 0;
				goto exit_get_dcache;
			}
		}
	 }

	memset(mbox_cmd, 0, sizeof(mbox_cmd));
	memset(mbox_sts, 0, sizeof(mbox_sts));
	mbox_cmd[0] = MBOX_CMD_GET_IPV6_DEST_CACHE;
	mbox_cmd[1] = ioctl->Instance;  /* Primary/Secondary */
	mbox_cmd[2] = LSDW(ql4im_ha->ioctl_dma_bufp);
	mbox_cmd[3] = MSDW(ql4im_ha->ioctl_dma_bufp);
	mbox_cmd[4] = ioctl->Reserved1;
	mbox_cmd[5] = pdcache->CacheBufferSize;

	status = ql4im_ha->drvr_ha->ql4_mbx(ql4im_ha->drvr_ha, 6, 7, &mbox_cmd[0], &mbox_sts[0]);

	ioctl->VendorSpecificStatus[0] = mbox_sts[0];
	ioctl->VendorSpecificStatus[1] = mbox_sts[1];
	ioctl->VendorSpecificStatus[2] = mbox_sts[2];
	ioctl->VendorSpecificStatus[3] = mbox_sts[3];
	ioctl->VendorSpecificStatus[4] = mbox_sts[4];
	ioctl->VendorSpecificStatus[5] = mbox_sts[5];
	ioctl->VendorSpecificStatus[6] = mbox_sts[6];

	if (status == QLA_ERROR) {
		DEBUG2( printk("scsi%d: %s: MBOX_CMD_GET_IPV6_DEST_CACHE "
				           "FAILED w/ status %04X\n",
                                           ql4im_ha->host_no, __func__, mbox_sts[0]));
		DEBUG2( printk("mbox[1] = 0x%04x\n"
					   "mbox[2] = 0x%04x\n"
					   "mbox[3] = 0x%04x\n"
					   "mbox[4] = 0x%04x\n"
					   "mbox[5] = 0x%04x\n"
					   "mbox[6] = 0x%04x\n",
                                           mbox_sts[1], mbox_sts[2],
                                           mbox_sts[3], mbox_sts[4],
					   mbox_sts[5], mbox_sts[6]));

		ioctl->Status = EXT_STATUS_MAILBOX;
		ioctl->ResponseLen = 0;
		goto exit_get_dcache;
	}

	if (pdcache->CacheBufferSize) {
		/*
		 * Copy DMA data to local IOCTL buffer
		 */
		memcpy(&pdcache->Buffer[0], ql4im_ha->ioctl_dma_bufv,
		    MIN(pdcache->CacheBufferSize,
			ioctl->ResponseLen - offsetof(EXT_DESTINATION_CACHE, Buffer)));
	}
	else {
		pdcache->CacheBufferSize = mbox_sts[5];
	}

	/*
	 * Transfer Data from local IOCTL buffer to the user's buffer
	 */
	if (copy_to_user(Q64BIT_TO_PTR(ioctl->ResponseAdr, ioctl->AddrMode),
	    pdcache, ioctl->ResponseLen) != 0) {
		DEBUG2(
		    printk("scsi%d: %s: unable to copy data to user's "
		    "memory area\n", ql4im_ha->host_no, __func__));

		status = (-EFAULT);
		ioctl->Status = EXT_STATUS_COPY_ERR;
		ioctl->ResponseLen = 0;
		goto exit_get_dcache;
	}

	ioctl->Status = EXT_STATUS_OK;

	DEBUG4(printk("scsi%d: %s: Succeeded\n",
	    ql4im_ha->host_no, __func__));
	DEBUG10(printk("scsi%d: DESTINATION CACHE dump:\n",
	    ql4im_ha->host_no));
	DEBUG10(ql4im_dump_bytes(pdcache, ioctl->ResponseLen);)

exit_get_dcache:
	ql4im_free_ioctl_scrap_mem(ql4im_ha);
	return(status);
}

/**************************************************************************
 * qla4extioctl_get_default_router_list
 *	This routine retrieves the IPv6 default router list of size of the
 *      list.
 *
 * Input:
 *	ql4im_ha    = adapter structure pointer.
 *	ioctl = IOCTL structure pointer.
 *
 * Output:
 *	None
 *
 * Returns:
 *	QLA_SUCCESS = success
 *	QLA_ERROR   = error
 *
 * Context:
 *	Kernel context.
 **************************************************************************/
static int
qla4extioctl_get_default_router_list(ql4im_scsi_qla_host_t *ql4im_ha, EXT_IOCTL_ISCSI *ioctl)
{
	int	status = 0;
	EXT_ROUTER_LIST *pdrlist = NULL;
	uint32_t mbox_cmd[MBOX_REG_COUNT];
	uint32_t mbox_sts[MBOX_REG_COUNT];

	/*
	 * Allocate local EXT_ROUTER_LIST buffer
	 */
	 if (ioctl->ResponseLen >= sizeof(EXT_ROUTER_LIST) &&
	     ql4im_get_ioctl_scrap_mem(ql4im_ha, (void **)&pdrlist,
	     ioctl->ResponseLen)) {
		 /* not enough memory */
		 DEBUG2(printk("scsi%d: %s: inst=%d scrap not big enough. "
		     "size requested=%ld.\n",
		     ql4im_ha->host_no, __func__, ql4im_ha->instance,
		     (ulong)ioctl->ResponseLen));

		 status = (-ENOMEM);
		 ioctl->Status = EXT_STATUS_NO_MEMORY;
		 ioctl->ResponseLen = 0;
		 goto exit_get_drlist;
	 }

	 /*
	  * Copy user's data to local buffer
	  */
	 if ((status = copy_from_user((uint8_t *)pdrlist,
	     Q64BIT_TO_PTR(ioctl->RequestAdr, ioctl->AddrMode), ioctl->RequestLen)) != 0) {
		 DEBUG2(
		     printk("scsi%d: %s: unable to copy data from user's "
		     "memory area\n", ql4im_ha->host_no, __func__));

		 status = (-EFAULT);
		 ioctl->Status = EXT_STATUS_COPY_ERR;
		 ioctl->ResponseLen = 0;
		 goto exit_get_drlist;
	 }

	 if (pdrlist->CacheBufferSize) {
		 /*
		  * Resize pre-allocated IOCTL DMA memory, if necesary
		  */
		if (!ql4im_ha->ioctl_dma_bufv || !ql4im_ha->ioctl_dma_bufp ||
		    (ql4im_ha->ioctl_dma_buf_len < pdrlist->CacheBufferSize)) {
			if (ql4im_resize_ioctl_dma_buf(ql4im_ha,
			    pdrlist->CacheBufferSize) != QLA_SUCCESS) {
				DEBUG2(
				    printk("scsi%d: %s: unable to allocate memory "
				    "for dma buffer.\n",
				    ql4im_ha->host_no, __func__));
	
				status = (-ENOMEM);
				ioctl->Status = EXT_STATUS_NO_MEMORY;
				ioctl->ResponseLen = 0;
				goto exit_get_drlist;
			}
		}
	 }

	memset(mbox_cmd, 0, sizeof(mbox_cmd));
	memset(mbox_sts, 0, sizeof(mbox_sts));
	mbox_cmd[0] = MBOX_CMD_GET_IPV6_DEF_ROUTER_LIST;
	mbox_cmd[1] = ioctl->Instance;  /* Primary/Secondary */
	mbox_cmd[2] = LSDW(ql4im_ha->ioctl_dma_bufp);
	mbox_cmd[3] = MSDW(ql4im_ha->ioctl_dma_bufp);
	mbox_cmd[4] = ioctl->Reserved1;
	mbox_cmd[5] = pdrlist->CacheBufferSize;

	status = ql4im_ha->drvr_ha->ql4_mbx(ql4im_ha->drvr_ha, 6, 6, &mbox_cmd[0], &mbox_sts[0]);

	ioctl->VendorSpecificStatus[0] = mbox_sts[0];
	ioctl->VendorSpecificStatus[1] = mbox_sts[1];
	ioctl->VendorSpecificStatus[2] = mbox_sts[2];
	ioctl->VendorSpecificStatus[3] = mbox_sts[3];
	ioctl->VendorSpecificStatus[4] = mbox_sts[4];
	ioctl->VendorSpecificStatus[5] = mbox_sts[5];

	if (status == QLA_ERROR) {
		DEBUG2( printk("scsi%d: %s: MBOX_CMD_GET_IPV6_DEF_ROUTER_LIST "
				           "FAILED w/ status %04X\n",
                                           ql4im_ha->host_no, __func__, mbox_sts[0]));
		DEBUG2( printk("mbox[1] = 0x%04x\n"
					   "mbox[2] = 0x%04x\n"
					   "mbox[3] = 0x%04x\n"
					   "mbox[4] = 0x%04x\n"
					   "mbox[5] = 0x%04x\n",
                                           mbox_sts[1], mbox_sts[2],
                                           mbox_sts[3], mbox_sts[4],
					   mbox_sts[5]));

		ioctl->Status = EXT_STATUS_MAILBOX;
		ioctl->ResponseLen = 0;
		goto exit_get_drlist;
	}

	if (pdrlist->CacheBufferSize) {
		/*
		 * Copy DMA data to local IOCTL buffer
		 */
		memcpy(&pdrlist->Buffer[0], ql4im_ha->ioctl_dma_bufv,
		    MIN(pdrlist->CacheBufferSize,
			ioctl->ResponseLen - offsetof(EXT_ROUTER_LIST, Buffer)));
	}
	else {
		pdrlist->CacheBufferSize = mbox_sts[5];
	}

	/*
	 * Transfer Data from local IOCTL buffer to the user's buffer
	 */
	if (copy_to_user(Q64BIT_TO_PTR(ioctl->ResponseAdr, ioctl->AddrMode),
	    pdrlist, ioctl->ResponseLen) != 0) {
		DEBUG2(
		    printk("scsi%d: %s: unable to copy data to user's "
		    "memory area\n", ql4im_ha->host_no, __func__));

		status = (-EFAULT);
		ioctl->Status = EXT_STATUS_COPY_ERR;
		ioctl->ResponseLen = 0;
		goto exit_get_drlist;
	}

	ioctl->Status = EXT_STATUS_OK;

	DEBUG4(printk("scsi%d: %s: Succeeded\n",
	    ql4im_ha->host_no, __func__));
	DEBUG10(printk("scsi%d: DEFAULT_ROUTER_LIST dump:\n",
	    ql4im_ha->host_no));
	DEBUG10(ql4im_dump_bytes(ql4im_ha->ioctl_dma_bufv, ioctl->ResponseLen);)

exit_get_drlist:
	ql4im_free_ioctl_scrap_mem(ql4im_ha);
	return(status);
}

/**************************************************************************
 * qla4extioctl_get_local_prefix_list
 *	This routine retrieves the IPv6 local prefix list or the size of
 *	the list.
 *
 * Input:
 *	ql4im_ha    = adapter structure pointer.
 *	ioctl = IOCTL structure pointer.
 *
 * Output:
 *	None
 *
 * Returns:
 *	QLA_SUCCESS = success
 *	QLA_ERROR   = error
 *
 * Context:
 *	Kernel context.
 **************************************************************************/
static int
qla4extioctl_get_local_prefix_list(ql4im_scsi_qla_host_t *ql4im_ha, EXT_IOCTL_ISCSI *ioctl)
{
	int	status = 0;
	EXT_PREFIX_LIST *plplist = NULL;
	uint32_t mbox_cmd[MBOX_REG_COUNT];
	uint32_t mbox_sts[MBOX_REG_COUNT];

	/*
	 * Allocate local EXT_PREFIX_LIST buffer
	 */
	 if (ioctl->ResponseLen >= sizeof(EXT_PREFIX_LIST) &&
	     ql4im_get_ioctl_scrap_mem(ql4im_ha, (void **)&plplist,
	     ioctl->ResponseLen)) {
		 /* not enough memory */
		 DEBUG2(
		     printk("scsi%d: %s: inst=%d scrap not big enough. "
		     "size requested=%ld.\n",
		     ql4im_ha->host_no, __func__, ql4im_ha->instance,
		     (ulong)ioctl->ResponseLen));

		 status = (-ENOMEM);
		 ioctl->Status = EXT_STATUS_NO_MEMORY;
		 ioctl->ResponseLen = 0;
		 goto exit_get_lplist;
	 }

	 /*
	  * Copy user's data to local buffer
	  */
	 if ((status = copy_from_user((uint8_t *)plplist,
	     Q64BIT_TO_PTR(ioctl->RequestAdr, ioctl->AddrMode), ioctl->RequestLen)) != 0) {
		 DEBUG2(
		     printk("scsi%d: %s: unable to copy data from user's "
		     "memory area\n", ql4im_ha->host_no, __func__));

		 status = (-EFAULT);
		 ioctl->Status = EXT_STATUS_COPY_ERR;
		 ioctl->ResponseLen = 0;
		 goto exit_get_lplist;
	 }

	 if (plplist->CacheBufferSize) {
		 /*
		  * Resize pre-allocated IOCTL DMA memory, if necesary
		  */
		if (!ql4im_ha->ioctl_dma_bufv || !ql4im_ha->ioctl_dma_bufp ||
		    (ql4im_ha->ioctl_dma_buf_len < plplist->CacheBufferSize)) {
			if (ql4im_resize_ioctl_dma_buf(ql4im_ha,
			    plplist->CacheBufferSize) != QLA_SUCCESS) {
				DEBUG2(
				    printk("scsi%d: %s: unable to allocate memory "
				    "for dma buffer.\n",
				    ql4im_ha->host_no, __func__));
	
				status = (-ENOMEM);
				ioctl->Status = EXT_STATUS_NO_MEMORY;
				ioctl->ResponseLen = 0;
				goto exit_get_lplist;
			}
		}
	 }

	memset(mbox_cmd, 0, sizeof(mbox_cmd));
	memset(mbox_sts, 0, sizeof(mbox_sts));
	mbox_cmd[0] = MBOX_CMD_GET_IPV6_LCL_PREFIX_LIST;
	mbox_cmd[1] = ioctl->Instance;  /* Primary/Secondary */
	mbox_cmd[2] = LSDW(ql4im_ha->ioctl_dma_bufp);
	mbox_cmd[3] = MSDW(ql4im_ha->ioctl_dma_bufp);
	mbox_cmd[4] = ioctl->Reserved1;
	mbox_cmd[5] = plplist->CacheBufferSize;

	status = ql4im_ha->drvr_ha->ql4_mbx(ql4im_ha->drvr_ha, 6, 6, &mbox_cmd[0], &mbox_sts[0]);

	ioctl->VendorSpecificStatus[0] = mbox_sts[0];
	ioctl->VendorSpecificStatus[1] = mbox_sts[1];
	ioctl->VendorSpecificStatus[2] = mbox_sts[2];
	ioctl->VendorSpecificStatus[3] = mbox_sts[3];
	ioctl->VendorSpecificStatus[4] = mbox_sts[4];
	ioctl->VendorSpecificStatus[5] = mbox_sts[5];

	if (status == QLA_ERROR) {
		DEBUG2( printk("scsi%d: %s: MBOX_CMD_GET_IPV6_LCL_PREFIX_LIST "
				           "FAILED w/ status %04X\n",
                                           ql4im_ha->host_no, __func__, mbox_sts[0]));
		DEBUG2( printk("mbox[1] = 0x%04x\n"
					   "mbox[2] = 0x%04x\n"
					   "mbox[3] = 0x%04x\n"
					   "mbox[4] = 0x%04x\n"
					   "mbox[5] = 0x%04x\n",
                                           mbox_sts[1], mbox_sts[2],
                                           mbox_sts[3], mbox_sts[4],
					   mbox_sts[5]));

		ioctl->Status = EXT_STATUS_MAILBOX;
		ioctl->ResponseLen = 0;
		goto exit_get_lplist;
	}

	if (plplist->CacheBufferSize) {
		/*
		 * Copy DMA data to local IOCTL buffer
		 */
		memcpy(&plplist->Buffer[0], ql4im_ha->ioctl_dma_bufv,
		    MIN(plplist->CacheBufferSize,
			ioctl->ResponseLen - offsetof(EXT_PREFIX_LIST, Buffer)));
	}
	else {
		plplist->CacheBufferSize = mbox_sts[5];
	}

	/*
	 * Transfer Data from local IOCTL buffer to the user's buffer
	 */
	if (copy_to_user(Q64BIT_TO_PTR(ioctl->ResponseAdr, ioctl->AddrMode),
	    plplist, ioctl->ResponseLen) != 0) {
		DEBUG2(
		    printk("scsi%d: %s: unable to copy data to user's "
		    "memory area\n", ql4im_ha->host_no, __func__));

		status = (-EFAULT);
		ioctl->Status = EXT_STATUS_COPY_ERR;
		ioctl->ResponseLen = 0;
		goto exit_get_lplist;
	}

	ioctl->Status = EXT_STATUS_OK;

	DEBUG4(printk("scsi%d: %s: Succeeded\n",
	    ql4im_ha->host_no, __func__));
	DEBUG10(printk("scsi%d: LOCAL_PREFIX_LIST dump:\n",
	    ql4im_ha->host_no));
	DEBUG10(ql4im_dump_bytes(ql4im_ha->ioctl_dma_bufv, ioctl->ResponseLen);)

exit_get_lplist:
	ql4im_free_ioctl_scrap_mem(ql4im_ha);
	return(status);
}

/**************************************************************************
 * qla4extioctl_get_statistics_iscsi_block
 *	This routine retrieves the HBA iSCSI statistical information.
 *
 * Input:
 *	ql4im_ha    = adapter structure pointer.
 *	ioctl = IOCTL structure pointer.
 *
 * Output:
 *	None
 *
 * Returns:
 *	QLA_SUCCESS = success
 *	QLA_ERROR   = error
 *
 * Context:
 *	Kernel context.
 **************************************************************************/
static int
qla4extioctl_get_statistics_iscsi_block(ql4im_scsi_qla_host_t *ql4im_ha, EXT_IOCTL_ISCSI *ioctl)
{
	int		status = 0;
	uint32_t	mbox_cmd[MBOX_REG_COUNT];
	uint32_t	mbox_sts[MBOX_REG_COUNT];

	DEBUG4(printk("scsi%d: %s: index [%d]\n",
			      ql4im_ha->host_no, __func__, ioctl->Instance));

	if (!ioctl->ResponseAdr || !ioctl->ResponseLen) {
		DEBUG2(printk("scsi%d: %s: invalid parameter\n",
		    ql4im_ha->host_no, __func__));

		status = (-EINVAL);
		ioctl->Status = EXT_STATUS_INVALID_PARAM;
		ioctl->ResponseLen = 0;
		goto exit_get_stats_iscsi_block;
	}

	if (ioctl->ResponseLen < sizeof(EXT_HBA_PORT_STAT_ISCSI_BLOCK)) {
		DEBUG2(printk("scsi%d: %s: RespLen too small (0x%x),  "
		    "need (0x%x).\n",
		    ql4im_ha->host_no, __func__, ioctl->ResponseLen,
		    (unsigned int) sizeof(EXT_HBA_PORT_STAT_ISCSI_BLOCK)));

		status = (-ENOMEM);
		ioctl->Status = EXT_STATUS_BUFFER_TOO_SMALL;
		ioctl->ResponseLen = 0;
		goto exit_get_stats_iscsi_block;
	}

	if ((ql4im_ha->ioctl_dma_buf_len < sizeof(EXT_HBA_PORT_STAT_ISCSI_BLOCK)) &&
	    (ql4im_resize_ioctl_dma_buf(ql4im_ha, sizeof(EXT_HBA_PORT_STAT_ISCSI_BLOCK))
	     != QLA_SUCCESS)) {
		DEBUG2(printk("scsi%d: %s: unable to allocate memory "
		    "for dma buffer.\n",
		    ql4im_ha->host_no, __func__));

		status = (-ENOMEM);
		ioctl->Status = EXT_STATUS_NO_MEMORY;
		ioctl->ResponseLen = 0;
		goto exit_get_stats_iscsi_block;
	}

	/*
	 * Make the mailbox call
	 */
	memset(mbox_cmd, 0, sizeof(mbox_cmd));
	memset(mbox_sts, 0, sizeof(mbox_sts));
	mbox_cmd[0] = MBOX_CMD_GET_MANAGEMENT_DATA;
	mbox_cmd[1] = ioctl->Instance;
	mbox_cmd[2] = LSDW(ql4im_ha->ioctl_dma_bufp);
	mbox_cmd[3] = MSDW(ql4im_ha->ioctl_dma_bufp);

	if (ql4im_ha->drvr_ha->ql4_mbx(ql4im_ha->drvr_ha, 4, 5, &mbox_cmd[0], &mbox_sts[0]) !=
	    QLA_SUCCESS) {
		DEBUG2(printk("scsi%d: %s: get mngmt data for index [%d] failed "
		    "w/ mailbox ststus 0x%x\n",
		    ql4im_ha->host_no, __func__, ioctl->Instance, mbox_sts[0]));

		ioctl->Status = EXT_STATUS_MAILBOX;
		ioctl->VendorSpecificStatus[0] = mbox_sts[0];
		ioctl->ResponseLen = 0;
		goto exit_get_stats_iscsi_block;
	}

	/*
	 * Copy the data from the dma buffer to the user's data space
	 */
	if (copy_to_user(Q64BIT_TO_PTR(ioctl->ResponseAdr, ioctl->AddrMode),
	    ql4im_ha->ioctl_dma_bufv, ioctl->ResponseLen) != 0) {
		DEBUG2(printk("scsi%d: %s: memory area too small\n",
		    ql4im_ha->host_no, __func__));

		status = (-EFAULT);
		ioctl->Status = EXT_STATUS_COPY_ERR;
		ioctl->ResponseLen = 0;
		goto exit_get_stats_iscsi_block;
	}

	ioctl->Status = EXT_STATUS_OK;
	ioctl->ResponseLen = mbox_sts[4];

exit_get_stats_iscsi_block:
	ql4im_free_ioctl_scrap_mem(ql4im_ha);
	return(status);
}

PDU_ENTRY *
ql4im_get_pdu(ql4im_scsi_qla_host_t *ql4im_ha, uint32_t length)
{
	PDU_ENTRY       *pdu;
	PDU_ENTRY       *free_pdu_top;
	PDU_ENTRY       *free_pdu_bottom;
	uint16_t        pdu_active;
	scsi_qla_host_t *ha = ql4im_ha->drvr_ha;

	if (ha->free_pdu_top == NULL) {
		DEBUG2(printk("scsi%d: %s: Out of PDUs!\n",
				ql4im_ha->host_no, __func__));
		return(NULL);
	}

	/* Save current state */
	free_pdu_top    = ha->free_pdu_top;
	free_pdu_bottom = ha->free_pdu_bottom;
	pdu_active = ha->pdu_active + 1;

	/* get next available pdu */
	pdu = free_pdu_top;
	free_pdu_top = pdu->Next;

	if (free_pdu_top == NULL)
		free_pdu_bottom = NULL;


	/* round up to nearest page */
	length = (length + (PAGE_SIZE-1)) & ~(PAGE_SIZE-1);


	/* Allocate pdu buffer PDU */
	pdu->Buff = pci_alloc_consistent(ha->pdev, length, &pdu->DmaBuff);
	if (pdu->Buff == NULL) {
		DEBUG2(printk("scsi%d: %s: Unable to allocate memory "
				"for PDU buffer\n",
				ql4im_ha->host_no, __func__));
		return(NULL);
	}

	memset(pdu->Buff, 0, length);

	/* Fill in remainder of PDU */
	pdu->BuffLen = length;
	pdu->SendBuffLen = 0;
	pdu->RecvBuffLen = 0;
	pdu->Next = NULL;

	ha->free_pdu_top = free_pdu_top;
	ha->free_pdu_bottom = free_pdu_bottom;
	ha->pdu_active = pdu_active;

	DEBUG2(printk("scsi%d: %s: Get PDU SUCCEEDED!  "
			"Top %p Bot %p PDU %p Buf %p DmaBuf %lx Length %x "
			"Active %d\n", ql4im_ha->host_no, __func__, free_pdu_top,
			free_pdu_bottom, pdu, pdu->Buff,
			(unsigned long)pdu->DmaBuff, pdu->BuffLen,
			pdu_active));
	return(pdu);
}

void ql4im_free_pdu(ql4im_scsi_qla_host_t *ql4im_ha, PDU_ENTRY *pdu)
{
	scsi_qla_host_t *ha = ql4im_ha->drvr_ha;

	if (ha->free_pdu_bottom == NULL) {
		ha->free_pdu_top = pdu;
		ha->free_pdu_bottom = pdu;
	}
	else {
		ha->free_pdu_bottom->Next = pdu;
		ha->free_pdu_bottom = pdu;
	}

	pci_free_consistent(ha->pdev, pdu->BuffLen, pdu->Buff, pdu->DmaBuff);
	ha->pdu_active--;

	DEBUG2(printk("scsi%d: %s: Top %p Bot %p PDU %p Buf %p DmaBuf %lx, "
			"Length %x Active %d\n", ql4im_ha->host_no, __func__,
			ha->free_pdu_top,  ha->free_pdu_bottom, pdu, pdu->Buff,
			(unsigned long) pdu->DmaBuff, pdu->BuffLen,
			ha->pdu_active));

	/* Clear PDU */
	pdu->Buff = NULL;
	pdu->BuffLen = 0;
	pdu->SendBuffLen = 0;
	pdu->RecvBuffLen = 0;
	pdu->Next = NULL;
	pdu->DmaBuff = 0;
}

static uint8_t
ql4_get_next_isns_tgt_name(ql4im_scsi_qla_host_t *ql4im_ha,
			 EXT_ISNS_TARGETS_BUFFER *isns_tgts,
			 uint32_t *isns_error)
{
	struct isnsp_response *isns_response;
	unsigned long   wtime;
	scsi_qla_host_t *drvr_ha = ql4im_ha->drvr_ha;

	set_bit(ISNS_FLAG_IOCTL_INVOKED_QUERY, &drvr_ha->isns.flags);
	drvr_ha->ql4_isns_send_dev_get_next(drvr_ha, &isns_tgts->LastIscsiName[0],
		&isns_tgts->Buffer[0], NULL);

	wtime = jiffies + (PDU_WAIT_TOV * HZ);
	while (!time_after_eq(jiffies, wtime)) {
		if (!test_bit(ISNS_FLAG_IOCTL_INVOKED_QUERY,
			&drvr_ha->isns.flags))
			break;

		DEBUG2(printk("."));
		set_current_state(TASK_UNINTERRUPTIBLE);
		schedule_timeout(10);
	}

	if (test_bit(ISNS_FLAG_IOCTL_INVOKED_QUERY, &drvr_ha->isns.flags)) {
		DEBUG2(printk("scsi%d: %s: Timed Out\n",
				      ql4im_ha->host_no, __func__));
		return (QLA_ERROR);
	}

	isns_response = (struct isnsp_response *) isns_tgts->Buffer;
	*isns_error = be32_to_cpu(isns_response->status_code);

	switch (*isns_error) {
		case ISNS_STS_SUCCESS:
/*
			isns_attr = (pisnsp_attribute) &isns_response->attributes[0];
			buffer_end = (uint8_t *) ((uint8_t *) &isns_message->payload[0] +
						   be16_to_cpu(isns_message->pdu_len));

			while (VALIDATE_ATTR(isns_attr, buffer_end)) {
				if(be32_to_cpu(isns_attr->tag) == ISNS_ATTR_ISCSI_NAME) {
				    strncpy(isns_tgts->LastIscsiName,
					    &isns_attr->val[0],
					    sizeof(isns_tgts->LastIscsiName));
				    DEBUG2(printk("scsi%d: %s: Next iSCSI Name = \"%s\"\n",
					ql4im_ha->host_no, __func__, isns_tgts->LastIscsiName));
				    break;
				}
				isns_attr = NEXT_ATTR(isns_attr);
			}
*/
			DEBUG2(printk("scsi%d: %s: Succeeded\n",
			    ql4im_ha->host_no, __func__));
			return (QLA_SUCCESS);

		case ISNS_STS_NO_SUCH_ENTRY:
			DEBUG2(printk("scsi%d: %s: No More targets\n",
			    ql4im_ha->host_no, __func__));
			return (QLA_SUCCESS);

		default:
			DEBUG2(printk("scsi%d: %s: Invalid Buffer\n",
			    ql4im_ha->host_no, __func__));
			return (QLA_ERROR);
	}
}

static uint8_t
ql4_get_isns_tgts_attrs(ql4im_scsi_qla_host_t *ql4im_ha,
			EXT_ISNS_TARGETS_BUFFER *isns_tgts)
{
	struct isnsp_response *isns_response;
	uint32_t isns_error;
	unsigned long   wtime;
	scsi_qla_host_t *drvr_ha = ql4im_ha->drvr_ha;

	set_bit(ISNS_FLAG_IOCTL_INVOKED_QUERY, &drvr_ha->isns.flags);
	drvr_ha->ql4_isns_send_dev_attr_qry(drvr_ha, &isns_tgts->LastIscsiName[0],
			&isns_tgts->Buffer[0], &isns_tgts->BufferSize);

	wtime = jiffies + (PDU_WAIT_TOV * HZ);
	while (!time_after_eq(jiffies, wtime)) {
		if (!test_bit(ISNS_FLAG_IOCTL_INVOKED_QUERY,
			&drvr_ha->isns.flags))
			break;

		set_current_state(TASK_UNINTERRUPTIBLE);
		schedule_timeout(10);
	}

	if (test_bit(ISNS_FLAG_IOCTL_INVOKED_QUERY, &drvr_ha->isns.flags)) {
		DEBUG2(printk("scsi%d: %s: Timed Out\n",
				      ql4im_ha->host_no, __func__));
		return (QLA_ERROR);
	}

	isns_response = (struct isnsp_response *) isns_tgts->Buffer;
	isns_error = be32_to_cpu(isns_response->status_code);

	switch (isns_error) {
		case ISNS_STS_SUCCESS:
			DEBUG2(printk("scsi%d: %s: Succeeded\n",
			    ql4im_ha->host_no, __func__));
			return (QLA_SUCCESS);

		case ISNS_STS_NO_SUCH_ENTRY:
			DEBUG2(printk("scsi%d: %s: No More targets\n",
			    ql4im_ha->host_no, __func__));
			return (QLA_SUCCESS);

		default:
			DEBUG2(printk("scsi%d: %s: Invalid Buffer\n",
			    ql4im_ha->host_no, __func__));
			return (QLA_ERROR);
	}
}

/**************************************************************************
 * qla4extioctl_get_isns_targets_buffer
 *      This routine retrieves the iSNS targets buffer.
 *
 * Input:
 *      ha    = adapter structure pointer.
 *      ioctl = IOCTL structure pointer.
 *
 * Output:
 *      None
 *
 * Returns:
 *      QLA_SUCCESS = success
 *      QLA_ERROR   = error
 *
 * Context:
 *      Kernel context.
 **************************************************************************/
static int
qla4extioctl_get_isns_targets_buffer(ql4im_scsi_qla_host_t *ql4im_ha, EXT_IOCTL_ISCSI *ioctl)
{
	int     status = 0;
	EXT_ISNS_TARGETS_BUFFER *isns_tgts = NULL;
	uint32_t isns_error;
	scsi_qla_host_t *drvr_ha = ql4im_ha->drvr_ha;

	DEBUG2(printk("scsi%d: %s: index [%d]\n",
			      ql4im_ha->host_no, __func__, ioctl->Instance));

	if (atomic_read(&drvr_ha->isns.state) == ISNS_STATE_TCP_DISCONNECTED) {
		DEBUG2(printk("scsi%d: %s: iSNS Service Not Started\n",
		    ql4im_ha->host_no, __func__));

		status = (-EPERM);
		ioctl->Status = EXT_STATUS_SERVICE_NOT_ENABLED;
		ioctl->ResponseLen = 0;
		goto exit_get_isns_tgts;
	}

	if (!test_bit(ISNS_FLAG_ISNS_SCN_REGISTERED, &drvr_ha->isns.flags)) {
		DEBUG2(printk("scsi%d: %s: iSNS SCN not registered. "
			"Cannot retrieve target database.\n",
			ql4im_ha->host_no, __func__));
		status = (-EPERM);
		ioctl->Status = EXT_STATUS_SERVICE_NOT_ENABLED;
		ioctl->ResponseLen = 0;
		goto exit_get_isns_tgts;
	}

	if (!ioctl->ResponseAdr || !ioctl->ResponseLen) {
		DEBUG2(printk("scsi%d: %s: invalid parameter\n",
		    ql4im_ha->host_no, __func__));

		status = (-EINVAL);
		ioctl->Status = EXT_STATUS_INVALID_PARAM;
		ioctl->ResponseLen = 0;
		goto exit_get_isns_tgts;
	}

	if (ioctl->ResponseLen < sizeof(EXT_ISNS_TARGETS_BUFFER)) {
		DEBUG2(printk("scsi%d: %s: RespLen too small (0x%x),  "
		    "need (0x%x).\n",
		    ql4im_ha->host_no, __func__, ioctl->ResponseLen,
		    (unsigned int) sizeof(EXT_ISNS_TARGETS_BUFFER)));

		status = (-ENOMEM);
		ioctl->Status = EXT_STATUS_BUFFER_TOO_SMALL;
		ioctl->ResponseLen = 0;
		goto exit_get_isns_tgts;
	}

	if ((ql4im_ha->ioctl_dma_buf_len < sizeof(EXT_ISNS_TARGETS_BUFFER)) &&
	    (ql4im_resize_ioctl_dma_buf(ql4im_ha, sizeof(EXT_ISNS_TARGETS_BUFFER))
	     != QLA_SUCCESS)) {
		DEBUG2(printk("scsi%d: %s: unable to allocate memory "
		    "for dma buffer.\n",
		    ql4im_ha->host_no, __func__));

		status = (-ENOMEM);
		ioctl->Status = EXT_STATUS_NO_MEMORY;
		ioctl->ResponseLen = 0;
		goto exit_get_isns_tgts;
	}

	isns_tgts = (EXT_ISNS_TARGETS_BUFFER *) ql4im_ha->ioctl_dma_bufv;
	memset(isns_tgts, 0, sizeof(EXT_ISNS_TARGETS_BUFFER));

	/* --- Copy input structure from user space --- */
	if ((status = copy_from_user((void *)isns_tgts,
	    Q64BIT_TO_PTR(ioctl->RequestAdr, ioctl->AddrMode), offsetof(EXT_ISNS_TARGETS_BUFFER, Buffer))) != 0) {
		DEBUG2(printk("scsi%d: %s: unable to copy data from "
		    "user's memory area\n", ql4im_ha->host_no, __func__));

		status = (-EFAULT);
		ioctl->Status = EXT_STATUS_COPY_ERR;
		ioctl->ResponseLen = 0;
		goto exit_get_isns_tgts;
	}
	/*
	 * Retrieve the iSNS targets buffer
	 */
	DEBUG2(printk("scsi%d: %s: Incoming iSCSI Name = \"%s\"\n",
	    ql4im_ha->host_no, __func__, isns_tgts->LastIscsiName));

	status = ql4_get_next_isns_tgt_name(ql4im_ha, isns_tgts, &isns_error);
	if (status == QLA_SUCCESS) {
		if (isns_error == ISNS_STS_NO_SUCH_ENTRY) {
			memset(isns_tgts->LastIscsiName, 0,
			       sizeof(isns_tgts->LastIscsiName));
			ioctl->ResponseLen = offsetof(EXT_ISNS_TARGETS_BUFFER, Buffer);
			ioctl->Status = EXT_STATUS_OK;
		} else {
			DEBUG2(printk("scsi%d: %s: Next iSCSI Name = \"%s\"\n",
					ql4im_ha->host_no, __func__, isns_tgts->LastIscsiName));
			status = ql4_get_isns_tgts_attrs(ql4im_ha, isns_tgts);
			DEBUG2(printk("scsi%d: %s: BufferSize = 0x%x\n",
					      ql4im_ha->host_no, __func__, isns_tgts->BufferSize));
			ioctl->ResponseLen = offsetof(EXT_ISNS_TARGETS_BUFFER, Buffer) +
				isns_tgts->BufferSize;
			ioctl->Status = EXT_STATUS_OK;
		}
	}

	if (status != QLA_SUCCESS) {
		status = (-EFAULT);
		ioctl->Status = EXT_STATUS_ERR;
		ioctl->ResponseLen = offsetof(EXT_ISNS_TARGETS_BUFFER, Buffer);
	}

	DEBUG10(printk("scsi%d: %s: dump isns_tgts structure: 0x%p,  "
			      "(0x%x)\n",
			      ql4im_ha->host_no, __func__, isns_tgts, ioctl->ResponseLen));
	DEBUG10(ql4im_dump_bytes(isns_tgts, ioctl->ResponseLen));

	/*
	 * Copy the data from the dma buffer to the user's data space
	 */
	if (copy_to_user(Q64BIT_TO_PTR(ioctl->ResponseAdr, ioctl->AddrMode),
	    ql4im_ha->ioctl_dma_bufv, ioctl->ResponseLen) != 0) {
		DEBUG2(printk("scsi%d: %s: memory area too small\n",
		    ql4im_ha->host_no, __func__));

		status = (-EFAULT);
		ioctl->Status = EXT_STATUS_COPY_ERR;
		ioctl->ResponseLen = 0;
		goto exit_get_isns_tgts;
	}

exit_get_isns_tgts:
	ql4im_free_ioctl_scrap_mem(ql4im_ha);
	return(status);
}

/**************************************************************************
 * qla4extioctl_get_data
 *	This routine calls get data IOCTLs based on the IOCTL Sub Code.
 *
 * Input:
 *	ql4im_ha    = adapter structure pointer.
 *	ioctl = IOCTL structure pointer.
 *
 * Output:
 *	None
 *
 * Returns:
 *	QLA_SUCCESS = success
 *	QLA_ERROR   = error
 *    	-EINVAL     = if the command is invalid
 *
 * Context:
 *	Kernel context.
 **************************************************************************/
static int
qla4extioctl_get_data(ql4im_scsi_qla_host_t *ql4im_ha, EXT_IOCTL_ISCSI *ioctl)
{
	switch (ioctl->SubCode) {
	case EXT_SC_GET_STATISTICS_GEN:
		return(qla4extioctl_get_statistics_gen(ql4im_ha, ioctl));

	case EXT_SC_GET_STATISTICS_ISCSI:
		return(qla4extioctl_get_statistics_iscsi(ql4im_ha, ioctl));

	case EXT_SC_GET_DEVICE_ENTRY_ISCSI:
	case EXT_SC_GET_DEVICE_ENTRY_DEFAULTS_ISCSI:
		return(qla4extioctl_get_device_entry_iscsi(ql4im_ha, ioctl));

	case EXT_SC_GET_INIT_FW_ISCSI:
	case EXT_SC_GET_INIT_FW_DEFAULTS_ISCSI:
		return(qla4extioctl_get_init_fw_iscsi(ql4im_ha, ioctl));

	case EXT_SC_GET_ISNS_SERVER:
		return(qla4extioctl_get_isns_server(ql4im_ha, ioctl));

	case EXT_SC_GET_ACB:
		return(qla4extioctl_get_acb(ql4im_ha, ioctl));

	case EXT_SC_GET_NEIGHBOR_CACHE:
		return(qla4extioctl_get_neighbor_cache(ql4im_ha, ioctl));

	case EXT_SC_GET_DESTINATION_CACHE:
		return(qla4extioctl_get_destination_cache(ql4im_ha, ioctl));

	case EXT_SC_GET_DEFAULT_ROUTER_LIST:
		return(qla4extioctl_get_default_router_list(ql4im_ha, ioctl));

	case EXT_SC_GET_LOCAL_PREFIX_LIST:
		return(qla4extioctl_get_local_prefix_list(ql4im_ha, ioctl));

	case EXT_SC_GET_STATISTICS_ISCSI_BLOCK:
		return(qla4extioctl_get_statistics_iscsi_block(ql4im_ha, ioctl));

	case EXT_SC_GET_ISNS_TARGETS_BUFFER:
		return(qla4extioctl_get_isns_targets_buffer(ql4im_ha, ioctl));

	default:
		DEBUG2(printk("qisioctl%d: %s: unsupported external get "
		    "data sub-command code (%X)\n",
		    ql4im_ha->host_no, __func__, ioctl->SubCode));

		ioctl->Status = EXT_STATUS_UNSUPPORTED_SUBCODE;
		return(-EINVAL);
	}
}

/**************************************************************************
 * qla4extioctl_rst_statistics_gen
 *	This routine clears the HBA general statistical information.
 *
 * Input:
 *	ql4im_ha    = adapter structure pointer.
 *	ioctl = IOCTL structure pointer.
 *
 * Output:
 *	None
 *
 * Returns:
 *	QLA_SUCCESS = success
 *	QLA_ERROR   = error
 *
 * Context:
 *	Kernel context.
 **************************************************************************/
static int
qla4extioctl_rst_statistics_gen(ql4im_scsi_qla_host_t *ql4im_ha,
				EXT_IOCTL_ISCSI *ioctl)
{
	scsi_qla_host_t *drvr_ha = ql4im_ha->drvr_ha;

	/*
	 * Reset the general statistics fields
	 */
	drvr_ha->adapter_error_count = 0;
	drvr_ha->device_error_count = 0;
	drvr_ha->total_io_count = 0;
	drvr_ha->total_mbytes_xferred = 0;
	drvr_ha->isr_count = 0;
	drvr_ha->link_failure_count = 0;
	drvr_ha->invalid_crc_count = 0;

	ioctl->Status = EXT_STATUS_OK;

	return(QLA_SUCCESS);
}

/**************************************************************************
 * qla4extioctl_rst_statistics_iscsi
 *	This routine clears the HBA iSCSI statistical information.
 *
 * Input:
 *	ql4im_ha    = adapter structure pointer.
 *	ioctl = IOCTL structure pointer.
 *
 * Output:
 *	None
 *
 * Returns:
 *	QLA_SUCCESS = success
 *	QLA_ERROR   = error
 *
 * Context:
 *	Kernel context.
 **************************************************************************/
static int
qla4extioctl_rst_statistics_iscsi(ql4im_scsi_qla_host_t *ql4im_ha,
				  EXT_IOCTL_ISCSI *ioctl)
{
	uint32_t mbox_cmd[MBOX_REG_COUNT];
	uint32_t mbox_sts[MBOX_REG_COUNT];

	/*
	 * Make the mailbox call
	 */
	memset(mbox_cmd, 0, sizeof(mbox_cmd));
	memset(mbox_sts, 0, sizeof(mbox_sts));
	mbox_cmd[0] = MBOX_CMD_GET_MANAGEMENT_DATA;
	mbox_cmd[1] = ioctl->Instance;
	mbox_cmd[2] = 0;
	mbox_cmd[3] = 0;

	if (ql4im_ha->drvr_ha->ql4_mbx(ql4im_ha->drvr_ha, 4, 1, &mbox_cmd[0], &mbox_sts[0]) !=
	    QLA_SUCCESS) {
		DEBUG2(printk("qisioctl%d: %s: get mngmt data for index [%d] failed! "
		    "w/ mailbox ststus 0x%x\n",
		    ql4im_ha->host_no, __func__, ioctl->Instance, mbox_sts[0]));

		ioctl->Status = EXT_STATUS_MAILBOX;
		ioctl->VendorSpecificStatus[0] = mbox_sts[0];
		ioctl->ResponseLen = 0;
		return(0);
	}

	ioctl->Status = EXT_STATUS_OK;
	return(0);
}

/**************************************************************************
 * qla4extioctl_set_device_entry_iscsi
 *	This routine configures a device with specific database entry data.
 *
 * Input:
 *	ql4im_ha    = adapter structure pointer.
 *	ioctl = IOCTL structure pointer.
 *
 * Output:
 *	None
 *
 * Returns:
 *	QLA_SUCCESS = success
 *	QLA_ERROR   = error
 *
 * Context:
 *	Kernel context.
 **************************************************************************/
static int
qla4extioctl_set_device_entry_iscsi(ql4im_scsi_qla_host_t *ql4im_ha,
				    EXT_IOCTL_ISCSI *ioctl)
{
	int		status = 0;
	DEV_DB_ENTRY	*pfw_ddb_entry;
	EXT_DEVICE_ENTRY_ISCSI *pdev_entry;

	if (ql4im_get_ioctl_scrap_mem(ql4im_ha, (void **)&pdev_entry,
	    sizeof(EXT_DEVICE_ENTRY_ISCSI))) {
		/* not enough memory */
		DEBUG2(printk("qisioctl%d: %s: inst=%d scrap not big enough. "
		    "size requested=%ld.\n",
		    ql4im_ha->host_no, __func__, ql4im_ha->instance,
		    (ulong)sizeof(EXT_DEVICE_ENTRY_ISCSI)));

		status = (-ENOMEM);
		ioctl->Status = EXT_STATUS_NO_MEMORY;
		ioctl->ResponseLen = 0;
		goto exit_set_dev_entry;
	}

	if (!ql4im_ha->ioctl_dma_bufv ||
	    !ql4im_ha->ioctl_dma_bufp ||
	    !ioctl->RequestAdr) {
		DEBUG2(printk("qisioctl%d: %s: memory allocation problem\n",
		    ql4im_ha->host_no, __func__));

		status = (-ENOMEM);
		ioctl->Status = EXT_STATUS_NO_MEMORY;
		ioctl->ResponseLen = 0;
		goto exit_set_dev_entry;
	}

	if (ql4im_ha->ioctl_dma_buf_len < sizeof(DEV_DB_ENTRY)) {
		if (ql4im_resize_ioctl_dma_buf(ql4im_ha, sizeof(DEV_DB_ENTRY)) !=
		    QLA_SUCCESS) {
			DEBUG2(printk("qisioctl%d: %s: unable to allocate memory "
			    "for dma buffer.\n",
			    ql4im_ha->host_no, __func__));

			status = (-ENOMEM);
			ioctl->Status = EXT_STATUS_NO_MEMORY;
			ioctl->ResponseLen = 0;
			goto exit_set_dev_entry;
		}
	}

	if (ioctl->RequestLen < sizeof(EXT_DEVICE_ENTRY_ISCSI)) {
		DEBUG2(printk("qisioctl%d: %s: memory area too small\n",
		    ql4im_ha->host_no, __func__));

		status = (-ENOMEM);
		ioctl->Status = EXT_STATUS_BUFFER_TOO_SMALL;
		ioctl->ResponseLen = 0;
		goto exit_set_dev_entry;
	}

	/*
	 * Copy the IOCTL EXT_DEVICE_ENTRY_ISCSI buffer from the user's
	 * data space
	 */
	if (copy_from_user((uint8_t *)pdev_entry,
                                     Q64BIT_TO_PTR(ioctl->RequestAdr,
						   ioctl->AddrMode),
				     ioctl->RequestLen) != 0) {
		DEBUG2(printk("qisioctl%d: %s: unable to copy data from user's "
		    "memory area\n", ql4im_ha->host_no, __func__));

		status = (-EFAULT);
		ioctl->Status = EXT_STATUS_COPY_ERR;
		ioctl->ResponseLen = 0;
		goto exit_set_dev_entry;
	}

	/*
	 * Transfer data from IOCTL's EXT_DEVICE_ENTRY_ISCSI buffer to
	 * Fw's DEV_DB_ENTRY buffer
	 */
	pfw_ddb_entry = ql4im_ha->ioctl_dma_bufv;
	memset(pfw_ddb_entry, 0, sizeof(DEV_DB_ENTRY));

	pfw_ddb_entry->options          = pdev_entry->Options;
	pfw_ddb_entry->TSID             = cpu_to_le16(pdev_entry->TargetSessID);
	pfw_ddb_entry->exeCount         = cpu_to_le16(pdev_entry->ExeCount);
	pfw_ddb_entry->ddbLink          = cpu_to_le16(pdev_entry->DDBLink);
	memcpy(pfw_ddb_entry->ISID, pdev_entry->InitiatorSessID,
	    sizeof(pdev_entry->InitiatorSessID));
	pfw_ddb_entry->exeThrottle =
	    cpu_to_le16(pdev_entry->DeviceInfo.ExeThrottle);
	pfw_ddb_entry->iSCSIMaxSndDataSegLen =
	    cpu_to_le16(pdev_entry->DeviceInfo.InitMarkerlessInt);
	pfw_ddb_entry->retryCount       = pdev_entry->DeviceInfo.RetryCount;
	pfw_ddb_entry->retryDelay       = pdev_entry->DeviceInfo.RetryDelay;
	pfw_ddb_entry->iSCSIOptions =
	    cpu_to_le16(pdev_entry->DeviceInfo.iSCSIOptions);
	pfw_ddb_entry->TCPOptions =
	    cpu_to_le16(pdev_entry->DeviceInfo.TCPOptions);
	pfw_ddb_entry->IPOptions =
	    cpu_to_le16(pdev_entry->DeviceInfo.IPOptions);
	pfw_ddb_entry->maxPDUSize =
	    cpu_to_le16(pdev_entry->DeviceInfo.MaxPDUSize);
	pfw_ddb_entry->firstBurstSize =
	    cpu_to_le16(pdev_entry->DeviceInfo.FirstBurstSize);
	pfw_ddb_entry->DefaultTime2Wait =
	    cpu_to_le16(pdev_entry->DeviceInfo.LogoutMinTime);
	pfw_ddb_entry->DefaultTime2Retain =
	    cpu_to_le16(pdev_entry->DeviceInfo.LogoutMaxTime);
	pfw_ddb_entry->maxOutstndngR2T =
	    cpu_to_le16(pdev_entry->DeviceInfo.MaxOutstandingR2T);
	pfw_ddb_entry->keepAliveTimeout =
	    cpu_to_le16(pdev_entry->DeviceInfo.KeepAliveTimeout);
	pfw_ddb_entry->RemoteTCPPortNumber =
	    cpu_to_le16(pdev_entry->DeviceInfo.PortNumber);
	pfw_ddb_entry->maxBurstSize =
	    cpu_to_le16(pdev_entry->DeviceInfo.MaxBurstSize);
	pfw_ddb_entry->taskMngmntTimeout =
	    cpu_to_le16(pdev_entry->DeviceInfo.TaskMgmtTimeout);
	memcpy(pfw_ddb_entry->targetAddr,
	       pdev_entry->DeviceInfo.TargetAddr,
	       sizeof(pdev_entry->DeviceInfo.TargetAddr));

	memcpy(pfw_ddb_entry->RemoteIPAddr,
	       pdev_entry->EntryInfo.IPAddr.IPAddress,
	       sizeof(pdev_entry->EntryInfo.IPAddr.IPAddress));
	memcpy(pfw_ddb_entry->iscsiName,
	       pdev_entry->EntryInfo.iSCSIName,
	       sizeof(pdev_entry->EntryInfo.iSCSIName));
	memcpy(pfw_ddb_entry->iSCSIAlias,
	       pdev_entry->EntryInfo.Alias,
               sizeof(pdev_entry->EntryInfo.Alias));

	/*
	 * Make the IOCTL call
	 */
	if (ql4im_mbx_set_ddb_entry(ql4im_ha->drvr_ha, ioctl->Instance, pfw_ddb_entry,
	    ql4im_ha->ioctl_dma_bufp) != QLA_SUCCESS) {
		DEBUG2(printk("qisioctl%d: %s: SET DDB Entry failed\n",
		    ql4im_ha->host_no, __func__));

		status = (-EFAULT);
		ioctl->Status = EXT_STATUS_ERR;
		ioctl->ResponseLen = 0;
		goto exit_set_dev_entry;
	}

	ioctl->Status = EXT_STATUS_OK;

exit_set_dev_entry:
	ql4im_free_ioctl_scrap_mem(ql4im_ha);
	return(status);
}

/**************************************************************************
 * qla4extioctl_set_init_fw_iscsi
 *	This routine configures a device with specific data entry data.
 *
 * Input:
 *	ql4im_ha    = adapter structure pointer.
 *	ioctl = IOCTL structure pointer.
 *
 * Output:
 *	None
 *
 * Returns:
 *	QLA_SUCCESS = success
 *	QLA_ERROR   = error
 *
 * Context:
 *	Kernel context.
 **************************************************************************/
static int
qla4extioctl_set_init_fw_iscsi(ql4im_scsi_qla_host_t *ql4im_ha,
			       EXT_IOCTL_ISCSI *ioctl)
{
	int			status = 0;
	EXT_INIT_FW_ISCSI	*pinit_fw = NULL;
	ADDRESS_CTRL_BLK	*pinit_fw_cb;
	uint32_t		mbox_cmd[MBOX_REG_COUNT];
	uint32_t		mbox_sts[MBOX_REG_COUNT];

	if (ql4im_get_ioctl_scrap_mem(ql4im_ha, (void **)&pinit_fw,
	    sizeof(EXT_INIT_FW_ISCSI))) {
		/* not enough memory */
		DEBUG2(printk("qisioctl%d: %s: inst=%d scrap not big enough. "
		    "size requested=%ld.\n",
		    ql4im_ha->host_no, __func__, ql4im_ha->instance,
		    (ulong)sizeof(EXT_INIT_FW_ISCSI)));

		status = (-ENOMEM);
		ioctl->Status = EXT_STATUS_NO_MEMORY;
		ioctl->ResponseLen = 0;
		goto exit_set_init_fw;
	}

	if (!ql4im_ha->ioctl_dma_bufv || !ql4im_ha->ioctl_dma_bufp ||
	    (ql4im_ha->ioctl_dma_buf_len < sizeof(INIT_FW_CTRL_BLK)) ||
	    (ioctl->RequestLen < sizeof(EXT_INIT_FW_ISCSI))) {
		DEBUG2(printk("qisioctl%d: %s: requst buffer too small\n",
		    ql4im_ha->host_no, __func__));

		status = (-ENOMEM);
		ioctl->Status = EXT_STATUS_BUFFER_TOO_SMALL;
		ioctl->ResponseLen = 0;
		goto exit_set_init_fw;
	}

	/*
	 * Copy the data from the user's buffer
	 */
	if (copy_from_user((uint8_t *)pinit_fw,
                                     Q64BIT_TO_PTR(ioctl->RequestAdr,
						   ioctl->AddrMode),
				     sizeof(EXT_INIT_FW_ISCSI)) != 0) {
		DEBUG2(printk("qisioctl%d: %s: unable to copy data to user's "
		    "memory area\n", ql4im_ha->host_no, __func__));

		status = (-EFAULT);
		ioctl->Status = EXT_STATUS_COPY_ERR;
		ioctl->ResponseLen = 0;
		goto exit_set_init_fw;
	}

	/*
	 * First get Initialize Firmware Control Block, so as not to
	 * destroy unaffected data
	 */
	memset(mbox_cmd, 0, sizeof(mbox_cmd));
	memset(mbox_sts, 0, sizeof(mbox_sts));
	mbox_cmd[0] = MBOX_CMD_GET_INIT_FW_CTRL_BLOCK;
	mbox_cmd[2] = LSDW(ql4im_ha->ioctl_dma_bufp);
	mbox_cmd[3] = MSDW(ql4im_ha->ioctl_dma_bufp);

	if (ql4im_ha->drvr_ha->ql4_mbx(ql4im_ha->drvr_ha, 5, 1,
				  &mbox_cmd[0], &mbox_sts[0]) == QLA_ERROR) {
		DEBUG2(printk("qisioctl%d: %s: command failed \n",
		    ql4im_ha->host_no, __func__));

		ioctl->Status = EXT_STATUS_MAILBOX;
		ioctl->VendorSpecificStatus[0] = mbox_sts[0];
		ioctl->ResponseLen = 0;
		goto exit_set_init_fw;
	}

	/*
	 * Transfer Data from Local buffer to DMA buffer
	 */
	pinit_fw_cb = (ADDRESS_CTRL_BLK *)ql4im_ha->ioctl_dma_bufv;

	pinit_fw_cb->Version         = pinit_fw->Version;
	pinit_fw_cb->FwOptions       = cpu_to_le16(pinit_fw->FWOptions);
	pinit_fw_cb->AddFwOptions    = cpu_to_le16(pinit_fw->AddFWOptions);
	memcpy(pinit_fw_cb->IPAddr, pinit_fw->IPAddr.IPAddress,
	    MIN(sizeof(pinit_fw_cb->IPAddr),
	    sizeof(pinit_fw->IPAddr.IPAddress)));
	memcpy(pinit_fw_cb->SubnetMask, pinit_fw->SubnetMask.IPAddress,
	    MIN(sizeof(pinit_fw_cb->SubnetMask),
	    sizeof(pinit_fw->SubnetMask.IPAddress)));
	memcpy(pinit_fw_cb->GatewayIPAddr, pinit_fw->Gateway.IPAddress,
	    MIN(sizeof(pinit_fw_cb->GatewayIPAddr),
	    sizeof(pinit_fw->Gateway.IPAddress)));
	memcpy(pinit_fw_cb->PriDNSIPAddr, pinit_fw->DNSConfig.IPAddr.IPAddress,
	    MIN(sizeof(pinit_fw_cb->PriDNSIPAddr),
	    sizeof(pinit_fw->DNSConfig.IPAddr.IPAddress)));
	memcpy(pinit_fw_cb->iSCSIAlias, pinit_fw->Alias,
	    MIN(sizeof(pinit_fw_cb->iSCSIAlias), sizeof(pinit_fw->Alias)));
	memcpy(pinit_fw_cb->iSCSINameString, pinit_fw->iSCSIName,
	    MIN(sizeof(pinit_fw_cb->iSCSINameString),
	    sizeof(pinit_fw->iSCSIName)));

	pinit_fw_cb->ExecThrottle =
	    cpu_to_le16(pinit_fw->DeviceInfo.ExeThrottle);
	pinit_fw_cb->ZIOCount         = pinit_fw->DeviceInfo.RetryCount;
	pinit_fw_cb->iSCSIOptions =
	    cpu_to_le16(pinit_fw->DeviceInfo.iSCSIOptions);
	pinit_fw_cb->TCPOptions = cpu_to_le16(pinit_fw->DeviceInfo.TCPOptions);
	pinit_fw_cb->IPOptions = cpu_to_le16(pinit_fw->DeviceInfo.IPOptions);
	pinit_fw_cb->MaxPDUSize = cpu_to_le16(pinit_fw->DeviceInfo.MaxPDUSize);
	pinit_fw_cb->FirstBurstSize =
	    cpu_to_le16(pinit_fw->DeviceInfo.FirstBurstSize);
	pinit_fw_cb->DefaultTime2Wait =
	    cpu_to_le16(pinit_fw->DeviceInfo.LogoutMinTime);
	pinit_fw_cb->DefaultTime2Retain =
	    cpu_to_le16(pinit_fw->DeviceInfo.LogoutMaxTime);
	pinit_fw_cb->MaxOutStndngR2T =
	    cpu_to_le16(pinit_fw->DeviceInfo.MaxOutstandingR2T);
	pinit_fw_cb->KeepAliveTimeout =
	    cpu_to_le16(pinit_fw->DeviceInfo.KeepAliveTimeout);
	pinit_fw_cb->PortNumber = cpu_to_le16(pinit_fw->DeviceInfo.PortNumber);
	pinit_fw_cb->MaxBurstSize =
	    cpu_to_le16(pinit_fw->DeviceInfo.MaxBurstSize);

	/*
	 * Send mailbox command
	 */

	memset(mbox_cmd, 0, sizeof(mbox_cmd));
	memset(mbox_sts, 0, sizeof(mbox_sts));
	mbox_cmd[0] = MBOX_CMD_INITIALIZE_FIRMWARE;
	mbox_cmd[2] = LSDW(ql4im_ha->ioctl_dma_bufp);
	mbox_cmd[3] = MSDW(ql4im_ha->ioctl_dma_bufp);
	if (ql4im_ha->acb_version == ACB_SUPPORTED) {
		mbox_cmd[4] = ql4im_ha->ifcb_size;
		mbox_cmd[5] = (IFCB_VER_MAX << 8) | IFCB_VER_MIN;
	}

	if ((status = ql4im_ha->drvr_ha->ql4_mbx(ql4im_ha->drvr_ha, 4, 1,
			&mbox_cmd[0], &mbox_sts[0])) == QLA_ERROR) {
		DEBUG2(printk("qisioctl%d: %s: command failed \n",
		    ql4im_ha->host_no, __func__));

		ioctl->Status = EXT_STATUS_MAILBOX;
		ioctl->VendorSpecificStatus[0] = mbox_sts[0];
		ioctl->ResponseLen = 0;
		goto exit_set_init_fw;
	}

	ioctl->Status = EXT_STATUS_OK;

	DEBUG10(printk("qisioctl%d: EXT_INIT_FW_ISCSI structure:\n", ql4im_ha->host_no));
	DEBUG10(ql4im_dump_bytes(pinit_fw, sizeof(EXT_INIT_FW_ISCSI));)

exit_set_init_fw:
	ql4im_free_ioctl_scrap_mem(ql4im_ha);
	return(status);
}

void ql4im_queue_aen(ql4im_scsi_qla_host_t *ql4im_ha, uint32_t *mbox_sts)
{
	scsi_qla_host_t *drvr_ha = ql4im_ha->drvr_ha;

        drvr_ha->aen_count++;
        if (drvr_ha->aen_count == 0) {
                /* We overflowed the counter */
                drvr_ha->aen_report = 0;
        }

        if (drvr_ha->aen_count - drvr_ha->aen_report == MAX_AEN_ENTRIES) {
                /* We wrapped the queue,
                 * update starting point for SDMAPI copy */
                drvr_ha->aen_report++;
        }

        if (drvr_ha->aen_in == (MAX_AEN_ENTRIES - 1))
                drvr_ha->aen_in = 0;
        else
                drvr_ha->aen_in++;

        memcpy(&drvr_ha->aen_q[drvr_ha->aen_in].mbox_sts[0],
               mbox_sts,
               sizeof(mbox_sts[0]) * MBOX_AEN_REG_COUNT);

}

void ql4im_queue_isns_sts_chg_aen(ql4im_scsi_qla_host_t *ql4im_ha,
				  uint32_t chg_type)
{
        uint32_t mbox_sts[MBOX_REG_COUNT];
        memset(mbox_sts, 0, sizeof(mbox_sts));
        mbox_sts[0] = MBOX_DRVR_ASTS_ISNS_STATUS_CHANGE;
        mbox_sts[1] = chg_type;
        ql4im_queue_aen(ql4im_ha, &mbox_sts[0]);

        DEBUG2(printk("scsi%d: %s: AEN 0x7002\n",
                ql4im_ha->host_no, __func__));
}

static inline uint8_t ql4_is_memzero(const char *m, size_t s)
{
	int i, val = 0;

	if (m) {
		for (i = 0; i < s; i++)
			val |= m[i];
	}
	return (val == 0);
}

/**************************************************************************
 * qla4extioctl_set_isns_server
 *	This routine retrieves the targets discovered via iSNS.
 *
 * Input:
 *	ql4im_ha    = adapter structure pointer.
 *	ioctl = IOCTL structure pointer.
 *
 * Output:
 *	None
 *
 * Returns:
 *	QLA_SUCCESS = success
 *	QLA_ERROR   = error
 *
 * Context:
 *	Kernel context.
 **************************************************************************/
static int
qla4extioctl_set_isns_server(ql4im_scsi_qla_host_t *ql4im_ha,
			     EXT_IOCTL_ISCSI *ioctl)
{
#if ISNS_SUPPORTED
	int			status = 0;
	uint32_t		mbox_cmd[MBOX_REG_COUNT];
	uint32_t		mbox_sts[MBOX_REG_COUNT];
	EXT_ISNS_SERVER		*pisns_server = NULL;
	FLASH_INIT_FW_CTRL_BLK *pflash_init_fw_cb = NULL;
	uint16_t		tcp_options;
	uint16_t		port_number;
	uint16_t		ipv6_tcp_options = 0;
	uint8_t			ip[16];
	uint8_t			tcp_changed = 0;
	uint8_t			port_changed = 0;
	uint8_t			ip_changed = 0;
	uint8_t			is_isns_active;

	scsi_qla_host_t *drvr_ha = ql4im_ha->drvr_ha;

	if (ql4im_get_ioctl_scrap_mem(ql4im_ha, (void **)&pisns_server,
	    sizeof(EXT_ISNS_SERVER))) {
		/* not enough memory */
		ioctl->Status = EXT_STATUS_NO_MEMORY;
		DEBUG2(printk("qisioctl%d: %s: inst=%d scrap not big enough. "
		    "size requested=%ld.\n",
		    ql4im_ha->host_no, __func__, ql4im_ha->instance,
		    (ulong)sizeof(EXT_ISNS_SERVER)));

		status = (-ENOMEM);
		ioctl->Status = EXT_STATUS_NO_MEMORY;
		ioctl->ResponseLen = 0;
		goto exit_set_isns_svr;
	}

	if (ioctl->RequestLen < sizeof(*pisns_server)) {
		DEBUG2(printk("qisioctl%d: %s: requst buffer too small (%d/%xh)\n",
		    ql4im_ha->host_no, __func__, ioctl->RequestLen,
		    ioctl->RequestLen));

		status = (-ENOMEM);
		ioctl->Status = EXT_STATUS_BUFFER_TOO_SMALL;
		ioctl->ResponseLen = 0;
		goto exit_set_isns_svr;
	}

	if (!ql4im_ha->ioctl_dma_bufv || !ql4im_ha->ioctl_dma_bufp ||
	    (ql4im_ha->ioctl_dma_buf_len < sizeof(FLASH_INIT_FW_CTRL_BLK))) {
		if (ql4im_resize_ioctl_dma_buf(ql4im_ha,
		    sizeof(DEV_DB_ENTRY)) != QLA_SUCCESS) {
			DEBUG2(printk("qisioctl%d: %s: unable to allocate memory "
			    "for dma buffer.\n",
			    ql4im_ha->host_no, __func__));

			status = (-ENOMEM);
			ioctl->Status = EXT_STATUS_NO_MEMORY;
			ioctl->ResponseLen = 0;
			goto exit_set_isns_svr;
		}
	}

	/*
	 * Copy iSNS Server info from the user's buffer
	 *---------------------------------------------*/
	if (copy_from_user((uint8_t *)pisns_server,
			   Q64BIT_TO_PTR(ioctl->RequestAdr, ioctl->AddrMode),
			   sizeof(EXT_ISNS_SERVER)) != 0) {
		DEBUG2(printk("qisioctl%d: %s: unable to copy data to user's "
		    "memory area\n", ql4im_ha->host_no, __func__));

		status = (-EFAULT);
		ioctl->Status = EXT_STATUS_COPY_ERR;
		ioctl->ResponseLen = 0;
		goto exit_set_isns_svr;
	}

	DEBUG10(printk("qisioctl%d: EXT_ISNS_SERVER structure:\n", ql4im_ha->host_no));
	DEBUG10(ql4im_dump_bytes(pisns_server, sizeof(EXT_ISNS_SERVER));)

	/* If IPv6 is not supported and an IPv6 address is specified -- ERROR */
	if (!is_ipv6_enabled(drvr_ha) &&
	    (pisns_server->IPAddr.Type == EXT_DEF_TYPE_ISCSI_IPV6)) {
		DEBUG2(printk("qisioctl%d: %s: Error: iSNS IPv6 address "
			"specified, but IPv6 not supported\n",
			ql4im_ha->host_no, __func__));
		status = (-EINVAL);
		ioctl->Status = EXT_STATUS_INVALID_PARAM;
		ioctl->ResponseLen = 0;
		goto exit_set_isns_svr;
	}

	DEBUG2(printk("%s: %s iSNS \n", __func__,
		pisns_server->PerformiSNSDiscovery ? "Enable" : "Disable"));

	if (pisns_server->PerformiSNSDiscovery == 0)
		set_bit(ISNS_FLAG_DISABLE_IN_PROGRESS, &drvr_ha->isns.flags);

	/*
	 * First get Flash Initialize Firmware Control Block, so as not to
	 * destroy unaffected data
	 *----------------------------------------------------------------*/
	memset(mbox_cmd, 0, sizeof(mbox_cmd));
	memset(mbox_sts, 0, sizeof(mbox_sts));
	mbox_cmd[0] = MBOX_CMD_READ_FLASH;
	mbox_cmd[1] = LSDW(ql4im_ha->ioctl_dma_bufp);
	mbox_cmd[2] = MSDW(ql4im_ha->ioctl_dma_bufp);
	mbox_cmd[3] = INT_ISCSI_INITFW_FLASH_OFFSET;
	mbox_cmd[4] = sizeof(FLASH_INIT_FW_CTRL_BLK);

	if (ql4im_ha->drvr_ha->ql4_mbx(ql4im_ha->drvr_ha, 5, 2,
				  &mbox_cmd[0], &mbox_sts[0]) == QLA_ERROR) {
		DEBUG2(printk("qisioctl%d: %s: READ_FLASH command failed \n",
		    ql4im_ha->host_no, __func__));

		ioctl->Status = EXT_STATUS_MAILBOX;
		ioctl->VendorSpecificStatus[0] = mbox_sts[0];
		ioctl->VendorSpecificStatus[1] = mbox_sts[1];
		ioctl->ResponseLen = 0;
		goto exit_set_isns_svr;
	}

	DEBUG4( printk("qisioctl%d: %s: READ_FLASH command successful \n",
	    ql4im_ha->host_no, __func__));

	/*
	 * Copy iSNS Server info to the flash_init_fw_cb
	 *----------------------------------------------*/
	pflash_init_fw_cb = (FLASH_INIT_FW_CTRL_BLK *)ql4im_ha->ioctl_dma_bufv;

	/* Save incoming flash info locally */
	memset(&ip[0], 0, sizeof(ip));
	if (pisns_server->IPAddr.Type == EXT_DEF_TYPE_ISCSI_IP)
		memcpy(&ip[0],
		       pflash_init_fw_cb->init_fw_cb.ipv4_isns_svr_ip,
		       MIN(sizeof(ip),
			   sizeof(pflash_init_fw_cb->init_fw_cb.ipv4_isns_svr_ip)));
	else
		memcpy(&ip[0],
		       pflash_init_fw_cb->init_fw_cb.ipv6_isns_svr_ip,
		       MIN(sizeof(ip),
			   sizeof(pflash_init_fw_cb->init_fw_cb.ipv6_isns_svr_ip)));

	// convert a couple of variables used for comparisons
	tcp_options = le16_to_cpu(pflash_init_fw_cb->init_fw_cb.TCPOptions);
	ipv6_tcp_options = le16_to_cpu(pflash_init_fw_cb->init_fw_cb.IPv6TCPOptions);
	port_number = le16_to_cpu(pflash_init_fw_cb->init_fw_cb.isns_svr_port);

	/* Update local copy of IFCB variables as determined by parameters
	 * from the user's iSNS structure. */
	if (pisns_server->PerformiSNSDiscovery) {
		if (pisns_server->IPAddr.Type == EXT_DEF_TYPE_ISCSI_IP)
			tcp_options |= TOPT_ISNSv4_ENABLE;
		else
			ipv6_tcp_options |= IPV6_TCPOPT_ISNSv6_ENABLE;

		port_number = EXT_DEF_ISNS_WELL_KNOWN_PORT;
		memcpy(&ip[0], pisns_server->IPAddr.IPAddress,
			MIN(sizeof(pisns_server->IPAddr.IPAddress),
			    sizeof(pflash_init_fw_cb->init_fw_cb.ipv6_isns_svr_ip)));
	} else {
		if (pisns_server->IPAddr.Type == EXT_DEF_TYPE_ISCSI_IP)
			tcp_options &= ~TOPT_ISNSv4_ENABLE;
		else
			ipv6_tcp_options &= ~IPV6_TCPOPT_ISNSv6_ENABLE;

		memset(&ip[0], 0, sizeof(ip));
	}

	DEBUG4( printk("qisioctl%d: %s: IPAddr %d.%d.%d.%d Port# %04d\n",
	    ql4im_ha->host_no, __func__,
	    pflash_init_fw_cb->init_fw_cb.ipv4_isns_svr_ip[0],
	    pflash_init_fw_cb->init_fw_cb.ipv4_isns_svr_ip[1],
	    pflash_init_fw_cb->init_fw_cb.ipv4_isns_svr_ip[2],
	    pflash_init_fw_cb->init_fw_cb.ipv4_isns_svr_ip[3],
	    port_number));

	/* Find out what changed */
	port_changed = (port_number !=
		le16_to_cpu(pflash_init_fw_cb->init_fw_cb.isns_svr_port));
	tcp_changed |= (tcp_options !=
		le16_to_cpu(pflash_init_fw_cb->init_fw_cb.TCPOptions));
	tcp_changed |= (ipv6_tcp_options !=
		le16_to_cpu(pflash_init_fw_cb->init_fw_cb.IPv6TCPOptions));
	if (pisns_server->IPAddr.Type == EXT_DEF_TYPE_ISCSI_IP)
		ip_changed |= (memcmp(&ip[0],
			pflash_init_fw_cb->init_fw_cb.ipv4_isns_svr_ip,
			MIN(sizeof(ip),
			    sizeof(pflash_init_fw_cb->init_fw_cb.ipv4_isns_svr_ip)))
				!= 0);
	else
		ip_changed |= (memcmp(&ip[0],
			pflash_init_fw_cb->init_fw_cb.ipv6_isns_svr_ip,
			MIN(sizeof(ip),
			    sizeof(pflash_init_fw_cb->init_fw_cb.ipv6_isns_svr_ip)))
				!= 0);

	if (ip_changed || port_changed || tcp_changed) {
		pflash_init_fw_cb->init_fw_cb.TCPOptions =
					cpu_to_le16(tcp_options);
		pflash_init_fw_cb->init_fw_cb.IPv6TCPOptions =
					cpu_to_le16(ipv6_tcp_options);
		pflash_init_fw_cb->init_fw_cb.isns_svr_port =
					cpu_to_le16(port_number);

		if (pisns_server->IPAddr.Type == EXT_DEF_TYPE_ISCSI_IP)
			memcpy(pflash_init_fw_cb->init_fw_cb.ipv4_isns_svr_ip,
				&ip[0], MIN(sizeof(ip),
				sizeof(pflash_init_fw_cb->init_fw_cb.ipv4_isns_svr_ip)));
		else
			memcpy(pflash_init_fw_cb->init_fw_cb.ipv6_isns_svr_ip,
				&ip[0], MIN(sizeof(ip),
				sizeof(pflash_init_fw_cb->init_fw_cb.ipv6_isns_svr_ip)));

		memset(mbox_cmd, 0, sizeof(mbox_cmd));
		memset(mbox_sts, 0, sizeof(mbox_sts));
		mbox_cmd[0] = MBOX_CMD_WRITE_FLASH;
		mbox_cmd[1] = LSDW(ql4im_ha->ioctl_dma_bufp);
		mbox_cmd[2] = MSDW(ql4im_ha->ioctl_dma_bufp);
		mbox_cmd[3] = INT_ISCSI_INITFW_FLASH_OFFSET;
		mbox_cmd[4] = sizeof(*pflash_init_fw_cb);
		mbox_cmd[5] = WRITE_FLASH_OPTION_COMMIT_DATA;

		if (ql4im_ha->drvr_ha->ql4_mbx(ql4im_ha->drvr_ha, 6, 2, &mbox_cmd[0],
		    &mbox_sts[0]) == QLA_ERROR) {
			DEBUG2(printk("qisioctl%d: %s: WRITE_FLASH command failed \n",
			    ql4im_ha->host_no, __func__));

			ioctl->Status = EXT_STATUS_MAILBOX;
			ioctl->VendorSpecificStatus[0] = mbox_sts[0];
			ioctl->VendorSpecificStatus[1] = mbox_sts[1];
			ioctl->ResponseLen = 0;
			goto exit_set_isns_svr;
		}

		DEBUG4(printk("qisioctl%d: %s: WRITE_FLASH command successful \n",
				ql4im_ha->host_no, __func__));
		DEBUG4(printk("qisioctl%d: Init Fw Ctrl Blk\n", ql4im_ha->host_no));
		DEBUG4(ql4im_dump_bytes(pflash_init_fw_cb,
		    sizeof(FLASH_INIT_FW_CTRL_BLK));)

		/* Update internal (cached) iSNS info */
		if (pisns_server->IPAddr.Type == EXT_DEF_TYPE_ISCSI_IPV6) {
			drvr_ha->ipv6_tcp_options = ipv6_tcp_options;
			if (ipv6_tcp_options & IPV6_TCPOPT_ISNSv6_ENABLE)
				set_bit(ISNS_FLAG_ISNS_ENABLED_IN_ISP,
					&drvr_ha->isns.flags);
			else
				clear_bit(ISNS_FLAG_ISNS_ENABLED_IN_ISP,
					&drvr_ha->isns.flags);
		} else {
			drvr_ha->tcp_options = tcp_options;
			if (tcp_options & TOPT_ISNSv4_ENABLE)
				set_bit(ISNS_FLAG_ISNS_ENABLED_IN_ISP,
					&drvr_ha->isns.flags);
			else
				clear_bit(ISNS_FLAG_ISNS_ENABLED_IN_ISP,
					&drvr_ha->isns.flags);
		}

		drvr_ha->ql4_isns_populate_server_ip(drvr_ha,
					&pflash_init_fw_cb->init_fw_cb);
		is_isns_active = drvr_ha->ql4_is_isns_active(drvr_ha);

		/* Start/Restart/Stop iSNS server, if requested */
		if ((pisns_server->FeaturesRequest &
		     EXT_DEF_IMMEDIATE_ISNS_CONFIG) != 0) {
			if (!ql4_is_memzero(&ip[0], sizeof(ip))) {
				status = drvr_ha->ql4_isns_start_svc(drvr_ha);
			} else if (is_isns_active == ISNS_STATUS_ACTIVE) {
				status = drvr_ha->ql4_isns_stop_svc(drvr_ha);
			}
			if (status == QLA_SUCCESS) {
				pisns_server->FeaturesResponse |=
					EXT_DEF_IMMEDIATE_ISNS_CONFIG;
			} else {
				pisns_server->FeaturesResponse &=
					~EXT_DEF_IMMEDIATE_ISNS_CONFIG;
			}
		}
	} else {
		DEBUG2(printk("%s: Nothing changed! "
			"Don't write flash\n", __func__));
	}

	/*
         * Return iSNS Connection Status , if requested.
         *---------------------------------------------------*/
	#if defined(QL_DEBUG_LEVEL_2) || defined(QL_DEBUG_LEVEL_4)
	{
	       memset(mbox_cmd, 0, sizeof(mbox_cmd));
	       memset(mbox_sts, 0, sizeof(mbox_sts));
	       mbox_cmd[0] = MBOX_CMD_GET_FW_STATE;

	       if (ql4im_ha->drvr_ha->ql4_mbx(ql4im_ha->drvr_ha, 1, 4, &mbox_cmd[0], &mbox_sts[0]) ==
		   QLA_SUCCESS) {
		       DEBUG4(printk("scsi%d: %s: GET ISNS SERVICE CONNECTION STATUS = \"%s\"\n",
			   ql4im_ha->host_no, __func__,
			   ((mbox_sts[3] & FW_ADDSTATE_ISNSv4_SVC_ENABLED) != 0)
			   ? "ACTIVE" : "INACTIVE"));
		       if ((pisns_server->FeaturesRequest & EXT_DEF_RETURN_ISNS_CONN_STATUS) != 0) {
			       pisns_server->iSNSServerConnOpen =
				       drvr_ha->addl_fw_state & FW_ADDSTATE_ISNSv4_SVC_ENABLED;
			       pisns_server->FeaturesResponse |= EXT_DEF_RETURN_ISNS_CONN_STATUS;
		       }
	       }
	       else {
		       pisns_server->FeaturesResponse &= ~EXT_DEF_RETURN_ISNS_CONN_STATUS;
		       DEBUG2(printk("scsi%d: %s: MBOX_CMD_GET_FW_STATE command "
			   "failed \n", ql4im_ha->host_no, __func__));
	       }
	}
	#endif

	/*
	 * Generate iSNS Status Change AEN to trigger App Layer to request
	 * iSNS Targets Buffer
	 *----------------------------*/
	ql4im_queue_isns_sts_chg_aen(ql4im_ha, ISNS_CHG_SERVER_OFFLINE);

	/*
	 * Complete IOCTL successfully
	 *----------------------------*/
	ioctl->Status = EXT_STATUS_OK;
	ioctl->ResponseLen = 0;

exit_set_isns_svr:
	clear_bit(ISNS_FLAG_DISABLE_IN_PROGRESS, &drvr_ha->isns.flags);
	ql4im_free_ioctl_scrap_mem(ql4im_ha);
	return(status);
#else
	ioctl->Status = EXT_STATUS_OK;
	ioctl->ResponseLen = 0;
	return (0);
#endif
}

/**************************************************************************
 * qla4extioctl_set_acb
 *	This routine modifies the address control block.
 *
 * Input:
 *	ql4im_ha    = adapter structure pointer.
 *	ioctl = IOCTL structure pointer.
 *
 * Output:
 *	None
 *
 * Returns:
 *	QLA_SUCCESS = success
 *	QLA_ERROR   = error
 *
 * Context:
 *	Kernel context.
 **************************************************************************/
static int
qla4extioctl_set_acb(ql4im_scsi_qla_host_t *ql4im_ha, EXT_IOCTL_ISCSI *ioctl)
{
	int	status = 0;
	uint32_t mbox_cmd[MBOX_REG_COUNT];
	uint32_t mbox_sts[MBOX_REG_COUNT];

	if (ql4im_ha->firmware_version[0] == 2) {
		DEBUG2(
		    printk("scsi%d: %s: Not supported by 2-series firmware.\n",
		    ql4im_ha->host_no, __func__));

		ioctl->Status = EXT_STATUS_UNSUPPORTED_SUBCODE;
		ioctl->ResponseLen = 0;
		goto exit_set_acb;
	}

	if (ioctl->RequestLen < EXT_DEF_ACB_SIZE) {
		DEBUG2(
		    printk("scsi%d: %s: requst buffer too small (%d/%xh)\n",
		    ql4im_ha->host_no, __func__, ioctl->RequestLen,
		    ioctl->RequestLen));

		status = (-ENOMEM);
		ioctl->Status = EXT_STATUS_BUFFER_TOO_SMALL;
		ioctl->ResponseLen = 0;
		goto exit_set_acb;
	}

	if (!ql4im_ha->ioctl_dma_bufv || !ql4im_ha->ioctl_dma_bufp ||
	    (ql4im_ha->ioctl_dma_buf_len < EXT_DEF_ACB_SIZE)) {
		if (ql4im_resize_ioctl_dma_buf(ql4im_ha, EXT_DEF_ACB_SIZE) !=
		    QLA_SUCCESS) {
			DEBUG2(
			    printk("scsi%d: %s: unable to allocate memory "
			    "for dma buffer.\n",
			    ql4im_ha->host_no, __func__));

			status = (-ENOMEM);
			ioctl->Status = EXT_STATUS_NO_MEMORY;
			ioctl->ResponseLen = 0;
			goto exit_set_acb;
		}
	}

	/*
	 * Transfer Data from the user's buffer to DMA buffer
	 */
	if (copy_from_user(ql4im_ha->ioctl_dma_bufv,
		Q64BIT_TO_PTR(ioctl->RequestAdr, ioctl->AddrMode), EXT_DEF_ACB_SIZE) != 0) {
		DEBUG2(
		    printk("scsi%d: %s: unable to copy data to user's "
		    "memory area\n", ql4im_ha->host_no, __func__));

		status = (-EFAULT);
		ioctl->Status = EXT_STATUS_COPY_ERR;
		ioctl->ResponseLen = 0;
		goto exit_set_acb;
	}

	memset(mbox_cmd, 0, sizeof(mbox_cmd));
	memset(mbox_sts, 0, sizeof(mbox_sts));
	mbox_cmd[0] = MBOX_CMD_SET_ACB;
	mbox_cmd[1] = ioctl->Instance;  /* Primary/Secondary */
	mbox_cmd[2] = LSDW(ql4im_ha->ioctl_dma_bufp);
	mbox_cmd[3] = MSDW(ql4im_ha->ioctl_dma_bufp);
	mbox_cmd[4] = ioctl->RequestLen;

	if (ql4im_ha->drvr_ha->ql4_mbx(ql4im_ha->drvr_ha, 5, 5, &mbox_cmd[0], &mbox_sts[0])
	    == QLA_ERROR) {
		DEBUG2( printk("scsi%d: %s: MBOX_CMD_SET_ACB "
				           "FAILED w/ status %04X\n",
                                           ql4im_ha->host_no, __func__, mbox_sts[0]));
		DEBUG2( printk("mbox[1] = 0x%04x\n"
					   "mbox[2] = 0x%04x\n"
					   "mbox[3] = 0x%04x\n"
					   "mbox[4] = 0x%04x\n",
                                           mbox_sts[1], mbox_sts[2],
                                           mbox_sts[3], mbox_sts[4]));

		ioctl->Status = EXT_STATUS_MAILBOX;
		ioctl->VendorSpecificStatus[0] = mbox_sts[0];
		ioctl->VendorSpecificStatus[1] = mbox_sts[1];
		ioctl->VendorSpecificStatus[2] = mbox_sts[2];
		ioctl->VendorSpecificStatus[3] = mbox_sts[3];
		ioctl->VendorSpecificStatus[4] = mbox_sts[4];
		ioctl->ResponseLen = 0;
		goto exit_set_acb;
	}

	ioctl->Status = EXT_STATUS_OK;
	ioctl->ResponseLen = mbox_sts[4];

	DEBUG4(printk("scsi%d: %s: Succeeded\n",
		ql4im_ha->host_no, __func__));
	DEBUG10(printk("scsi%d: ACB structure dump:\n", ql4im_ha->host_no));
	DEBUG10(ql4im_dump_bytes(ql4im_ha->ioctl_dma_bufv, EXT_DEF_ACB_SIZE);)

	/*
	 * Update local ifcb data
	 */

	memset(mbox_cmd, 0, sizeof(mbox_cmd));
	memset(mbox_sts, 0, sizeof(mbox_sts));
	mbox_cmd[0] = MBOX_CMD_GET_INIT_FW_CTRL_BLOCK;
	mbox_cmd[1] = 0;
	mbox_cmd[2] = LSDW(ql4im_ha->ioctl_dma_bufp);
	mbox_cmd[3] = MSDW(ql4im_ha->ioctl_dma_bufp);
        mbox_cmd[4] = ql4im_ha->ifcb_size;

	if (ql4im_ha->drvr_ha->ql4_mbx(ql4im_ha->drvr_ha, 4, 1, &mbox_cmd[0], &mbox_sts[0])
	    != QLA_SUCCESS) {
		DEBUG2(printk("qisioctl%d: %s: command  failed w/ status %04X\n",
                              ql4im_ha->host_no, __func__, mbox_sts[0]));
		ioctl->Status = EXT_STATUS_MAILBOX;
		ioctl->VendorSpecificStatus[0] = mbox_sts[0];
		goto exit_set_acb;
	}

	ql4im_update_local_ifcb(ql4im_ha->drvr_ha, ql4im_ha->ioctl_dma_bufv);
	status = QLA_SUCCESS;

exit_set_acb:
	return(status);
}

/**************************************************************************
 * qla4extioctl_set_data
 *	This routine calls set data IOCTLs based on the IOCTL Sub Code.
 *
 * Input:
 *	ql4im_ha    = adapter structure pointer.
 *	ioctl = IOCTL structure pointer.
 *
 * Output:
 *	None
 *
 * Returns:
 *	QLA_SUCCESS = success
 *	QLA_ERROR   = error
 *    	-EINVAL     = if the command is invalid
 *
 * Context:
 *	Kernel context.
 **************************************************************************/
static int qla4extioctl_set_data(ql4im_scsi_qla_host_t *ql4im_ha,
				 EXT_IOCTL_ISCSI *ioctl)
{
	switch (ioctl->SubCode) {
	case EXT_SC_RST_STATISTICS_GEN:
		return(qla4extioctl_rst_statistics_gen(ql4im_ha, ioctl));

	case EXT_SC_RST_STATISTICS_ISCSI:
		return(qla4extioctl_rst_statistics_iscsi(ql4im_ha, ioctl));

	case EXT_SC_SET_DEVICE_ENTRY_ISCSI:
		return(qla4extioctl_set_device_entry_iscsi(ql4im_ha, ioctl));

	case EXT_SC_SET_INIT_FW_ISCSI:
		return(qla4extioctl_set_init_fw_iscsi(ql4im_ha, ioctl));

	case EXT_SC_SET_ISNS_SERVER:
		return(qla4extioctl_set_isns_server(ql4im_ha, ioctl));

	case EXT_SC_SET_ACB:
		return(qla4extioctl_set_acb(ql4im_ha, ioctl));

	default:
		DEBUG2(printk("qisioctl%d: %s: unsupported set data sub-command "
		    "code (%X)\n",
		    ql4im_ha->host_no, __func__, ioctl->SubCode));

		ioctl->Status = EXT_STATUS_UNSUPPORTED_SUBCODE;
		ioctl->ResponseLen = 0;
		return(-EINVAL);
	}
	return(0);
}

/**************************************************************************
 * qla4xxx_ioctl_sleep_done
 *	This routine is the callback function to wakeup ioctl completion
 *	semaphore for the ioctl request that is waiting.
 *
 * Input:
 *   	sem - pointer to the ioctl completion semaphore.
 *
 * Output:
 *	None
 *
 * Returns:
 *	None
 *
 * Context:
 *	Kernel context.
 **************************************************************************/
static void
qla4xxx_ioctl_sleep_done (struct semaphore * sem)
{
	if (sem != NULL) {
		DEBUG4(printk("qisioctl: %s: wake up sem, count=%d.\n", __func__,
		    atomic_read(&sem->count)));
		up(sem);
	}
}

/**************************************************************************
 * qla4xxx_ioctl_sem_init
 *	This routine initializes the ioctl timer and semaphore used to wait
 *	for passthru completion.
 *
 * Input:
 *	ql4im_ha    = adapter structure pointer.
 *
 * Output:
 *	None
 *
 * Returns:
 *	None
 *
 * Context:
 *	Kernel context.
 **************************************************************************/
void
ql4im_ioctl_sem_init (ql4im_scsi_qla_host_t *ql4im_ha)
{
	init_timer(&(ql4im_ha->ioctl->ioctl_cmpl_timer));
	ql4im_ha->ioctl->ioctl_cmpl_timer.expires = jiffies +
			(ql4im_ha->ioctl->ioctl_tov * HZ);
	ql4im_ha->ioctl->ioctl_cmpl_timer.data =
		(ulong)&ql4im_ha->ioctl->ioctl_cmpl_sem;
	ql4im_ha->ioctl->ioctl_cmpl_timer.function =
                (void (*)(ulong))qla4xxx_ioctl_sleep_done;
	add_timer(&(ql4im_ha->ioctl->ioctl_cmpl_timer));
}

/**************************************************************************
 * qla4xxx_scsi_pass_done
 *	This routine resets the ioctl progress flag and wakes up the ioctl
 * 	completion semaphore.
 *
 * Input:
 *   	cmd - pointer to the passthru Scsi cmd structure which has completed.
 *
 * Output:
 *	None
 *
 * Returns:
 *	None
 *
 * Context:
 *	Interrupt context.
 **************************************************************************/
static void
qla4xxx_scsi_pass_done(struct scsi_cmnd *cmd)
{
	scsi_qla_host_t 	*drvr_ha = (scsi_qla_host_t *) cmd->device->host->hostdata;
	ql4im_scsi_qla_host_t 	*ql4im_ha;

	ql4im_ha = ql4im_get_adapter_handle(drvr_ha->instance);

	if (ql4im_ha) {
	/* First check to see if the command has previously timed-out
	 * because we don't want to get the up/down semaphore counters off.
	 */
		if (ql4im_ha->ioctl->ioctl_scsi_pass_in_progress == 1) {
			ql4im_ha->ioctl->ioctl_scsi_pass_in_progress = 0;
			ql4im_ha->ioctl->ioctl_tov = 0;
			ql4im_ha->ioctl->ioctl_err_cmd = NULL;

			if (del_timer(&ql4im_ha->ioctl->ioctl_cmpl_timer))
				up(&ql4im_ha->ioctl->ioctl_cmpl_sem);
		}
	}
	else {
		DEBUG2(printk("qisioctl%d: %s: NULL ql4im_ha handle\n",
		    ql4im_ha->host_no, __func__));
	}
	return;
}

/**************************************************************************
 * qla4extioctl_scsi_passthru
 *	This routine
 *
 * Input:
 *	ql4im_ha    = adapter structure pointer.
 *	ioctl = IOCTL structure pointer.
 *
 * Output:
 *	None
 *
 * Map of DMA Buffer:
 *    +-------------------------+
 *    | EXT_SCSI_PASSTHRU_ISCSI |
 *    +-------------------------+
 *    | [SCSI READ|WRITE data]  |
 *    +-------------------------+
 *
 * Returns:
 *	QLA_SUCCESS = success
 *	QLA_ERROR   = error
 *
 * Context:
 *	Kernel context.
 **************************************************************************/
static int
qla4extioctl_scsi_passthru(ql4im_scsi_qla_host_t *ql4im_ha, EXT_IOCTL_ISCSI *ioctl)
{
	int			status = 0;
	ddb_entry_t		*ddb_entry;
	int			i;
	EXT_SCSI_PASSTHRU_ISCSI	*pscsi_pass = NULL;
	struct scsi_device	*pscsi_device = NULL;
	struct scsi_cmnd	*pscsi_cmd = NULL;
	struct request		*request = NULL;
	srb_t			*srb;
	uint32_t		dma_buf_len;
	os_tgt_t		*tgt_entry;
	os_lun_t		*lun_entry;
	fc_port_t		*fcport;
	unsigned long  		wait_cnt;

	if (!ADAPTER_UP(ql4im_ha->drvr_ha)) {
		DEBUG2(printk("qisioctl%d: %s: command not pocessed, "
		    "adapter link down.\n",
		    ql4im_ha->host_no, __func__));
		ioctl->Status = EXT_STATUS_HBA_NOT_READY;
		ioctl->ResponseLen = 0;
		return(QLA_ERROR);
	}

	if (ql4im_get_ioctl_scrap_mem(ql4im_ha, (void **)&pscsi_cmd,
	    sizeof(struct scsi_cmnd))) {
		/* not enough memory */
		DEBUG2(printk("qisioctl%d: %s: inst=%d scrap not big enough. "
		    "size requested=%ld.\n",
		    ql4im_ha->host_no, __func__, ql4im_ha->instance,
		    (ulong)sizeof(struct scsi_cmnd)));

		status = (-ENOMEM);
		ioctl->Status = EXT_STATUS_NO_MEMORY;
		ioctl->ResponseLen = 0;
		goto error_exit_scsi_pass;
	}

	if (ql4im_get_ioctl_scrap_mem(ql4im_ha, (void **)&pscsi_device,
	    sizeof(struct scsi_device))) {
		/* not enough memory */
		ioctl->Status = EXT_STATUS_NO_MEMORY;
		DEBUG2(printk("qisioctl%d: %s: inst=%d scrap not big enough. "
		    "size requested=%ld.\n",
		    ql4im_ha->host_no, __func__, ql4im_ha->instance,
		    (ulong)sizeof(struct scsi_device)));

		status = (-ENOMEM);
		ioctl->Status = EXT_STATUS_NO_MEMORY;
		ioctl->ResponseLen = 0;
		goto error_exit_scsi_pass;
	}

	if (ql4im_get_ioctl_scrap_mem(ql4im_ha, (void **)&request,
	    sizeof(struct request))) {
		/* not enough memory */
		ioctl->Status = EXT_STATUS_NO_MEMORY;
		DEBUG2(printk("qisioctl%d: %s: inst=%d scrap not big enough. "
		    "size requested=%ld.\n",
		    ql4im_ha->host_no, __func__, ql4im_ha->instance,
		    (ulong)sizeof(struct request)));

		status = (-ENOMEM);
		ioctl->Status = EXT_STATUS_NO_MEMORY;
		ioctl->ResponseLen = 0;
		goto error_exit_scsi_pass;
	}
	
	if (ql4im_get_ioctl_scrap_mem(ql4im_ha, (void **)&pscsi_pass,
	    (sizeof(EXT_SCSI_PASSTHRU_ISCSI)))) {
		/* not enough memory */
		ioctl->Status = EXT_STATUS_NO_MEMORY;
		DEBUG2(printk("qisioctl%d: %s: inst=%d scrap not big enough. "
		    "size requested=%ld.\n",
		    ql4im_ha->host_no, __func__, ql4im_ha->instance,
		    (ulong)sizeof(EXT_SCSI_PASSTHRU_ISCSI)));

		status = (-ENOMEM);
		ioctl->Status = EXT_STATUS_NO_MEMORY;
		ioctl->ResponseLen = 0;
		goto error_exit_scsi_pass;
	}

	memset(pscsi_device, 0, sizeof(struct scsi_device));
	memset(pscsi_pass, 0, sizeof(EXT_SCSI_PASSTHRU_ISCSI));
	memset(pscsi_cmd, 0, sizeof(struct scsi_cmnd));
	pscsi_cmd->device = pscsi_device;
	pscsi_cmd->request = request;
	pscsi_cmd->request->nr_hw_segments = 1;

	/* ---- Get passthru structure from user space ---- */
	if (copy_from_user((uint8_t *)pscsi_pass,
                                     Q64BIT_TO_PTR(ioctl->RequestAdr,
						   ioctl->AddrMode),
                                     sizeof(EXT_SCSI_PASSTHRU_ISCSI)) != 0) {
		DEBUG2(printk("qisioctl%d: %s: unable to copy passthru struct "
		    "from user's memory area.\n",
		    ql4im_ha->host_no, __func__));

		status = (-EFAULT);
		ioctl->Status = EXT_STATUS_COPY_ERR;
		ioctl->ResponseLen = 0;
		goto error_exit_scsi_pass;
	}

	DEBUG10(printk("qisioctl%d: %s: incoming  EXT_SCSI_PASSTHRU_ISCSI structure:\n",
	    ql4im_ha->host_no, __func__));
	DEBUG10(ql4im_dump_bytes(pscsi_pass, sizeof(EXT_SCSI_PASSTHRU_ISCSI));)

	/* ---- Make sure device exists ---- */
	tgt_entry = ql4im_lookup_target_by_SCSIID(ql4im_ha->drvr_ha,
						    pscsi_pass->Addr.Bus,
	    pscsi_pass->Addr.Target);
	if (tgt_entry ==  NULL) {
		DEBUG2(printk("qisioctl%d: %s: unable to find target queue for "
		    "tgt %d.\n",
		    ql4im_ha->host_no, __func__, pscsi_pass->Addr.Target));

		status = (-EFAULT);
		ioctl->Status = EXT_STATUS_ERR;
		ioctl->ResponseLen = 0;
		goto error_exit_scsi_pass;
	}

	lun_entry = ql4im_lookup_lun_handle(ql4im_ha->drvr_ha, tgt_entry,
	    pscsi_pass->Addr.Lun);
	if (lun_entry ==  NULL) {
		fc_lun_t *fclun;

		/* ---- Create temporary lun --- */
		if (ql4im_get_ioctl_scrap_mem(ql4im_ha, (void **)&lun_entry,
                                                sizeof(*lun_entry))) {
			/* not enough memory */
                        DEBUG2(printk("qisioctl%d: %s: inst=%d scrap not big enough. "
			    "size requested=%ld.\n",
			    ql4im_ha->host_no, __func__, ql4im_ha->instance,
			    (ulong)sizeof(*lun_entry)));

			status = (-ENOMEM);
			ioctl->Status = EXT_STATUS_NO_MEMORY;
			ioctl->ResponseLen = 0;
			goto error_exit_scsi_pass;
		}

		if (ql4im_get_ioctl_scrap_mem(ql4im_ha, (void **)&fclun,
                                                sizeof(*fclun))) {
			/* not enough memory */
			ioctl->Status = EXT_STATUS_NO_MEMORY;
			DEBUG2(printk("qisioctl%d: %s: inst=%d scrap not big enough. "
			    "size requested=%ld.\n",
			    ql4im_ha->host_no, __func__, ql4im_ha->instance,
			    (ulong)sizeof(*fclun)));

			status = (-ENOMEM);
			ioctl->Status = EXT_STATUS_NO_MEMORY;
			ioctl->ResponseLen = 0;
			goto error_exit_scsi_pass;
                }

		memset(lun_entry, 0, sizeof(*lun_entry));
		memset(fclun, 0, sizeof(*fclun));

		fcport = tgt_entry->fcport;

		fclun->lun = pscsi_pass->Addr.Lun;
		fclun->fcport = fcport;
		fclun->device_type = TYPE_DISK;
		
		lun_entry->fclun = fclun;
		lun_entry->fclun->fcport = fcport;
		lun_entry->lun_state = LS_LUN_READY;
		spin_lock_init(&lun_entry->lun_lock);

		goto scsipt_lun_created;
	}

	if (lun_entry->fclun ==  NULL) {
		DEBUG2(printk("qisioctl%d: %s: unable to find fclun of lun queue "
		    "for lun %d.\n",
		    ql4im_ha->host_no, __func__, pscsi_pass->Addr.Lun));

		status = (-EFAULT);
		ioctl->Status = EXT_STATUS_ERR;
		ioctl->ResponseLen = 0;
		goto error_exit_scsi_pass;
	}

	fcport = lun_entry->fclun->fcport;
	if (fcport ==  NULL) {
		DEBUG2(printk("qisioctl%d: %s: unable to find fcport of lun queue "
		    "for lun %d.\n",
		    ql4im_ha->host_no, __func__, pscsi_pass->Addr.Lun));

		status = (-EFAULT);
		ioctl->Status = EXT_STATUS_ERR;
		ioctl->ResponseLen = 0;
		goto error_exit_scsi_pass;
	}

scsipt_lun_created:
	ddb_entry = fcport->ddbptr;
	if (ddb_entry == NULL) {
		DEBUG2(printk("qisioctl%d: %s: invalid device (b%d,t%d) specified.\n",
		    ql4im_ha->host_no, __func__,
		    pscsi_pass->Addr.Bus, pscsi_pass->Addr.Target));

		status = (-EINVAL);
		ioctl->Status = EXT_STATUS_DEV_NOT_FOUND;
		ioctl->ResponseLen = 0;
		goto error_exit_scsi_pass;
	}

	/* ---- Make sure device is in an active state ---- */
	if (ddb_entry->fw_ddb_device_state != DDB_DS_SESSION_ACTIVE) {
		DEBUG2(printk("qisioctl%d: %s: device (b%d,t%d) not in active state\n",
		    ql4im_ha->host_no, __func__,
		    pscsi_pass->Addr.Bus, pscsi_pass->Addr.Target));

		ioctl->Status = EXT_STATUS_DEVICE_NOT_READY;
		ioctl->ResponseLen = 0;
		goto error_exit_scsi_pass;
	}

	/* ---- Allocate larger DMA buffer, if neccessary ---- */
	dma_buf_len = MAX(ioctl->ResponseLen - sizeof(EXT_SCSI_PASSTHRU_ISCSI),
	    ioctl->RequestLen - sizeof(EXT_SCSI_PASSTHRU_ISCSI));

	if (ql4im_ha->ioctl_dma_buf_len < dma_buf_len &&
	    ql4im_resize_ioctl_dma_buf(ql4im_ha, dma_buf_len) != QLA_SUCCESS) {
		DEBUG2(printk("qisioctl%d: %s: ERROR cannot allocate requested "
		    "DMA buffer size 0x%x.\n",
		    ql4im_ha->host_no, __func__, dma_buf_len));

		status = (-ENOMEM);
		ioctl->Status = EXT_STATUS_NO_MEMORY;
		ioctl->ResponseLen = 0;
		goto error_exit_scsi_pass;
	}

	memset(ql4im_ha->ioctl_dma_bufv, 0, ql4im_ha->ioctl_dma_buf_len);

	/* ---- Retrieve srb from pool ---- */
	srb = ql4im_del_from_free_srb_q_head(ql4im_ha->drvr_ha);
	if (srb == NULL) {
		DEBUG2( printk("qisioctl%d: %s: srb not available\n",
		    ql4im_ha->host_no, __func__));
		goto error_exit_scsi_pass;
	}

	/* ---- Fill in the SCSI command structure ---- */
	pscsi_cmd->device->channel = pscsi_pass->Addr.Bus;
	pscsi_cmd->device->id = pscsi_pass->Addr.Target;
	pscsi_cmd->device->lun = pscsi_pass->Addr.Lun;
	pscsi_cmd->device = pscsi_device;
	pscsi_cmd->device->host = ql4im_ha->drvr_ha->host;
	pscsi_cmd->request_buffer = ql4im_ha->ioctl_dma_bufv;
	pscsi_cmd->scsi_done = qla4xxx_scsi_pass_done;
	pscsi_cmd->timeout_per_command = IOCTL_PASSTHRU_TOV * HZ;

	CMD_SP(pscsi_cmd) = (char *) srb;
	srb->cmd = pscsi_cmd;
	srb->fw_ddb_index = ddb_entry->fw_ddb_index;
	srb->lun = pscsi_cmd->device->lun;
	srb->flags = SRB_IOCTL_CMD;
	srb->tgt_queue = tgt_entry;
	srb->lun_queue = lun_entry;
	srb->fclun = lun_entry->fclun;
	srb->ha = fcport->ha;

	if (pscsi_pass->CdbLength == 6 || pscsi_pass->CdbLength == 10 ||
	    pscsi_pass->CdbLength == 12 || pscsi_pass->CdbLength == 16) {
		pscsi_cmd->cmd_len = pscsi_pass->CdbLength;
	} else {
		DEBUG2(printk("qisioctl%d: %s: Unsupported CDB length 0x%x \n",
		    ql4im_ha->host_no, __func__, pscsi_cmd->cmd_len));

		status = (-EINVAL);
		ioctl->Status = EXT_STATUS_INVALID_PARAM;
		ioctl->ResponseLen = 0;
		goto error_exit_scsi_pass_free_srb;
	}

	if (pscsi_pass->Direction == EXT_DEF_SCSI_PASSTHRU_DATA_IN) {
		pscsi_cmd->sc_data_direction = DMA_FROM_DEVICE;
		pscsi_cmd->request_bufflen = ioctl->ResponseLen -
		    sizeof(EXT_SCSI_PASSTHRU_ISCSI);

	} else if (pscsi_pass->Direction ==  EXT_DEF_SCSI_PASSTHRU_DATA_OUT) {
		pscsi_cmd->sc_data_direction = DMA_TO_DEVICE;
		pscsi_cmd->request_bufflen = ioctl->RequestLen -
		    sizeof(EXT_SCSI_PASSTHRU_ISCSI);

		/* Sending user data from ioctl->ResponseAddr to SCSI
		 * command buffer
		 */
		if (copy_from_user((uint8_t *)
					     pscsi_cmd->request_buffer,
					     Q64BIT_TO_PTR(ioctl->RequestAdr,
							   ioctl->AddrMode) +
                                             sizeof(EXT_SCSI_PASSTHRU_ISCSI),
                                             pscsi_cmd->request_bufflen) != 0) {
			DEBUG2(printk("qisioctl%d: %s: unable to copy write buffer "
			    "from user's memory area.\n",
			    ql4im_ha->host_no, __func__));

			status = (-EFAULT);
			ioctl->Status = EXT_STATUS_COPY_ERR;
			ioctl->ResponseLen = 0;
			goto error_exit_scsi_pass_free_srb;
		}
	} else {
		pscsi_cmd->sc_data_direction = DMA_NONE;
		pscsi_cmd->request_buffer  = NULL;
		pscsi_cmd->request_bufflen = 0;
	}

	memcpy(pscsi_cmd->cmnd, pscsi_pass->Cdb, pscsi_cmd->cmd_len);

	DEBUG4(printk("qisioctl%d:%d:%d:%d: %s: CDB = ",
                      ql4im_ha->host_no, pscsi_cmd->device->channel,
		      pscsi_cmd->device->id,
                       pscsi_cmd->device->lun, __func__));

	for (i = 0; i < pscsi_cmd->cmd_len; i++)
		DEBUG4(printk("%02X ", pscsi_cmd->cmnd[i]));

	DEBUG4(printk("\n"));

	/* ---- prepare for receiving completion ---- */
	ql4im_ha->ioctl->ioctl_scsi_pass_in_progress = 1;
	ql4im_ha->ioctl->ioctl_tov = pscsi_cmd->timeout_per_command;

	ql4im_ioctl_sem_init(ql4im_ha);
	CMD_COMPL_STATUS(pscsi_cmd)  = IOCTL_INVALID_STATUS;
	CMD_PASSTHRU_TYPE(pscsi_cmd) = (void *)1;

	/* ---- send command to adapter ---- */
	DEBUG4(printk("qisioctl%d:%d:%d:%d: %s: sending command.\n",
	    ql4im_ha->host_no, pscsi_cmd->device->channel, pscsi_cmd->device->id,
	    pscsi_cmd->device->lun, __func__));

	/* Attach the timer to srb and let the srb timer
	 * do the error recovery in case of timeout.
	 * Instead of blinding waiting for some sec's
	 * and expecting things to be sane, even though
	 * the low level has knowledge of the srb. Unfortunately
 	 * the ioctl's srb was being added to the retry_q
	 * etc post AEN event causing random panic.
	 */
	
	/* Time the command via our standard driver-timer */
        if ((pscsi_cmd->timeout_per_command / HZ) > MIN_CMD_TOV)
                ql4im_add_timer_to_cmd(srb,
                    (pscsi_cmd->timeout_per_command/HZ) - QLA_CMD_TIMER_DELTA);
        else
		srb->flags |= SRB_NO_TIMER;

	if (ql4im_send_command_to_isp(ql4im_ha->drvr_ha, srb) != QLA_SUCCESS) {
		DEBUG2(printk("qisioctl%d: %s: error sending cmd to isp\n",
		    ql4im_ha->host_no, __func__));
		status = (-EINVAL);
		ioctl->Status = EXT_STATUS_DEV_NOT_FOUND;
		ioctl->ResponseLen = 0;
		goto error_exit_scsi_pass_free_srb;
	}

	down(&ql4im_ha->ioctl->ioctl_cmpl_sem);

	/*******************************************************
	 *						       *
	 *             Passthru Completion                     *
	 *						       *
	 *******************************************************/

	/* Poll for completion */
	wait_cnt = jiffies + IOCTL_PASSTHRU_TOV * HZ;
	while (!time_after_eq(jiffies, wait_cnt)) {

		if (ql4im_ha->ioctl->ioctl_scsi_pass_in_progress == 0) {
			break;
		}

		DEBUG2( printk("."));
		set_current_state(TASK_UNINTERRUPTIBLE);
		schedule_timeout(10);
	}

	/* ---- check for timeout --- */
	if (ql4im_ha->ioctl->ioctl_scsi_pass_in_progress == 1) {
		DEBUG2(printk("qisioctl%d: %s: ERROR = command timeout.\n",
		    ql4im_ha->host_no, __func__));

		ioctl->Status = EXT_STATUS_ERR;

		ql4im_ha->ioctl->ioctl_scsi_pass_in_progress = 0;
		goto error_exit_scsi_pass_free_srb;
	}

	/* NOTE: At this point cmd was completed in status_entry and
	 *       srb has already been returned to pool */

	/* --- Return info from status entry --- */
	ioctl->DetailStatus = CMD_SCSI_STATUS(pscsi_cmd);
	pscsi_pass->Reserved[0] = (uint8_t) CMD_SCSI_STATUS(pscsi_cmd);
	pscsi_pass->Reserved[1] = (uint8_t) CMD_COMPL_STATUS(pscsi_cmd);
	pscsi_pass->Reserved[2] = (uint8_t) CMD_ACTUAL_SNSLEN(pscsi_cmd);
	pscsi_pass->Reserved[3] = (uint8_t) CMD_HOST_STATUS(pscsi_cmd);
	pscsi_pass->Reserved[6] = (uint8_t) CMD_ISCSI_RESPONSE(pscsi_cmd);
	pscsi_pass->Reserved[7] = (uint8_t) CMD_STATE_FLAGS(pscsi_cmd);

	if (CMD_ACTUAL_SNSLEN(pscsi_cmd)) {
		memcpy(pscsi_pass->SenseData, pscsi_cmd->sense_buffer,
		    MIN(CMD_ACTUAL_SNSLEN(pscsi_cmd),
		    sizeof(pscsi_pass->SenseData)));

		DEBUG10(printk("qisioctl%d: %s: sense data dump:\n",
		    ql4im_ha->host_no, __func__));
		DEBUG10(ql4im_dump_bytes(pscsi_pass->SenseData,
					   sizeof(pscsi_pass->SenseData));)
	}

	/* ---- check for command completion --- */
	if (CMD_COMPL_STATUS(pscsi_cmd) == IOCTL_INVALID_STATUS) {
		DEBUG2(printk("qisioctl%d:%d:%d:%d: %s: ERROR = "
		    "command not completed.\n",
		    ql4im_ha->host_no, pscsi_cmd->device->channel,
		    pscsi_cmd->device->id,
		    pscsi_cmd->device->lun, __func__));

		ioctl->Status = EXT_STATUS_ERR;
		goto error_exit_scsi_pass;

	} else if (CMD_HOST_STATUS(pscsi_cmd) == DID_OK) {

		ioctl->Status = EXT_STATUS_OK;

	} else if (CMD_COMPL_STATUS(pscsi_cmd) == SCS_DATA_UNDERRUN) {
		DEBUG2(printk("qisioctl%d: %s: Data underrun.  Resid = 0x%x\n",
		    ql4im_ha->host_no, __func__, CMD_RESID_LEN(pscsi_cmd)));

		ioctl->Status = EXT_STATUS_DATA_UNDERRUN;
		pscsi_pass->Reserved[4] = MSB(CMD_RESID_LEN(pscsi_cmd));
		pscsi_pass->Reserved[5] = LSB(CMD_RESID_LEN(pscsi_cmd));

	} else if (CMD_COMPL_STATUS(pscsi_cmd) == SCS_DATA_OVERRUN) {
		DEBUG2(printk("qisioctl%d: %s: Data overrun.  Resid = 0x%x\n",
		    ql4im_ha->host_no, __func__, CMD_RESID_LEN(pscsi_cmd)));

		ioctl->Status = EXT_STATUS_DATA_OVERRUN;
		pscsi_pass->Reserved[4] = MSB(CMD_RESID_LEN(pscsi_cmd));
		pscsi_pass->Reserved[5] = LSB(CMD_RESID_LEN(pscsi_cmd));

	} else {
		DEBUG2(printk("qisioctl%d: %s: Command completed in ERROR. "
		    "cs=%04x, ss=%-4x\n", ql4im_ha->host_no, __func__,
		    CMD_COMPL_STATUS(pscsi_cmd), CMD_SCSI_STATUS(pscsi_cmd)));

		if (CMD_SCSI_STATUS(pscsi_cmd) != SCSI_GOOD) {
			ioctl->Status = EXT_STATUS_SCSI_STATUS;
		} else {
			ioctl->Status = EXT_STATUS_ERR;
		}
	}

	/* ---- Copy SCSI Passthru structure with updated sense buffer
	 *      to user space ----
	 */
	if (copy_to_user(Q64BIT_TO_PTR(ioctl->ResponseAdr, ioctl->AddrMode),
			 pscsi_pass, sizeof(EXT_SCSI_PASSTHRU_ISCSI)) != 0) {
		DEBUG2(printk("qisioctl%d: %s: unable to copy passthru struct "
		    "to user's memory area.\n",
		    ql4im_ha->host_no, __func__));

		status = (-EFAULT);
		ioctl->Status = EXT_STATUS_COPY_ERR;
		goto error_exit_scsi_pass;
	}

	DEBUG10(printk("qisioctl%d: %s: outgoing EXT_SCSI_PASSTHRU_ISCSI structure:\n",
	    ql4im_ha->host_no, __func__));
	DEBUG10(ql4im_dump_bytes(Q64BIT_TO_PTR(ioctl->ResponseAdr,
					       ioctl->AddrMode),
                                 sizeof(EXT_SCSI_PASSTHRU_ISCSI));)

	/* ---- Copy SCSI READ data from SCSI command buffer
	*       to user space ---- */
	if (pscsi_pass->Direction == EXT_DEF_SCSI_PASSTHRU_DATA_IN) {
		void __user *xfer_ptr = Q64BIT_TO_PTR(ioctl->ResponseAdr,
						  ioctl->AddrMode) +
				    sizeof(EXT_SCSI_PASSTHRU_ISCSI);
		uint32_t xfer_len = ioctl->ResponseLen -
				    sizeof(EXT_SCSI_PASSTHRU_ISCSI);


		/* Update ResponseLen if a data underrun occurred */
		if (CMD_COMPL_STATUS(pscsi_cmd) == SCS_DATA_UNDERRUN &&
		    CMD_RESID_LEN(pscsi_cmd)) {
			xfer_len -= CMD_RESID_LEN(pscsi_cmd);
		}

		if (copy_to_user(xfer_ptr, pscsi_cmd->request_buffer,
		    xfer_len) != 0) {
			DEBUG2(printk("qisioctl%d: %s: unable to copy READ data "
			    "to user's memory area.\n",
			    ql4im_ha->host_no, __func__));

			status = (-EFAULT);
			ioctl->Status = EXT_STATUS_COPY_ERR;
			goto error_exit_scsi_pass;
		}

		DEBUG10(printk("qisioctl%d: %s: outgoing READ data:  (0x%p)\n",
		    ql4im_ha->host_no, __func__, xfer_ptr));

		DEBUG10(ql4im_dump_bytes(xfer_ptr, xfer_len);)
	}

	goto exit_scsi_pass;

error_exit_scsi_pass_free_srb:
	ql4im_request_cleanup(ql4im_ha->drvr_ha, srb);

error_exit_scsi_pass:
	ioctl->ResponseLen = 0;

exit_scsi_pass:
	ql4im_free_ioctl_scrap_mem(ql4im_ha);
	return(status);
}

/**************************************************************************
 * qla4extioctl_iscsi_passthru
 *	This routine sends iSCSI pass-through to destination.
 *
 * Input:
 *	ql4im_ha    = adapter structure pointer.
 *	ioctl = IOCTL structure pointer.
 *
 * Output:
 *	None
 *
 * Returns:
 *	QLA_SUCCESS = success
 *	QLA_ERROR   = error
 *
 * Context:
 *	Kernel context.
 **************************************************************************/
static int
qla4extioctl_iscsi_passthru(ql4im_scsi_qla_host_t *ql4im_ha, EXT_IOCTL_ISCSI *ioctl)
{
	int status = 0;

	DEBUG4(printk("qisioctl%d: %s: UNSUPPORTED\n",
	    ql4im_ha->host_no, __func__));
	
	status = (-EINVAL);
	ioctl->Status = EXT_STATUS_UNSUPPORTED_SUBCODE;
	ioctl->ResponseLen = 0;
	return(status);
}

/**************************************************************************
 * qla4extioctl_disable_acb
 *	This routine disables the access control block
 *
 * Input:
 *	ql4im_ha    = adapter structure pointer.
 *	ioctl = IOCTL structure pointer.
 *
 * Output:
 *	None
 *
 * Returns:
 *	QLA_SUCCESS = success
 *	QLA_ERROR   = error
 *
 * Context:
 *	Kernel context.
 **************************************************************************/
static int
qla4extioctl_disable_acb(ql4im_scsi_qla_host_t *ql4im_ha, EXT_IOCTL_ISCSI *ioctl)
{
	int	status = 0;
	uint32_t mbox_cmd[MBOX_REG_COUNT];
	uint32_t mbox_sts[MBOX_REG_COUNT];

	memset(mbox_cmd, 0, sizeof(mbox_cmd));
	memset(mbox_sts, 0, sizeof(mbox_sts));
	mbox_cmd[0] = MBOX_CMD_DISABLE_ACB;
	mbox_cmd[1] = ioctl->Instance;  /* Primary/Secondary */
	mbox_cmd[2] = ioctl->Reserved1;	/* Command Option */

	if (ql4im_ha->drvr_ha->ql4_mbx(ql4im_ha->drvr_ha, 3, 1, &mbox_cmd[0], &mbox_sts[0])
	    == QLA_SUCCESS) {
		DEBUG4(printk("scsi%d: %s: Succeeded\n",
			ql4im_ha->host_no, __func__));

		ioctl->Status = EXT_STATUS_OK;
	} else {
		if (mbox_sts[0] == MBOX_STS_INTERMEDIATE_COMPLETION) {
			unsigned long  wait_cnt;

			/* Poll AF_WAIT4DISABLE_ACB_COMPLETE bit for
			 * AEN 8027 completion */
			wait_cnt = jiffies + WAIT_CMD_TOV * HZ;
			while (!time_after_eq(jiffies, wait_cnt)) {

				if (test_bit(AF_WAIT4DISABLE_ACB_COMPLETE,
					&ql4im_ha->drvr_ha->flags) == 0) {
					break;
				}

				DEBUG2( printk("."));
				set_current_state(TASK_UNINTERRUPTIBLE);
				schedule_timeout(1 * HZ);
			}

			if (test_bit(AF_WAIT4DISABLE_ACB_COMPLETE,
					&ql4im_ha->drvr_ha->flags) == 0) {
				DEBUG4(printk("scsi%d: %s: Succeeded\n",
				    ql4im_ha->host_no, __func__));
				ioctl->Status = EXT_STATUS_OK;
				ioctl->ResponseLen = 0;
				goto exit_disable_acb;
			} else {

				DEBUG2( printk("scsi%d: %s: "
				    "Intermediate Timed Out\n",
				    ql4im_ha->host_no, __func__));

				ioctl->Status = EXT_STATUS_PENDING;
				ioctl->ResponseLen = 0;
				goto exit_disable_acb;
			}
		} else {
			DEBUG2( printk("scsi%d: %s: FAILED w/ Status %04X\n",
			    ql4im_ha->host_no, __func__, mbox_sts[0]));

			ioctl->Status = EXT_STATUS_MAILBOX;
			ioctl->VendorSpecificStatus[0] = mbox_sts[0];
			ioctl->ResponseLen = 0;
		}
	}

exit_disable_acb:
	return(status);
}

/**************************************************************************
 * qla4extioctl_send_router_sol
 *	This routine requests the driver to send a router solicitation.
 *
 * Input:
 *	ql4im_ha    = adapter structure pointer.
 *	ioctl = IOCTL structure pointer.
 *
 * Output:
 *	None
 *
 * Returns:
 *	QLA_SUCCESS = success
 *	QLA_ERROR   = error
 *
 * Context:
 *	Kernel context.
 **************************************************************************/
static int
qla4extioctl_send_router_sol(ql4im_scsi_qla_host_t *ql4im_ha, EXT_IOCTL_ISCSI *ioctl)
{
	int	status = 0;
	EXT_SEND_ROUTER_SOL *psend_router_sol = NULL;
	uint32_t mbox_cmd[MBOX_REG_COUNT];
	uint32_t mbox_sts[MBOX_REG_COUNT];

	if (ioctl->RequestLen < sizeof(EXT_SEND_ROUTER_SOL)) {
		DEBUG2(
		    printk("scsi%d: %s: requst buffer too small (%d/%xh)\n",
		    ql4im_ha->host_no, __func__, ioctl->RequestLen,
		    ioctl->RequestLen));

		status = (-ENOMEM);
		ioctl->Status = EXT_STATUS_BUFFER_TOO_SMALL;
		ioctl->ResponseLen = 0;
		goto exit_send_ipv6_router_sol;
	}

	if (ql4im_get_ioctl_scrap_mem(ql4im_ha, (void **)&psend_router_sol,
					sizeof(*psend_router_sol))) {
		/* not enough memory */
		DEBUG2(printk("scsi%d: %s: inst=%d scrap not big enough. "
                              "size requested=%ld.\n",
                              ql4im_ha->host_no, __func__, ql4im_ha->instance,
                              (ulong)sizeof(*psend_router_sol)));

		status = (-ENOMEM);
		ioctl->Status = EXT_STATUS_NO_MEMORY;
		ioctl->ResponseLen = 0;
		goto exit_send_ipv6_router_sol;
	}

	/*
	 * Transfer Data from the user's buffer to DMA buffer
	 */
	if (copy_from_user(psend_router_sol,
                           Q64BIT_TO_PTR(ioctl->RequestAdr, ioctl->AddrMode),
			   sizeof(EXT_SEND_ROUTER_SOL)) != 0) {
		DEBUG2(
		    printk("scsi%d: %s: unable to copy data to user's "
		    "memory area\n", ql4im_ha->host_no, __func__));

		status = (-EFAULT);
		ioctl->Status = EXT_STATUS_COPY_ERR;
		ioctl->ResponseLen = 0;
		goto exit_send_ipv6_router_sol;
	}

	memset(mbox_cmd, 0, sizeof(mbox_cmd));
	memset(mbox_sts, 0, sizeof(mbox_sts));
	mbox_cmd[0] = MBOX_CMD_SEND_IPV6_ROUTER_SOL;
	mbox_cmd[1] = ioctl->Instance;  /* Primary/Secondary */
	mbox_cmd[2] = psend_router_sol->Flags;
	memcpy(&mbox_cmd[3], &psend_router_sol->Addr.IPAddress[0], 4);
	memcpy(&mbox_cmd[4], &psend_router_sol->Addr.IPAddress[4], 4);
	memcpy(&mbox_cmd[5], &psend_router_sol->Addr.IPAddress[8], 4);
	memcpy(&mbox_cmd[6], &psend_router_sol->Addr.IPAddress[12], 4);


	status = ql4im_ha->drvr_ha->ql4_mbx(ql4im_ha->drvr_ha, 7, 2, &mbox_cmd[0], &mbox_sts[0]);

	ioctl->VendorSpecificStatus[0] = mbox_sts[0];
	ioctl->VendorSpecificStatus[1] = mbox_sts[1];
	
	if (status == QLA_ERROR) {
		DEBUG2( printk("scsi%d: %s: "
					   "MBOX_CMD_SEND_IPV6_ROUTER_SOL "
				           "FAILED w/ status %04X\n",
                                           ql4im_ha->host_no, __func__, mbox_sts[0]));
		DEBUG2( printk("mbox[1] = 0x%04x\n", mbox_sts[1]));

		ioctl->Status = EXT_STATUS_MAILBOX;
		ioctl->ResponseLen = 0;
		goto exit_send_ipv6_router_sol;
	}

	ioctl->Status = EXT_STATUS_OK;
	ioctl->ResponseLen = mbox_sts[4];

	DEBUG4(printk("scsi%d: %s: Succeeded\n",
		ql4im_ha->host_no, __func__));

exit_send_ipv6_router_sol:
	ql4im_free_ioctl_scrap_mem(ql4im_ha);
	return(status);
}

/**************************************************************************
 * qla4extioctl_get_hbacnt
 *	This routine retrieves the number of supported HBAs found.
 *
 * Input:
 *	ql4im_ha    = adapter structure pointer.
 *	ioctl = IOCTL structure pointer.
 *
 * Output:
 *	None
 *
 * Returns:
 *	QLA_SUCCESS = success
 *	QLA_ERROR   = error
 *
 * Context:
 *	Kernel context.
 **************************************************************************/
static int
qla4extioctl_get_hbacnt(EXT_IOCTL_ISCSI *ioctl)
{
	int		status = 0;
	EXT_HBA_COUNT	hba_cnt;

	hba_cnt.HbaCnt = ql4im_get_hba_count();
	if ((status = copy_to_user(Q64BIT_TO_PTR(ioctl->ResponseAdr,
						 ioctl->AddrMode),
	    &hba_cnt, sizeof(hba_cnt))) != 0) {
		DEBUG2(printk("qisioctl: %s: failed to copy data\n", __func__));

		status = (-EFAULT);
		ioctl->Status = EXT_STATUS_COPY_ERR;
		ioctl->ResponseLen = 0;
		goto exit_get_hbacnt;
	}

	DEBUG4(printk("qisioctl: %s: hbacnt is %d\n", __func__, hba_cnt.HbaCnt));
	ioctl->Status = EXT_STATUS_OK;

exit_get_hbacnt:
	return(status);
}

static int qla4intioctl_diag_mode(ql4im_scsi_qla_host_t *ql4im_ha,
			 EXT_IOCTL_ISCSI *ioctl) {
	int status = 0;
	u_long wait_time, reset_time;
	struct scsi_qla_host *ha = ql4im_ha->drvr_ha;

	ioctl->Status = EXT_STATUS_OK;

	switch (ioctl->SubCode) {
	case INT_SC_SET_DIAG_MODE:
		DEBUG2(printk("%s: Set Quiescent mode\n", __func__));
		if (test_bit(DPC_QUIESCE_ACTIVE, &ha->dpc_flags)) {
			printk("%s: Device already in quiescent mode\n",
				__func__);
			break;
		}

		set_bit(AF_QUIESCE_OWNER, &ha->flags);
		set_bit(DPC_HA_NEED_QUIESCENT, &ha->dpc_flags);

		wait_time = jiffies + (30 * HZ);
		while (!time_after_eq(jiffies, wait_time)) {
			ssleep(1);
			if (test_bit(DPC_QUIESCE_ACTIVE, &ha->dpc_flags)) {
				ioctl->Status = EXT_STATUS_OK;
				break;
			}
		}

		/* AF_QUIESCE_OWNER is reset after qsnt mode is cleared */
		if (!test_bit(DPC_QUIESCE_ACTIVE, &ha->dpc_flags)) {
			printk("%s: Setting quiescent mode timed out, "
				"ha flags=0x%lx," " dpc_flags=0x%lx!\n",
				__func__, ha->flags, ha->dpc_flags);
			ioctl->Status = EXT_STATUS_ERR;
			status = QLA_ERROR;
		} else
			DEBUG2(printk("%s: Device quiescent mode set\n",
				__func__));
		break;

	case INT_SC_RESET_DIAG_MODE:
		if (!test_bit(AF_QUIESCE_OWNER, &ha->flags) &&
		    !test_bit(DPC_QUIESCE_ACTIVE, &ha->dpc_flags)) {
			printk("%s: Device not in qsnt mode "
				 "or function not quiesce owner, "
				 "ha->flags=0x%lx, ha->dpc_flags=0x%lx!\n",
				 __func__, ha->flags, ha->dpc_flags);
			break;
		}
		DEBUG2(printk("%s: Reset Quiescent mode\n", __func__));
		set_bit(DPC_RESET_QUIESCENT, &ha->dpc_flags);
		reset_time = jiffies + (30 * HZ);
		while (!time_after_eq(jiffies, reset_time)) {
			ssleep(1);
			if (!test_bit(DPC_QUIESCE_ACTIVE,
				&ha->dpc_flags)) {
				ioctl->Status = EXT_STATUS_OK;
				break;
			}
		}

		if (test_bit(DPC_QUIESCE_ACTIVE, &ha->dpc_flags)) {
			printk("%s: Resetting quiescent mode "
				 "timed out!!!\n", __func__);
			ioctl->Status = EXT_STATUS_ERR;
		} else
			DEBUG2(printk("%s: Device quiescent mode reset\n",
				__func__));
		clear_bit(AF_QUIESCE_OWNER, &ha->flags);
		break;

	default:
	       DEBUG2(printk("%s: Invalid Quiescent mode request\n",
		       __func__));
	       ioctl->Status = EXT_STATUS_INVALID_REQUEST;
	}

	return status;
}

/*
 * qla4intioctl_get_diag_data
 *  This routine is used to get diagnostics data from the f/w.
 *  Supports TEST_INTERNAL_LOOPBACK_DIAG and TEST_EXTERNAL_LOOPBACK_DIAG.
 *  Ioctl can be enhanced for other test codes when F/w supports it.
 *
 *  Returns:
 *  QLA_SUCCESS - command completed successfully, either with or without
 *          errors in the Status field of the main ioctl structure
 *      -EINVAL      - command is invalid
 *      -ENOMEM      - memory allocation failed
 */
static int qla4intioctl_get_diag_data(ql4im_scsi_qla_host_t *ql4im_ha,
                            EXT_IOCTL_ISCSI *ioctl) {
	int status = 0;
	INT_DIAG_TEST_RESULT results;
	uint32_t mbox_cmd[MBOX_REG_COUNT];
	uint32_t mbox_sts[MBOX_REG_COUNT];
	struct scsi_qla_host *ha;
	uint32_t config = 0;

	ha = ql4im_ha->drvr_ha;
	ioctl->Status = EXT_STATUS_OK;
	memset(&mbox_cmd, 0, sizeof(mbox_cmd));
	memset(&mbox_sts, 0, sizeof(mbox_sts));
	memset(&results, 0, sizeof(INT_DIAG_TEST_RESULT));

	if (!ioctl->RequestAdr ||
		ioctl->RequestLen > sizeof(INT_DIAG_TEST_RESULT)) {
		DEBUG2(printk("%s: memory area too small\n", __func__));
		ioctl->Status = EXT_STATUS_BUFFER_TOO_SMALL;
		status = -ENOMEM;
		goto exit_iscsi_diag;
	}

	if ((status = copy_from_user(&results,
		Q64BIT_TO_PTR(ioctl->RequestAdr, ioctl->AddrMode),
		ioctl->RequestLen) != 0)) {
		DEBUG2(printk("%s: unable to copy data from "
			"user's memory area status=%d\n", __func__, status));
		ioctl->Status = EXT_STATUS_COPY_ERR;
		goto exit_iscsi_diag;
	}

	/* Check if it's call for beacon operation */
	if (results.mbCmd[0] == MBOX_CMD_UPDATE_BEACON) {
		qla4intioctl_update_beacon(ql4im_ha, ioctl, results.mbCmd, mbox_sts);
		goto beacon_exit;
	}

	if (IS_QLA8032(ha)) {
		if (qla4_83xx_get_port_config(ha, &config) != QLA_SUCCESS) {
			DEBUG2(printk("%s: get port config failed\n",
					__func__));
			ioctl->Status = EXT_STATUS_MAILBOX;
			status = -EINVAL;
			goto exit_iscsi_diag;
		}
		DEBUG2(printk("%s: default config=%04X\n",
				__func__, config));

		if (results.mbCmd[1] == TEST_INTERNAL_LOOPBACK_DIAG)
			config |= ENABLE_INTERNAL_LOOPBACK;

		else if (results.mbCmd[1] == TEST_EXTERNAL_LOOPBACK_DIAG)
			config |= ENABLE_EXTERNAL_LOOPBACK;

		config &= ~ENABLE_DCBX;

		DEBUG2(printk("%s: setting new config=%04X\n",
				__func__, config));

		ha->notify_idc_comp = 1;
		ha->notify_link_up_comp = 1;

		if (qla4_83xx_set_port_config(ha, &config) != QLA_SUCCESS) {
			DEBUG2(printk("%s: set port config failed for "
					"config=%04X\n", __func__, config));
			ha->notify_idc_comp = 0;
			ha->notify_link_up_comp = 0;
			ioctl->Status = EXT_STATUS_MAILBOX;
			status = -EINVAL;
			goto exit_iscsi_diag;
		}

		if (!wait_for_completion_timeout(&ha->idc_comp, (5 * HZ))) {
			DEBUG2(printk("%s: IDC Complete notification "
					"not received\n", __func__));
			ha->notify_idc_comp = 0;
			ha->notify_link_up_comp = 0;
			status = -EINVAL;
			goto restore;
		} else {
			DEBUG2(printk("%s: IDC Complete notification "
					"received\n", __func__));
		}
		ha->notify_idc_comp = 0;

		if (!wait_for_completion_timeout(&ha->link_up_comp, (30 * HZ))) {
			DEBUG2(printk("%s: Link UP notification not "
					"received\n", __func__));
			ha->notify_link_up_comp = 0;
			status = -EINVAL;
			goto restore;
		} else {
			DEBUG2(printk("%s: Link UP notification "
					"received\n", __func__));
		}
		ha->notify_link_up_comp = 0;
	}

	/*
	 * Send the mailbox command
	 */
	mbox_cmd[0] = MBOX_CMD_DIAGNOSTICS_TEST_RESULTS;
	mbox_cmd[1] = results.mbCmd[1];

	switch (mbox_cmd[1])  {
	case TEST_INTERNAL_LOOPBACK_DIAG:
	case TEST_EXTERNAL_LOOPBACK_DIAG:
	       /* Update mbox[2-7] with src_addr, dest_addr, transfer_size,
		* flags from ioctl and call ha->ql4mbx
		*/
		mbox_cmd[2] = results.mbCmd[2]; /* Source low_addr */
		mbox_cmd[3] = results.mbCmd[3]; /* source high_addr */
		mbox_cmd[4] = results.mbCmd[4]; /* destination low_addr */
		mbox_cmd[5] = results.mbCmd[5]; /* destination high_addr */
		mbox_cmd[6] = results.mbCmd[6]; /* transfer size */
		mbox_cmd[7] = results.mbCmd[7]; /* flags */
		break;

	default:
		DEBUG2(printk("%s: diagnostics results "
		       "opcode %d not supported \n", __func__, mbox_cmd[1]));

		ioctl->Status = EXT_STATUS_UNSUPPORTED_SUBCODE;
		status = -EINVAL;
		goto exit_iscsi_diag;
	}
	ioctl->Status = EXT_STATUS_OK;

	if (ha->ql4_mbx(ha, MBOX_REG_COUNT, MBOX_REG_COUNT, &mbox_cmd[0],
			&mbox_sts[0]) != QLA_SUCCESS) {
		DEBUG2(printk("%s: command failed \n", __func__));

		ioctl->Status = EXT_STATUS_ERR;

		switch (mbox_cmd[1]) {
		case TEST_INTERNAL_LOOPBACK_DIAG:
		case TEST_EXTERNAL_LOOPBACK_DIAG:
		       ioctl->DetailStatus = mbox_sts[0]; /* Completion Stat*/
		       ioctl->Reserved1 = mbox_sts[1]; /* Failure Type */
		       DEBUG2(printk("%s: diagnostics results "
			       "completion_status=0x%x, failure_type=%d\n",
			       __func__, mbox_sts[0], mbox_sts[1]));
		       break;

		default:
		       break;
		}
	}

	if (IS_QLA8032(ha)) {
restore:
		/* Restore  to original state */
		if (qla4_83xx_get_port_config(ha, &config) != QLA_SUCCESS) {
			DEBUG2(printk("%s: restore get port config "
					"failed\n", __func__));
			ioctl->Status = EXT_STATUS_MAILBOX;
			status = -EINVAL;
			goto exit_iscsi_diag;
		}
		DEBUG2(printk("%s: restore default config=%04X\n",
				__func__, config));

		if (results.mbCmd[1] == TEST_INTERNAL_LOOPBACK_DIAG)
			config &= ~ENABLE_INTERNAL_LOOPBACK;
		else if (results.mbCmd[1] == TEST_EXTERNAL_LOOPBACK_DIAG)
			config &= ~ENABLE_EXTERNAL_LOOPBACK;

		config |= ENABLE_DCBX;

		DEBUG2(printk("%s: restoring new config=%04X\n",
				__func__, config));

		if (qla4_83xx_set_port_config(ha, &config) != QLA_SUCCESS) {
			DEBUG2(printk("%s: restore set port config "
					"failed for config=%04X\n",
					__func__, config));
			ioctl->Status = EXT_STATUS_MAILBOX;
			status = -EINVAL;
			goto exit_iscsi_diag;
		}
	}

beacon_exit:
	memcpy(results.mbSts, mbox_sts, sizeof(results.mbSts));
	if ((status = copy_to_user(
		Q64BIT_TO_PTR(ioctl->ResponseAdr, ioctl->AddrMode),
		&results, ioctl->ResponseLen) != 0)) {
		DEBUG2(printk("%s: unable to copy data to "
			"user's memory area status=%d\n", __func__, status));
		ioctl->Status = EXT_STATUS_COPY_ERR;
	}

exit_iscsi_diag:
	DEBUG2(printk("%s: diagnostics/beacon results status=%d, "
		"ioctl->status=%d, detailed status=0x%x, failure_type=%d\n",
		__func__, status, ioctl->Status, mbox_sts[0], mbox_sts[1]));

	return status;
}

static int
qla4extioctl_get_hostno(ql4im_scsi_qla_host_t *ql4im_ha, EXT_IOCTL_ISCSI *ioctl)
{
	int status = 0;

	ioctl->HbaSelect = ql4im_ha->host_no;
	ioctl->Status = EXT_STATUS_OK;

	DEBUG4(printk("qisioctl%d: %s: instance is %d\n",
	    ql4im_ha->host_no, __func__, ql4im_ha->instance));

	return(status);
}

static int
qla4extioctl_driver_specific(ql4im_scsi_qla_host_t *ql4im_ha, EXT_IOCTL_ISCSI *ioctl)
{
	int			status = 0;
	EXT_LN_DRIVER_DATA      data;

	if (ioctl->ResponseLen < sizeof(EXT_LN_DRIVER_DATA)) {
		status = (-ENOMEM);
		ioctl->Status = EXT_STATUS_BUFFER_TOO_SMALL;
		ioctl->ResponseLen = 0;

		DEBUG2(printk("qisioctl: %s: ERROR ResponseLen too small.\n", __func__));

		goto exit_driver_specific;
	}

	data.DrvVer.Major = ql4im_ha->drv_major;
	data.DrvVer.Minor = ql4im_ha->drv_minor;
	data.DrvVer.Patch = ql4im_ha->drv_patch;
	data.DrvVer.Beta  = ql4im_ha->drv_beta;

	if (IS_QLA4010(ql4im_ha->drvr_ha))
		data.AdapterModel = EXT_DEF_QLA4010_DRIVER;
	else
		data.AdapterModel = EXT_DEF_QLA4022_DRIVER;

	if (copy_to_user(Q64BIT_TO_PTR(ioctl->ResponseAdr, ioctl->AddrMode),
			      &data, sizeof(EXT_LN_DRIVER_DATA)) != 0) {
		DEBUG2(printk("qisioctl: %s: ERROR copy resp buf\n", __func__));

		status = (-EFAULT);
		ioctl->Status = EXT_STATUS_COPY_ERR;
		ioctl->ResponseLen = 0;
	}

exit_driver_specific:
	return(status);
}

static ioctl_tbl_row_t IOCTL_CMD_TBL[] =
{
	{EXT_CC_QUERY, "EXT_CC_QUERY"},
	{EXT_CC_REG_AEN, "EXT_CC_REG_AEN"},
	{EXT_CC_GET_AEN, "EXT_CC_GET_AEN"},
	{EXT_CC_GET_DATA, "EXT_CC_GET_DATA"},
	{EXT_CC_SET_DATA, "EXT_CC_SET_DATA"},
	{EXT_CC_SEND_SCSI_PASSTHRU, "EXT_CC_SEND_SCSI_PASSTHRU"},
	{EXT_CC_SEND_ISCSI_PASSTHRU, "EXT_CC_SEND_ISCSI_PASSTHRU"},
	{EXT_CC_DISABLE_ACB, "EXT_CC_DISABLE_ACB"},
	{EXT_CC_SEND_ROUTER_SOL, "EXT_CC_SEND_ROUTER_SOL"},
	{EXT_CC_GET_HBACNT, "EXT_CC_GET_HBACNT"},
	{INT_CC_LOGOUT_ISCSI, "INT_CC_LOGOUT_ISCSI"},
	{EXT_CC_GET_HOST_NO, "EXT_CC_GET_HOST_NO"},
	{INT_CC_DIAG_PING, "INT_CC_DIAG_PING"},
	{INT_CC_DIAG_MODE, "INT_CC_DIAG_MODE"},
	{INT_CC_DIAG_TEST, "INT_CC_DIAG_TEST"},
	{INT_CC_GET_DATA, "INT_CC_GET_DATA"},
	{INT_CC_SET_DATA, "INT_CC_SET_DATA"},
	{INT_CC_HBA_RESET, "INT_CC_HBA_RESET"},
	{INT_CC_COPY_FW_FLASH, "INT_CC_COPY_FW_FLASH"},
	{INT_CC_IOCB_PASSTHRU, "INT_CC_IOCB_PASSTHRU"},
	{INT_CC_RESTORE_FACTORY_DEFAULTS, "INT_CC_RESTORE_FACTORY_DEFAULTS"},
	{0, "UNKNOWN"}
};

static ioctl_tbl_row_t IOCTL_SCMD_QUERY_TBL[] =
{
	{EXT_SC_QUERY_HBA_ISCSI_NODE, "EXT_SC_QUERY_HBA_ISCSI_NODE"},
	{EXT_SC_QUERY_HBA_ISCSI_PORTAL, "EXT_SC_QUERY_HBA_ISCSI_PORTAL"},
	{EXT_SC_QUERY_DISC_ISCSI_NODE, "EXT_SC_QUERY_DISC_ISCSI_NODE"},
	{EXT_SC_QUERY_DISC_ISCSI_PORTAL, "EXT_SC_QUERY_DISC_ISCSI_PORTAL"},
	{EXT_SC_QUERY_DRIVER, "EXT_SC_QUERY_DRIVER"},
	{EXT_SC_QUERY_FW, "EXT_SC_QUERY_FW"},
	{EXT_SC_QUERY_CHIP, "EXT_SC_QUERY_CHIP"},
	{EXT_SC_QUERY_IP_STATE, "EXT_SC_QUERY_IP_STATE"},
	{EXT_SC_QUERY_DEVICE_CURRENT_IP, "EXT_SC_QUERY_DEVICE_CURRENT_IP"},
	{0, "UNKNOWN"}
};

static ioctl_tbl_row_t IOCTL_SCMD_EGET_DATA_TBL[] =
{
	{EXT_SC_GET_STATISTICS_GEN, "EXT_SC_GET_STATISTICS_GEN"},
	{EXT_SC_GET_STATISTICS_ISCSI, "EXT_SC_GET_STATISTICS_ISCSI"},
	{EXT_SC_GET_DEVICE_ENTRY_ISCSI, "EXT_SC_GET_DEVICE_ENTRY_ISCSI"},
	{EXT_SC_GET_DEVICE_ENTRY_DEFAULTS_ISCSI, "EXT_SC_GET_DEVICE_ENTRY_DEFAULTS_ISCSI"},
	{EXT_SC_GET_INIT_FW_ISCSI, "EXT_SC_GET_INIT_FW_ISCSI"},
	{EXT_SC_GET_INIT_FW_DEFAULTS_ISCSI, "EXT_SC_GET_INIT_FW_DEFAULTS_ISCSI"},
	{EXT_SC_GET_ISNS_SERVER, "EXT_SC_GET_ISNS_SERVER"},
	{EXT_SC_GET_ISNS_DISCOVERED_TARGETS, "EXT_SC_GET_ISNS_DISCOVERED_TARGETS"},
	{EXT_SC_GET_ACB, "EXT_SC_GET_ACB"},
	{EXT_SC_GET_NEIGHBOR_CACHE, "EXT_SC_GET_NEIGHBOR_CACHE"},
	{EXT_SC_GET_DESTINATION_CACHE, "EXT_SC_GET_DESTINATION_CACHE"},
	{EXT_SC_GET_DEFAULT_ROUTER_LIST, "EXT_SC_GET_DEFAULT_ROUTER_LIST"},
	{EXT_SC_GET_LOCAL_PREFIX_LIST, "EXT_SC_GET_LOCAL_PREFIX_LIST"},
	{EXT_SC_GET_STATISTICS_ISCSI_BLOCK, "EXT_SC_GET_STATISTICS_ISCSI_BLOCK"},
	{EXT_SC_GET_ISNS_TARGETS_BUFFER, "EXT_SC_GET_ISNS_TARGETS_BUFFER"},
	{0, "UNKNOWN"}
};

static ioctl_tbl_row_t IOCTL_SCMD_ESET_DATA_TBL[] =
{
	{EXT_SC_RST_STATISTICS_GEN, "EXT_SC_RST_STATISTICS_GEN"},
	{EXT_SC_RST_STATISTICS_ISCSI, "EXT_SC_RST_STATISTICS_ISCSI"},
	{EXT_SC_SET_DEVICE_ENTRY_ISCSI, "EXT_SC_SET_DEVICE_ENTRY_ISCSI"},
	{EXT_SC_SET_INIT_FW_ISCSI, "EXT_SC_SET_INIT_FW_ISCSI"},
	{EXT_SC_SET_ISNS_SERVER, "EXT_SC_SET_ISNS_SERVER"},
	{EXT_SC_SET_ACB, "EXT_SC_SET_ACB"},
	{0, "UNKNOWN"}
};

char *IOCTL_TBL_STR(int cc, int sc)
{
	ioctl_tbl_row_t *r;
	int cmd;

	switch (cc) {
	case EXT_CC_QUERY:
		r = IOCTL_SCMD_QUERY_TBL;
		cmd = sc;
		break;
	case EXT_CC_GET_DATA:
		r = IOCTL_SCMD_EGET_DATA_TBL;
		cmd = sc;
		break;
	case EXT_CC_SET_DATA:
		r = IOCTL_SCMD_ESET_DATA_TBL;
		cmd = sc;
		break;
	case INT_CC_GET_DATA:
		r = IOCTL_SCMD_IGET_DATA_TBL;
		cmd = sc;
		break;
	case INT_CC_SET_DATA:
		r = IOCTL_SCMD_ISET_DATA_TBL;
		cmd = sc;
		break;

	default:
		r = IOCTL_CMD_TBL;
		cmd = cc;
		break;
	}

	while (r->cmd != 0) {
		if (r->cmd == cmd) break;
		r++;
	}
	return(r->s);

}

/**************************************************************************
 * qla4xxx_send_ioctl
 * 	This the main entry point for all ioctl requests
 *
 * Input:
 *	dev - pointer to SCSI device structure
 *	cmd - internal or external ioctl command code
 *	arg - pointer to the main ioctl structure
 *
 *	Instance field in ioctl structure - to determine which device to
 *	perform ioctl
 *	HbaSelect field in ioctl structure - to determine which adapter to
 *	perform ioctl
 *
 * Output:
 *	The resulting data/status is returned via the main ioctl structure.
 *
 *	When Status field in ioctl structure is valid for normal command errors
 * 	this function returns 0 (QLA_SUCCESS).
 *
 *      All other return values indicate ioctl/system specific error which
 *	prevented the actual ioctl command from completing.
 *
 * Returns:
 *	 QLA_SUCCESS - command completed successfully, either with or without
 *			errors in the Status field of the main ioctl structure
 *    	-EFAULT      - arg pointer is NULL or memory access error
 *    	-EINVAL      - command is invalid
 *    	-ENOMEM      - memory allocation failed
 *
 * Context:
 *	Kernel context.
 **************************************************************************/
int
qla4xxx_send_ioctl(struct scsi_device *dev, int cmd, void __user *arg)
{
	EXT_IOCTL_ISCSI *pioctl = NULL;
	ql4im_scsi_qla_host_t *ql4im_ha = NULL;
	int status = 0;	/* ioctl status; errno value when function returns */
	int tmp_stat;

	/* Catch any non-exioct ioctls */
	if (_IOC_TYPE(cmd) != QLMULTIPATH_MAGIC) {
		DEBUG2(printk("qisioctl: %s: invalid magic number received.\n",
		    __func__));

		status = (-EINVAL);
		goto exit_qla4xxx_ioctl;
	}

	//DEBUG4(printk("qisioctl: %s: received cmd %x.\n",
	//    __func__, cmd));

	/* Allocate ioctl structure buffer to support multiple concurrent
	 * entries. NO static structures allowed.
	 */
	pioctl = QL_KMEM_ZALLOC(sizeof(EXT_IOCTL_ISCSI));
	if (pioctl == NULL) {
		/* error */
		printk(KERN_WARNING "qisioctl: ERROR in main ioctl buffer allocation.\n");
		status = (-ENOMEM);
		goto exit_qla4xxx_ioctl;
	}

	/*
	 * Check to see if we can access the ioctl command structure
	 */
	if (!access_ok(VERIFY_WRITE, arg, sizeof(EXT_IOCTL_ISCSI))) {
		DEBUG2(printk("qisioctl: %s: EXT_IOCTL_ISCSI access error.\n",
		    __func__));

		status = (-EFAULT);
		goto exit_qla4xxx_ioctl;
	}

	/*
	 * Copy the ioctl command structure from user space to local structure
	 */
	if ((status = copy_from_user((uint8_t *)pioctl, arg,
	    sizeof(EXT_IOCTL_ISCSI)))) {
		DEBUG2(printk("qisioctl: %s: EXT_IOCTL_ISCSI copy error.\n",
		    __func__));

		goto exit_qla4xxx_ioctl;
	}

	DEBUG10(printk("EXT_IOCTL_ISCSI structure dump: \n");)
	DEBUG10(ql4im_dump_dwords(pioctl, sizeof(*pioctl));)

        /* check signature of this ioctl */
	if (memcmp(pioctl->Signature, EXT_DEF_REGULAR_SIGNATURE,
	    sizeof(EXT_DEF_REGULAR_SIGNATURE)) != 0) {
		DEBUG2(printk("qisioctl: %s: signature did not match. "
		    "received cmd=%x arg=%p signature=%s.\n",
		    __func__, cmd, arg, pioctl->Signature));
		pioctl->Status = EXT_STATUS_INVALID_PARAM;
		status = copy_to_user(arg, (void *)pioctl,
		    sizeof(EXT_IOCTL_ISCSI));

		goto exit_qla4xxx_ioctl;
	}

        /* check version of this ioctl */
        if (pioctl->Version > EXT_VERSION) {
                printk(KERN_WARNING
                    "qisioctl: ioctl interface version not supported = %d.\n",
                    pioctl->Version);

		pioctl->Status = EXT_STATUS_UNSUPPORTED_VERSION;
		status = copy_to_user(arg, (void *)pioctl,
		    sizeof(EXT_IOCTL_ISCSI));
		goto exit_qla4xxx_ioctl;
        }

	/*
	 * Retrieving HBA Count does not require dependance on an adapter instance.
	 * Apps need the ability to retrieve hba count without assuming that
	 * driver instance 0 is present. In a hot swap environment, instance 0
	 * may not be present.
	 */
	if (cmd == (int)EXT_CC_GET_HBACNT) {
		status = qla4extioctl_get_hbacnt(pioctl);
		/*
		 * Copy the updated ioctl structure back to the user
		 */
		tmp_stat = copy_to_user(arg, (void *)pioctl, sizeof(EXT_IOCTL_ISCSI));
		if (status == 0) {
			status = tmp_stat;
		}
		goto exit_qla4xxx_ioctl;
	}

	/*
	 * Get the adapter handle for the corresponding adapter instance
	 */
	ql4im_ha = ql4im_get_adapter_handle(pioctl->HbaSelect);
	if (ql4im_ha == NULL) {
		DEBUG2(printk("qisioctl: %s: NULL host adapter structure\n", __func__));

		pioctl->Status = EXT_STATUS_DEV_NOT_FOUND;
		status = copy_to_user(arg, (void *)pioctl,
		    sizeof(EXT_IOCTL_ISCSI));
		goto exit_qla4xxx_ioctl;
	}

	if (ql4im_ha->ioctl == NULL) {
		DEBUG2(printk("qisioctl: %s: NULL host adapter ioctl structure \n",
			      __func__));

		pioctl->Status = EXT_STATUS_DEV_NOT_FOUND;
		status = copy_to_user(arg, (void *)pioctl,
		    sizeof(EXT_IOCTL_ISCSI));
		goto exit_qla4xxx_ioctl;
	}

       	if (!test_bit(AF_ONLINE, &ql4im_ha->drvr_ha->flags)) {
		if (cmd == (int)INT_CC_GET_DATA &&
		    pioctl->SubCode == INT_SC_GET_CORE_DUMP) {
			/* Allow Core Dump IOCTLs if adapter is offline  */
		}
		else if (ql4im_wait_for_hba_online(ql4im_ha->drvr_ha) == QLA_ERROR) {
			DEBUG2(printk("qisioctl: %s: IOCTL (%s) not pocessed, "
				"adapter offline.\n", __func__, IOCTL_TBL_STR(cmd, pioctl->SubCode)));
			pioctl->Status = EXT_STATUS_HBA_NOT_READY;
			goto exit_qla4xxx_ioctl;
		}
	}

	DEBUG4(printk("qisioctl%d: ioctl+ (%s)\n", ql4im_ha->host_no,
	    IOCTL_TBL_STR(cmd, pioctl->SubCode)));

	/*
	 * Get permission to process ioctl command. Only one will proceed
	 * at a time.
	 */
	if (ql4im_down_timeout(&ql4im_ha->ioctl->ioctl_sem,
                               (INTERNAL_PASSTHRU_TOV * HZ)) != 0) {
		/* error timed out */
		DEBUG2(printk("%s: ERROR timeout getting ioctl "
		    "access. host no=%d.\n", __func__, pioctl->HbaSelect);)

		pioctl->Status = EXT_STATUS_BUSY;
		status = copy_to_user(arg, pioctl, sizeof(*pioctl));

		kfree(pioctl);
		return (status);
	}

	/*
	 * If the DPC is active, wait for it to complete before proceeding
	 */
	while (ql4im_ha->drvr_ha->dpc_active) {
		set_current_state(TASK_UNINTERRUPTIBLE);
		schedule_timeout(1*HZ);
	}

	ql4im_ha->drvr_ha->i_start = jiffies;
	ql4im_ha->drvr_ha->i_end = 0;
	ql4im_ha->drvr_ha->f_start = 0;
	ql4im_ha->drvr_ha->f_end = 0;

	/*
	 * Issue the ioctl command
	 */
	switch (cmd) {
	case EXT_CC_QUERY:
		status = qla4extioctl_query(ql4im_ha, pioctl);
		break;

	case EXT_CC_REG_AEN:
		status = qla4extioctl_reg_aen(ql4im_ha, pioctl);
		break;

	case EXT_CC_GET_AEN:
		status = qla4extioctl_get_aen(ql4im_ha, pioctl);
		break;

	case EXT_CC_GET_DATA:
		status = qla4extioctl_get_data(ql4im_ha, pioctl);
		break;

	case EXT_CC_SET_DATA:
		status = qla4extioctl_set_data(ql4im_ha, pioctl);
		break;

	case EXT_CC_SEND_SCSI_PASSTHRU:
		status = qla4extioctl_scsi_passthru(ql4im_ha, pioctl);
		break;

	case EXT_CC_SEND_ISCSI_PASSTHRU:
		status = qla4extioctl_iscsi_passthru(ql4im_ha, pioctl);
		break;

	case EXT_CC_DISABLE_ACB:
		status = qla4extioctl_disable_acb(ql4im_ha, pioctl);
		break;

	case EXT_CC_SEND_ROUTER_SOL:
		status = qla4extioctl_send_router_sol(ql4im_ha, pioctl);
		break;

	case EXT_CC_GET_HOST_NO:
		status = qla4extioctl_get_hostno(ql4im_ha, pioctl);
		break;

	case EXT_CC_DRIVER_SPECIFIC:
		status = qla4extioctl_driver_specific(ql4im_ha, pioctl);
		break;

	case INT_CC_RESTORE_FACTORY_DEFAULTS:
		status = qla4intioctl_restore_factory_defaults(ql4im_ha, pioctl);
		break;

	case INT_CC_LOGOUT_ISCSI:
		status = qla4intioctl_logout_iscsi(ql4im_ha, pioctl);
		break;

	case INT_CC_DIAG_MODE:
		status = qla4intioctl_diag_mode(ql4im_ha, pioctl);
		break;

	case INT_CC_DIAG_TEST:
		status = qla4intioctl_get_diag_data(ql4im_ha, pioctl);
		break;

	case INT_CC_DIAG_PING:
		status = qla4intioctl_ping(ql4im_ha, pioctl);
		break;

	case INT_CC_GET_DATA:
		status = qla4intioctl_get_data(ql4im_ha, pioctl);
		break;

	case INT_CC_SET_DATA:
		status = qla4intioctl_set_data(ql4im_ha, pioctl);
		break;

	case INT_CC_HBA_RESET:
		status = qla4intioctl_hba_reset(ql4im_ha, pioctl);
		break;

	case INT_CC_COPY_FW_FLASH:
		status = qla4intioctl_copy_fw_flash(ql4im_ha, pioctl);
		break;

	case INT_CC_IOCB_PASSTHRU:
		status = qla4intioctl_iocb_passthru(ql4im_ha, pioctl);
		break;

	default:
		DEBUG2(printk("qisioctl%d: %s: unsupported command code (%x)\n",
		    ql4im_ha->host_no, __func__, cmd));

		status = (-EINVAL);
		pioctl->Status = EXT_STATUS_UNSUPPORTED_SUBCODE;
	}

	/*
	 * Copy the updated ioctl structure back to the user
	 */
	tmp_stat = copy_to_user(arg, (void *)pioctl, sizeof(EXT_IOCTL_ISCSI));
	if (status == 0) {
		status = tmp_stat;
	}

	ql4im_ha->drvr_ha->i_end = jiffies;

	up(&ql4im_ha->ioctl->ioctl_sem);

	DEBUG4(printk("qisioctl%d: ioctl- (%s)\n", ql4im_ha->host_no,
	    IOCTL_TBL_STR(cmd, pioctl->SubCode)));

exit_qla4xxx_ioctl:

	if (pioctl)
		QL_KMEM_FREE(pioctl);

	return(status);
}

/*
 * Overrides for Emacs so that we almost follow Linus's tabbing style.
 * Emacs will notice this stuff at the end of the file and automatically
 * adjust the settings for this buffer only.  This must remain at the end
 * of the file.
 * ---------------------------------------------------------------------------
 * Local variables:
 * c-indent-level: 2
 * c-brace-imaginary-offset: 0
 * c-brace-offset: -2
 * c-argdecl-indent: 2
 * c-label-offset: -2
 * c-continued-statement-offset: 2
 * c-continued-brace-offset: 0
 * indent-tabs-mode: nil
 * tab-width: 8
 * End:
 */

